@@ -1,5 +1,24 @@
 #include "atlconf.h"
 
+/*
+ * Defined some macros for checking booleans passed by -Si
+ */
+#define SI_FKO 0
+#define SI_BOZOL1 1
+#define SI_ARCHDEF 2
+#define SI_IEEE 3
+#define SI_LATUNE 4
+#define SI_NOF77  5
+#define SI_NOCYGWIN 6
+#define SI_OMP 6
+#define SI_LAREF 7
+#define SI_ADCLOSEP 8
+#define SI_SKPTHRCHK 9
+#define SI_INDTHR 10
+#define SI_IS_TRUE(bv_, b_) (((bv_)>>(b_))&1)
+#define SI_SET_BIT(bv_, b_) (bv_) |= 1 << (b_)
+#define SI_UNSET_BIT(bv_, b_) (bv_) &= ~(1 << (b_))
+
 void PrintUsage(char *name, int iarg, char *arg)
 {
    fprintf(stderr, "\nERROR around arg %d (%s).\n", iarg,
@@ -31,6 +50,10 @@ void PrintUsage(char *name, int iarg, char *arg)
    fprintf(stderr, "   -f <#> : size (in KB) to flush before timing\n");
    fprintf(stderr,
            "   -t <#> : set # of threads (-1: autodect; 0: no threading)\n");
+   fprintf(stderr,
+           "   -tl <#> <list> : set # of threads, use list of affinity IDs\n");
+   fprintf(stderr,
+           "   -r <#>: set the number of floating point registers to #\n");
    fprintf(stderr, "   -m <mhz> : set clock rate\n");
    fprintf(stderr, "   -S[i/s] <handle> <val>  : special int/string arg\n");
    fprintf(stderr,
@@ -43,14 +66,16 @@ void PrintUsage(char *name, int iarg, char *arg)
            "          (eg., NEON, 3DNow!)\n");
    fprintf(stderr,
            "      -Si latune <1/0> : do/don't tune F77 LAPACK routines\n");
+      fprintf(stderr,
+        "      -Si indthr <0/1> : Don't/do force independent threads only\n");
       fprintf(stderr,
         "      -Si nof77 <0/1> : Have/don't have fortran compiler\n");
-      fprintf(stderr,
-           "   -tl <#> <list> : set # of threads, use list of affinity IDs\n");
       fprintf(stderr,
         "      -Si omp <0/1> : don'tuse/use OpenMP for threading\n");
+#if 0
       fprintf(stderr,
 "      -Si antthr <0/1/2> : nobuild/build/use Antoine's code for threading\n");
+#endif
       fprintf(stderr,
               "      -Si lapackref <0/1>: Netlib lapack is not/is unpacked\n");
       fprintf(stderr, "                           to $BLDdir/src/lapack/ref\n");
@@ -78,22 +103,16 @@ void GetFlags(int nargs,                /* nargs as passed into main */
               int *vec,                 /* Vector ISA extension bitfield */
               enum MACHTYPE *mach,     /* machine/arch to assume */
               int *mhz,                /* Clock rate in Mhz */
-              int *ptrbits             /* # of bits in ptr: 32/64 */,
+              int *ptrbits             /* # of bits in ptr: -32/32/64 */,
+              int *NREGS,
               int *nthreads,           /* # of threads */
               int **tids,              /* thread affinity ID list */
-              int *omp,                /* Build OpenMP version of threading? */
-              int *AntThr,             /* Build Antoine's threads? */
               char **comps,
               char **gccflags,        /* append flags for user-contrib gcc */
               char **outfile,
               char **srcdir,          /* path to top of source directory */
               char **bindir,          /* path to top of binary directory */
-              int *bozol1,            /* Use untuned L1 */
-              int *UseArchDef,        /* Use arch defaults */
-              int *IEEE,              /* enforce IEEE FP standard? */
-              int *latune,            /* Tune LAPACK? */
-              int *NoF77,
-              int *lapackref,
+              int *SIflag,
               char **f2cdefs,         /* F77-to-C interface defines */
               char **ecdefs,          /* extra cpp defines to add to CDEFS */
               char **pmake,           /* parallel make command */
@@ -113,10 +132,10 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    char ln[1024];
 
    *verb = 0;
+   *mach = MACHOther;
    *ADd = NULL;
    *srcdir = *bindir = NULL;
-    *bozol1 = 0;
-    *IEEE = *latune = *UseArchDef = 1;
+    *NREGS = 0;
     *flapack = NULL;
     *f77lib = NULL;
     *smaflags = *dmaflags = NULL;
@@ -130,15 +149,16 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    *flush = 0;
    *ptrbits = 0;
    *mhz = 0;
-   *mach = 0;
    *vec = 0;
    *asmb = 0;
    *OS = 0;
    *verb = 0;
-   *NoF77 = 0;
+   *SIflag = 0;
+   SI_SET_BIT(*SIflag, SI_IEEE);
+   SI_SET_BIT(*SIflag, SI_LATUNE);
+   SI_SET_BIT(*SIflag, SI_ARCHDEF);
    *nthreads = -1;
    *tids = NULL;
-   *omp = *AntThr = 0;
    *pmake = NULL;
    for (i=1; i < nargs; i++)
    {
@@ -146,6 +166,11 @@ void GetFlags(int nargs,                /* nargs as passed into main */
          PrintUsage(args[0], i, args[i]);
       switch(args[i][1])
       {
+      case 'r':
+         if (++i >= nargs)
+            PrintUsage(args[0], i, "out of arguments");
+         *NREGS = atoi(args[i]);
+         break;
       case 't':
          if (++i >= nargs)
             PrintUsage(args[0], i, "out of arguments");
@@ -162,6 +187,19 @@ void GetFlags(int nargs,                /* nargs as passed into main */
             }
          }
          break;
+      case 'A':
+         if (++i >= nargs)
+            PrintUsage(args[0], i, "out of arguments");
+         if(args[i][0] >= '0' && args[i][0] <= '9') /* giving a # */
+            *mach = atoi(args[i]);
+         else /* giving a architecture name */
+         {
+            for (k=1; k < NMACH; k++)
+               if (!strcmp(args[i], machnam[k]))
+                  break;
+            *mach = (k == NMACH) ? 0 : k;
+         }
+         break;
       case 'f':
          if (++i >= nargs)
             PrintUsage(args[0], i, "out of arguments");
@@ -177,11 +215,6 @@ void GetFlags(int nargs,                /* nargs as passed into main */
             PrintUsage(args[0], i, "out of arguments");
          *mhz = atoi(args[i]);
          break;
-      case 'A':
-         if (++i >= nargs)
-            PrintUsage(args[0], i, "out of arguments");
-         *mach = atoi(args[i]);
-         break;
       case 'V':
          if (++i >= nargs)
             PrintUsage(args[0], i, "out of arguments");
@@ -221,21 +254,79 @@ void GetFlags(int nargs,                /* nargs as passed into main */
          else
             sp = NewStringCopy(args[i]);
          if (!strcmp(sp0, "archdef"))
-            *UseArchDef = k;
+         {
+            if (k)
+            {
+               SI_SET_BIT(*SIflag, SI_ARCHDEF);
+               if (k == 2)
+                  SI_SET_BIT(*SIflag, SI_ADCLOSEP);
+            }
+            else
+               SI_UNSET_BIT(*SIflag, SI_ARCHDEF);
+         }
          else if (!strcmp(sp0, "ieee"))
-            *IEEE = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_IEEE);
+            else
+               SI_UNSET_BIT(*SIflag, SI_IEEE);
+         }
          else if (!strcmp(sp0, "bozol1"))
-            *bozol1 = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_BOZOL1);
+            else
+               SI_UNSET_BIT(*SIflag, SI_BOZOL1);
+         }
          else if (!strcmp(sp0, "latune"))
-            *latune = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_LATUNE);
+            else
+               SI_UNSET_BIT(*SIflag, SI_LATUNE);
+         }
          else if (!strcmp(sp0, "omp"))
-            *omp = k;
-         else if (!strcmp(sp0, "antthr"))
-            *AntThr = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_OMP);
+            else
+               SI_UNSET_BIT(*SIflag, SI_OMP);
+         }
+         else if (!strcmp(sp0, "latune"))
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_LATUNE);
+            else
+               SI_UNSET_BIT(*SIflag, SI_LATUNE);
+         }
          else if (!strcmp(sp0, "lapackref"))
-            *lapackref = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_LAREF);
+            else
+               SI_UNSET_BIT(*SIflag, SI_LAREF);
+         }
+         else if (!strcmp(sp0, "fko"))
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_FKO);
+            else
+               SI_UNSET_BIT(*SIflag, SI_FKO);
+         }
          else if (!strcmp(sp0, "nof77"))
-            *NoF77 = k;
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_NOF77);
+            else
+               SI_UNSET_BIT(*SIflag, SI_NOF77);
+         }
+         else if (!strcmp(sp0, "indthr"))
+         {
+            if (k)
+               SI_SET_BIT(*SIflag, SI_INDTHR);
+            else
+               SI_UNSET_BIT(*SIflag, SI_INDTHR);
+         }
          else if (!strcmp(sp0, "kern"))
             gcc3 = sp;
          else if (!strcmp(sp0, "ADdir") || !strcmp(sp0, "addir"))
@@ -326,7 +417,10 @@ void GetFlags(int nargs,                /* nargs as passed into main */
          {
             if (++i >= nargs)
                PrintUsage(args[0], i, "out of arguments");
-            *gccflags = args[i];
+            if (args[i-2][1] == 'C')
+               comps[k] = args[i];
+            else
+               *gccflags = args[i];
          }
          else
          {
@@ -377,7 +471,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    }
    *f2cdefs = fdefs;
    *ecdefs = cdefs;
-   if (*ptrbits != 32 && *ptrbits != 64)
+   if (*ptrbits != 32 && *ptrbits != -32 && *ptrbits != 64)
       *ptrbits = 0;
 }
 char *GetPtrbitsFlag(enum OSTYPE OS, enum MACHTYPE arch, int ptrbits,
@@ -392,7 +486,8 @@ char *GetPtrbitsFlag(enum OSTYPE OS, enum MACHTYPE arch, int ptrbits,
    if (MachIsIA64(arch))
       return(sp);
    if (MachIsMIPS(arch))
-      return((ptrbits == 64) ? "-mabi=64" : "-mabi=n32");
+      return(sp);  /* patch by Sebastien Villemot for modern Linux */
+/*      return((ptrbits == 64) ? "-mabi=64" : "-mabi=n32"); */
    if (MachIsS390(arch))
       return((ptrbits == 64) ? "-m64" : "-m31");
    if (!CompIsGcc(comp))
@@ -419,16 +514,96 @@ char *GetPtrbitsFlag(enum OSTYPE OS, enum MACHTYPE arch, int ptrbits,
    else if (OS == OSAIX)
       sp = (ptrbits == 64) ? "-maix64" : "-maix32";
    else if ((MachIsX86(arch) || MachIsPPC(arch) || MachIsUS(arch)) ||
-            arch == IbmPwr7 || arch == IbmPwr6 || arch == Pwre6500)
+            MachIsPWR(arch))
    {
       if (ptrbits == 64)
          sp = "-m64";
       else if (ptrbits == 32)
          sp = "-m32";
+      else if (ptrbits == -32)
+         sp = "-mx32";
    }
    return(sp);
 }
 
+char *GetArch(enum MACHTYPE mach, int ISAX, int SIflag, int ptrbits)
+{
+   char *ap;
+   int i, k;
+
+   i = strlen(machnam[mach]);
+   #if defined(__powerpc64__) && (__BYTE_ORDER__  ==  __ORDER_LITTLE_ENDIAN__)
+      i += 2; /* LE */
+   #endif
+   i += 2;   /* 64/32 */
+   if (ISAX)
+      i += strlen(ISAXNAM[ISAX]);
+   if (!SI_IS_TRUE(SIflag,SI_IEEE))
+      i += 7;  /* NONIEEE */
+   ap = malloc(i+1);
+   assert(ap);
+   k = sprintf(ap, "%s", machnam[mach]);
+   #if defined(__powerpc64__) && (__BYTE_ORDER__  ==  __ORDER_LITTLE_ENDIAN__)
+      k += sprintf(ap+k, "%dLE", ptrbits);
+   #else
+      k += sprintf(ap+k, "%d", ptrbits);
+   #endif
+   if (ISAX)
+      k += sprintf(ap+k, "%s", ISAXNAM[ISAX]);
+   if (!SI_IS_TRUE(SIflag,SI_IEEE))
+      sprintf(ap+k, "NONIEEE");
+   assert (k <= i);
+   return(ap);
+}
+/*
+ * RETURNS: string with path to archdef to use.  If np was exact match,
+ *   *BAD is zeroed, else set non-zero.
+ */
+char *GetArchDef(char *srcd, char *mach, int np, int *BAD)
+{
+   char *ap=NULL;
+   int k, i, l;
+   *BAD = 0;
+   k = l = strlen(srcd);
+   l += 1 + strlen(mach);
+   l += 4 + 8 + 2;        /* 4 digits np, 8 for .tar.bz2, 2 p & strtrm */
+   ap = malloc(l+1);
+   assert(ap);
+   assert(np < 10000);
+   if (srcd[k-1] == '/')
+      i = sprintf(ap, "%s%sp%d.tar.bz2", srcd, mach, np);
+   else
+      i = sprintf(ap, "%s/%sp%d.tar.bz2", srcd, mach, np);
+   assert(i < l);
+   if (!FileIsThere(ap))
+   {
+      int d;
+      for (d=1; d < 64; d++)
+      {
+         int p;
+         *BAD = 1;
+         p = np + d;
+         if (p < 10000)
+         {
+            i = sprintf(ap, "%s/%sp%d.tar.bz2", srcd, mach, p);
+            assert(i < l);
+            if (FileIsThere(ap))
+               return(ap);
+         }
+         p = np - d;
+         if (p > 0)
+         {
+            i = sprintf(ap, "%s/%sp%d.tar.bz2", srcd, mach, p);
+            assert(i < l);
+            if (FileIsThere(ap))
+               return(ap);
+         }
+      }
+      free(ap);
+      ap = NULL;
+   }
+   return(ap);
+}
 int main(int nargs, char **args)
 {
    enum OSTYPE OS;
@@ -438,24 +613,29 @@ int main(int nargs, char **args)
    int ptrbits, l2size;
    int delay=0;  /* change this to come from "special" ints in GetFlags */
    int THREADS=0;
+   int nreg=0;
    int Use3DNow=0;  /* this needs to come from getflags */
-   int ncpu, omp, AntThr, lapackref;
-   int USEDEFL1, USEARCHDEF, LATUNE, USEIEEE, USEMINGW;
+   int ncpu, omp=0, AntThr=0, lapackref;
+   int SIflag, USEDEFL1, USEMINGW, FKO, NOMATCH;
    int *tids;
    #define NPREC 4
    char pres[NPREC] = {'s', 'd', 'c', 'z'};
    char *targ, *sp, *pmake, *flapack, *ADd;
    char *comps[3*NCOMP], *comp, *flags, *srcdir, *blddir, *f2cdefs, *cdefs;
    char *outfile, *smaflags, *dmaflags, *f77lib, *gccflags, *goodgcc;
+   char *arch, *archdef;
    char targarg[256], ln[1024];
    FILE *fpout;
    char *adnames[NARDEF] = {"sKERNDEF", "dKERNDEF", "sMMDEF", "dMMDEF"};
 
    GetFlags(nargs, args, &verb, &OS, (enum ASMDIA*) &asmb, &vecexts, &mach,
-            &mhz, &ptrbits, &ncpu, &tids, &omp, &AntThr, comps, &gccflags,
-            &outfile, &srcdir, &blddir, &USEDEFL1, &USEARCHDEF, &USEIEEE,
-            &LATUNE, &nof77, &lapackref, &f2cdefs, &cdefs, &pmake, &flapack,
-            &smaflags, &dmaflags, &f77lib, &ADd, &l2size, &targ);
+            &mhz, &ptrbits, &nreg, &ncpu, &tids, comps, &gccflags,
+            &outfile, &srcdir, &blddir, &SIflag, &f2cdefs, &cdefs, &pmake,
+            &flapack, &smaflags, &dmaflags, &f77lib, &ADd, &l2size, &targ);
+   USEDEFL1 = SI_IS_TRUE(SIflag,SI_BOZOL1);
+   nof77 = SI_IS_TRUE(SIflag,SI_NOF77);
+   lapackref = SI_IS_TRUE(SIflag,SI_LAREF);
+   FKO = SI_IS_TRUE(SIflag,SI_FKO);
    if (ncpu > 1) THREADS = 1;
    if (!outfile)
       fpout = stdout;
@@ -463,11 +643,36 @@ int main(int nargs, char **args)
       fpout = fopen(outfile, "w");
    assert(fpout);
    assert(srcdir && blddir);
+/*
+ * If archdef directory not given, default to srcdir/CONFIG/ARCHS[/WIN64]
+ */
+   if (!ADd)
+   {
+      int l;
+      l = strlen(srcdir);
+      if (srcdir[l-1] == '/')
+         l--;
+      ADd = malloc(l+21);
+      assert(ADd);
+      strncpy(ADd, srcdir, l);
+      strcpy(ADd+l, "/CONFIG/ARCHS");
+      if (OS == OSWin64 && ptrbits == 64)
+         strcat(ADd, "/WIN64");
+   }
 /*
  * Update l2size, and set f2cdefs/cdefs if they are null
  */
-   if (!l2size) l2size = 4*1024*1024;
-   else l2size *= 1024;
+   if (!l2size)
+   {
+      if (mach == IntPhi)
+         l2size = 1024*1024;
+      else if (ptrbits == 64)
+         l2size = 32*1024*1024;
+      else
+         l2size = 4*1024*1024;
+   }
+   else
+      l2size *= 1024;
    if (!f2cdefs) f2cdefs = "";
 /*
  * Append any appended flags, and then we have just compilers and flags
@@ -529,7 +734,7 @@ int main(int nargs, char **args)
          ISAX = i;
 
    fprintf(fpout, "#  ----------------------------\n");
-   fprintf(fpout, "#  Make.inc for ATLAS3.10.2\n");
+   fprintf(fpout, "#  Make.inc for ATLAS3.11.41\n");
    fprintf(fpout, "#  ----------------------------\n\n");
 
    fprintf(fpout, "#  ----------------------------------\n");
@@ -540,13 +745,9 @@ int main(int nargs, char **args)
    fprintf(fpout, "#  -------------------------------------------------\n");
    fprintf(fpout, "#  Name indicating the platform to configure BLAS to\n");
    fprintf(fpout, "#  -------------------------------------------------\n");
-   fprintf(fpout, "   ARCH = %s", machnam[mach]);
-   fprintf(fpout, "%d", ptrbits);
-   if (ISAX)
-      fprintf(fpout, "%s", ISAXNAM[ISAX]);
-   if (!USEIEEE)
-      fprintf(fpout, "NONIEEE");
-   fprintf(fpout, "\n\n");
+   arch = GetArch(mach, ISAX, SIflag, ptrbits);
+   archdef = GetArchDef(ADd, arch, ncpu, &NOMATCH);
+   fprintf(fpout, "   ARCH = %s\n\n", arch);
 
    fprintf(fpout, "#  ----------------------------\n");
    fprintf(fpout, "#  Paths to various directories\n");
@@ -559,6 +760,7 @@ int main(int nargs, char **args)
    fprintf(fpout, "   LIBdir = $(BLDdir)/lib\n\n");
    fprintf(fpout, "   SYSdir = $(BLDdir)/tune/sysinfo\n");
    fprintf(fpout, "   GMMdir = $(BLDdir)/src/blas/gemm\n");
+   fprintf(fpout, "   AMMdir = $(BLDdir)/src/blas/ammm\n");
    fprintf(fpout, "   GMVdir = $(BLDdir)/src/blas/gemv\n");
    fprintf(fpout, "   GR1dir = $(BLDdir)/src/blas/ger\n");
    fprintf(fpout, "   L1Bdir = $(BLDdir)/src/blas/level1\n");
@@ -574,10 +776,13 @@ int main(int nargs, char **args)
    fprintf(fpout, "   L1Tdir = $(BLDdir)/tune/blas/level1\n");
    fprintf(fpout, "   L3Tdir = $(BLDdir)/tune/blas/level3\n");
    fprintf(fpout, "   FLAdir = $(BLDdir)/src/lapack/reference\n");
-   if (ADd)
-      fprintf(fpout, "   ADdir  = %s\n", ADd);
-   else
-      fprintf(fpout, "   ADdir  = $(SRCdir)/CONFIG/ARCHS\n");
+   fprintf(fpout, "   ADdir  = %s\n", ADd);
+   free(ADd);
+   if (FKO)
+   {
+      fprintf(fpout, "   FKOSRCdir = $(SRCdir)/iFKO\n");
+      fprintf(fpout, "   FKOBLDdir = $(BLDdir)/iFKO\n");
+   }
    fprintf(fpout, "\n");
 
    fprintf(fpout,
@@ -586,7 +791,7 @@ int main(int nargs, char **args)
 "#  Name and location of scripts for running executables during tuning\n");
    fprintf(fpout,
 "#  ---------------------------------------------------------------------\n");
-   fprintf(fpout, "   ATLRUN = $(BLDdir)/bin/ATLrun.sh\n");
+   fprintf(fpout, "   ATLRUN = $(BLDdir)/ATLrun.sh\n");
    fprintf(fpout, "   ATLFWAIT = $(BLDdir)/bin/xatlas_waitfile\n\n");
 
    fprintf(fpout, "#  ---------------------\n");
@@ -596,6 +801,7 @@ int main(int nargs, char **args)
    fprintf(fpout, "   CBLASlib = $(LIBdir)/libcblas.a\n");
    fprintf(fpout, "   F77BLASlib = $(LIBdir)/libf77blas.a\n");
    fprintf(fpout, "   LAPACKlib = $(LIBdir)/liblapack.a\n");
+   fprintf(fpout, "   UAMMlib = $(LIBdir)/libuamm.a\n");
    if (THREADS)
    {
       fprintf(fpout, "   PTCBLASlib = $(LIBdir)/libptcblas.a\n");
@@ -624,6 +830,10 @@ int main(int nargs, char **args)
    fprintf(fpout, "#  Architecture identifying flags\n");
    fprintf(fpout, "#  ------------------------------\n");
    fprintf(fpout, "   ARCHDEFS =");
+   if (ptrbits == 32)
+      fprintf(fpout, " -DATL_PTR32");
+   else
+      fprintf(fpout, " -DATL_PTR64");
    if (OS != OSOther)
       fprintf(fpout, " -DATL_OS_%s", osnam[OS]);
    if (mach != MACHOther)
@@ -660,7 +870,7 @@ int main(int nargs, char **args)
          fprintf(fpout, " %d", tids[k]);
    }
    else
-      fprintf(fpout, "TIDLIST=");
+      fprintf(fpout, "   TIDLIST=");
    fprintf(fpout, "\n\n");
 
    fprintf(fpout,
@@ -677,9 +887,18 @@ int main(int nargs, char **args)
 
    sprintf(ln, "%s/CONFIG/src/CompMake.txt", srcdir);
    DisplayFile(ln, fpout, 0);
-   fprintf(fpout, "   NPROC=%d\n", ncpu);
+/*
+ * For PHI, have plenty of cores, so don't press luck by using core that
+ * may be busy communicating with host
+ */
+   if (mach == IntPhi && ncpu > 4)
+      fprintf(fpout, "   NPROC=%d\n", ncpu-4);
+   else
+      fprintf(fpout, "   NPROC=%d\n", ncpu);
    fprintf(fpout, "   CDEFS = $(L2SIZE) $(INCLUDES) $(F2CDEFS) $(ARCHDEFS)");
-   if (!USEIEEE)
+   if (SI_IS_TRUE(SIflag,SI_FKO))
+      fprintf(fpout, " -DATL_FKO=1");
+   if (!SI_IS_TRUE(SIflag,SI_IEEE))
       fprintf(fpout, " -DATL_NONIEEE=1");
 /*
  * Dump -m32/-m64 to CDEFS if it is in normal flags, so that generic flags
@@ -691,6 +910,8 @@ int main(int nargs, char **args)
       fprintf(fpout, " -m32");
    else if (strstr(comps[NCOMP+DKC_], " -m64"))
       fprintf(fpout, " -m64");
+   else if (strstr(comps[NCOMP+DKC_], " -mx32"))
+      fprintf(fpout, " -mx32");
    if (cdefs) fprintf(fpout, " %s", cdefs);
    if (THREADS)
    {
@@ -739,10 +960,32 @@ int main(int nargs, char **args)
           comps[i][j-2] == 'c' && comps[i][j-1] == 'c')
          break;
    }
-   goodgcc = (i < NCOMP) ? comps[i] : "gcc";
-   fprintf(fpout, "   GOODGCC = %s", goodgcc);
+   if (i < NCOMP)
+      goodgcc = comps[i];
+   else
+      goodgcc = comps[GCC_] ? comps[GCC_] : "gcc";
+   if (mach == IntPhi)
+      fprintf(fpout, "   GOODGCC = icc");
+/*
+ * Get rid of any -ansi flag from gcc, since it causes cpp not to be able
+ * to handle # in arguments, which is needed for some assemblies, part. ARM
+ */
+   else
+   {
+      char *sp;
+      sp = strstr(goodgcc, "-ansi");
+      if (sp)
+         sp[0] = sp[1] = sp[2] = sp[3] = sp[4] = ' ';
+      fprintf(fpout, "   GOODGCC = %s", goodgcc);
+   }
    if (gccflags)
+   {
+      char *sp;
+      sp = strstr(gccflags, "-ansi");
+      if (sp)
+         sp[0] = sp[1] = sp[2] = sp[3] = sp[4] = ' ';
       fprintf(fpout, " %s", gccflags);
+   }
    GetGccVers(goodgcc, &i, &j, &k, &k);
    if (OSIsWin(OS) && ptrbits != 64)  /* stop gcc breaking ABI */
       fprintf(fpout, " -mstackrealign");
@@ -754,6 +997,11 @@ int main(int nargs, char **args)
          fprintf(fpout, " -fPIC");
    #endif
    fprintf(fpout, "\n");
+   if (FKO)
+   {
+      fprintf(fpout, "   FKO = $(FKOBLDdir)/fko\n");
+      fprintf(fpout, "   FKOC = $(FKOBLDdir)/fkoc\n");
+   }
    fprintf(fpout, "   KC = $(DKC)\n   KCFLAGS = $(CDEFS) $(DKCFLAGS)\n");
 
    fprintf(fpout, "   LDFLAGS =");
@@ -767,6 +1015,8 @@ int main(int nargs, char **args)
             fprintf(fpout, " -melf_i386");
          else if (ptrbits == 64)
             fprintf(fpout, " -melf_x86_64");
+         else if (ptrbits == -32)
+            fprintf(fpout, " -melf_x32_x86_64");
          if (OS == OSFreeBSD)
             fprintf(fpout, "_fbsd");
       }
@@ -787,6 +1037,8 @@ int main(int nargs, char **args)
       fprintf(fpout, " -nofor_main");
    if (USEMINGW)
       fprintf(fpout, "\n   ARCHIVER = $(BLDdir)/mgwar\n");
+   else if (mach == TI_C66_BM)
+      fprintf(fpout, "\n   ARCHIVER = ar6x # For TI_C66_BM.\n");
    else
       fprintf(fpout, "\n   ARCHIVER = ar\n");
    fprintf(fpout, "   ARFLAGS  = r\n");
@@ -806,7 +1058,10 @@ int main(int nargs, char **args)
    fprintf(fpout, "   TAR    = tar\n");
    fprintf(fpout, "   BZIP   = bzip2\n");
    fprintf(fpout, "   BUNZIP = bunzip2\n");
-   fprintf(fpout, "   PMAKE  = %s\n\n", pmake ? pmake : "$(MAKE)");
+   if (mach == IntPhi)
+      fprintf(fpout, "   PMAKE  = $(MAKE) -j 4\n\n");
+   else
+      fprintf(fpout, "   PMAKE  = %s\n\n", pmake ? pmake : "$(MAKE)");
 /*
  * Need to add libs to GetFlags and update GetSysLib to do this right
 */
@@ -824,8 +1079,10 @@ int main(int nargs, char **args)
    fprintf(fpout, "   SSLAPACKlib =  # set to serial system lapack\n");
    if (THREADS)
    {
-      if (OSIsWin(OS))
+      if (USEMINGW || (ptrbits == 32 && OSIsWin(OS)))
          fprintf(fpout, "   LIBS = -lkernel32 -lm\n\n");
+      else if (OSIsWin(OS))
+         fprintf(fpout, "   LIBS = -lm\n\n");
       else
          fprintf(fpout, "   LIBS = -lpthread -lm\n\n");
    }
@@ -833,11 +1090,11 @@ int main(int nargs, char **args)
       fprintf(fpout, "   LIBS = -lm\n\n");
 
    fprintf(fpout,
-   "#  --------------------------------------------------------------------\n");
+   "#  ----------------------------------------------------------\n");
    fprintf(fpout,
-   "#  Compiler names for architectural defaults and flags to atlas_install\n");
+   "#  Info for architectural defaults and flags to atlas_install\n");
    fprintf(fpout,
-   "#  --------------------------------------------------------------------\n");
+   "#  ----------------------------------------------------------\n");
    for (i=0; i < NCOMP; i++)
    {
       sp = NewStringCopy(COMPNAME[i]);
@@ -860,6 +1117,8 @@ int main(int nargs, char **args)
          }
          else sp = NewStringCopy("gcc");
       }
+      else if (CompIsClang(comps[i]))
+         sp = NewStringCopy((i==F77_) ? "gfortran" : "clang");
       else
       {
          sp = NameWithoutPath(comps[i]);
@@ -869,8 +1128,42 @@ int main(int nargs, char **args)
       fprintf(fpout, "%s\n", sp);
       free(sp);
    }
-   fprintf(fpout, "   INSTFLAGS = -1 %d -a %d -l %d\n\n",
-           USEDEFL1, USEARCHDEF, LATUNE);
+   fprintf(fpout, "   ADnew = %sp$(NPROC)\n", arch);
+   free(arch);
+   if (archdef)
+   {
+      fprintf(fpout, "   ADtar = %s\n", archdef);
+      i = strlen(archdef) - 8;
+      archdef[i] = '\0';
+      for (i--; i > 0 && archdef[i] != '/'; i--);
+      if (i)
+         i++;
+      fprintf(fpout, "   ADuse = $(BLDdir)/ARCHS/%s\n", archdef+i);
+   }
+   else
+   {
+      fprintf(fpout, "   ADtar = \n");
+      fprintf(fpout, "   ADuse = \n");
+   }
+   if (!SI_IS_TRUE(SIflag,SI_ARCHDEF) || !archdef)
+   {
+      if (USEDEFL1)
+         fprintf(fpout, "   ADtarg = do_bozoL1\n");
+      else
+         fprintf(fpout, "   ADtarg = do_nothing\n");
+   }
+   else if (NOMATCH && !SI_IS_TRUE(SIflag,SI_ADCLOSEP))
+      fprintf(fpout, "   ADtarg = do_basic\n");
+   else
+      fprintf(fpout, "   ADtarg = do_full\n");
+   if (!nreg)
+      fprintf(fpout, "   INSTFLAGS = -1 %d -a %d -l %d -A %d\n\n",
+              USEDEFL1, SI_IS_TRUE(SIflag,SI_ARCHDEF),
+              SI_IS_TRUE(SIflag,SI_LATUNE),  SI_IS_TRUE(SIflag,SI_INDTHR));
+   else
+      fprintf(fpout, "   INSTFLAGS = -1 %d -a %d -l %d -A %d -r %d\n\n",
+              USEDEFL1, SI_IS_TRUE(SIflag,SI_ARCHDEF),
+              SI_IS_TRUE(SIflag,SI_LATUNE), SI_IS_TRUE(SIflag,SI_INDTHR), nreg);
 
 fprintf(fpout,
    "#  -------------------------------------------------------------------\n");
