@@ -3,19 +3,18 @@
 /*
  * The number of results, and their ordering (this must match the file).
  */
-#define NBENCH 8
-#define SELKMM 0
-#define GENKMM 1
-#define KMM_NT 2
-#define KMM_TN 3
-#define KMV_N  4
-#define KMV_T  5
-#define KGER   6
-#define MM_BIG 7
-#define CLKRATE 8
+#define NBENCH 6
+#define KMV_N  0
+#define KMV_T  1
+#define KGER   2
+#define KGER2  3
+#define SELKMM 4
+#define MM_BIG 5
+#define CLKRATE 6
+
 
 char *BNCHNAMES[NBENCH] =
-   {"kSelMM", "kGenMM", "kMM_NT", "kMM_TN", "kMV_N", "kMV_T", "kGER", "BIG_MM"};
+   {"kMV_N", "kMV_T", "kGER", "kGER2", "kgemm", "BIG_MM"};
 
 void PrintUsage(char *name, int iarg, char *flag)
 {
@@ -85,9 +84,7 @@ double RunBigMM(char pre, double clkrate, int *N)
    double mf0, mf1;
    FILE *fpin;
 
-   offset = (pre == 's' || pre == 'd') ? 50 : 61;
-   sprintf(cmnd, "cd bin ; make x%cmmtst_big", pre);
-   assert(!system(cmnd));
+   offset = 67;
 /*
  * Don't run case taking longer than 1 minute, assuming 1flop/cycle
  */
@@ -102,15 +99,15 @@ double RunBigMM(char pre, double clkrate, int *N)
    {
       assert(n > 200);
       remove("big.out");
-      sprintf(cmnd, "./bin/x%cmmtst_big -n %d -Test 0 > big.out\n", pre, n);
+      sprintf(cmnd, "cd bin ; make IRun_%cbigmm nb=%d", pre, n);
       n -= 200;
 /*      fprintf(stderr, "cmnd='%s'", cmnd); */
    }
    while(system(cmnd));
    *N = n + 200;
 /*
- * This section parses xdmmtst output to get mflop; note that it is fragile,
- * so if we change the formatting of xdmmtst, we must change this!
+ * This section parses xdsl3time_ab output to get mflop; note it is fragile,
+ * so if we change the formatting of xdsl3time, we must change this!
  */
    fpin = fopen("big.out", "r");
    assert(fpin);
@@ -121,7 +118,6 @@ double RunBigMM(char pre, double clkrate, int *N)
       assert(fgets(res, 2048, fpin));
    mf0 = atof(res+offset);
    fprintf(stderr, "res+off=%s\n", res+offset);
-   assert(fgets(res, 2048, fpin));
    mf1 = atof(res+offset);
    fclose(fpin);
    fprintf(stderr, "BIG_MM N=%d, mf=%.2f,%.2f!\n", *N, mf0, mf1);
@@ -230,14 +226,12 @@ void PrintNameDefs(FILE *fpout)
    fprintf(fpout,
 "\nThe times labeled Reference are for ATLAS as installed by the authors.\n");
    fprintf(fpout, "NAMING ABBREVIATIONS:\n");
-   fprintf(fpout, "   kSelMM : selected matmul kernel (may be hand-tuned)\n");
-   fprintf(fpout, "   kGenMM : generated matmul kernel\n");
-   fprintf(fpout, "   kMM_NT : worst no-copy kernel\n");
-   fprintf(fpout, "   kMM_TN : best no-copy kernel\n");
-   fprintf(fpout, "   BIG_MM : large GEMM timing (usually N=1600); estimate of asymptotic peak\n");
    fprintf(fpout, "   kMV_N  : NoTranspose matvec kernel\n");
    fprintf(fpout, "   kMV_T  : Transpose matvec kernel\n");
    fprintf(fpout, "   kGER   : GER (rank-1 update) kernel\n");
+   fprintf(fpout, "   kGER2  : GER2 (rank-2 update) kernel\n");
+   fprintf(fpout, "   kSelMM : selected matmul kernel (may be hand-tuned)\n");
+   fprintf(fpout, "   BIG_MM : large GEMM timing (usually N=1600); estimate of asymptotic peak\n");
    fprintf(fpout,
            "Kernel routines are not called by the user directly, and their\n");
    fprintf(fpout, "performance is often somewhat different than the total\n");
