@@ -1,33 +1,9 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1998 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1998 R. Clint Whaley
  */
 #include "atlconf.h"
+#include "atlconf_misc.h"
 
 #include <time.h>
 #include <ctype.h>
@@ -198,7 +174,7 @@ void ATL_mprintf(int np, ...)
          vsprintf(ln, form, argptr);
          assert(strlen(ln) < 1024);/* sanity test only, will not stop overrun */
          va_end(argptr);
-         for (i=0; i < np; i++) if (fp[i]) fprintf(fp[i], ln);
+         for (i=0; i < np; i++) if (fp[i]) fprintf(fp[i], "%s", ln);
       #else
          for (i=0; i < np; i++) if (fp[i]) vfprintf(fp[i], form, argptr);
          va_end(argptr);
@@ -220,6 +196,20 @@ int GetFirstInt(char *ln)
    return(iret);
 }
 
+int GetFirstHex(char *ln)
+{
+   int i, iret=0;
+   for (i=0; ln[i]; i++)
+   {
+      if (isxdigit(ln[i]))
+      {
+         sscanf(ln+i, "%x", &iret);
+         break;
+      }
+   }
+   return(iret);
+}
+
 long long GetFirstLong(char *ln)
 {
    int i;
@@ -228,7 +218,22 @@ long long GetFirstLong(char *ln)
    {
       if (isdigit(ln[i]))
       {
-         sscanf(ln+i, "%ld", &iret);
+         sscanf(ln+i, "%Ld", &iret);
+         break;
+      }
+   }
+   return(iret);
+}
+
+long long GetFirstLongHex(char *ln)
+{
+   int i;
+   long long iret=0;
+   for (i=0; ln[i]; i++)
+   {
+      if (isxdigit(ln[i]))
+      {
+         sscanf(ln+i, "%lx", &iret);
          break;
       }
    }
@@ -268,6 +273,20 @@ int GetLastInt(char *ln)
    return(iret);
 }
 
+int GetLastHex(char *ln)
+{
+   int i, iret=0;
+   for (i=0; ln[i]; i++);
+   if (i > 0) for (i--; i > 0 && !isxdigit(ln[i]); i--);
+   if (i > 0 || (i == 0 && isxdigit(ln[0])))
+   {
+      while(isxdigit(ln[i]) && i > 0) i--;
+      if (!isxdigit(ln[i])) i++;
+      sscanf(ln+i, "%x", &iret);
+   }
+   return(iret);
+}
+
 long long GetLastLong(char *ln)
 {
    int i;
@@ -283,6 +302,89 @@ long long GetLastLong(char *ln)
    return(iret);
 }
 
+long long GetLastLongHex(char *ln)
+{
+   int i;
+   long iret=0;
+   for (i=0; ln[i]; i++);
+   if (i > 0) for (i--; i > 0 && !isxdigit(ln[i]); i--);
+   if (i > 0 || (i == 0 && isxdigit(ln[0])))
+   {
+      while(isxdigit(ln[i]) && i > 0) i--;
+      if (!isxdigit(ln[i])) i++;
+      sscanf(ln+i, "%lx", &iret);
+   }
+   return(iret);
+}
+
+long long GetLastLongWithRound(char *ln)
+/*
+ * Gets a long from end of line, assuming the line may be in decimal
+ * format.  If given 1.6 it will return 2.  However, if given 1e256,
+ * it will return 256!
+ */
+{
+   int i;
+   long iret=0;
+   double d;
+
+   for (i=0; ln[i]; i++);  /* find end of string */
+   if (!i)                 /* empty string */
+     return(0);            /* returns 0 */
+/*
+ * Now skip any non-digit info at end of string (eg., MHz)
+ */
+   for(i--; i > 0 && !isdigit(ln[i]); i--);
+   if (!isdigit(ln[i])) /* no digits in string */
+      return(0);        /* returns 0 */
+/*
+ * Go backwards until we find a non-digit to possibly end the number
+ */
+   for (i--; i > 0 && isdigit(ln[i]); i--);
+   if (i > 0)  /* may be more to this number */
+   {
+      if (ln[i] == '.')
+         for(i--; i >0 && isdigit(ln[i]); i--);
+   }
+   else if (ln[0] != '.' && !isdigit(ln[0]))
+      i = 1;
+   sscanf(ln+i, "%lf", &d);
+   iret = (int)(d+0.5);
+   return(iret);
+}
+
+char *ATL_fgetln(FILE *fpin)
+/*
+ * RETURNS: dynamically allocates string containing a line from fpin, regardless
+ *          of how long that line is; RETURNS NULL if EOF
+ */
+{
+   char *ln, *sp;
+   int len=128, i;
+   ln = malloc(len*sizeof(char));
+   assert(ln);
+   if (!fgets(ln, len, fpin))
+      return(NULL);
+   i = strlen(ln);
+/*
+ * Keep extending lnlen and reading in more chars until \n is found
+ */
+   while (ln[i-1] != '\n')
+   {
+      char *ln2;
+      len <<= 1;
+      ln2 = malloc(len*sizeof(char));
+      assert(ln2);
+      strcpy(ln2, ln);
+      free(ln);
+      ln = ln2;
+      if (!fgets(ln+i, len-i, fpin))
+         return(ln);
+      i += strlen(ln+i);
+   }
+   return(ln);
+}
+
 int fNumLines(char *fnam)
 {
    FILE *fp;
@@ -295,47 +397,6 @@ int fNumLines(char *fnam)
    return(i);
 }
 
-char *CmndResults(char *targ, char *cmnd)
-{
-   static char tnam[128];
-   static int FirstTime=1;
-   char ln[512];
-
-   if (FirstTime)
-   {
-      FirstTime = 0;
-      assert(tmpnam(tnam));
-   }
-   if (targ) sprintf(ln, "ssh %s \"%s\" > %s 2>&1 \n", targ, cmnd, tnam);
-   else sprintf(ln, "%s > %s 2>&1\n", cmnd, tnam);
-   if (!system(ln)) return(tnam);
-   else return(NULL);
-}
-
-int CmndOneLine(char *targ, char *cmnd, char *ln)
-/*
- * executes a system call with contents of cmnd, returns the output in ln;
- * Returns value returned by system call
- * if targ is set, we ssh to that machine
- */
-{
-   int i;
-   FILE *fp;
-   char *tnam;
-
-   ln[0] = '\0';
-   tnam = CmndResults(targ, cmnd);
-   if (tnam)
-   {
-      fp = fopen(tnam, "r");
-      assert(fp);
-      if (!fgets(ln, 2048, fp)) ln[0] = '\0';
-      fclose(fp);
-      return(0);
-   }
-   else ln[0] = '\0';
-   return(1);
-}
 
 int GetIntBeforeWord(char *word, char *ln)
 /*
@@ -369,7 +430,7 @@ void GetEnter(FILE *fpout)
 {
    char ln[128];
    fprintf(fpout, "---------- PRESS ENTER TO CONTINUE ---------- ");
-   fgets(ln, 128, stdin);
+   assert(fgets(ln, 128, stdin));
 }
 
 int DisplayFile(char *fnam, FILE *fpout, int nlines)
@@ -469,11 +530,18 @@ enum ARCHFAM ProbeArchFam(char *targ)
  */
 {
    enum ARCHFAM fam=AFOther;
-   char cmnd[256], res[256];
+   char *cmnd, *res;
    char *uname;
+   int i;
+
    uname = FindUname(targ);
+   i = strlen(uname) + 4;
+
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
    sprintf(cmnd, "%s -m", uname);
-   if ( !CmndOneLine(targ, cmnd, res) )
+   res = atlsys_1L(targ, cmnd, 0, 0);
+   if (res)
    {
       if (strstr(res, "ppc") || strstr(res, "Power Macintosh") ||
           strstr(res, "powerpc")) fam = AFPPC;
@@ -482,9 +550,14 @@ enum ARCHFAM ProbeArchFam(char *targ)
       else if (strstr(res, "ia64")) fam = AFIA64;
       else if (strstr(res, "mips")) fam = AFMIPS;
       else if (strstr(res, "arm")) fam = AFARM;
+      else if (strstr(res, "aarch64")) fam = AFARM;
+      else if (strstr(res, "s390")) fam = AFS390;
       else if ( strstr(res, "i686") || strstr(res, "i586") ||
                 strstr(res, "i486") || strstr(res, "i386") ||
-                strstr(res, "x86") || strstr(res, "x86_64") ) fam = AFX86;
+                strstr(res, "x86") || strstr(res, "x86_64")  ||
+                strstr(res, "k1om")
+               ) fam = AFX86;
+      free(res);
    }
 /*
  * Try uname -p if uname -m didn't work
@@ -492,7 +565,8 @@ enum ARCHFAM ProbeArchFam(char *targ)
    if (fam == AFOther)
    {
       sprintf(cmnd, "%s -p", uname);
-      if ( !CmndOneLine(targ, cmnd, res) )
+      res = atlsys_1L(targ, cmnd, 0, 0);
+      if (res)
       {
          if (strstr(res, "ppc") || strstr(res, "Power Macintosh") ||
              strstr(res, "powerpc")) fam = AFPPC;
@@ -504,8 +578,12 @@ enum ARCHFAM ProbeArchFam(char *targ)
                    strstr(res, "x86_64") ) fam = AFX86;
          else if (strstr(res, "mips")) fam = AFMIPS;
          else if (strstr(res, "arm")) fam = AFARM;
+         else if (strstr(res, "aarch64")) fam = AFARM;
+         else if (strstr(res, "s390")) fam = AFS390;
+         free(res);
       }
    }
+   free(cmnd);
    return(fam);
 }
 
@@ -594,6 +672,7 @@ char *GetPathWithoutName(char *file)
    for (lastslash=i=0; file[i]; i++)
       if (file[i] == '/')
          lastslash = i;
+   i = lastslash;
    ch = file[i];
    file[i] = '\0';
    sp = NewStringCopy(file);
@@ -641,74 +720,200 @@ int GetIntVers(char *str, int *nskip)
 
 void GetGccVers(char *gcc, int *comp, int *major, int *minor, int *patch)
 /*
- * comp: 0: gcc;  1: egcs;  2: pgcc, 3: apple's gcc
+ * comp: 0: gcc;  1: egcs;  2: pgcc, 3: apple's gcc, 4: clang
  */
 {
-   char ln[512], ln2[512];
+   char *cmnd, *res;
    int i, j;
 
    *comp = *major = *minor = *patch = -1;
-   sprintf(ln, "%s --version", gcc);
-   if (CmndOneLine(NULL, ln, ln2) == 0)
+   i = strlen(gcc) + 12;
+   cmnd = malloc(i * sizeof(char));
+   assert(cmnd);
+   sprintf(cmnd, "%s --version", gcc);
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
-      if (strstr(ln2, "Apple Computer"))
+      if (strstr(res, "clang") || strstr(res, "LLVM"))
+         *comp = 4;
+      else if (strstr(res, "Apple Computer") || strstr(res, "Apple Inc"))
          *comp = 3;
 /*
  *    Skip compiler name, which may have digits in it
  */
-      for (i=0; ln2[i] && !isspace(ln2[i]); i++);
-      *major = GetIntVers(ln2+i, &j); j += i;
+      for (i=0; res[i] && !isspace(res[i]); i++);
+      *major = GetIntVers(res+i, &j); j += i;
       if (*major != -1)
       {
-         *minor = GetIntVers(ln2+j, &i); j += i;
+         *minor = GetIntVers(res+j, &i); j += i;
          if (*minor != -1)
          {
-            *patch = GetIntVers(ln2+j, &i); j += i;
-            if (strstr(ln2, "egcs")) *comp = 1;
-            else if (strstr(ln2, "pgcc")) *comp = 2;
-            else *comp = 0;
+            *patch = GetIntVers(res+j, &i); j += i;
+            if (strstr(res, "egcs")) *comp = 1;
+            else if (strstr(res, "pgcc")) *comp = 2;
+            else if (*comp == -1) *comp = 0;
          }
       }
+      free(res);
    }
 }
 
+char *GetPathEnvVar(void)
+/*
+ * returns users path with inter-name spaces replaced by "\ "
+ * and dirs separated by spaces
+ */
+{
+   char *path, *pp, *p;
+   int i, n;
+
+   path = getenv("PATH");
+   if (!path)
+      return(NULL);
+
+   n = strlen(path);
+   p = pp = malloc((2*n+1)*sizeof(char));
+   assert(pp);
+   for (i=0; i < n; i++)
+   {
+      if (path[i] == ':')
+         *p++ = ' ';
+      else if (path[i] == ' ')
+      {
+         *p = '\\';
+	 p[1] = ' ';
+	 p += 2;
+      }
+      else
+         *p++ = path[i];
+   }
+   *p = '\0';
+   return(pp);
+}
+
 int CompIsGcc(char *comp)
 /*
  * Tries to detect if compiler is gcc w/o scoping name of compiler
+ * However, rejects compilers with c89 and c90 in name, since these
+ * guys turn off inline assembly, which hurts the ATLAS install on performance.
  */
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
+   char *cmpname;
+   int i;
 
+   cmpname = NameWithoutPath(comp);
+   if (strstr(cmpname, "c89") || strstr(cmpname, "c90"))
+   {
+      free(cmpname);
+      return(0);
+   }
+   free(cmpname);
+   i = strlen(comp) + 16;
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
    sprintf(cmnd, "%s --version", comp);
-   if (CmndOneLine(NULL, cmnd, res) == 0)
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
 /*
  *    The direct comps added because Ubuntu stopped printing (GCC) in the
  *    version line.  Don't want to search for just "gcc", since that will
  *    match with pgcc
  */
-      if (strstr(res, "(GCC)") || strstr(res, " GCC ") ||
-          strstr(res, "GNU Fortran") || strstr(res, "(GCC ") ||
-          (res[0] == 'g' && res[1] == 'c' && res[2] == 'c' && res[3] == ' ')
+      if (strstr(res, "(GCC)")  || strstr(res, " GCC ") ||
+          strstr(res, "gcc-mp") || strstr(res, "(GCC ") ||
+          strstr(res, "GNU Fortran") ||
+          (res[0] == 'g' && res[1] == 'c' && res[2] == 'c' &&
+           (res[3] == ' ' || res[3] == '-'))
          )
+      {
+         free(res);
          return(1);
+      }
+      free(res);
    }
    return(0);
 }
+
+int CompIsClang(char *comp)
+/*
+ * Tries to detect if compiler is clang w/o scoping name of compiler
+ */
+{
+   char *cmnd, *res;
+   char *cmpname;
+   int i, iret=0;
+
+   i = strlen(comp) + 16;
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
+   sprintf(cmnd, "%s --version", comp);
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
+   {
+      iret = (strstr(res, "clang") || strstr(res, "LLVM"));
+      free(res);
+   }
+   return(iret);
+}
+
+int CompIsMinGW(char *comp)
+/*
+ * Tries to detect if compiler is MinGW compiler
+ */
+{
+   if (CompIsGcc(comp))
+   {
+      char *cmnd, *res;
+      int i;
+      i = strlen(comp) + 24;
+      cmnd = malloc(sizeof(char)*i);
+      assert(cmnd);
+      sprintf(cmnd, "%s -v 2>&1 | fgrep mingw", comp);
+      res = atlsys_1L(NULL, cmnd, 0, 0);
+      free(cmnd);
+      if (res)
+      {
+         if (strstr(res, "mingw"))
+         {
+            free(res);
+            return(1);
+         }
+         free(res);
+      }
+   }
+   return(0);
+}
+
 int CompIsAppleGcc(char *comp)
 /*
  * Tries to detect if compiler is Apple's funked-up gcc
  */
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
 
    if (CompIsGcc(comp))
    {
-      res[0] = '\0';
+      int i;
+      i = strlen(comp) + 24;
+      cmnd = malloc(sizeof(char)*i);
+      assert(cmnd);
       sprintf(cmnd, "%s -v 2>&1 | fgrep Apple", comp);
-      CmndOneLine(NULL, cmnd, res);
-      if (strstr(res, "Apple"))
-         return(1);
+      res = atlsys_1L(NULL, cmnd, 0, 0);
+      free(cmnd);
+      if (res)
+      {
+         if (strstr(res, "Apple"))
+         {
+            free(res);
+            return(1);
+         }
+         free(res);
+      }
    }
    return(0);
 }
@@ -718,26 +923,46 @@ int CompIsMIPSpro(char *comp)
  * RETURNS: 1 if compiler is MIPSpro compiler, 0 otherwise
  */
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
+   int i;
 
+   i = strlen(comp) + 4;
+   cmnd = malloc(sizeof(char)*i);
+   assert(cmnd);
    sprintf(cmnd, "%s -v", comp);
-   if (CmndOneLine(NULL, cmnd, res) == 0)
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
       if (strstr(res, "MIPSpro Compiler"))
+      {
+         free(res);
          return(1);
+      }
+      free(res);
    }
    return(0);
 }
 
 int CompIsPathScale(char *comp)
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
+   int i;
 
+   i = strlen(comp) + 4;
+   cmnd = malloc(sizeof(char)*i);
+   assert(cmnd);
    sprintf(cmnd, "%s -v", comp);
-   if (CmndOneLine(NULL, cmnd, res) == 0)
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
       if (strstr(res, "PathScale"))
+      {
+         free(res);
          return(1);
+      }
+      free(res);
    }
    return(0);
 }
@@ -747,13 +972,23 @@ int CompIsSunWorkshop(char *comp)
  * RETURNS: 1 if compiler is Sun WorkShop compiler, 0 otherwise
  */
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
+   int i;
 
+   i = strlen(comp) + 4;
+   cmnd = malloc(sizeof(char)*i);
+   assert(cmnd);
    sprintf(cmnd, "%s -V", comp);
-   if (CmndOneLine(NULL, cmnd, res) == 0)
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
       if (strstr(res, "Sun WorkShop"))
+      {
+         free(res);
          return(1);
+      }
+      free(res);
    }
    return(0);
 }
@@ -763,72 +998,135 @@ int CompIsIBMXL(char *comp)
  * RETURNS: 1 if compiler is an IBM XL compiler, 0 otherwise
  */
 {
-   char cmnd[1024], res[1024];
+   char *cmnd, *res;
+   int i;
 
+   i = strlen(comp) + 11;
+   cmnd = malloc(sizeof(char)*i);
+   assert(cmnd);
    sprintf(cmnd, "%s -qversion", comp);
-   if (CmndOneLine(NULL, cmnd, res) == 0)
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
       if (strstr(res, "IBM XL"))
+      {
+         free(res);
          return(1);
+      }
+      free(res);
    }
    return(0);
 }
 
-char *NewStringCopy(char *old)
+char **GetLinesFromFile
+(
+   FILE *fpin,    /* stream to read from */
+   char **curlns  /* NULL-termed list of lines to prepend to file list ret */
+)
 /*
- * RETURNS: newly allocates string containing copy of string old
+ * Read all lines from file fnam, suffix them to lines in curlns, delete
+ * curlns, and return them as NULL-terminated array of strings
  */
 {
-   char *new;
-   new = malloc(sizeof(char)*(strlen(old)+1));
-   strcpy(new, old);
-   return(new);
-}
-char *NewAppendedString0(char *old, char *app)
-/*
- * RETURNS: string holding : old + app
- * NOTE: frees old string after copy
- */
-{
-   char *new;
-   if (!old)
-   {
-      new = malloc(sizeof(char)*(strlen(app)+1));
-      assert(new);
-      strcpy(new, app);
-   }
-   else
-   {
-      new = malloc(sizeof(char)*(strlen(old) + strlen(app)+1));
-      assert(new);
-      strcpy(new, old);
-      strcat(new, app);
-      free(old);
-   }
-   return(new);
-}
-char *NewAppendedString(char *old, char *app)
-/*
- * RETURNS: string holding : old + " " + app
- * NOTE: frees old string after copy
- */
+   char **lns=NULL;
+   int i, n, nc=0, N;
+   char ln[2048];
 
-{
-   char *new;
-   if (!old)
+   assert(fpin);
+   for (n=0; fgets(ln, 2048, fpin); n++);  /* count number of lines in file */
+   rewind(fpin);
+   if (curlns)
+      for (nc=0; curlns[nc]; nc++);
+   N = n+nc;
+   if (N < 1)
+      return(NULL);
+   lns = malloc((N+1)*sizeof(char*));
+   assert(lns);
+   if (nc)
    {
-      new = malloc(sizeof(char)*(strlen(app)+1));
-      assert(new);
-      strcpy(new, app);
+      for (i=0; i < nc; i++)
+         lns[i] = curlns[i];
+      free(curlns);
    }
-   else
+   for (i=0; i < n; i++)
    {
-      new = malloc(sizeof(char)*(strlen(old) + strlen(app)+2));
-      assert(new);
-      strcpy(new, old);
-      strcat(new, " ");
-      strcat(new, app);
-      free(old);
+      int k;
+      char *sp;
+      assert(fgets(ln, 2048, fpin));
+      k = strlen(ln)+1;
+      lns[i+nc] = sp = malloc(k*sizeof(char));
+      assert(sp);
+      strcpy(sp, ln);
+/*
+ *    Get rid of whitespace on end of line
+ */
+      for (k -= 2; isspace(sp[k]); k--) sp[k] = '\0';
    }
-   return(new);
+   lns[N] = NULL;
+   return(lns);
+}
+
+void PrintAllStringsInList
+(
+   char *exp,
+   char **strs  /* NULL terminated arrayof strings to be printed */
+)
+{
+   int i;
+   if (!strs)
+   {
+      printf("%s: NULL\n", exp);
+      return;
+   }
+   printf("%s:\n", exp);
+   for (i=0; strs[i]; i++)
+      printf("   '%s'\n", strs[i]);
+}
+void KillAllStringsInList
+(
+   char **strs  /* NULL terminated arrayof strings to be freed */
+)
+{
+   int i;
+   if (!strs)
+      return;
+   for (i=0; strs[i]; i++)
+      free(strs[i]);
+   free(strs);
+}
+
+char *FreeListGetString
+(
+   char **strs, /* NULL-terminated array of strings to be freed */
+   int n        /* only string you want to retain from array (returned) */
+)
+{
+   char *ret;
+   int i;
+
+   if (!strs)
+      return(NULL);
+   ret = strs[n];
+   for (i=0; strs[i]; i++)
+   {
+      if (i != n)
+         free(strs[i]);
+   }
+   free(strs);
+   return(ret);
+}
+
+char **NewOneStringList
+(
+   char **strs, /* NULL-terminated array of strings to be freed */
+   int n        /* only string you want to retain from array (returned) */
+)
+{
+   char **nstrs;
+   nstrs = malloc(2*sizeof(char*));
+   assert(nstrs);
+   nstrs[0] = FreeListGetString(strs, n);
+   nstrs[1] = NULL;
+   return(nstrs);
 }
