@@ -151,7 +151,7 @@ int ProbeThrottle()
    return(iret);
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    int flags, CacheLevel, ncpu, mhz, bits, sure;
    enum MACHTYPE arch=MACHOther;
@@ -183,5 +183,5 @@ main(int nargs, char **args)
       printf("NCACHES=0\n");
    if (flags & PCacheSize)
       printf("%d Cache size (kb) = 0\n", CacheLevel);
-   exit(0);
+   return(0);
 }
