@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v@(ver)
- *                    (C) Copyright 2006 R. Clint Whaley
- *
+ * Automatically Tuned Linear Algebra Software v@(ver)
+ * (C) Copyright 2006 R. Clint Whaley
  * Code contributers : R. Clint Whaley, Dean Gaudet
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 /*
  * This code written for ATLAS use by R. Clint Whaley based on code and info
@@ -129,6 +103,9 @@ int ProbeArch(char *vendor, unsigned *family, unsigned *model, int *x86_64)
 #define EF_K8        0x02F    /* Hammer */
 #define EF_ITAN      0x020    /* Itanium */
 #define EF_K8b      16        /* 3rd gen opteron */
+#define EF_k1om     11
+#define EF_K15h     21        /* AMD K15 (eg, bulldozer, piledriver, etc.) */
+#define EF_ZEN      23        /* AMD Zen, specifically Ryzen */
 
 enum FAM {ERR,         /* cannot decipher */
           i486,        /* 486 & AMD 5x86 and Cyrix 5x86 */
@@ -136,6 +113,9 @@ enum FAM {ERR,         /* cannot decipher */
           P6,          /* Intel PIII, Core and AMD K7 (orig athlon) */
           P7,          /* Intel P4, AMD hammer, Efficeon */
           P8B,         /* 3rd generation hammer */
+          K15h,        /* AMD K15 */
+          Zen,         /* AMD Zen/Ryzen */
+          k1om,        /* XeonPHI */
           ITAN};       /* Intel Itanium */
 
 enum FAM GetFamily(int efam)  /* efam = (family+ext fam) from cpuid */
@@ -158,6 +138,9 @@ enum FAM GetFamily(int efam)  /* efam = (family+ext fam) from cpuid */
    case EF_K8_P4_EFF:         /* P4, Hammer, Efficien */
       iret = P7;
       break;
+   case EF_K15h:
+      iret = K15h;
+      break;
    case EF_K8_ITAN:           /* Hammer, Itanium */
    case EF_K8:                /* Hammer */
       iret = P7;
@@ -169,6 +152,12 @@ enum FAM GetFamily(int efam)  /* efam = (family+ext fam) from cpuid */
    case EF_ITAN0:             /* Itanium */
       iret = ITAN;
       break;
+   case EF_k1om:
+      iret = k1om;
+      break;
+   case EF_ZEN :
+      iret = Zen;
+      break;
    default:
       iret = ERR;
    }
@@ -197,7 +186,7 @@ enum VEND str2vend(char *vendor)
  * Specific chip (family, but disambiguated using vendor string
  */
 enum CHIP {CERR, Pentium, IntP6, Pentium4, Itanium, K7, Hammer, HammerB,
-           Crusoe, Efficeon};
+           AMDK15h, AMDZen, Crusoe, Efficeon, K1om};
 
 enum CHIP Family2Chip(char *vendor, enum FAM family)
 /*
@@ -242,6 +231,15 @@ enum CHIP Family2Chip(char *vendor, enum FAM family)
       if (ivend == AMD)
          iret = HammerB;
       break;
+   case K15h:
+      iret = AMDK15h;
+      break;
+   case Zen:
+      iret = AMDZen;
+      break;
+   case k1om:
+      iret = K1om;
+      break;
    case   ITAN:        /* Intel Itanium */
       iret = Itanium;
       break;
@@ -262,6 +260,18 @@ enum MACHTYPE Chip2Mach(enum CHIP chip, int model, int x8664)
 
    switch(chip)
    {
+   case AMDZen:
+      iret = MACHOther;
+      if (model == 1)
+         iret = AmdRyzen;
+      break;
+   case AMDK15h:
+      iret = MACHOther;
+      if (model == 1)
+         iret = AmdDozer;
+      else if (model == 2)
+         iret = AmdDriver;
+      break;
    case Pentium:
       switch(model)
       {
@@ -314,11 +324,29 @@ enum MACHTYPE Chip2Mach(enum CHIP chip, int model, int x8664)
       case 0x1F:
       case 0x25:
       case 0x2C:
+      case 0x2E:
          iret = IntCorei1;
          break;
+      case 0x2D:
+      case 0x3A:
       case 0x2A:
+      case 0x3E:
          iret = IntCorei2;
 	 break;
+      case 0x45:
+      case 0x3C:
+         iret = IntCorei3;
+         break;
+      case 85:
+         iret = IntCorei4X;
+         break;
+      case 94:
+         iret = IntCorei4;
+         break;
+      case 0x3F:  /* 63 */
+      case 79:    /* 0x4F */
+         iret = IntCorei3EP;
+         break;
       default:
          iret = MACHOther;
       }
@@ -375,6 +403,9 @@ enum MACHTYPE Chip2Mach(enum CHIP chip, int model, int x8664)
    case Efficeon:
       iret = TMEff;
       break;
+   case K1om:
+      iret = IntPhi;
+      break;
    case Crusoe:  /* unsupported */
    default:
       iret = MACHOther;
@@ -466,8 +497,8 @@ main(int nargs, char **args)
    }
    if (ierr)
    {
-      fprintf(stderr, "ERROR: enum fam=%d, chip=%d, mach=%d\n",
-              fam, chip, mach);
+      fprintf(stderr, "ERROR: enum fam=%d, chip=%d, model=%d, mach=%d\n",
+              fam, chip, model, mach);
       printf("ERROR %d: vendor='%s', family=%d, model=%d, x86_64=%d\n",
              ierr, vendor, family, model, x86_64);
    }
