@@ -2,49 +2,68 @@
 
 enum OSTYPE ProbeOS(int verb, char *targ)
 {
-   int ierr;
+   int i, ierr=0;
    char ln[1024], ln2[1024];
-   enum OSTYPE i, OS;
+   char *cmnd, *res;
+   enum OSTYPE OS;
    char *unam;
 
    if (verb) printf("Probing to make operating system determination:\n");
    unam = FindUname(targ);
 
-   sprintf(ln2, "%s -s", unam);
-   ierr = CmndOneLine(targ, ln2, ln);
-   if (ierr == 0)
+   i = strlen(unam) + 4;
+   cmnd = malloc(sizeof(char)*i);
+   assert(cmnd);
+   sprintf(cmnd, "%s -s", unam);
+   res = atlsys_1L(targ, cmnd, verb, 0);
+   if (res)
    {
-/*
- *    Accept GNU (HURD) as Linux, since they seem to use same stuff;
- *    This is patch from Sylvestre Ledru; I have no direct experience wt HURD
- */
-      if(strstr(ln, "Linux") || strstr(ln, "GNU")) OS = OSLinux;
-      else if(strstr(ln, "FreeBSD")) OS = OSFreeBSD;
-      else if (strstr(ln, "Darwin")) OS = OSOSX;
-      else if(strstr(ln, "SunOS"))
+      if(strstr(res, "Linux")) OS = OSLinux;
+      else if(strstr(res, "FreeBSD")) OS = OSFreeBSD;
+      else if (strstr(res, "Darwin")) OS = OSOSX;
+      else if(strstr(res, "SunOS"))
       {
-         sprintf(ln2, "%s -r", unam);
-         CmndOneLine(targ, ln2, ln);
-         if (ln[0] == '4') OS = OSSunOS4;
+         sprintf(cmnd, "%s -r", unam);
+         free(res);
+         res = atlsys_1L(targ, cmnd, verb, 0);
+         if (res[0] == '4') OS = OSSunOS4;
          else OS = OSSunOS;
       }
-      else if(strstr(ln, "OSF1")) OS = OSOSF1;
-      else if(strstr(ln, "IRIX")) OS = OSIRIX;
-      else if(strstr(ln, "AIX")) OS = OSAIX;
-      else if(strstr(ln, "Interix")) OS = OSWinSFU;
-      else if(strstr(ln, "WIN"))
+      else if(strstr(res, "OSF1")) OS = OSOSF1;
+      else if(strstr(res, "IRIX")) OS = OSIRIX;
+      else if(strstr(res, "AIX")) OS = OSAIX;
+      else if(strstr(res, "WIN"))
       {
-         if (strstr(ln, "95") || strstr(ln, "98") || strstr(ln, "_ME"))
+         if (strstr(res, "95") || strstr(res, "98") || strstr(res, "_ME"))
             OS = OSWin9x;
 /*
  *       Need to confirm what is returned under cygwin for XP, etc.
  */
-         else if (strstr(ln, "NT")) OS = OSWinNT;
+         else if (strstr(res, "WOW64")) OS =OSWin64;
+         else if (strstr(res, "CYGWIN"))
+         {
+            free(res);
+            sprintf(cmnd, "%s -m", unam);
+            res = atlsys_1L(targ, cmnd, verb, 0);
+            if (strstr(res, "x86_64") || strstr(res, "AMD64") ||
+                strstr(res, "Intel64"))
+               OS = OSWin64;
+            else
+               OS = OSWinNT;
+         }
+         else if (strstr(res, "NT")) OS = OSWinNT;
          else ierr = 1;
       }
-      else if (strstr(ln, "HP-UX")) OS = OSHPUX;
+      else if (strstr(res, "HP-UX")) OS = OSHPUX;
+/*
+ *    Accept GNU (HURD) as Linux, since they seem to use same stuff;
+ *    This is patch from Sylvestre Ledru; I have no direct experience wt HURD
+ */
+      else if(strstr(res, "GNU")) OS = OSLinux;
       else ierr = 1;
+      free(res);
    }
+   free(cmnd);
    if (ierr)
    {
       printf("\n\nUnable to determine OS, quitting\n\n");
