@@ -23,6 +23,7 @@ int GetFlags(int nargs, char **args, int *CacheLevel, enum OSTYPE *OS,
          if (++i >= nargs)
             PrintUsage(args[0], i);
          *targ = args[i];
+          break;
       case 's':
          if (++i >= nargs)
             PrintUsage(args[0], i);
@@ -88,10 +89,11 @@ int ProbeOneInt(enum OSTYPE OS0, enum ASMDIA asmd0, char *targ0,
  * keeps trying them until out of probes or one returns good (non-zero) value
  */
 {
-   char cmnd[2048], res[2048], osname[128], targ[512];
+   char osname[128];
+   char *cmnd, *res, *frm, *targ;
    enum OSTYPE OS;
    enum ASMDIA asmd;
-   int iret=0;
+   int i, iret=0;
 
    *sure = 0;
    asmd = (asmd0 > ASM_None && asmd0 < NASMD) ? asmd0 : gas_x86_32;
@@ -103,42 +105,58 @@ int ProbeOneInt(enum OSTYPE OS0, enum ASMDIA asmd0, char *targ0,
    else
       strlowcpy(osname, osnam[OS]);
    if (targ0)
+   {
+      i = strlen(targ0) + 24;
+      targ = malloc(sizeof(char)*i);
+      assert(targ);
       sprintf(targ, "atlrun=atlas_runX targ=%s", targ0);
-   else targ[0] = '\0';
+   }
+   else
+   {
+      targ = malloc(sizeof(char));
+      assert(targ);
+      targ[0] = '\0';
+   }
 
 /*
  * If Assembler right or unspecified, try x86 probe
  */
-   if (asmd == gas_x86_32 || asmd == gas_x86_64)
+   if (asmd == gas_x86_32 || asmd == gas_x86_64 || asmd == gas_wow64)
    {
-      sprintf(cmnd, "make IRunArchInfo_x86 MYFLAGS=\"-DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'",
-              osnam[OS], ASMNAM[asmd], flag, targ, find);
-      if (!CmndOneLine(NULL, cmnd, res))
+      frm = "make IRunArchInfo_x86 MYFLAGS=\"-DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'";
+      i = strlen(frm) + strlen(osnam[OS]) + strlen(ASMNAM[asmd]) + strlen(flag)
+          + strlen(targ) + strlen(find) + 1;
+      cmnd = malloc(sizeof(char)*i);
+      assert(cmnd);
+      sprintf(cmnd, frm, osnam[OS], ASMNAM[asmd], flag, targ, find);
+      res = atlsys_1L(NULL, cmnd, 0, 0);
+      if (res)
       {
          iret = GetFirstInt(res);
          *sure = GetLastInt(res);
+         free(res);
       }
+      free(cmnd);
    }
 /*
  * If that didn't work, try OS-specific probe
  */
    if (!iret)
    {
-      sprintf(cmnd, "make IRunArchInfo_%s MYFLAGS=\"-DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'",
-              osname, osnam[OS], ASMNAM[asmd], flag, targ, find);
-      if (!CmndOneLine(NULL, cmnd, res))
+      frm = "make IRunArchInfo_%s MYFLAGS=\"-DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'";
+      i = 1 + strlen(frm) + strlen(osname) + 3+strlen(osnam[OS]) +
+          strlen(ASMNAM[asmd]) + strlen(flag) + strlen(targ) + strlen(find);
+      cmnd = malloc(sizeof(char)*i);
+      assert(cmnd);
+      sprintf(cmnd, frm, osname, osnam[OS], ASMNAM[asmd], flag, targ, find);
+      res = atlsys_1L(NULL, cmnd, 0, 0);
+      if (res)
       {
          iret = GetFirstInt(res);
          *sure = GetLastInt(res);
+         free(res);
       }
-/*
- *    Interix has its own probe under Windows, if all else fails
- */
-      if (!iret && OS == OSWinSFU)
-      {
-         sprintf(cmnd, "make IRunArchInfo_sfu MYFLAGS=\"-DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'",
-              osnam[OS], ASMNAM[asmd], flag, targ, find);
-      }
+      free(cmnd);
    }
    return(iret);
 }
@@ -150,31 +168,49 @@ int ConfirmPtrbits(enum OSTYPE OS0, enum ASMDIA asmd0, char *targ0,
  * case
  */
 {
-   char cmnd[2048], res[2048], osname[128], targ[512];
+   char osname[128];
+   char *cmnd, *res, *targ;
+   char *frm="make IRunArchInfo_%s MYFLAGS=\"-m64 -DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'";
    enum OSTYPE OS;
    enum ASMDIA asmd;
-   int iret=0;
+   int i, iret=0;
 
    *sure = 0;
    asmd = (asmd0 > ASM_None && asmd0 < NASMD) ? asmd0 : gas_x86_32;
    OS = (OS0 > OSOther && OS0 < NOS) ? OS0 : OSLinux;
-   if (OS == OSOSX)
-      strlowcpy(osname, osnam[OSFreeBSD]);
-   else
-      strlowcpy(osname, osnam[OS]);
+   i = (OS == OSOSX) ? OSFreeBSD : OS;
+   assert(strlen(osnam[i]) < 128);
+   strlowcpy(osname, osnam[i]);
    if (targ0)
+   {
+      i = strlen(targ0) + 24;
+      targ = malloc(sizeof(char)*i);
+      assert(targ);
       sprintf(targ, "atlrun=atlas_runX targ=%s", targ0);
-   else targ[0] = '\0';
+   }
+   else
+   {
+      targ = malloc(sizeof(char));
+      assert(targ);
+      targ[0] = '\0';
+   }
 
 /*
  * Try OS-specific probe, compiling with -m64 (assumes gcc workalike)
  */
-   sprintf(cmnd, "make IRunArchInfo_%s MYFLAGS=\"-m64 -DATL_OS_%s -DATL_%s\" args=\"%s\" %s | fgrep '%s'",
-           osname, osnam[OS], ASMNAM[asmd], flag, targ, find);
-   if (!CmndOneLine(NULL, cmnd, res))
+   i = 1 + strlen(frm) + strlen(osnam) + strlen(osnam[OS]) +
+       strlen(ASMNAM[asmd]) + strlen(flag) + strlen(targ) + strlen(find);
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
+   sprintf(cmnd, frm, osname, osnam[OS], ASMNAM[asmd], flag, targ, find);
+   free(targ);
+   res = atlsys_1L(NULL, cmnd, 0, 0);
+   free(cmnd);
+   if (res)
    {
       iret = GetFirstInt(res);
       *sure = GetLastInt(res);
+      free(res);
    }
    return(iret);
 }
@@ -191,7 +227,8 @@ int main(int nargs, char **args)
    if (flags & Parch)
    {
       arch = ProbeOneInt(OS, asmd, targ, "-a", "MACHTYPE=", &sure);
-      if (arch == MACHOther && (asmd == gas_x86_32 || asmd == gas_x86_64))
+      if (arch == MACHOther &&
+          (asmd == gas_x86_32 || asmd == gas_x86_64 || asmd == gas_wow64))
          arch = x86X;
       if (flags & Pverb)
          printf("Architecture detected as %s.\n", machnam[arch]);
@@ -207,7 +244,7 @@ int main(int nargs, char **args)
              ProbeOneInt(OS, asmd, targ, "-t", "CPU THROTTLE=", &sure));
    if (flags & P64)
    {
-      if (asmd == gas_x86_64)
+      if (asmd == gas_x86_64 || asmd == gas_wow64)
       {
          sure = 1;
          bits = 64;
