@@ -3,22 +3,37 @@
 int RunASMProbe(char *targ, int verb, enum OSTYPE OS, char *asmnam)
 {
    char ln[2048], ln2[2048];
+   char *cmnd, *res, *frm;
+   int i = 0;
    if (targ)
-      sprintf(ln2, "make IRun_%s atlrun=atlas_runX args=\"-v %d\" MYFLAGS=\"-DATL_OS_%s\" targ=%s 2> /dev/null | fgrep SUCCESS",
-              asmnam, verb, osnam[OS], targ);
-   else
-      sprintf(ln2, "make IRun_%s args=\"-v %d\" MYFLAGS=\"-DATL_OS_%s\" 2> /dev/null | fgrep SUCCESS",
-              asmnam, verb, osnam[OS]);
-   if (verb > 1)
-      fprintf(stderr, "system(%s)\n", ln2);
-   if(!CmndOneLine(NULL, ln2, ln))
    {
-      if (strstr(ln, "SUCCESS"))
+      frm = "make IRun_%s atlrun=atlas_runX args=\"-v %d\" MYFLAGS=\"-DATL_OS_%s\" targ=%s 2> /dev/null | fgrep SUCCESS";
+      i = strlen(targ);
+   }
+   else
+      frm = "make IRun_%s args=\"-v %d\" MYFLAGS=\"-DATL_OS_%s\" 2> /dev/null | fgrep SUCCESS";
+
+   i += strlen(frm) + strlen(asmnam) + 11 + strlen(osnam[OS]) + 1;
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
+   if (targ)
+      sprintf(cmnd, frm, asmnam, verb, osnam[OS], targ);
+   else
+      sprintf(cmnd, frm, asmnam, verb, osnam[OS]);
+   if (verb > 1)
+      fprintf(stderr, "system(%s)\n", cmnd);
+   res = atlsys_1L(NULL, cmnd, verb, 0);
+   free(cmnd);
+   if (res)
+   {
+      if (strstr(res, "SUCCESS"))
       {
          if (verb)
             fprintf(stdout, "   %s: DETECTED!\n", asmnam);
+         free(res);
          return(1);
       }
+      free(res);
    }
    if (verb)
       fprintf(stdout, "   %s: NO.\n", asmnam);
@@ -35,6 +50,25 @@ enum ASMDIA ProbeASM(char *targ, int verb, enum OSTYPE OS)
 }
 
 
+/*
+ * Defined some macros for checking booleans passed by -Si
+ */
+#define SI_FKO 0
+#define SI_BOZOL1 1
+#define SI_ARCHDEF 2
+#define SI_IEEE 3
+#define SI_LATUNE 4
+#define SI_NOF77  5
+#define SI_NOCYGWIN 6
+#define SI_OMP 6
+#define SI_LAREF 7
+#define SI_ADCLOSEP 8
+#define SI_SKPTHRCHK 9
+#define SI_INDTHR 10
+#define SI_IS_TRUE(bv_, b_) (((bv_)>>(b_))&1)
+#define SI_SET_BIT(bv_, b_) (bv_) |= 1 << (b_)
+#define SI_UNSET_BIT(bv_, b_) (bv_) &= ~(1 << (b_))
+
 void PrintUsage(char *name, int iarg, char *arg)
 {
    fprintf(stderr, "\nERROR around arg %d (%s).\n", iarg,
