@@ -2,15 +2,23 @@
 
 int probe_name(char *targarg, int verb, char **usrcomps)
 {
-   char cmnd[2048], res[1024];
+   char *frm = "make IRunF2C_name %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"%s\" | fgrep 'F2C name'";
+   char *cmnd, *res;
    enum F2CNAME f2cname = f2c_NamErr;
+   int i;
 
-   sprintf(cmnd, "make IRunF2C_name %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"%s\" | fgrep 'F2C name'",
-           targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
+   i = 1 + strlen(frm) + strlen(targarg) + strlen(usrcomps[F77_]) +
+       strlen(usrcomps[F77_+NCOMP]) + strlen(usrcomps[ICC_]) +
+       strlen(usrcomps[ICC_+NCOMP]);
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
+   sprintf(cmnd, frm, targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
            usrcomps[ICC_], usrcomps[ICC_+NCOMP]);
    if (verb > 1)
       fprintf(stderr, "cmnd = '%s'\n", cmnd);
-   if (!CmndOneLine(NULL, cmnd, res))
+   res = atlsys_1L(NULL, cmnd, verb, 0);
+   free(cmnd);
+   if (res)
    {
       if (verb > 1)
          fprintf(stderr, "res = '%s'\n", res);
@@ -22,6 +30,7 @@ int probe_name(char *targarg, int verb, char **usrcomps)
          f2cname = f2c_NoChange;
       else if (strstr(res, "UpCase"))
          f2cname = f2c_UpCase;
+      free(res);
    }
    if (verb)
       printf("F2C Name Decoration = %s\n", f2c_namestr[f2cname]);
@@ -30,14 +39,22 @@ int probe_name(char *targarg, int verb, char **usrcomps)
 
 int probe_int(char *targarg, int verb, char **usrcomps, int f2cname)
 {
-   char cmnd[2048], res[1024];
+   char *frm="make IRunF2C_int %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"-D%s %s\" | fgrep 'F2C int'";
+   char *cmnd, *res;
    enum F2CINT f2c_int = f2c_IntErr;
-   sprintf(cmnd, "make IRunF2C_int %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"-D%s %s\" | fgrep 'F2C int'",
-           targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
+   int i;
+   i = 1 + strlen(frm) + strlen(targarg) + strlen(usrcomps[F77_])
+       + strlen(usrcomps[F77_+NCOMP]) + strlen(usrcomps[ICC_])
+       + strlen(f2c_namestr[f2cname]) + strlen(usrcomps[ICC_+NCOMP]);
+   cmnd = malloc(i*sizeof(char));
+   assert(cmnd);
+   sprintf(cmnd, frm, targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
            usrcomps[ICC_], f2c_namestr[f2cname], usrcomps[ICC_+NCOMP]);
    if (verb > 1)
       fprintf(stderr, "cmnd = '%s'\n", cmnd);
-   if (!CmndOneLine(NULL, cmnd, res))
+   res = atlsys_1L(NULL, cmnd, verb, 0);
+   free(cmnd);
+   if (res)
    {
       if (verb > 1)
          fprintf(stderr, "res = '%s'\n", res);
@@ -49,6 +66,7 @@ int probe_int(char *targarg, int verb, char **usrcomps, int f2cname)
          f2c_int = FintClong;
       else if (strstr(res, " C short"))
          f2c_int = FintCshort;
+      free(res);
    }
    if (verb)
       printf("F2C int = %s\n", f2c_intstr[f2c_int]);
@@ -57,22 +75,33 @@ int probe_int(char *targarg, int verb, char **usrcomps, int f2cname)
 
 int probe_str(char *targarg, int verb, char **usrcomps, int f2cname, int f2cint)
 {
-   char cmnd[2048], res[1024];
+   char *frm = "make IRunF2C_str %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"-D%s -D%s -DString%s %s\" | fgrep 'F2C string'";
    enum F2CSTRING f2cstr = f2c_StrErr;
-   int i;
+   int i, len;
 
+   len = 1 + strlen(frm) + strlen(targarg) + strlen(usrcomps[F77_]) +
+        strlen(usrcomps[F77_+NCOMP]) + strlen(usrcomps[ICC_]) +
+        strlen(f2c_namestr[f2cname]) + strlen(f2c_intstr[f2cint]) +
+        strlen(usrcomps[ICC_+NCOMP]);
    for (i=1; i < 5; i++)
    {
-      sprintf(cmnd, "make IRunF2C_str %s F77=\"%s\" F77FLAGS=\"%s\" CC=\"%s\" CCFLAGS=\"-D%s -D%s -DString%s %s\" | fgrep 'F2C string'",
-              targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
+      char *cmnd, *res;
+      int k;
+      k = len + strlen(f2c_strstr[i]);
+      cmnd = malloc(k*sizeof(char));
+      assert(cmnd);
+      sprintf(cmnd, frm, targarg, usrcomps[F77_], usrcomps[F77_+NCOMP],
               usrcomps[ICC_], f2c_namestr[f2cname], f2c_intstr[f2cint],
               f2c_strstr[i], usrcomps[ICC_+NCOMP]);
       if (verb > 1)
          fprintf(stderr, "cmnd = '%s'\n", cmnd);
-      if (!CmndOneLine(NULL, cmnd, res))
+      res = atlsys_1L(NULL, cmnd, verb, 0);
+      free(cmnd);
+      if (res)
       {
          if (verb > 1)
             fprintf(stderr, "res = '%s'\n", res);
+         free(res);
          f2cstr = i;
          break;
       }
@@ -82,6 +111,25 @@ int probe_str(char *targarg, int verb, char **usrcomps, int f2cname, int f2cint)
    return(f2cstr);
 }
 
+/*
+ * Defined some macros for checking booleans passed by -Si
+ */
+#define SI_FKO 0
+#define SI_BOZOL1 1
+#define SI_ARCHDEF 2
+#define SI_IEEE 3
+#define SI_LATUNE 4
+#define SI_NOF77  5
+#define SI_NOCYGWIN 6
+#define SI_OMP 6
+#define SI_LAREF 7
+#define SI_ADCLOSEP 8
+#define SI_SKPTHRCHK 9
+#define SI_INDTHR 10
+#define SI_IS_TRUE(bv_, b_) (((bv_)>>(b_))&1)
+#define SI_SET_BIT(bv_, b_) (bv_) |= 1 << (b_)
+#define SI_UNSET_BIT(bv_, b_) (bv_) &= ~(1 << (b_))
+
 void PrintUsage(char *name, int iarg, char *arg)
 {
    fprintf(stderr, "\nERROR around arg %d (%s).\n", iarg,
@@ -140,6 +188,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
          sp = args[i];
          k = -1;
          if (*sp == 'i' && sp[1] == 'c') k = ICC_;
+         else if (*sp == 'g' && sp[1] == 'c') k = GCC_;
          else if (*sp == 'i' && sp[1] == 'f') k = F77_;
          else if (*sp == 's' && sp[1] == 'k') k = SKC_;
          else if (*sp == 'd' && sp[1] == 'k') k = DKC_;
@@ -148,6 +197,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
          else if (*sp == 'x' && sp[1] == 'c') k = XCC_;
          if (*sp == 'a' && (sp[1] == 'l' || sp[1] == 'c'))
          {  /* only appended flags can be applied to all compilers */
+            const int SKIPGCC=(sp[2] != 'g'), SKIPF=(sp[1] == 'c');
             if (args[i-1][1] == 'F')
             {
                if (args[i-1][2] == 'a')
@@ -169,7 +219,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
             if (++i >= nargs)
                PrintUsage(args[0], i, "out of arguments");
             for (k=k0; k < kn; k++)
-               if (sp[1] == 'l' || k-2*NCOMP != F77_)
+               if ((!SKIPF || k-k0 != F77_) && (!SKIPGCC || k-k0 != GCC_))
                   comps[k] = args[i];
          }
          else
@@ -221,7 +271,7 @@ int main (int nargs, char **args)
    int f2cname, f2cint, f2cstr;
    char *usrcomps[3*NCOMP];
    char *targ, *targarg;
-   int ierr = 0;
+   int i, ierr = 0;
 
    GetFlags(nargs, args, &verb, usrcomps, &targ);
    if (targ)
@@ -247,5 +297,8 @@ int main (int nargs, char **args)
    }
    if (targ)
       free(targarg);
+   for (i=0; i < 3*NCOMP; i++)
+      if (usrcomps[i])
+         free(usrcomps[i]);
    return(ierr);
 }
