@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -265,6 +240,26 @@ int main(int nargs, char **args)
                j = sprintf(ln+ic, "/link:DEFAULTLIB:%s", &args[i][2]);
             slashsub(ln+ic+17);
          }
+         else if (args[i][1] == 'm' && args[i][2] == '3'
+                  && args[i][3] == '2' && args[i][4] == '\0')
+            j=0;  /* do not pass on -m32/64 to windows compilers */
+         else if (args[i][1] == 'm' && args[i][2] == '3'
+                  && args[i][3] == '2' && args[i][4] == '\0')
+            j=0;  /* do not pass on -m32/64 to windows compilers */
+         else if (args[i][1]=='m' && args[i][2]=='s' && args[i][3]=='s' &&
+                  args[i][4] == 'e')
+         {
+            if (args[i][5] == '\0')
+               j = sprintf(ln+ic, "/arch:sse");
+            else
+               j = sprintf(ln+ic, "/arch:sse2");
+         }
+         else if (args[i][1]=='m' && args[i][2]=='a' && args[i][3]=='v' &&
+                  args[i][4] == 'x')
+            j = sprintf(ln+ic, "/arch:avx");
+         else if (args[i][1] == 'm' && args[i][2] == '3'
+                  && args[i][3] == '2' && args[i][4] == '\0')
+              ;  /* do not pass on -m32/64 to windows compilers */
          else
          {
             if (args[i][1] == 'c') COMP=1;
