@@ -126,6 +126,10 @@
       #define FKO_IVLEN 8
    #endif
    #ifdef AVX
+      #define ArchHasMemBroadcast
+      #define ArchHasFPthreeOps
+      #define ArchHasVFPthreeOps
+      #define ArchHasVINTthreeOps
       #define FP_VEC
       #define FKO_SVLEN 8
       #define DP_VEC
@@ -136,6 +140,11 @@
    #else /* by default SSE4.1*/
       /*#define INT_VEC*/
       /*#define FKO_IVLEN 4*/
+/*
+ *    added synthetic inst in SSE to support mem broadcast
+ */
+      #define ArchHasMemBroadcast
+      #define SSE3   /* sse4.1 includes sse3 */
       #define FP_VEC
       #define FKO_SVLEN 4
       #define DP_VEC
@@ -704,7 +713,7 @@
    #endif
 #else
    #ifdef X86_64
-      #define HAMMER
+      #define COREI
    #endif
    #ifdef PIII
       #define NCACHE 2
@@ -727,7 +736,7 @@
       #else
          extern short LINESIZE[NCACHE];
       #endif
-   #else /* P4/P4E defaults */
+   #elif defined(p4) /* P4/P4E */
       #define NCACHE 1
       #ifdef ARCH_DECLARE
          /*short LINESIZE[NCACHE] = {128};*/
@@ -735,6 +744,20 @@
       #else
          extern short LINESIZE[NCACHE];
       #endif
+   #elif defined(COREI) /* corei, AMD defaults */
+      #define NCACHE 3
+      #ifdef ARCH_DECLARE
+         short LINESIZE[NCACHE] = {64,64,64};
+      #else
+         extern short LINESIZE[NCACHE];
+      #endif
+   #else /* defaults */
+      #define NCACHE 2
+      #ifdef ARCH_DECLARE
+         short LINESIZE[NCACHE] = {64,64};
+      #else
+         extern short LINESIZE[NCACHE];
+      #endif
    #endif
 #endif
 /*
@@ -758,4 +781,5 @@ void FeedbackArchInfo(FILE *fpout);
 int RevealArchMemUses(void);
 short vtype2elem(int type);
 short iName2Reg(char *rname);
+short GetVecAlignByte();
 #endif
