@@ -74,6 +74,9 @@ enum inst
    CMPAND,                      /* cc0, r1, r2/c : set [cc] based on r1 & r2 */
    CMP,                         /* cc#, r1, r2/c: set [cc] based on r1 - r2 */
    NEG,                         /* [r0], [r1] : r0 = -r1 */
+   LEA2,                        /* lea r0, [r1, r2, 2] */ 
+   LEA4,                        /* lea r0, [r1, r2, 4] */ 
+   LEA8,                        /* lea r0, [r1, r2, 8] */ 
 /*   ABS, ; abs commented out because not widely supported */
 /*
  * 32-bit integer (64-bit systems only)
@@ -95,7 +98,10 @@ enum inst
    CMPS,                        /* set [cc](r0) based on r1 - r2 */
    MOVS,                        /* [r0], [r1] : r0 = r1 */
    NEGS,                        /* [r0], [r1] : r0 = -r1 */
-   ABSS,
+   LEAS2,                        /* lea r0, [r1, r2, 2] */ 
+   LEAS4,                        /* lea r0, [r1, r2, 4] */ 
+   LEAS8,                        /* lea r0, [r1, r2, 8] */ 
+   /*ABSS,*/
 /*
  * Jump instructions
  */
@@ -198,6 +204,7 @@ enum inst
    VDNEG,                      /* [vr0], [vr1] : vr0 = -(vr1) */
    VDMAX,                      /* vr0 = max(vr1,vr2) */
    VDMIN,                      /* vr0 = min(vr1,vr2) */
+   VDHADD,                     /* vr0 = HADD(vr1), HADD(vr2)*/
 /*
  * Majedul: intorducing special load instruction with broadcast ability
  * NOTE: it will only be used to load data. repeatble optimizations skip it
@@ -215,6 +222,8 @@ enum inst
           /* [int32] is split into 8 4 bit words; 1st word indicates which */
           /* should reside in vr0[0], 4th in vr0[3];  Words are numbered */
           /* starting in vr0[0], and ending in vr1[N], N=veclen-1 */
+   VDHIHALF,                  /* vr0~2[1,0]: vr0[0] = vr1[1], vr0[1] = vr2[1] */
+   VDLOHALF,                  /* vr0~2[1,0]: vr0[0] = vr1[0], vr0[1] = vr2[0] */
 /*
  * Single precision vector instructions
  * [memA] is a vector-aligned mem @ [mem] is any alignment
@@ -245,6 +254,7 @@ enum inst
    VFNEG,                      /* [vr0], [vr1] : vr0 = -(vr1) */
    VFMAX,                      /* vr0 = max(vr1,vr2) */
    VFMIN,                      /* vr0 = min(vr1,vr2) */
+   VFHADD,                     /* vr0 = HADD(vr1), HADD(vr2)*/
 /*
  * Majedul: intorducing special load instruction with broadcast ability
  * NOTE: it will only be used to load data. repeatble optimizations skip it
@@ -262,6 +272,8 @@ enum inst
           /* [int32] is split into 8 4 bit words; 1st word indicates which */
           /* should reside in vr0[0], 4th in vr0[3];  Words are numbered */
           /* starting in vr0[0], and ending in vr1[N], N=veclen-1 */
+   VFHIHALF,                  /* vr0~2[1,0]: vr0[0] = vr1[1], vr0[1] = vr2[1] */
+   VFLOHALF,                  /* vr0~2[1,0]: vr0[0] = vr1[0], vr0[1] = vr2[0] */
 /*
  * x86-only instructions
  */
@@ -496,6 +508,9 @@ char *instmnem[] =
    "CMPAND",
    "CMP",
    "NEG",
+   "LEA2",
+   "LEA4",
+   "LEA8",
 /*   ABS, ; abs commented out because not widely supported */
 /*
  * 32-bit integer (64-bit systems only)
@@ -517,7 +532,10 @@ char *instmnem[] =
    "UDIVS",
    "CMPS",
    "NEGS",
-   "ABSS",
+   "LEAS2",
+   "LEAS4",
+   "LEAS8", 
+   /*"ABSS",*/
 /*
  * Jump instructions
  */
@@ -605,6 +623,7 @@ char *instmnem[] =
    "VDNEG",
    "VDMAX",
    "VDMIN",
+   "VDHADD",
 /*
  * Majedul: intorducing special load instruction with broadcast ability
  */
@@ -615,6 +634,8 @@ char *instmnem[] =
    "VDCMOV1",
    "VDCMOV2",
    "VDSHUF",
+   "VDHIHALF",
+   "VDLOHALF",
 /*
  * Single precision vector inst
  */
@@ -639,6 +660,7 @@ char *instmnem[] =
    "VFNEG",
    "VFMAX",
    "VFMIN",
+   "VFHADD",
 /*
  * Majedul: intorducing special load instruction with broadcast ability
  */
@@ -649,6 +671,8 @@ char *instmnem[] =
    "VFCMOV1",
    "VFCMOV2",
    "VFSHUF",
+   "VFHIHALF",
+   "VFLOHALF",
 /*
  * x86-only instructions
  */
@@ -948,7 +972,8 @@ char *instmnem[] =
                       (i_) == FCMOVD1 || (i_) == FCMOVD2 || (i_) == VFCMOV1 || \
                       (i_) == VFCMOV2 || (i_) == VDCMOV1 || (i_) == VDCMOV2 || \
                       (i_) == CMOV1 || (i_) == CMOV2 || (i_) == VSCMOV1 || \
-                      (i_) == VSCMOV2 || (i_) == VICMOV1 || (i_) == VICMOV2 )
+                      (i_) == VSCMOV2 || (i_) == VICMOV1 || (i_) == VICMOV2 || \
+                      (i_) == LEA2 || (i_) == LEA4 || (i_) == LEA8 )
 /*
  * NOTE: FMA4 in AMD can be re ordered, src2 can be mem. need to consider
  * this again while implemting other FMAC like: FMA3 ... 
@@ -962,8 +987,9 @@ char *instmnem[] =
  * Majedul: These are the instructions where destination is inherently used as
  * one of the sources. This is due to map 4 operands instruction into our 
  * three operand LIL instruction.
- * NOTE: SHUFFLE and ireg2vreg instructions may keep the destination unchanged
+ * NOTE: 1. SHUFFLE and ireg2vreg instructions may keep the destination unchanged
  * at certain positions. So, destination is also implicitly used for them
+ *       2. DIV, UDIV, DIVS, UDIVS : handled specially 
  */
 #define IS_DEST_INUSE_IMPLICITLY(i_) ((i_) == FMAC || (i_) == FMACD || \
                                       (i_) == VFMAC || (i_) == VDMAC || \
@@ -995,7 +1021,7 @@ char *instmnem[] =
                               (i_) == MULS   || (i_) == UMULS || \
                               (i_) == DIVS   || (i_) == UDIVS || \
                               (i_) == CMPS   || (i_) == MOVS || \
-                              (i_) == NEGS   || (i_) == ABSS )
+                              (i_) == NEGS  )
 
 #define IS_SHUFFLE_OP(i_)   ( (i_) == VFSHUF   || (i_) == VDSHUF || \
                               (i_) == VSSHUF   || (i_) == VISHUF )
@@ -1005,6 +1031,9 @@ char *instmnem[] =
 
 /*#define IS_MOVS(i_) ( (i_) == VDMOVS || (i_) == VFMOVS )*/
 
+#define IS_PREF(i_) ( (i_) == PREFR || (i_) == PREFW )
+
+
 INSTQ *NewInst(BBLOCK *myblk, INSTQ *prev, INSTQ *next, enum inst ins,
                short dest, short src1, short src2);
 INSTQ *InsNewInst(BBLOCK *myblk, INSTQ *prev, INSTQ *next, enum inst ins,
@@ -1022,6 +1051,8 @@ void WriteLILToBinFile(char *fname, BBLOCK *bbase);
 void ReadLILFromBinFile(char *fname);
 void PrintInst(FILE *fpout, BBLOCK *bbase);
 void PrintThisInst(FILE *fpout, int i, INSTQ *ip);
+void PrintThisInstQ(FILE *fpout, INSTQ *ip);
 INSTQ *InsertInstBeforeQEntry(INSTQ *list, INSTQ *add);
 int FindInstNum(BBLOCK *blk, INSTQ *inst);
+INSTQ *FindFirstLILforHIL(INSTQ *ipX);
 #endif
