@@ -13,6 +13,8 @@ void KillAllLocinit(struct locinit *libase);
 void *NewPtrTable(int *n, void *old, int chunk);
 short *NewShortTable(int *n, short *old, int chunk);
 ILIST *NewIlist(INSTQ *inst, ILIST *next);
+ILIST *NewIlistAtEnd(INSTQ *inst, ILIST *top); 
+ILIST *NewIlistInBetween(INSTQ *inst, ILIST *prev, ILIST *next);
 ILIST *KillIlist(ILIST *ip);
 void KillAllIlist(ILIST *ilist);
 struct ptrinfo *FindPtrinfo(struct ptrinfo *base, short ptr);
@@ -22,5 +24,7 @@ struct locinit *NewLI(short id, short con, struct locinit *next);
 void KillAllLI(struct locinit *die);
 int FindInShortList(int n, short *sp, short val);
 int AddToShortList(int n, short *sp, short val);
-
+LPLIST *NewLPlist(LPLIST *next, LOOPQ *loop);
+void KillAllLPlist(LPLIST *lp);
+LPLIST *FindLoopNest(LOOPQ *lp0);
 #endif
