@@ -57,28 +57,42 @@ short type2shift(int type)
       #endif
    return(len);
 }
+
+#ifdef X86
 short vtype2elem(int type)
 /*
  * returns element count in vector depending on the vector type
  */
 {
    short nelem;
-#if defined(AVX)
-   if (type == T_VDOUBLE)
-      nelem = 4;
-   else if (type == T_VFLOAT) 
-      nelem = 8;
-#else
-   if (type == T_VDOUBLE)
-      nelem = 2;
-   else if (type == T_VFLOAT) 
-      nelem = 4;
-#endif
+   #if defined(AVX)
+      if (type == T_VDOUBLE)
+         nelem = 4;
+      else if (type == T_VFLOAT) 
+         nelem = 8;
+   #else
+      if (type == T_VDOUBLE)
+         nelem = 2;
+      else if (type == T_VFLOAT) 
+         nelem = 4;
+   #endif
    else
       fko_error(__LINE__, "Must be a vector type!");
    return nelem;
 }
-#ifdef X86
+
+short GetVecAlignByte()
+/*
+ * alignment needed in bytes for vector unit
+ */
+{
+   #if defined(AVX)
+      return(32);
+   #else
+      return(16);
+   #endif
+}
+
 int RevealArchMemUses(void)
 /*
  * reveals all the mem uses for DTabs/DTnzero, 
@@ -87,12 +101,13 @@ int RevealArchMemUses(void)
  * may be violated!!
  */
 {
+   int j;
    BBLOCK *bp;
    INSTQ *ip, *ipN;
    enum inst inst;
    /*short op, ir;*/
-   short op3, vr;
-   int nchanges, arinst, j;
+   short op3, vr, ir;
+   int nchanges;
    extern int DTabs, DTabsd, DTnzero, DTnzerod;
    /*extern int DTabss, DTabsds, DTnzeros, DTnzerods;*/
    extern BBLOCK *bbbase;
@@ -107,12 +122,25 @@ int RevealArchMemUses(void)
  *       Since x86 only supports vector operation for and/xor, we always change
  *       following instructions into compatible x86 inst.
  */
+         if (inst == FABS || inst == FABSD || inst == FNEG || inst == FNEGD 
+             || inst == VFABS || inst == VDABS || inst == VFNEG 
+             || inst == VDNEG)
+         {
+            j = ireg2type(-ip->inst[1]);
+            ir = GetReg(j);
+            while(ir == -ip->inst[1] || ir == -ip->inst[2])
+            {
+               ir = GetReg(j);
+            }
+         }
+         else continue;
+         
          /*if (ip->inst[3] >= 0)*/
          switch(inst)
          {
             case FABS:
                op3 = SToff[DTabs-1].sa[2];
-               vr = GetReg(T_VFLOAT);
+               vr = ir - FREGBEG + VFREGBEG;
                ipN = InsNewInst(NULL, NULL, ip, VFLD, -vr, op3, 0);
                ip->inst[0] = VFSABS;
                ip->inst[3] = -vr;
@@ -123,7 +151,7 @@ int RevealArchMemUses(void)
                break;
             case FABSD:
                op3 = SToff[DTabsd-1].sa[2];
-               vr = GetReg(T_VDOUBLE);
+               vr = ir - DREGBEG + VDREGBEG;
                ipN = InsNewInst(NULL, NULL, ip, VDLD, -vr, op3, 0);
                ip->inst[0] = VDSABS;
                ip->inst[3] = -vr;
@@ -134,7 +162,7 @@ int RevealArchMemUses(void)
                break;
             case FNEG:
                op3 = SToff[DTnzero-1].sa[2];
-               vr = GetReg(T_VFLOAT);
+               vr = ir - FREGBEG + VFREGBEG;
                ipN = InsNewInst(NULL, NULL, ip, VFLD, -vr, op3, 0);
                ip->inst[0] = VFSNEG;
                ip->inst[3] = -vr;
@@ -145,7 +173,7 @@ int RevealArchMemUses(void)
                break;
             case FNEGD:
                op3 = SToff[DTnzerod-1].sa[2];
-               vr = GetReg(T_VDOUBLE);
+               vr = ir - DREGBEG + VDREGBEG;
                ipN = InsNewInst(NULL, NULL, ip, VDLD, -vr, op3, 0);
                ip->inst[0] = VDSNEG;
                ip->inst[3] = -vr;
@@ -157,9 +185,7 @@ int RevealArchMemUses(void)
             case VFABS:
                op3 = SToff[DTabs-1].sa[2];
                j = T_VFLOAT;
-               vr = GetReg(j);
-               while (vr == -ip->inst[1] || vr == -ip->inst[2])
-                  vr = GetReg(j);
+               vr = ir;
                ipN = InsNewInst(NULL, NULL, ip, VFLD, -vr, op3, 0);
                ip->inst[3] = -vr;
                GetReg(-1);
@@ -169,10 +195,7 @@ int RevealArchMemUses(void)
                   break;
             case VDABS:
                op3 = SToff[DTabsd-1].sa[2];
-               j = T_VDOUBLE;
-               vr = GetReg(j);
-               while (vr == -ip->inst[1] || vr == -ip->inst[2])
-                  vr = GetReg(j);
+               vr = ir;
                ipN = InsNewInst(NULL, NULL, ip, VDLD, -vr, op3, 0);
                ip->inst[3] = -vr;
                GetReg(-1);
@@ -182,10 +205,7 @@ int RevealArchMemUses(void)
                break;
             case VFNEG:
                op3 = SToff[DTnzero-1].sa[2];
-               j = T_VFLOAT;
-               vr = GetReg(j);
-               while (vr == -ip->inst[1] || vr == -ip->inst[2])
-                  vr = GetReg(j);
+               vr = ir;
                ipN = InsNewInst(NULL, NULL, ip, VFLD, -vr, op3, 0);
                ip->inst[3] = -vr;
                GetReg(-1);
@@ -195,23 +215,21 @@ int RevealArchMemUses(void)
                break;
             case VDNEG:
                op3 = SToff[DTnzerod-1].sa[2];
-               j = T_VDOUBLE;
-               vr = GetReg(j);
-               while (vr == -ip->inst[1] || vr == -ip->inst[2])
-                  vr = GetReg(j);
+               vr = ir;
                ipN = InsNewInst(NULL, NULL, ip, VDLD, -vr, op3, 0);
                ip->inst[3] = -vr;
                GetReg(-1);
                CalcThisUseSet(ipN);
                CalcThisUseSet(ip);
                nchanges++;
-               break;   
+               break;  
+            default:
+               break;
          }
       }
    }
    if (nchanges)
-      CFUSETU2D = INDEADU2D = 0;
-
+      CFUSETU2D = INUSETU2D = INDEADU2D = 0;
    return nchanges;
 }
 #endif
@@ -524,7 +542,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTnzerod;
          SToff[k].sa[2] = -DTnzerod;
-         SToff[k].sa[3] = DTnzerod;
+         /*SToff[k].sa[3] = DTnzerod;*/
+         SetDTcon(k+1, DTnzerod);
       }
    }
    if (DTabsd)
@@ -541,7 +560,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTabsd;
          SToff[k].sa[2] = -DTabsd;
-         SToff[k].sa[3] = DTabsd;
+         /*SToff[k].sa[3] = DTabsd;*/
+         SetDTcon(k+1, DTabsd);
       }
    }
    if (DTnzero)
@@ -558,7 +578,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTnzero;
          SToff[k].sa[2] = -DTnzero;
-         SToff[k].sa[3] = DTnzero;
+         /*SToff[k].sa[3] = DTnzero;*/
+         SetDTcon(k+1, DTnzero);
       }
    }
    if (DTabs)
@@ -575,7 +596,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTabs;
          SToff[k].sa[2] = -DTabs;
-         SToff[k].sa[3] = DTabs;
+         /*SToff[k].sa[3] = DTabs;*/
+         SetDTcon(k+1, DTabs);
       }
    }
    #ifdef X86_32
@@ -593,7 +615,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTx87;
          SToff[k].sa[2] = -DTx87;
-         SToff[k].sa[3] = DTx87;
+         /*SToff[k].sa[3] = DTx87;*/
+         SetDTcon(k+1, DTx87);
       }
    }
    if (DTx87d)
@@ -610,7 +633,8 @@ void CreateSysLocals()
          SToff[k].sa[0] = -REG_SP;
          SToff[k].sa[1] = DTx87d;
          SToff[k].sa[2] = -DTx87d;
-         SToff[k].sa[3] = DTx87d;
+         /*SToff[k].sa[3] = DTx87d;*/
+         SetDTcon(k+1, DTx87d);
       }
    }
    #endif
@@ -746,7 +770,7 @@ void FPConstStore(INSTQ *next, short id, short con,
       long *lp;
    #endif
    #if defined(PPC) || defined(SPARC)
-      short i;
+      INT_DTC i;
    #endif
    #ifndef X86
       short k;
@@ -768,7 +792,7 @@ void FPConstStore(INSTQ *next, short id, short con,
             InsNewInst(NULL, NULL, next, FZEROD, -dreg, 0, 0);
             InsNewInst(NULL, NULL, next, FSTD, SToff[id-1].sa[2], -dreg, 0);
          #else
-            i = SToff[SToff[id-1].sa[2]-1].sa[3];
+            i = GetDTcon(SToff[SToff[id-1].sa[2]-1].sa[3]);
             k = AddDerefEntry(-REG_SP, STderef, -STderef, i, con);
             InsNewInst(NULL, NULL, next, XOR, -reg, -reg, -reg);
             InsNewInst(NULL, NULL, next, ST, k, -reg, 0);
@@ -809,7 +833,7 @@ void FPConstStore(INSTQ *next, short id, short con,
  */
          #elif defined(SPARC)
             ip = (int*) &d;
-            i = SToff[SToff[id-1].sa[2]-1].sa[3];
+            i = GetDTcon(SToff[SToff[id-1].sa[2]-1].sa[3]);
             k = AddDerefEntry(-REG_SP, STderef, -STderef, i, con);
             bitload(next, reg, 12, *ip);
             InsNewInst(NULL, NULL, next, ST, k, -reg, 0);
@@ -821,7 +845,7 @@ void FPConstStore(INSTQ *next, short id, short con,
  */
          #elif defined(PPC)
             ip = (int*) &d;
-            i = SToff[SToff[id-1].sa[2]-1].sa[3];
+            i = GetDTcon(SToff[SToff[id-1].sa[2]-1].sa[3]);
             k = AddDerefEntry(-REG_SP, STderef, -STderef, i, con);
             bitload(next, reg, 16, *ip);
             InsNewInst(NULL, NULL, next, ST, k, -reg, 0);
@@ -849,7 +873,7 @@ void FPConstStore(INSTQ *next, short id, short con,
             InsNewInst(NULL, NULL, next, FST, SToff[id-1].sa[2], -freg, 0);
          #else
             k = AddDerefEntry(-REG_SP, STderef, -STderef, 
-                              SToff[SToff[id-1].sa[2]-1].sa[3], con);
+                              GetDTcon(SToff[SToff[id-1].sa[2]-1].sa[3]), con);
             InsNewInst(NULL, NULL, next, XOR, -reg, -reg, -reg);
             InsNewInst(NULL, NULL, next, ST, k, -reg, 0);
             SignalSet(next, id, k, freg);
@@ -860,7 +884,7 @@ void FPConstStore(INSTQ *next, short id, short con,
          ip = (int*) &f;
          #ifndef X86
             k = AddDerefEntry(-REG_SP, STderef, -STderef, 
-                              SToff[SToff[id-1].sa[2]-1].sa[3], con);
+                              GetDTcon(SToff[SToff[id-1].sa[2]-1].sa[3]), con);
          #endif
          #ifdef X86
             InsNewInst(NULL, NULL, next, MOV, -reg, STiconstlookup(*ip), 0);
@@ -941,8 +965,8 @@ void Extern2Local(INSTQ *next, int rsav)
 {
    extern int NPARA, DTnzerod, DTnzero, DTabsd, DTabs; 
    /*extern int        DTnzerods, DTnzeros, DTabsds, DTabss;*/
-   extern int FKO_FLAG;
-   short i, j=0, flag, ir, k, kk, reg1=0, freg, dreg, vfreg, vdreg;
+   /*extern int FKO_FLAG;*/
+   short i, j=0, flag, ir, k, reg1=0, freg, dreg, vfreg, vdreg;
    int USED;
    #ifdef X86_64
       int nof, ni, nd, dr;
@@ -1327,7 +1351,7 @@ void Extern2Local(INSTQ *next, int rsav)
                if (USED)
                {
                   k = AddDerefEntry(-REG_SP, STderef, -STderef, 
-                                    SToff[SToff[paras[i]].sa[2]-1].sa[3],
+                                 GetDTcon(SToff[SToff[paras[i]].sa[2]-1].sa[3]),
                                     paras[i]+1);
                   InsNewInst(NULL, NULL, next, ST, k, -ir, 0);
                   SignalSet(next, paras[i]+1, k, freg);
@@ -1353,7 +1377,7 @@ void Extern2Local(INSTQ *next, int rsav)
                j++;
                if (USED)
                {
-                  kk = SToff[SToff[paras[i]].sa[2]-1].sa[3];
+                  kk = GetDTcon(SToff[SToff[paras[i]].sa[2]-1].sa[3]);
                   k = AddDerefEntry(-REG_SP, STderef, -STderef, kk, paras[i]+1);
                   InsNewInst(NULL, NULL, next, ST, k, -ir, 0);
                   nam[2] = j + '0';
@@ -1372,7 +1396,7 @@ void Extern2Local(INSTQ *next, int rsav)
                j++;
                if (USED)
                {
-                  kk = SToff[SToff[paras[i]].sa[2]-1].sa[3];
+                  kk = GetDTcon(SToff[SToff[paras[i]].sa[2]-1].sa[3]);
                   k = AddDerefEntry(-REG_SP, STderef, -STderef, kk, paras[i]+1);
                   InsNewInst(NULL, NULL, next, ST, k, -ir, 0);
 
