@@ -175,6 +175,16 @@ void DoMove(short dest, short src)
    {
       type = FLAG2PTYPE(sflag);
       rsrc = GetReg(type);
+/*
+ *    FIXME: need to add const_init for each floating point const, by this way
+ *    we can have a placeholder for that const in stack. We should/will create 
+ *    the placeholder whenever user uses a floating point const, but until it's
+ *    upto user to manage it correctly. 
+ *    Don't need any place-holder for the value 0.0. we can use FZERO LIL 
+ *    inst for that. It won't maintain the load->Arith->store structure of LIL.
+ *    Need to check whether it creates any problem in later transformation!!!
+ */
+#if 0      
       if (IS_CONST(sflag) && (type == T_INT) && SToff[src-1].i == 0)
          InsNewInst(NULL, NULL, NULL, XOR, -rsrc, -rsrc, -rsrc);
       else
@@ -189,6 +199,41 @@ void DoMove(short dest, short src)
          InsNewInst(NULL, NULL, NULL, mov, -rsrc, src, 0);
       }
       LocalStore(dest, rsrc);
+#else
+      if (FLAG2TYPE(STflag[dest-1]) != FLAG2TYPE(sflag))
+         fko_error(__LINE__, "Conversions of constant not yet supported");
+
+      if (type == T_INT)
+      { 
+         if (SToff[src-1].i == 0)
+            InsNewInst(NULL, NULL, NULL, XOR, -rsrc, -rsrc, -rsrc);
+         else
+         {
+            mov = MOV;
+            InsNewInst(NULL, NULL, NULL, mov, -rsrc, src, 0);
+         }
+      }
+      else if (type == T_FLOAT)
+      {
+         if (SToff[src-1].f == 0.0)
+            InsNewInst(NULL, NULL, NULL, FZERO, -rsrc, 0, 0);
+         else
+            fko_error(__LINE__, 
+               "Floating point const other than zero must be defined before");
+      }
+      else if (type == T_DOUBLE)
+      {
+         if (SToff[src-1].d == 0.0)
+            InsNewInst(NULL, NULL, NULL, FZEROD, -rsrc, 0, 0);
+         else
+            fko_error(__LINE__, 
+               "Floating point const other than zero must be defined before");
+      }
+      else
+         fko_error(__LINE__, "unsupported constant!");
+      
+      LocalStore(dest, rsrc);
+#endif
    }
    else if (FLAG2TYPE(STflag[dest-1]) == FLAG2TYPE(sflag))
    {
@@ -210,6 +255,11 @@ short AddArrayDeref(short array, short index, int offset)
    if (IS_DOUBLE(flag)) mul = 8;
    else if (IS_CHAR(flag)) mul = 1;
    assert(!IS_VEC(flag));
+/*
+ * FIXED: we multiply the offset with mul always... for architectures which 
+ * don't support LoadMul/ConstAndIndex, they will be fixed in FixDeref() 
+ */
+   offset *= mul;
    return(AddDerefEntry(array, index, mul, offset, array));
 }
 
@@ -226,48 +276,75 @@ static void FixDeref(short ptr)
  */
 {
    short type;
+   short sta;
 
    ptr--;
    type = FLAG2TYPE(STflag[SToff[ptr].sa[0]-1]);
 /* fprintf(stderr, "FixDeref: [%d, %d, %d, %d]\n", SToff[ptr].sa[0], 
  *         SToff[ptr].sa[1], SToff[ptr].sa[2], SToff[ptr].sa[3]); */
 /*
- * Load beginning of array
+ * FIXED: incase of optimized 2D ptr, we want to keep the DT intact.
+ * it is applied only on x86. so, it's safe to keep that unchanged
+ * NOTE: mulitplying offset with datatype is done while creating the DT entry.
  */
-   SToff[ptr].sa[0] = -LocalLoad(SToff[ptr].sa[0]);
-/*
- * Multiply constant by mul
- */
-   if (SToff[ptr].sa[2]) SToff[ptr].sa[3] *= SToff[ptr].sa[2];
-/*
- * Load index register if needed
- */
-   if (SToff[ptr].sa[1])
+   sta = STarrColPtrlookup(SToff[ptr].sa[0]); 
+   if ( (FKO_FLAG & IFF_OPT2DPTR) 
+         && sta && STarr[sta-1].ndim > 1)
    {
-      SToff[ptr].sa[1] = -LocalLoad(SToff[ptr].sa[1]);
 /*
- *    Some architectures cannot multiply the index register by some (or any)
- *    constants, and in this case generate an extra shift instruction
+ *    Load beginning of array
  */
-      if (!ArchHasLoadMul(SToff[ptr].sa[2]))
+      SToff[ptr].sa[0] = -LocalLoad(SToff[ptr].sa[0]);
+/*
+ *    Load index register if needed
+ *    we kept mul and con intact 
+ */
+      if (SToff[ptr].sa[1])
+         SToff[ptr].sa[1] = -LocalLoad(SToff[ptr].sa[1]);      
+   }
+   else /* kept the old code and logic unchanged for all other case*/
+   {
+/*
+ *    Load beginning of array
+ */
+      SToff[ptr].sa[0] = -LocalLoad(SToff[ptr].sa[0]);
+/*
+ *    Multiply constant by mul
+ *    FIXED: for const index, we already multiply datasize at the beginning
+ *    ref AddArrayDeref()
+ */
+      /*if (SToff[ptr].sa[2]) SToff[ptr].sa[3] *= SToff[ptr].sa[2];*/
+/*
+ *    Load index register if needed
+ */
+      if (SToff[ptr].sa[1])
       {
-         InsNewInst(NULL, NULL, NULL, SHL, SToff[ptr].sa[1], SToff[ptr].sa[1], 
-                    STiconstlookup(type2shift(type)));
-         SToff[ptr].sa[2] = 1;
-      }
+         SToff[ptr].sa[1] = -LocalLoad(SToff[ptr].sa[1]);
 /*
- *    On machines with fixed-size instructions, you usually need to choose
- *    _either_ an index register, _or_ a constant addition.  If we have both
- *    on such a machine, add the constant to the index register
+ *       Some architectures cannot multiply the index register by some (or any)
+ *       constants, and in this case generate an extra shift instruction
  */
-      #ifndef ArchConstAndIndex
-         if (SToff[ptr].sa[3])
+         if (!ArchHasLoadMul(SToff[ptr].sa[2]))
          {
-            InsNewInst(NULL, NULL, NULL, ADD, SToff[ptr].sa[1], 
-                       SToff[ptr].sa[1], STiconstlookup(SToff[ptr].sa[3]));
-            SToff[ptr].sa[3] = 0;
+            InsNewInst(NULL, NULL, NULL, SHL, SToff[ptr].sa[1], SToff[ptr].sa[1], 
+                       STiconstlookup(type2shift(type)));
+            SToff[ptr].sa[2] = 1;
          }
-      #endif
+/*
+ *       On machines with fixed-size instructions, you usually need to choose
+ *       _either_ an index register, _or_ a constant addition.  If we have both
+ *       on such a machine, add the constant to the index register
+ */
+         #ifndef ArchConstAndIndex
+            if (SToff[ptr].sa[3])
+            {
+               InsNewInst(NULL, NULL, NULL, ADD, SToff[ptr].sa[1], 
+                          SToff[ptr].sa[1], 
+                          STiconstlookup(GetDTcon(SToff[ptr].sa[3])));
+               SToff[ptr].sa[3] = 0;
+            }
+         #endif
+      }
    }
 }
 
@@ -581,20 +658,17 @@ void Handle2dArrayPtrArith(short dest, short src0, char op, short src1)
    }
 }
 
-
 void DoArith(short dest, short src0, char op, short src1)
 {
-   short rd, rs0, rs1, type;
+   short rd, rs0, rs1, rs2, type;
    enum inst inst;
    extern int DTnzerod, DTnzero, DTabs, DTabsd;
 
    if (IS_PTR(STflag[dest-1]))
    {
-#if 1
       if (SToff[dest-1].sa[3]) /* 2D array pointer? */
          Handle2dArrayPtrArith(dest, src0, op, src1);
       else
-#endif
          HandlePtrArith(dest, src0, op, src1);
       return;
    }
@@ -623,13 +697,58 @@ void DoArith(short dest, short src0, char op, short src1)
          else
             InsNewInst(NULL, NULL, NULL, LD, -rs1, SToff[src1-1].sa[2], 0);
          InsNewInst(NULL, NULL, NULL, DIV, -rd, -rs0, -rs1);
-         fprintf(stderr, "DIV %d, %d, %d\n", -rd, -rs0, -rs1);
+         /*fprintf(stderr, "DIV %d, %d, %d\n", -rd, -rs0, -rs1);*/
          LocalStore(dest, rd);
          GetReg(-1);
          return;
       }
    #endif
-   rd = rs0 = LocalLoad(src0);
+/*
+ * if 3 operands supported and dest != src0, use separate regs for destination
+ * NOTE: for MAC, dest is also in use, so load dest later
+ */
+   rs0 = LocalLoad(src0);
+   if (op != 'm' && dest != src0)
+   {
+      switch (type)
+      {
+         case T_INT:
+            #ifdef ArchHasINTthreeOps
+               rd = GetReg(type);
+            #else
+               rd = rs0;
+            #endif
+            break;
+         case T_FLOAT: case T_DOUBLE: 
+            #ifdef ArchHasFPthreeOps
+               rd = GetReg(type);
+            #else
+               rd = rs0;
+            #endif
+            break;
+         case T_VINT:
+            #ifdef ArchHasVINTthreeOps
+               rd = GetReg(type);
+            #else
+               rd = rs0;
+            #endif
+            break;
+         case T_VFLOAT: case T_VDOUBLE:
+            #ifdef ArchHasVFPthreeOps
+               rd = GetReg(type);
+            #else
+               rd = rs0;
+            #endif
+            break;
+         default: 
+            fko_error(__LINE__, "Unknown type!!!");
+      }
+   }
+   else
+      rd = rs0;
+/*
+ * 2nd src
+ */
    if (op != 'n' && op != 'a')
    {
       if (src0 != src1)
@@ -643,6 +762,9 @@ void DoArith(short dest, short src0, char op, short src1)
    if ( (op == '%' || op == '>' || op == '<') && 
         (type != T_INT && type != T_SHORT) )
       fko_error(__LINE__,"modulo and shift not defined for non-integer types");
+/*
+ * find appropriate inst
+ */
    switch(op)
    {
    case '+': /* dest = src0 + src1 */
@@ -657,14 +779,12 @@ void DoArith(short dest, short src0, char op, short src1)
       case T_DOUBLE:
          inst = FADDD;
          break;
-#if 1
       case T_VFLOAT:
          inst = VFADD;
          break;
       case T_VDOUBLE:
          inst = VDADD;
          break;
-#endif
       }
       break;
    case '-': /* dest = src0 - src1 */
@@ -679,14 +799,12 @@ void DoArith(short dest, short src0, char op, short src1)
       case T_DOUBLE:
          inst = FSUBD;
          break;
-#if 1
       case T_VFLOAT:
          inst = VFSUB;
          break;
       case T_VDOUBLE:
          inst = VDSUB;
          break;
-#endif
       }
       break;
    case '*': /* dest = src0 * src1 */
@@ -701,14 +819,12 @@ void DoArith(short dest, short src0, char op, short src1)
       case T_DOUBLE:
          inst = FMULD;
          break;
-#if 1
       case T_VFLOAT:
          inst = VFMUL;
          break;
       case T_VDOUBLE:
          inst = VDMUL;
          break;
-#endif
       }
       break;
    case '/': /* dest = src0 / src1 */
@@ -723,14 +839,12 @@ void DoArith(short dest, short src0, char op, short src1)
       case T_DOUBLE:
          inst = FDIVD;
          break;
-#if 1
       case T_VFLOAT:
          inst = VFDIV;
          break;
       case T_VDOUBLE:
          inst = VDDIV;
          break;
-#endif
       }
       break;
    case '%': /* dest = src0 % src1 */
@@ -750,48 +864,56 @@ void DoArith(short dest, short src0, char op, short src1)
          #ifdef ArchHasMAC
             rd = LocalLoad(dest);
             if (type == T_FLOAT) inst = FMAC;
-#if 1            
             else if (type == T_DOUBLE) inst = FMACD;
             else if (type == T_VFLOAT) inst = VFMAC;
             else if (type == T_VDOUBLE) inst = VDMAC;
             else 
                fko_error(__LINE__, "MAC not supported with this type!");
-#else
-            else inst = FMACD; /*Majedul: was a typo, corrected it. */
-#endif
          #else
-            if (type == T_FLOAT)
+            switch(type)
             {
-               InsNewInst(NULL, NULL, NULL, FMUL, -rs0, -rs0, -rs1);
-               inst = FADD;
+               case T_FLOAT: 
+                  #ifdef ArchHasFPthreeOps
+                     rs2 = GetReg(type);
+                  #else
+                     rs2 = rs0;
+                  #endif
+                  InsNewInst(NULL, NULL, NULL, FMUL, -rs2, -rs0, -rs1);
+                  inst = FADD;
+                  break;
+               case T_DOUBLE:
+                  #ifdef ArchHasFPthreeOps
+                     rs2 = GetReg(type);
+                  #else
+                     rs2 = rs0;
+                  #endif
+                  InsNewInst(NULL, NULL, NULL, FMULD, -rs2, -rs0, -rs1);
+                  inst = FADDD;
+                  break;
+               case T_VFLOAT: 
+                  #ifdef ArchHasVFPthreeOps
+                     rs2 = GetReg(type);
+                  #else
+                     rs2 = rs0;
+                  #endif
+                  InsNewInst(NULL, NULL, NULL, VFMUL, -rs2, -rs0, -rs1);
+                  inst = VFADD;
+               case T_VDOUBLE:
+                  #ifdef ArchHasVFPthreeOps
+                     rs2 = GetReg(type);
+                  #else
+                     rs2 = rs0;
+                  #endif
+                  InsNewInst(NULL, NULL, NULL, VDMUL, -rs2, -rs0, -rs1);
+                  inst = VDADD;
+                  break;
+               default: 
+                  fko_error(stderr, "MAC not supported with this type!");
             }
-#if 1
-            else if (type == T_DOUBLE)
-            {
-               InsNewInst(NULL, NULL, NULL, FMULD, -rs0, -rs0, -rs1);
-               inst = FADDD;
-            }
-            else if (type == T_VFLOAT)
-            {
-               InsNewInst(NULL, NULL, NULL, VFMUL, -rs0, -rs0, -rs1);
-               inst = VFADD;
-            }
-            else if (type == T_VDOUBLE)
-            {
-               InsNewInst(NULL, NULL, NULL, VDMUL, -rs0, -rs0, -rs1);
-               inst = VDADD;
-            }
-            else
-               fko_error(__LINE__, "MAC not supported with this type!");
-
-#else
-            else
-            {
-               InsNewInst(NULL, NULL, NULL, FMULD, -rs0, -rs0, -rs1);
-               inst = FADDD;
-            }
-#endif            
-            rs1 = rs0;
+/*
+ *          for addition, dest is also src0
+ */
+            rs1 = rs2;
             rs0 = rd = LocalLoad(dest);
          #endif
       }
@@ -1767,9 +1889,6 @@ void DoReduce(short sid, short vid, char op, short iconst)
       else
          fko_error(__LINE__, 
                   "supported element count in vector: 2,4,8!!!");
-#endif
-#if 1
-      fprintf(stderr, "0x%x\n", k);
 #endif
       r0 = GetReg(FLAG2TYPE(flag));
       InsNewInst(NULL, NULL, NULL, vld, -r0, SToff[vid-1].sa[2], 0);
