@@ -17,27 +17,87 @@
  *    already freed)
  *
  */
-
 char **STname;
 union valoff *SToff;
 int *STflag;
 short *STpts2;
-struct arrayinfo *STarr; /* to save information of multi-dim array*/
-
+struct arrayinfo *STarr = NULL; /* to save information of multi-dim array*/
+INT_DTC *DTcon = NULL;
 
 static int N=0, Nalloc=0; /* for Symbol table */
 static int Narr=0, TNarr=0; /* for array table */
-static int niloc=0, nlloc=0, nfloc=0, ndloc=0, nvfloc=0, nvdloc=0;
+static int Ndc=0, TNdc=0;
 
-#if 1
+static int niloc=0, nlloc=0, nfloc=0, ndloc=0, nvfloc=0, nvdloc=0;
 static int nviloc=0;
-#endif
 
 int LOCSIZE=0, LOCALIGN=0, NPARA=0;
 
-#define STCHUNK 1024    /* increased for safety ...*/
+#define STCHUNK 1024 
 #define DTCHUNK 1024
+#define DCCHUNK 256
+#define MAXENTRY 32767 
 
+static void GetNewSymtab(int chunk)
+{
+   char **nam;
+   int *flg;
+   short *pts2;
+   union valoff *off;
+   int i, n;
+   n = Nalloc + chunk;
+   nam = malloc(sizeof(char*)*n);
+   assert(nam);
+   off = malloc(n*sizeof(union valoff));
+   flg = malloc(n*sizeof(int));
+   pts2 = malloc(n*sizeof(short));
+   assert(off && flg && pts2);
+   if (Nalloc > 0)
+   {
+      for (i=0; i < N; i++)
+      {
+         nam[i] = STname[i];
+         off[i].d = SToff[i].d;
+         flg[i] = STflag[i];
+         pts2[i] = STpts2[i];
+      }
+      free(STname);
+      free(SToff);
+      free(STflag);
+      free(STpts2);
+   }
+   STname = nam;
+   SToff = off;
+   STflag = flg;
+   STpts2 = pts2;
+   Nalloc = n;
+}
+
+static short STnew(char *name, int flag, union valoff off)
+{
+   if (N == Nalloc) GetNewSymtab(STCHUNK);
+   if (name)
+   {
+      STname[N] = malloc(strlen(name)+1);
+      assert(STname[N]);
+      strcpy(STname[N], name);
+   }
+   else STname[N] = NULL;
+   STflag[N] = flag;
+   SToff[N] = off;
+   STpts2[N] = 0;
+#if 1
+   assert(N!=MAXENTRY);
+#endif
+   return(++N);
+}
+
+short STdef(char *name, int flag, int off)
+{
+   union valoff offset;
+   offset.i = off;
+   return(STnew(name, flag, offset));
+}
 
 static void NewArrTable(int chunk)
 /*
@@ -97,6 +157,9 @@ short AddSTarr(short ptr, short ndim, short *ldas)
  */
    STarr[Narr].cldas = NULL; 
    STarr[Narr].colptrs = NULL; 
+#if 1
+   assert(Narr!=MAXENTRY);
+#endif
    return (++Narr);
 }
 
@@ -123,64 +186,85 @@ int UpdateSTarrUnroll(short id, short *ulist)
    return(1);
 }
 
-static void GetNewSymtab(int chunk)
+static void GetNewDtcTable(int chunk)
 {
-   char **nam;
-   int *flg;
-   short *pts2;
-   union valoff *off;
    int i, n;
-   n = Nalloc + chunk;
-   nam = malloc(sizeof(char*)*n);
-   assert(nam);
-   off = malloc(n*sizeof(union valoff));
-   flg = malloc(n*sizeof(int));
-   pts2 = malloc(n*sizeof(short));
-   assert(off && flg && pts2);
-   if (Nalloc > 0)
+   INT_DTC *newdtc;
+   n = TNdc + chunk;
+   newdtc = malloc(sizeof(INT_DTC)*n);
+   assert(newdtc);
+   if (TNdc > 0)
    {
-      for (i=0; i < N; i++)
-      {
-         nam[i] = STname[i];
-         off[i].d = SToff[i].d;
-         flg[i] = STflag[i];
-         pts2[i] = STpts2[i];
-      }
-      free(STname);
-      free(SToff);
-      free(STflag);
-      free(STpts2);
+      for (i=0; i < Ndc; i++)
+         newdtc[i] = DTcon[i];
+      free(DTcon);
    }
-   STname = nam;
-   SToff = off;
-   STflag = flg;
-   STpts2 = pts2;
-   Nalloc = n;
+   DTcon = newdtc;
+   TNdc = n;
 }
 
-static short STnew(char *name, int flag, union valoff off)
+static int DTCnew(INT_DTC val)
 {
-   if (N == Nalloc) GetNewSymtab(STCHUNK);
-   if (name)
+   if (Ndc == TNdc)
+      GetNewDtcTable(DCCHUNK);
+   DTcon[Ndc] = val;
+#if 1
+   assert(Ndc!=MAXENTRY);
+#endif
+   return(++Ndc);
+}
+
+short DTClookup(INT_DTC val)
+/*
+ * Assumption: short is used to store the index of DTcon in DT table (sa[3]). 
+ * So, we return short here. the assumption here is : we won't need more than
+ * 2^15 entry of the table
+ */
+{
+   int i;   
+   for (i=0; i < Ndc; i++)
    {
-      STname[N] = malloc(strlen(name)+1);
-      assert(STname[N]);
-      strcpy(STname[N], name);
+      if (DTcon[i] == val)
+         return(i+1);
    }
-   else STname[N] = NULL;
-   STflag[N] = flag;
-   SToff[N] = off;
-   STpts2[N] = 0;
-   return(++N);
+   i = DTCnew(val);
+   assert(!((i<<1)&(1<<15))); /* <= 2^14-1*/
+   return(i);
 }
 
-short STdef(char *name, int flag, int off)
+INT_DTC GetDTcon(int val)
 {
-   union valoff offset;
-   offset.i = off;
-   return(STnew(name, flag, offset));
+   if (val & 1)
+   {
+      return(DTcon[(val>>1)-1]);
+   }
+   return(val);
 }
 
+void SetDTcon(int dt, INT_DTC con)
+{
+   int SHORT_MAX = 32767;
+   int SHORT_MIN = -32768;
+   INT_DTC val;
+/*
+ * we will save the const into a table if it is bigger than short type (16 bit)
+ * or, the const is an odd number. Normally, const can't be odd since we 
+ * multiply it with size of datatype. 
+ */
+   if ( (con > SHORT_MAX || con < SHORT_MIN)
+         || (con & 1) )
+   {
+      val = DTClookup(con); 
+      val = (val << 1) | 1;
+      SToff[dt-1].sa[3] = val;
+      /*fprintf(stderr, "***********con=%d, val=%d\n");*/
+   }
+   else /*FIXME: most common case, make it as if-statement */
+   {
+      /*assert(!(con&1));*/
+      SToff[dt-1].sa[3] = con;
+   }
+}
 
 short STdconstlookup(double f)
 /*
@@ -258,6 +342,10 @@ short STlconstlookup(long ic)
          return(i+1);
    }
    val.l = ic;
+/*
+ * NOTE: it will only be needed in X8664 and in original FKO, T_SHORT is used
+ * to represent the long const. see line:333 in l2a.c 
+ */
    return(STnew(NULL, CONST_BIT | T_SHORT, val));
 }
 
@@ -584,6 +672,8 @@ void NumberLocalsByType()
    for (k=0; k != N; k++)
    {
       fl = STflag[k];
+      /*if ((IS_PARA(fl) || IS_LOCAL(fl)) && SToff[SToff[k].sa[2]-1].sa[0] 
+            && !IS_VECELEM(fl))*/
       if ((IS_PARA(fl) || IS_LOCAL(fl)) && SToff[SToff[k].sa[2]-1].sa[0])
       {
          type = FLAG2PTYPE(fl);
@@ -628,6 +718,10 @@ void CreateLocalDerefs()
 {
    short k;
    int fl;
+   /*short st; */
+   /*int i, vl, stype;*/
+   /*char ln[512];*/
+
    for (k=0; k != N; k++)
    {
       fl = STflag[k];
@@ -636,29 +730,74 @@ void CreateLocalDerefs()
          SToff[k].sa[0] = SToff[k].i;
          SToff[k].sa[2] = AddDerefEntry(-REG_SP, k+1, -k-1, 0, k+1);
          SToff[k].sa[3] = 0; /* by default 0, remain 0 for 1D array */
-/*
- *       NOTE: after creating deref entry for each parameter, sa[3] is 
- *       available. we use this sa[3] ST entry to point the STarr
- *       shifted to CreateArrayLdas() function.
- */
-#if 0
-         if (IS_PTR(STflag[k]))
-         {
-            for (i=0; i < Narr; i++)
-            {
-               if (STarr[i].ptr == k+1)
-               {
-                  SToff[k].sa[3] = i+1;
-               }
-            }
-         }
-#endif
       }
       else if (IS_LOCAL(fl))
+      {
+#if 0         
+         if (!IS_VECELEM(fl))
+            SToff[k].sa[2] = AddDerefEntry(-REG_SP, k+1, -k-1, 0, k+1);
+         else
+            SToff[k].sa[2] = AddDerefEntry(-REG_SP, SToff[k].sa[0], -k-1, 0, 
+                                           k+1);
+/*
+ *       adding scalars for each vector elements
+ *       For local, SToff[].sa[0] is unused. we used this to point the parent 
+ *       vector ST
+ */
+         if (IS_VEC(fl))
+         {
+            SToff[k].sa[2] = AddDerefEntry(-REG_SP, k+1, -k-1, 0, k+1);
+            SToff[k].sa[1] = 0; /* will be changed later! */
+            vl = vtype2elem(FLAG2TYPE(fl));
+            if (IS_VFLOAT(fl))
+               stype = T_FLOAT;
+            else if (IS_VDOUBLE(fl))
+               stype = T_DOUBLE;
+            else if (IS_VINT(fl))
+               stype = T_INT;
+
+            for (i=0; i < vl; i++)
+            {
+               sprintf(ln, "_%s_%d", STname[k], i);
+               st = STdef(ln, VELEM_BIT | LOCAL_BIT | stype, 0);
+               SToff[st-1].sa[0] = k+1;
+               SToff[st-1].sa[1] = i+1;
+            }
+         }
+         else if (IS_VECELEM(fl))
+            SToff[k].sa[2] = AddDerefEntry(-REG_SP, SToff[k].sa[0],-k-1,0, k+1);
+         else
+            SToff[k].sa[2] = AddDerefEntry(-REG_SP, k+1, -k-1, 0, k+1);
+#else
          SToff[k].sa[2] = AddDerefEntry(-REG_SP, k+1, -k-1, 0, k+1);
+#endif
+      }
    }
+#if 0
+   PrintST(stderr);
+   exit(0);
+#endif
 }
 
+#if 0
+short FindSTVecElem(short vid, int el)
+{
+   short i;
+   int fl;
+
+   for (i=0; i != N; i++)
+   {
+      fl = STflag[i];
+      if (IS_VECELEM(fl))
+      {
+         if (SToff[i].sa[0] == vid && el == SToff[i].sa[1])
+            return(i+1);
+      }
+   }
+   return(0);
+}
+#endif
+
 void UpdateLocalDerefs(int isize)
 /*
  * Given numbered locals, creates derefs for local access, assuming local
@@ -679,6 +818,26 @@ void UpdateLocalDerefs(int isize)
    for (k=0; k != N; k++)
    {
       fl = STflag[k];
+#if 0
+      if (IS_VECELEM(fl) && SToff[SToff[k].sa[2]-1].sa[0]) /* vector element */
+      {
+         if (IS_FLOAT(fl)) 
+         {
+            SToff[SToff[k].sa[2]-1].sa[3] = 
+               SToff[SToff[SToff[k].sa[0]-1].sa[2]-1].sa[3] 
+               + (SToff[k].sa[1]-1) * 4;
+         }
+         else if (IS_DOUBLE(fl))
+         {
+            SToff[SToff[k].sa[2]-1].sa[3] = 
+               SToff[SToff[SToff[k].sa[0]-1].sa[2]-1].sa[3] 
+               + (SToff[k].sa[1]-1) * 8;
+         }
+         else
+            fko_error(__LINE__, "vtype not imp yet");
+      }
+      else
+#endif
       if ((IS_PARA(fl) || IS_LOCAL(fl)) && SToff[SToff[k].sa[2]-1].sa[0])
       {
 /* fprintf(stderr, "Updating local %s\n", STname[k]); */
@@ -864,10 +1023,13 @@ void MarkFinalUnusedLocals(BBLOCK *bbase)
 {
    INSTQ *ip;
    short k, i;
-   enum inst inst;
+   /*enum inst inst;*/
    BBLOCK *bp;
-   extern int DTabsd, DTnzerod, DTabs, DTnzero, DTx87, DTx87d;
-   extern int DTabsds, DTnzerods, DTabss, DTnzeros;
+   #ifdef X86_32
+      extern int DTx87, DTx87d;
+   #endif
+   /*extern int DTabsd, DTnzerod, DTabs, DTnzero;*/
+   /*extern int DTabsds, DTnzerods, DTabss, DTnzeros;*/
 
 /*
  * Start out by marking all locals as unused, except the system const
@@ -954,6 +1116,7 @@ void KillStaticData(void)
  * all of which except mul & con are indexes into the symbol table.  Mul and
  * con are simply short constants.
  * Addressing is: ptr+reg*mul+con
+ * NOTE: con now can be an index of INT_DTC table
  */
 short FindDerefEntry(short ptr, short ireg, short mul, short con)
 {
@@ -961,16 +1124,19 @@ short FindDerefEntry(short ptr, short ireg, short mul, short con)
    for (i=0; i != N; i++)
    {
       if ( IS_DEREF(STflag[i]) && SToff[i].sa[0] == ptr && 
-           SToff[i].sa[2] == mul && SToff[i].sa[3] == con &&
+           SToff[i].sa[2] == mul && GetDTcon(SToff[i].sa[3]) == con &&
            (SToff[i].sa[1] == ireg || (!ireg && SToff[i].sa[1] > 0)) )
               return(i+1);
    }
    return(0);
 }
 
-short AddDerefEntry(short ptr, short reg, short mul, short con, short pts2)
+short AddDerefEntry(short ptr, short reg, short mul, int con, short pts2)
 {
    int i;
+   /*int icon;
+   int SHORT_MAX = 32767; 
+   int SHORT_MIN = -32768; */
 /*
  * Majedul: pts2 can be 0, see FinalizeEpilogue. 
  */
@@ -983,8 +1149,9 @@ short AddDerefEntry(short ptr, short reg, short mul, short con, short pts2)
    SToff[i].sa[0] = ptr;
    SToff[i].sa[1] = reg;
    SToff[i].sa[2] = mul;
-   SToff[i].sa[3] = con;
    STpts2[i] = pts2;
+   SetDTcon(i+1, con);
+   
    return(i+1);
 }
 
@@ -1202,6 +1369,10 @@ void PrintSymtabStaticMember(FILE *fpout)
    fprintf(fpout, "ndloc = %d\n", ndloc);
    fprintf(fpout, "nvfloc = %d\n", nvfloc);
    fprintf(fpout, "nvdloc = %d\n", nvdloc);
+   fprintf(fpout, "Narr = %d\n", Narr);
+   fprintf(fpout, "TNarr = %d\n", TNarr);
+   fprintf(fpout, "Ndc = %d\n", Ndc);
+   fprintf(fpout, "TNdc = %d\n", TNdc);
    fprintf(fpout, "-------------------------\n");
 }
 
@@ -1564,4 +1735,19 @@ void CreateArrColPtrs()
 #endif
 }
 
-
+void KillSTarr()
+{
+   int i;
+   for (i=0; i < Narr; i++)
+   {
+      if (STarr[i].ldas)
+         free(STarr[i].ldas);
+      if (STarr[i].urlist)
+         free(STarr[i].urlist);
+      if (STarr[i].colptrs)
+         free(STarr[i].colptrs);
+      if (STarr[i].cldas)
+         free(STarr[i].cldas);
+   }
+   if(STarr) free(STarr);
+}
