@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.2
- *                    (C) Copyright 1998 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * Copyright (C) 1998, 2015 R. Clint Whaley
  */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -270,6 +244,7 @@ long long GetFirstLong(char *ln)
    }
    return(iret);
 }
+
 long long GetFirstLongHex(char *ln)
 {
    int i;
@@ -317,6 +292,7 @@ int GetLastInt(char *ln)
    }
    return(iret);
 }
+
 int GetLastHex(char *ln)
 {
    int i, iret=0;
@@ -345,6 +321,7 @@ long long GetLastLong(char *ln)
    }
    return(iret);
 }
+
 long long GetLastLongHex(char *ln)
 {
    int i;
@@ -369,21 +346,30 @@ long long GetLastLongWithRound(char *ln)
 {
    int i;
    long iret=0;
-   for (i=0; ln[i]; i++);
-   if (i > 0)
+   double d;
+
+   for (i=0; ln[i]; i++);  /* find end of string */
+   if (!i)                 /* empty string */
+     return(0);            /* returns 0 */
+/*
+ * Now skip any non-digit info at end of string (eg., MHz)
+ */
+   for(i--; i > 0 && !isdigit(ln[i]); i--);
+   if (!isdigit(ln[i])) /* no digits in string */
+      return(0);        /* returns 0 */
+/*
+ * Go backwards until we find a non-digit to possibly end the number
+ */
+   for (i--; i > 0 && isdigit(ln[i]); i--);
+   if (i > 0)  /* may be more to this number */
    {
-      for (i--; i > 0 && !isdigit(ln[i]); i--);
-      if (i > 0 && ln[i] == '.')  /* allow skip of 1 decimal point */
-         for (i--; i > 0 && !isdigit(ln[i]); i--);
-   }
-   if (i > 0 || (i == 0 && (isdigit(ln[0]) || ln[0] == '.')))
-   {
-      double d;
-      while(isdigit(ln[i]) && i > 0) i--;
-      if (!isdigit(ln[i])) i++;
-      sscanf(ln+i, "%lf", &d);
-      iret = (int)(d+0.5);
+      if (ln[i] == '.')
+         for(i--; i >0 && isdigit(ln[i]); i--);
    }
+   else if (ln[0] != '.' && !isdigit(ln[0]))
+      i = 1;
+   sscanf(ln+i, "%lf", &d);
+   iret = (int)(d+0.5);
    return(iret);
 }
 
@@ -408,8 +394,11 @@ static void ATL_Cassert0(size_t cond, char *exp, char *logfile, int line)
          "directory.  Be sure to include this file in any help request.\n");
    }
    if (fperr) fclose(fperr);
-   system("cat ../../CONFIG/error.txt >> INSTALL_LOG/ERROR.LOG");
-   system("cat ../../CONFIG/error.txt");
+/*
+ * Ignore ifs: fucking gcc/glibc forces me to make code less maintainable
+ */
+   if(system("cat ../../CONFIG/error.txt >> INSTALL_LOG/ERROR.LOG"));
+   if(system("cat ../../CONFIG/error.txt"));
    exit(-1);
 }
 #define ATL_Cassert(cond_, exp_, logfile_) \
@@ -425,9 +414,10 @@ void GetInstLogFile(char *nam, int pre, int *muladd, int *pf, int *lat,
    fp = fopen(nam, "r");
    if (fp == NULL) fprintf(stderr, "file %s not found!!\n\n", nam);
    assert(fp);
-   fgets(ln, 128, fp);
-   fscanf(fp, " %d %d %d %d %d %d %d %d %d %d %lf\n",
-          muladd, lat, pf, nb, mu, nu, ku, ForceFetch, ifetch, nfetch, mflop);
+   assert(fgets(ln, 128, fp));
+   assert(fscanf(fp, " %d %d %d %d %d %d %d %d %d %d %lf\n",
+          muladd, lat, pf, nb, mu, nu, ku, ForceFetch, ifetch, nfetch, mflop)
+          == 11);
    fclose(fp);
 }
 
@@ -444,10 +434,10 @@ void PrintBanner(char *fnam, int START, int sec, int subsec, int subsubsec)
    GetDate(&month, &day, &year, &hour, &min);
    fprintf(fp, "\n%s%s%s", sep, sep, sep);
    if (START)
-      fprintf(fp, "*       BEGAN ATLAS3.10.2 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d     *\n",
+      fprintf(fp, "*       BEGAN ATLAS3.11.4 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d     *\n",
               sec, subsec, subsubsec, month, day, year, hour, min);
    else
-      fprintf(fp, "*      FINISHED ATLAS3.10.2 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d   *\n",
+      fprintf(fp, "*      FINISHED ATLAS3.11.4 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d   *\n",
                       sec, subsec, subsubsec, month, day, year, hour, min);
    fprintf(fp, "%s%s%s\n\n\n", sep, sep, sep);
    fclose(fp);
@@ -483,9 +473,9 @@ void PrintStartStop(FILE *fp0, FILE *fp1, int nspc, int START, int sec,
          fprintf(fp1, "\n\n");
    }
    if (fp0)
-      fprintf(fp0, ln);
+      fprintf(fp0, "%s", ln);
    if (fp1)
-      fprintf(fp1, ln);
+      fprintf(fp1, "%s", ln);
 }
 
 int LnIsCont(char *ln)
@@ -514,8 +504,8 @@ void GetuMMRES(char pre, int ID, int *muladd, int *lat,
    sprintf(ln, "../tune/blas/gemm/%ccases.dsc", pre);
    fp = fopen(ln, "r");
    assert(fp);
-   fgets(ln, 1024, fp);  /* skip comment line */
-   fgets(ln, 1024, fp);  /* get number of user cases line */
+   assert(fgets(ln, 1024, fp));  /* skip comment line */
+   assert(fgets(ln, 1024, fp));  /* get number of user cases line */
    assert(sscanf(ln, " %d", &n) == 1);
 /*
  * Now search file for ID
@@ -548,11 +538,11 @@ void GetMMRES(char pre, int *muladd, int *lat, int *nb, int *pref,
    sprintf(ln, "INSTALL_LOG/%cMMRES", pre);
    fp = fopen(ln, "r");
    assert(fp);
-   fgets(ln, 256, fp);
-   fgets(ln, 256, fp);
+   assert(fgets(ln, 256, fp));
+   assert(fgets(ln, 256, fp));
    assert(sscanf(ln, " %d %d %d %d %d %d %d %d %d %d %lf", muladd, lat, pref,
                  nb, mu, nu, ku, ff, iff, nf, mf) == 11);
-   fgets(ln, 256, fp);
+   assert(fgets(ln, 256, fp));
    if ( fgets(ln, 256, fp) )  /* user-supplied GEMM was best */
    {
       assert(fscanf(fp, " %d %d %lf \"%[^\"]\" \"%[^\"]",
@@ -567,98 +557,7 @@ void GetMMRES(char pre, int *muladd, int *lat, int *nb, int *pref,
    fclose(fp);
 }
 
-/*
- *             Automatically Tuned Linear Algebra Software v3.10.2
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Code contributers : R. Clint Whaley, Antoine P. Petitet
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- */
-
-int ATL_lcm(const int M, const int N)
-/*
- * Returns least common multiple (LCM) of two positive integers M & N by
- * computing greatest common divisor (GCD) and using the property that
- * M*N = GCD*LCM.
- */
-{
-   register int tmp, max, min, gcd=0;
-
-   if (M != N)
-   {
-      if (M > N) { max = M; min = N; }
-      else { max = N; min = M; }
-      if (min > 0)  /* undefined for negative numbers */
-      {
-         do  /* while (min) */
-         {
-            if ( !(min & 1) ) /* min is even */
-            {
-               if ( !(max & 1) ) /* max is also even */
-               {
-                  do
-                  {
-                     min >>= 1;
-                     max >>= 1;
-                     gcd++;
-                     if (min & 1) goto MinIsOdd;
-                  }
-                  while ( !(max & 1) );
-               }
-               do min >>=1 ; while ( !(min & 1) );
-            }
-/*
- *          Once min is odd, halve max until it too is odd.  Then, use
- *          property that gcd(max, min) = gcd(max, (max-min)/2)
- *          for odd max & min
- */
-MinIsOdd:
-            if (min != 1)
-            {
-               do  /* while (max >= min */
-               {
-                  max -= (max & 1) ? min : 0;
-                  max >>= 1;
-               }
-               while (max >= min);
-            }
-            else return( (M*N) / (1<<gcd) );
-            tmp = max;
-            max = min;
-            min = tmp;
-         }
-         while(tmp);
-      }
-      return( (M*N) / (max<<gcd) );
-   }
-   else return(M);
-}
-
-void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
+void GoToTown(int ARCHDEF, int L1DEF, int TuneLA, int NREG, int INDTHR)
 {
    const char TR[2] = {'N','T'};
    char prec[4] = {'d', 's', 'z', 'c'}, pre, upre, *typ;
@@ -675,7 +574,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    int idU, maU, latU, muU, nuU, kuU, il1mul, pfA;
    double mfU, mf4x1, mf4x4, mf, mfp, mmmf, mfpeak[2], l1mul;
    FILE *fp, *fpsum, *fpabr;
-   ATL_mmnode_t *mmp;
+   ATL_mmnode_t *mmp, *mmb;
 
    PrintBanner("INSTALL_LOG/SUMMARY.LOG", 1, 0, 0, 0);
    fpsum = fopen("INSTALL_LOG/SUMMARY.LOG", "a");
@@ -683,29 +582,29 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
 
    ATL_Cassert(tmpnam(tnam), "GETTING TEMPFILE", NULL);
 
-   if (L1DEF)
-   {
-      ATL_Cassert(system("make IBozoL1.grd\n")==0,
-                  "USING BOZO L1 DEFAULTS", NULL);
-   }
-   if (ARCHDEF)
-      DefInstall = !system("make IArchDef.grd\n");
+   DefInstall = !system("make IArchDef.grd\n");
 
    PrintStartStop(stdout, fpsum, 0, 1, 1, 0, 0, "SYSTEM PROBE/AUX COMPILE");
 
 
    sprintf(ln2, "INSTALL_LOG/Stage1.log");
    PrintBanner("INSTALL_LOG/Stage1.log", 1, 1, 0, 0);
-   sprintf(ln, "%s IStage1 %s INSTALL_LOG/Stage1.log\n", fmake, redir);
+   #if defined(ATL_OS_Win64)
+      #if defined(ATL_GAS_ARM64)
+         typ = "winARM64";
+      #else
+         typ = "win64";
+      #endif
+   #elif defined(ATL_OS_Win9x) || defined(ATL_OS_WinNT)
+      typ = "ia32";
+   #else
+      typ = "mut";
+   #endif
+   sprintf(ln, "%s IStage1 nreg=%d LOCK=%s prim=%s %s INSTALL_LOG/Stage1.log\n",
+           fmake, NREG, typ, INDTHR?"ITprim2":"ITprim1", redir);
    ATL_Cassert(system(ln)==0, "Stage 1 install", ln2);
    fp = fopen("INSTALL_LOG/L1CacheSize", "r");
 
-   ATL_Cassert(fp, "CACHESIZE SEARCH", ln2);
-   ATL_Cassert(fscanf(fp, "%d", &i) == 1, "CACHESIZE SEARCH", ln2);
-   fclose(fp);
-   fprintf(stdout, "\n\n   Level 1 cache size calculated as %dKB\n", i);
-   fprintf(fpsum, "   Level 1 cache size calculated as %dKB.\n\n", i);
-   iL1 = i;
 
    for (ip=0; ip < 2; ip++)
    {
@@ -760,33 +659,6 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
          upre = 'd';
          break;
       }
-
-      sprintf(ln, "TUNING PREC=\'%c\' (precision %d of %d)", pre, ip+1, nprec);
-      PrintStartStop(stdout, fpsum, 0, 1, 2, ip+1, 0, ln);
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 1,
-                     "BUILDING BLOCK MATMUL TUNE");
-/*
- *    If necessary, install matmul for this type
- */
-      sprintf(fnam, "INSTALL_LOG/%cMMRES.sum", pre);
-      if (!FileExists(fnam))  /* need to run search or make link */
-      {
-         sprintf(ln2, "INSTALL_LOG/%cMMSEARCH.LOG", pre);
-         PrintBanner(ln2, 1, 2, ip+1, 1);
-         if (DefInstall)
-         {
-            sprintf(ln, "%s IRunMMDef pre=%c %s %s\n", fmake, pre, redir, ln2);
-            fprintf(stdout, ln);
-            ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
-         }
-         sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 1);
-      }
-      mmp = ReadMMFileWithPath(pre, "INSTALL_LOG", "MMRES.sum");
-      assert(mmp);
-      nb = (mmp->next) ? mmp->next->nbB : mmp->nbB;
       #ifdef ATL_CPUMHZ
          fprintf(fpabr, "Clock_rate=%d Mhz\n", ATL_CPUMHZ);
          mfp = ATL_CPUMHZ;
@@ -801,202 +673,21 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
       #endif
       fprintf(fpabr, "%% clock      MFLOP  ROUTINE/PROBLEM\n");
       fprintf(fpabr, "=======  =========  ===============\n");
-      if (mmp->next && mmp->next->mflop[0] > mmp->mflop[0])
-      {
-         ATL_mprintf(2, fpsum, stdout,
-            "      The best matmul kernel was %s, NB=%d, written by %s\n",
-                    mmp->next->rout, nb,
-                    mmp->next->auth ? mmp->next->auth : "R. Clint Whaley");
-         ATL_mprintf(2, fpsum, stdout,
-            "      Performance: %.2fMFLOPS (%5.2f percent of %s)\n",
-                     mmp->next->mflop[0], (mmp->next->mflop[0]/mfp)*100.0,
-                     peakstr);
-         ATL_mprintf(2, fpsum, stdout,
-                     "        (Gen case got %.2fMFLOPS)\n", mmp->mflop[0]);
-         mmmf = mmp->next->mflop[0];
-      }
-      else
-      {
-         mmmf = mmp->mflop[0];
-         ATL_mprintf(2, fpsum, stdout,
-"      %cL1MATMUL: lat=%d, nb=%d, pf=%d, mu=%d, nu=%d, ku=%d, if=%d, nf=%d;\n",
-                     pre, mmp->lat, nb, mmp->pref, mmp->mu, mmp->nu, mmp->ku,
-                     mmp->iftch, mmp->nftch);
-         ATL_mprintf(2, fpsum, stdout,
-            "                 Performance: %.2f (%5.2f percent of %s)\n",
-                     mmmf, (mmmf/mfp)*100.0, peakstr);
-      }
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mmmf/mfp)*100.0, mmmf,
-              "Chosen kgemm");
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mmp->mflop[0]/mfp)*100.0,
-              mmp->mflop[0], "Generated kgemm");
-      KillAllMMNodes(mmp);
 
-      sprintf(fnam, "INSTALL_LOG/%cNCNB", pre);
-      if (!FileExists(fnam))
-      {
-         sprintf(ln, "%s %s pre=%c %s %s", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
-      }
-      fp = fopen(fnam, "r");
-      ATL_Cassert(fp, "OPENING NCNB", NULL);
-      ATL_Cassert(fscanf(fp, " %d", &ncnb) == 1, "READING NCNB", NULL);
-      fclose(fp);
-
-      for (ia=0; ia < 2; ia++)
-      {
-         for (ib=0; ib < 2; ib++)
-         {
-            sprintf(fnam, "INSTALL_LOG/%cbest%c%c_%dx%dx%d", pre, TR[ia], TR[ib],
-                    ncnb, ncnb, ncnb);
-            if (!FileExists(fnam))
-            {
-               sprintf(ln, "%s %s pre=%c nb=%d %s %s",
-                       fmake, fnam, pre, ncnb, redir, ln2);
-               fprintf(stdout, ln);
-               ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
-            }
-            GetInstLogFile(fnam, pre, &muladd, &pf, &lat, &nb, &mu, &nu, &ku,
-                           &ffetch, &ifetch, &nfetch, &mf);
-            fprintf(stdout,
-   "      NCgemm%c%c : muladd=%d, lat=%d, pf=%d, nb=%d, mu=%d, nu=%d ku=%d,\n",
-                    TR[ia], TR[ib], muladd, lat, pf, nb, mu, nu, ku);
-            fprintf(stdout,"                 ForceFetch=%d, ifetch=%d nfetch=%d\n",
-                    ffetch, ifetch, nfetch);
-            fprintf(stdout,
-"                 Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-                    mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-            fprintf(fpsum,
-"      mm%c%c   : ma=%d, lat=%d, nb=%d, mu=%d, nu=%d ku=%d, ff=%d, if=%d, nf=%d\n",
-                    TR[ia], TR[ib], muladd, lat, nb, mu, nu, ku,
-                    ffetch, ifetch, nfetch);
-            fprintf(fpsum,
-"               Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-                    mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-            if (ia != ib)
-               fprintf(fpabr, "%7.1f %10.1f  kgemm%c%c\n", (mf/mfp)*100.0, mf,
-                       TR[ia], TR[ib]);
-         }
-      }
-
-      sprintf(ln, "%s MMinstall pre=%c %s %s\n", fmake, pre, redir, ln2);
-      fprintf(stdout, ln);
-      ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
-
-      fprintf(fpsum, "\n");
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 1, NULL);
-/*
- *    If necessary, find cacheedge
- */
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 2, "CacheEdge DETECTION");
-      sprintf(ln2, "INSTALL_LOG/%cMMCACHEEDGE.LOG", pre);
-      if (!FileExists("INSTALL_LOG/atlas_cacheedge.h"))
-      {
-         PrintBanner(ln2, 1, 2, ip+1, 2);
-         sprintf(ln, "%s INSTALL_LOG/atlas_cacheedge.h pre=%c %s %s\n",
-                 fmake, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "CACHEEDGE DETECTION", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 2);
-      }
-      fp = fopen("INSTALL_LOG/atlas_cacheedge.h", "r");
-      ATL_Cassert(fp, "CACHE EDGE DETECTION", NULL);
-      ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-      ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-      ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-      if (fgets(ln3, 256, fp))
-      {
-         ATL_Cassert(sscanf(ln+21, " %d", &i)==1, "CACHE EDGE DETECTION", NULL);
-      }
-      else i = 0;
-      fprintf(fpsum, "      CacheEdge set to %d bytes\n", i);
-      fclose(fp);
-/*
- *    Determine [ZD,CS]NKB, if necessary
- */
-      if (pre == 'z' || pre == 'c')
-      {
-         sprintf(ln3, "INSTALL_LOG/atlas_%c%cNKB.h", pre, upre);
-         if (!FileExists(ln3))
-         {
-            sprintf(ln, "%s %s pre=%c %s %s\n",
-                    fmake, ln3, pre, redir, ln2);
-            fprintf(stdout, ln);
-            ATL_Cassert(system(ln)==0, "CACHEEDGE DETECTION", ln2);
-         }
-         fp = fopen(ln3, "r");
-         ATL_Cassert(fp, "CACHE EDGE DETECTION", NULL);
-         ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-         ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-         ATL_Cassert(fgets(ln, 256, fp), "CACHE EDGE DETECTION", NULL);
-         if (fgets(ln3, 256, fp))
-         {
-            ATL_Cassert(sscanf(ln+21, " %d", &i)==1,
-                               "CACHE EDGE DETECTION", NULL);
-         }
-         else i = 0;
-         fprintf(fpsum, "      %c%cNKB set to %d bytes\n", pre, upre, i);
-         fclose(fp);
-      }
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 2, NULL);
-/*
- *    If necessary, determine Xover for this data type
- */
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 3, "SMALL/LARGE CROSSOVER");
-      sprintf(fnam, "INSTALL_LOG/%cXover.h", pre);
-      if (!FileExists(fnam))  /* need to run Xover tests */
-      {
-         sprintf(ln2, "INSTALL_LOG/%cMMCROSSOVER.LOG", pre);
-         PrintBanner(ln2, 1, 2, ip+1, 3);
-            fprintf(stdout,
-              "\n\n   STAGE 2-%d-3: COPY/NO-COPY CROSSOVER DETECTION\n", ip+1);
-         fprintf(fpsum,
-              "\n\n   STAGE 2-%d-3: COPY/NO-COPY CROSSOVER DETECTION\n", ip+1);
-
-         sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "COPY/NO-COPY CROSSOVER DETECTION", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 3);
-         fprintf(stdout, "      done.\n");
-         fprintf(fpsum , "      done.\n");
-      }
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 3, NULL);
-
-      sprintf(ln2, "INSTALL_LOG/%cL3TUNE.LOG", pre);
-      PrintBanner(ln2, 1, 2, ip+1, 5);
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 4, "L3BLAS TUNE");
-      if (pre == 's' || pre == 'd')
-      {
-         sprintf(ln, "%s INSTALL_LOG/atlas_%ctrsmXover.h pre=%c %s %s\n",
-                 fmake, pre, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
-      }
-      else
-      {
-         sprintf(ln, "%s Il3lib pre=%c %s %s\n", fmake, pre, redir, ln2);
-         fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
-      }
-      sprintf(ln, "%s %ccblaslib %s %s\n", fmake, pre, redir, ln2); /* cblas */
-      fprintf(stdout, ln);
-      ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
-      PrintBanner(ln2, 0, 2, ip+1, 5);
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 4, "L3BLAS TUNE");
-
-
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 5, "GEMV TUNE");
+      sprintf(ln, "TUNING PREC=\'%c\' (precision %d of %d)",
+               pre, ip+1, nprec);
+      PrintStartStop(stdout, fpsum, 0, 1, 2, ip+1, 0, ln);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 1, "GEMVN TUNE");
       sprintf(fnam, "INSTALL_LOG/%cMVNK.sum", pre);
       if (!FileExists(fnam))
       {
          sprintf(ln2, "INSTALL_LOG/%cMVNTUNE.LOG", pre);
-         PrintBanner(ln2, 1, 2, ip+1, 7);
+         PrintBanner(ln2, 1, 2, ip+1, 1);
          sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
+         fprintf(stdout, "%s", ln);
          ATL_Cassert(system(ln)==0, "MVNTUNE", ln2);
          ATL_Cassert(FileIsThere(fnam), "MVNTUNE", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 7);
+         PrintBanner(ln2, 0, 2, ip+1, 1);
       }
       {
          ATL_mvnode_t *kb, *kp;
@@ -1016,20 +707,22 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
          mf = Mmax(mf, kp->mflop[2]);
          KillAllMVNodes(kb);
          ATL_mprintf(2, fpsum, stdout,
-"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
+                     "              Performance = %.2f (%5.2f of %s)\n",
+                     mf, (mf / mfp)*100.0, peakstr2);
          fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvN");
       }
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 1, "GEMVN TUNE");
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 2, "GEMVT TUNE");
       sprintf(fnam, "INSTALL_LOG/%cMVTK.sum", pre);
       if (!FileExists(fnam))
       {
          sprintf(ln2, "INSTALL_LOG/%cMVTTUNE.LOG", pre);
-         PrintBanner(ln2, 1, 2, ip+1, 7);
+         PrintBanner(ln2, 1, 2, ip+1, 2);
          sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
+         fprintf(stdout, "%s", ln);
          ATL_Cassert(system(ln)==0, "MVTTUNE", ln2);
          ATL_Cassert(FileIsThere(fnam), "MVTTUNE", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 7);
+         PrintBanner(ln2, 0, 2, ip+1, 2);
       }
       {
          ATL_mvnode_t *kb, *kp;
@@ -1049,33 +742,33 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
          mf = Mmax(mf, kp->mflop[2]);
          KillAllMVNodes(kb);
          ATL_mprintf(2, fpsum, stdout,
-"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
+                     "              Performance = %.2f (%5.2f of %s)\n",
+                     mf, (mf / mfp)*100.0, peakstr2);
          fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvT");
       }
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 5, "GEMV TUNE");
-      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 6, "GER TUNE");
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 2, "GEMVT TUNE");
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 3, "GER  TUNE");
       sprintf(fnam, "INSTALL_LOG/%cR2K.sum", pre);
       if (!FileExists(fnam))
       {
          sprintf(ln2, "INSTALL_LOG/%cR1TUNE.LOG", pre);
          PrintBanner(ln2, 1, 2, ip+1, 7);
          sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
-         fprintf(stdout, ln);
+         fprintf(stdout, "%s", ln);
          ATL_Cassert(system(ln)==0, "R1TUNE", ln2);
          ATL_Cassert(FileIsThere(fnam), "R1TUNE", ln2);
-         PrintBanner(ln2, 0, 2, ip+1, 7);
-         fnam[14] = '1';
+         PrintBanner(ln2, 0, 2, ip+1, 3);
       }
       {
          ATL_r1node_t *r1b, *r1B;
+         fnam[14] = '1';
          r1b = ReadR1File(fnam);
          ATL_Cassert(r1b, "R1TUNE", NULL);
          ATL_R1SplitContexts(r1b, &r1B, NULL, NULL, NULL);
          r1b = r1B;
          r1B = ATL_LastR1Node(r1B);
          ATL_mprintf(2, fpsum, stdout,
-              "      ger : main kernel %d:%s written by %s\n", r1B->ID,
+              "      ger1 : main kernel %d:%s written by %s\n", r1B->ID,
                      r1B->rout?r1B->rout : "Generated",
                      r1B->auth?r1B->auth : "R. Clint Whaley");
          ATL_mprintf(2, fpsum, stdout,
@@ -1085,13 +778,86 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
          mf = Mmax(mf, r1B->mflop[2]);
          KillAllR1Nodes(r1b);
          ATL_mprintf(2, fpsum, stdout,
-"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kger");
-         fclose(fpabr);
+                     "              Performance = %.2f (%5.2f of %s)\n",
+                     mf, (mf / mfp)*100.0, peakstr2);
+         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kger1");
+         fnam[14] = '2';
+         r1b = ReadR1File(fnam);
+         ATL_Cassert(r1b, "R1TUNE", NULL);
+         ATL_R1SplitContexts(r1b, &r1B, NULL, NULL, NULL);
+         r1b = r1B;
+         r1B = ATL_LastR1Node(r1B);
+         ATL_mprintf(2, fpsum, stdout,
+              "      ger2 : main kernel %d:%s written by %s\n", r1B->ID,
+                     r1B->rout?r1B->rout : "Generated",
+                     r1B->auth?r1B->auth : "R. Clint Whaley");
+         ATL_mprintf(2, fpsum, stdout,
+                     "            mu=%d, nu=%d, using %d Cache Elements\n",
+                     r1B->MU, r1B->NU, r1B->CacheElts);
+         mf = Mmax(r1B->mflop[0], r1B->mflop[1]);
+         mf = Mmax(mf, r1B->mflop[2]);
+         KillAllR1Nodes(r1b);
+         ATL_mprintf(2, fpsum, stdout,
+                     "              Performance = %.2f (%5.2f of %s)\n",
+                     mf, (mf / mfp)*100.0, peakstr2);
+         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kger2");
       }
-      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 6, "GER TUNE");
-      PrintStartStop(stdout, fpsum, 0, 0, 2, 0, 0, "TYPE-DEPENDENT TUNING");
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 3, "GER  TUNE");
+/*
+ *    Detection of CacheEdge currently missing
+ */
+/*
+ *    GEMM install
+ */
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 4, "GEMM TUNE");
+      sprintf(ln2, "INSTALL_LOG/%cAMMTUNE.LOG", pre);
+      if (pre == 'd' || pre == 's' || pre == 'q')
+         sprintf(ln, "%s AMMinstall pre=%c %s %s\n", fmake, pre, redir, ln2);
+      else /* complex */
+         sprintf(ln, "%s cAMMinstall pre=%c %s %s\n", fmake, pre, redir, ln2);
+      fprintf(stdout, "%s", ln);
+      ATL_Cassert(system(ln)==0, "GEMM TUNE", ln2);
+      mmb = ReadMMFileWithPath(pre, "INSTALL_LOG", "ipgen.sum");
+      ATL_Cassert(mmb, "ACCESS-MAJOR MATMUL TUNE", ln2);
+      for (mmp=mmb; mmp->next; mmp = mmp->next);
+      ATL_mprintf(2, fpsum, stdout,
+         "\n      The best matmul kernel was %s, MB=%d, NB=%d, KB=%d\n",
+                     mmp->rout, mmp->mbB, mmp->nbB, mmp->kbB);
+      ATL_mprintf(2, fpsum, stdout, "      written %s\n",
+                  mmp->auth ? mmp->auth : "G: R. Clint Whaley");
+      mf = mmp->mflop[0];
+      ATL_mprintf(2, fpsum, stdout,
+                  "      Performance: %.2fMFLOPS (%.2f of %s)\n",
+                  mf, (mf / mfp)*100.0, peakstr2);
+      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemm");
+      KillAllMMNodes(mmb);
+      fclose(fpabr);
+
+      fprintf(fpsum, "\n");
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 1, NULL);
+
+      sprintf(ln2, "INSTALL_LOG/%cL3TUNE.LOG", pre);
+      PrintBanner(ln2, 1, 2, ip+1, 5);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 4, "L3BLAS TUNE");
+      if (pre == 's' || pre == 'd')
+      {
+         sprintf(ln, "%s INSTALL_LOG/atlas_%ctrsmXover.h pre=%c %s %s\n",
+                 fmake, pre, pre, redir, ln2);
+         fprintf(stdout, "%s", ln);
+         ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
+      }
+      else
+      {
+         sprintf(ln, "%s Il3lib pre=%c %s %s\n", fmake, pre, redir, ln2);
+         fprintf(stdout, "%s", ln);
+         ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
+      }
+      sprintf(ln, "%s %ccblaslib %s %s\n", fmake, pre, redir, ln2); /* cblas */
+      fprintf(stdout, "%s", ln);
+      ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
+      PrintBanner(ln2, 0, 2, ip+1, 5);
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 4, "L3BLAS TUNE");
+
    }
    PrintStartStop(stdout, fpsum, 0, 0, 2, 0, 0, "TYPE-DEPENDENT TUNING");
    PrintStartStop(stdout, fpsum, 0, 1, 3, 0, 0, "GENERAL LIBRARY BUILD");
@@ -1099,18 +865,34 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    sprintf(ln2, "INSTALL_LOG/LIBBUILD.LOG");
    PrintBanner(ln2, 1, 3, 1, 1);
    sprintf(ln, "%s IBuildLibs %s %s\n", fmake, redir, ln2);
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "LIBRARY BUILD", ln2);
    ATL_Cassert(FileIsThere(fnam), "LIBRARY BUILD", ln2);
    PrintBanner(ln2, 0, 3, 1, 1);
    PrintStartStop(stdout, fpsum, 0, 0, 3, 0, 0, "GENERAL LIBRARY BUILD");
 
+   for (ip=2; ip < 4; ip++)
+   {
+      char pre=prec[ip];
+      PrintStartStop(stdout, fpsum, 3, 1, 3, ip-1, 2,
+                     "COMPLEX AMM NB REFINEMENT");
+      sprintf(ln2, "INSTALL_LOG/atlas_%camm_sum.h", pre);
+      if (!FileExists(ln2))
+      {
+         sprintf(ln2, "INSTALL_LOG/%cAMMNBTUNE.LOG", pre);
+         PrintBanner(ln2, 1, 3, ip-1, 2);
+         sprintf(ln, "%s cAMMinstall pre=%c %s %s\n", fmake, pre, redir, ln2);
+         fprintf(stdout, "%s", ln);
+         ATL_Cassert(system(ln)==0, "COMPLEX AMM NB REFINEMENT", ln2);
+         PrintBanner(ln2, 0, 3, ip-1, 2);
+      }
+   }
    PrintStartStop(stdout, fpsum, 0, 1, 4, 0, 0, "POST-BUILD TUNING");
    sprintf(ln2, "INSTALL_LOG/POSTTUNE.LOG");
    PrintBanner(ln2, 1, 4, 1, 1);
    PrintStartStop(stdout, fpsum, 3, 1, 4, 1, 1, "TRSM TUNE");
    sprintf(ln, "%s IPostTune %s %s\n", fmake, redir, ln2);
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "POST-BUILD TUNE", ln2);
    PrintStartStop(stdout, fpsum, 3, 0, 4, 1, 0, NULL);
    ATL_Cassert(FileIsThere(fnam), "POST-BUILD TUNE", ln2);
@@ -1121,7 +903,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    sprintf(ln2, "INSTALL_LOG/PTTUNE.LOG");
    PrintBanner(ln2, 1, 4, 2, 0);
    sprintf(ln, "%s IPTtune %s %s\n", fmake, redir, ln2);
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "THREADING TUNE", ln2);
    PrintStartStop(stdout, fpsum, 3, 0, 4, 2, 0, "THREADING TUNE");
 
@@ -1129,7 +911,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    sprintf(ln2, "INSTALL_LOG/LIBPTBUILD.LOG");
    PrintBanner(ln2, 1, 4, 2, 1);
    sprintf(ln, "%s IBuildPtlibs %s %s\n", fmake, redir, ln2);
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "PTLIBRARY BUILD", ln2);
    PrintBanner(ln2, 0, 4, 2, 1);
    PrintStartStop(stdout, fpsum, 3, 0, 4, 2, 1, "THREADING BUILD");
@@ -1144,7 +926,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
          PrintStartStop(stdout, fpsum, 6, 1, 4, 3, ip+1, ln2);
          sprintf(ln2, "INSTALL_LOG/%cLATUNE.LOG", pre);
          PrintBanner(ln2, 1, 4, 3, ip+1);
-         fprintf(stdout, ln);
+         fprintf(stdout, "%s", ln);
          sprintf(ln, "%s ILATune pre=%c %s %s\n", fmake, pre, redir, ln2);
          ATL_Cassert(system(ln)==0, "LAPACK TUNE", ln2);
          PrintStartStop(stdout, fpsum, 6, 0, 4, 3, ip+1, NULL);
@@ -1162,7 +944,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    #else
       sprintf(ln, "%s IBuildLibs %s %s\n", fmake, redir, ln2);
    #endif
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "STATIC LIBRARY UPDATE", ln2);
    PrintBanner(ln2, 0, 5, 1, 1);
    PrintStartStop(stdout, fpsum, 3, 0, 5, 1, 0, "FINAL STATIC LIBRARY UPDATE");
@@ -1172,7 +954,7 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
    sprintf(ln2, "INSTALL_LOG/LIBDYBUILD.LOG");
    PrintBanner(ln2, 1, 5, 2, 1);
    sprintf(ln, "%s IBuildDyLibs %s %s\n", fmake, redir, ln2);
-   fprintf(stdout, ln);
+   fprintf(stdout, "%s", ln);
    ATL_Cassert(system(ln)==0, "DYLIBRARY BUILD", ln2);
    PrintBanner(ln2, 0, 5, 2, 1);
    PrintStartStop(stdout, fpsum, 3, 0, 5, 2, 0, NULL);
@@ -1187,27 +969,42 @@ void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
 
 void PrintUsage(char *nam)
 {
-   fprintf(stderr, "\n\nUSAGE: %s [-a <#(use archdef)> -1 <#(bozoL1)>]\n\n",
-           nam);
+   fprintf(stderr, "\n\nUSAGE: %s [flags]:\n", nam);
+   fprintf(stderr, "   -a 1/0: do/don't use architectural defaults\n");
+   fprintf(stderr, "   -1 0/1: don't/do use bozo L1BLAS guesses if "
+                   "no archdef\n");
+   fprintf(stderr, "   -A 0/1: don't/do eliminate non-independent"
+           " affinity thread IDs\n");
    exit(-1);
 }
 
-void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF, int *TuneLA)
+void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF, int *TuneLA,
+              int *NREG, int *INDTHR)
 {
    int i;
 
-   *TuneLA = *L1DEF = 0;
+   *INDTHR = *NREG = *TuneLA = *L1DEF = 0;
    *ARCHDEF = 1;
    for (i=1; i < nargs; i++)
    {
       if (args[i][0] != '-') PrintUsage(args[0]);
       switch(args[i][1])
       {
+      case 'A':
+         if (++i > nargs)
+            PrintUsage(args[0]);
+         *INDTHR = atoi(args[i]);
+         break;
       case 'a':
          if (++i > nargs)
             PrintUsage(args[0]);
          *ARCHDEF = atoi(args[i]);
          break;
+      case 'r':
+         if (++i > nargs)
+            PrintUsage(args[0]);
+         *NREG = atoi(args[i]);
+         break;
       case 'l':
          if (++i > nargs)
             PrintUsage(args[0]);
@@ -1226,8 +1023,8 @@ void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF, int *TuneLA)
 
 int main(int nargs, char *args[])
 {
-   int L1DEF, ARCHDEF, TuneLA;
-   GetFlags(nargs, args, &ARCHDEF, &L1DEF, &TuneLA);
-   GoToTown(ARCHDEF, L1DEF, TuneLA);
+   int L1DEF, ARCHDEF, TuneLA, NREG, INDTHR;
+   GetFlags(nargs, args, &ARCHDEF, &L1DEF, &TuneLA, &NREG, &INDTHR);
+   GoToTown(ARCHDEF, L1DEF, TuneLA, NREG, INDTHR);
    return(0);
 }
