@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #if defined(ATL_AFFINITY) || defined(ATL_NOAFFINITY)
    #ifndef ATL_USEPTHREADS
@@ -33,7 +8,7 @@
    #endif
 #endif
 
-/* #define TEST_GPMM */
+/* #define TEST_RKKM 1 */
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -42,9 +17,13 @@
 #include "atlas_lvl3.h"
 #include "atlas_tst.h"
 #include "cblas.h"
+#include "atlas_bitvec.h"
 #ifdef ATL_USEPTHREADS
    #include "atlas_tlvl3.h"
 #endif
+#ifdef TRUST_REF
+   #include "atlas_reflevel3.h"
+#endif
 #if defined(ATL_RKXOVER) || defined(TEST_RANKK)
    #define TRUST_BIGNORK
    void Mjoin(PATL,bignork_mm)
@@ -87,7 +66,7 @@ int Mjoin(PATL,gemm_rK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
 #ifdef ATL_DeclareSlens
 F77_INTEGER ATL_Slen1, ATL_Slen2;
 #endif
-double time00();
+double time00(void);
 
 void printmat(char *mat, int M, int N, TYPE *A, int lda)
 {
@@ -154,10 +133,15 @@ void matnan(int M, int N, TYPE *C, int ldc, int seed)
 }
 #endif
 
+static int IK0=0, IKN=0, incIK=1, IIK=0;
+
 #if defined(TRUST_C)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(Mjoin(cblas_,PRE),gemm) \
          (CblasColMajor, TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TRUST_REF)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,refgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #elif defined(ATL_NOAFFINITY) || defined(ATL_AFFINITY)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,tgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -176,7 +160,7 @@ void matnan(int M, int N, TYPE *C, int ldc, int seed)
 #elif defined(TRUST_JIT)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,mmJITcp)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
-#elif defined(ATL_USEPTHREADS) && !defined(TEST_F77)
+#elif (defined(ATL_USEPTHREADS) && !defined(TEST_F77))
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #else
@@ -188,6 +172,21 @@ void matnan(int M, int N, TYPE *C, int ldc, int seed)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,gpmm)(PackGen, TA, PackGen, TB, PackGen, m, n, k, al, \
                        A, 0, 0, lda, B, 0, 0, ldb, be, C, 0, 0, ldc)
+#elif defined(TEST_AMMREC)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,ammmREC)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_RKKM)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,ammm_rkKm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_AMMNKM)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,ammmNKM)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_AMM2)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      TEST_MM(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_AMMM)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,ammmNMK)(IIK,TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #elif defined(ATL_NOAFFINITY) || defined(ATL_AFFINITY)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,tgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -238,8 +237,8 @@ void matnan(int M, int N, TYPE *C, int ldc, int seed)
 #endif
 
 int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
-	   SCALAR alpha, TYPE *A, int lda, TYPE *B, int ldb, SCALAR beta,
-	   TYPE *C, int ldc, TYPE *D, int ldd)
+	   SCALAR alpha, TYPE *A, size_t lda, TYPE *B, size_t ldb, SCALAR beta,
+	   TYPE *C, size_t ldc, TYPE *D, size_t ldd)
 {
    char *pc;
 #ifdef TREAL
@@ -251,7 +250,8 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
    #define MBETA *beta, beta[1]
    char *form="%4d   %c   %c %4d %4d %4d  %5.1f %5.1f  %5.1f %5.1f  %6.2f %6.1f %4.2f   %3s\n";
 #endif
-   int ii, jj, i, j=0, PASSED, nerrs, Na, Nb;
+   ATL_BV_t *errBV=NULL;
+   int ii, jj, i, j=0, PASSED, nerrs, Na, Nb, ibad=(-1), jbad=(-1);
    double t0, t1, t2, t3, mflop;
    TYPE maxval, f1, ferr;
    static TYPE feps=0.0;
@@ -310,14 +310,17 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
      for (i=0; i != nL2; i++) j += L2[i];
      }*/
 
-   /* invalidate L2 cache */
-   #if !ATL_LINEFLUSH
-      l2ret = ATL_flushcache( -1 );
-   #else
-      ATL_flushCacheByAddr(ldc*N, C);
-      ATL_flushCacheByAddr(ldb*Nb, B);
-      ATL_flushCacheByAddr(lda*Na, A);
-   #endif
+   if (CACHESIZE > 0)
+   {
+      /* invalidate L2 cache */
+      #if !ATL_LINEFLUSH
+         l2ret = ATL_flushcache( -1 );
+      #else
+         ATL_flushCacheByAddr(ldc*N, C);
+         ATL_flushCacheByAddr(ldb*Nb, B);
+         ATL_flushCacheByAddr(lda*Na, A);
+      #endif
+   }
 
    t0 = time00();
    trusted_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
@@ -342,14 +345,17 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
       matgen(M, N, D, ldd, M*N);
    #endif
 
-   /* invalidate L2 cache */
-   #if !ATL_LINEFLUSH
-      l2ret = ATL_flushcache( -1 );
-   #else
-      ATL_flushCacheByAddr(ldc*N, D);
-      ATL_flushCacheByAddr(ldb*Nb, B);
-      ATL_flushCacheByAddr(lda*Na, A);
-   #endif
+   if (CACHESIZE > 0)
+   {
+      /* invalidate L2 cache */
+      #if !ATL_LINEFLUSH
+         l2ret = ATL_flushcache( -1 );
+      #else
+         ATL_flushCacheByAddr(ldc*N, D);
+         ATL_flushCacheByAddr(ldb*Nb, B);
+         ATL_flushCacheByAddr(lda*Na, A);
+      #endif
+   }
    t0 = time00();
    test_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, D, ldd);
 
@@ -400,15 +406,18 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
       {
          for (i=0; i != M SHIFT; i++)
          {
+            int eltbad=0;
             #ifdef ATL_NANC
                if (D[i] != D[i])
                {
+                  eltbad=1;
                   nerrs++;
                   PASSED = 0;
                   printf("NaN in test_gemm at C(%d,%d)\n", i, j);
                }
                if (C[i] != C[i])
                {
+                  eltbad=1;
                   nerrs++;
                   PASSED = 0;
                   printf("NaN in trusted_gemm at C(%d,%d)\n", i, j);
@@ -418,6 +427,11 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
             if (f1 < 0.0) f1 = -f1;
             if (f1 > ferr)
             {
+               eltbad=1;
+               if (ibad == -1)
+                  ibad = i+1;
+               if (jbad == -1)
+                  jbad = j+1;
                nerrs++;
                PASSED = 0;
                pc = "NO!";
@@ -430,17 +444,25 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
             }
             else if (D[i] != D[i])
             {
+               eltbad=1;
                nerrs++;
                PASSED = 0;
                pc = "NaN";
                maxval = D[i];
             }
+            if (eltbad && N < 80 && (M SHIFT) < 80)
+            {
+               if (!errBV)
+                  errBV = ATL_NewBV((M SHIFT)*N);
+               ATL_SetBitBV(errBV, i*N+j);
+            }
          }
          D += ldd SHIFT;
          C += ldc SHIFT;
       }
       if (maxval != 0.0)
-         fprintf(stderr, "ERROR: nerr=%d, i=%d, j=%d, maxval=%e\n", nerrs, ii,jj, maxval);
+         fprintf(stderr, "ERROR: nerr=%d, first(%d,%d), maxval(%d,%d)=%e\n",
+                 nerrs, ibad, jbad, ii,jj, maxval);
    }
    else pc = "---";
    if (t1 == t2) t3 = 1.0;
@@ -455,7 +477,12 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
             speedup_rk[K] = (t3 >= 1.0) ? t3*100 : 0;
       }
    #endif
+   #ifdef TEST_AMMM
+   printf(form, IIK, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   itst++;
+   #else
    printf(form, itst++, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   #endif
 #else
    itst++;
    PASSED = 1;
@@ -464,11 +491,16 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
    #if !ATL_LINEFLUSH
       l2ret = ATL_flushcache( 0 );
    #endif
+   if (errBV)
+   {
+      ATL_print2dBV(M SHIFT, N, errBV);
+      ATL_FreeBV(errBV);
+   }
    return(PASSED);
 }
 
 int mmcase0(int MFLOP, size_t CACHESIZE, char TA, char TB, int M, int N, int K,
-	    SCALAR alpha, int lda, int ldb, SCALAR beta, int ldc)
+	    SCALAR alpha, size_t lda, size_t ldb, SCALAR beta, size_t ldc)
 {
    char *pc;
 #ifdef TREAL
@@ -556,6 +588,9 @@ int mmcase0(int MFLOP, size_t CACHESIZE, char TA, char TB, int M, int N, int K,
       nset = 2;
    vp = malloc(ATL_MulBySize(nset*inc)+ATL_Cachelen);
    ATL_assert(vp);
+   #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+      ATL_NumaTouchSpread(ATL_MulBySize(nset*inc)+ATL_Cachelen, vp);
+   #endif
    C = c = ATL_AlignPtr(vp);
    a = A = C + incC;
    b = B = A + incA;
@@ -627,7 +662,11 @@ int mmcase0(int MFLOP, size_t CACHESIZE, char TA, char TB, int M, int N, int K,
             speedup_rk[K] = (t3 >= 1.0) ? t3*100 : 0;
       }
    #endif
+   #ifdef TEST_AMMM
+   printf(form, IIK, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   #else
    printf(form, itst-4, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   #endif
    itst++;
    free(vp);
    return(1);
@@ -635,7 +674,7 @@ int mmcase0(int MFLOP, size_t CACHESIZE, char TA, char TB, int M, int N, int K,
 
 void PrintUsage(char *nam)
 {
-   fprintf(stderr, "USAGE: %s -Side <nsides> L/R -Uplo <nuplo> L/U -Atrans <ntrans> n/t/c -Btrans <ntrans> n/t/c -Diag <ndiags> N/U -M <m1> <mN> <minc> -N <n1> <nN> <ninc> <k1> <kN> <kinc> -n <n> -m <m> -k <k> -a <nalphas> <alpha1> ... <alphaN> -b <nbetas> <beta1> ... <betaN> -Test <0/1> -F <mflops> -C <cachesize>\n", nam);
+   fprintf(stderr, "USAGE: %s -Side <nsides> L/R -Uplo <nuplo> L/U -Atrans <ntrans> n/t/c -Btrans <ntrans> n/t/c -Diag <ndiags> N/U -M <m1> <mN> <minc> -N <n1> <nN> <ninc> <k1> <kN> <kinc> -n <n> -m <m> -k <k> -a <nalphas> <alpha1> ... <alphaN> -b <nbetas> <beta1> ... <betaN> -Test <0/1> -F <mflops> -i <IKERN> -I <IK1> <IKN> <incIK> -C <cachesize> -W <warmup> \n", nam);
    exit(-1);
 }
 
@@ -646,7 +685,8 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
               enum ATLAS_DIAG **Diag, int *M0, int *MN, int *Minc,
               int *N0, int *NN, int *Ninc, int *K0, int *KN, int *Kinc,
               int *nalphas, TYPE **alphas, int *nbetas, TYPE **betas,
-              int *LDA_IS_M, int *MFLOP, int *CACHESIZE, int *nrep)
+              int *LDA_IS_M, int *MFLOP, int *CACHESIZE, int *nrep,
+              int *WARM)
 
 {
    char ch;
@@ -660,6 +700,7 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
    *M0 = *N0 = *K0 = -1;
    *nuplo = *nta = *ntb = *nside = *ndiag = *nalphas = *nbetas = -1;
    *MFLOP = *LDA_IS_M = 0;
+   *WARM = 1;
 #ifdef L2SIZE
    *CACHESIZE = L2SIZE;               /* Size of largest cache to flush */
 #else
@@ -670,12 +711,25 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
    {
       switch(args[i][1])
       {
+      case 'W':
+         *WARM = atoi(args[++i]);
+         break;
       case '#':
          *nrep = atoi(args[++i]);
          break;
       case 'F':
          *MFLOP = atoi(args[++i]);
          break;
+      #ifdef TEST_AMMM
+      case 'i':
+         IK0 = IKN = atoi(args[++i]);
+         break;
+      case 'I':
+         IK0 = atoi(args[++i]);
+         IKN = atoi(args[++i]);
+         incIK = atoi(args[++i]);
+         break;
+      #endif
       case 'C':
             if( args[i+1] == NULL ) PrintUsage( args[0] );
 	    *CACHESIZE = 1024*atoi(args[++i]);
@@ -925,12 +979,13 @@ int main(int nargs, char *args[])
    enum ATLAS_UPLO *Uplo;
    enum ATLAS_TRANS *TransA, *TransB, TAc, TBc;
    enum ATLAS_DIAG *Diag;
-   int CACHESIZE;
+   int CACHESIZE, WARM;
 
    GetFlags(nargs, args, &TEST, &nside, &Side, &nuplo, &Uplo,
             &nTA, &TransA, &nTB, &TransB, &ndiag, &Diag,
             &M0, &MN, &incM, &N0, &NN, &incN, &K0, &KN, &incK,
-            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP,&CACHESIZE, &nrep);
+            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP,&CACHESIZE, &nrep,
+            &WARM);
 
    if (M0 == -1)
    {
@@ -955,6 +1010,12 @@ int main(int nargs, char *args[])
          fprintf(stderr, "Not enough memory to run tests!!\n");
          exit(-1);
       }
+      #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+         ATL_NumaTouchSpread(MN*KN*ATL_sizeof, A);
+         ATL_NumaTouchSpread(NN*KN*ATL_sizeof, B);
+         ATL_NumaTouchSpread(MN*NN*ATL_sizeof, C);
+         ATL_NumaTouchSpread(MN*NN*ATL_sizeof, D);
+      #endif
    }
 /*
  * Page the code in from disk, so first timing doesn't blow
@@ -966,10 +1027,13 @@ int main(int nargs, char *args[])
       #else
          k = 100;
       #endif
-      mmcase0(10, 1, 'n', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 'n', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 't', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 't', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
+      if (WARM)
+      {
+         mmcase0(10, 1, 'n', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
+         mmcase0(10, 1, 'n', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
+         mmcase0(10, 1, 't', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
+         mmcase0(10, 1, 't', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
+      }
    }
    else
    {
@@ -981,8 +1045,11 @@ int main(int nargs, char *args[])
       matgen(m, n, C, m, m*n);
       TA = TB = 'N';
       TAc = TBc = AtlasNoTrans;
-      trusted_gemm(TAc, TBc, m, n, k, alp1, A, m, B, k, bet1, C, m);
-      test_gemm(TAc, TBc, m, n, k, alp1, A, m, B, k, bet1, C, m);
+      if (WARM)
+      {
+         trusted_gemm(TAc, TBc, m, n, k, alp1, A, m, B, k, bet1, C, m);
+         test_gemm(TAc, TBc, m, n, k, alp1, A, m, B, k, bet1, C, m);
+      }
    }
 
 #ifdef TREAL
@@ -1017,7 +1084,6 @@ int main(int nargs, char *args[])
                   {
                      for (ib=0; ib != nbeta; ib++)
                      {
-                        itst++;
                         if (LDA_IS_M)
                         {
                            if (TA == 'n' || TA == 'N') lda = m;
@@ -1034,6 +1100,11 @@ int main(int nargs, char *args[])
                            else ldb = NN;
                            ldc = MN;
                         }
+                        #ifdef TEST_AMMM
+                        for (IIK=IK0; IIK <= IKN; IIK += incIK)
+                        {
+                        #endif
+                        itst++;
                         for (r=0; r < nrep; r++)
                         {
                            if (MFLOP)
@@ -1082,6 +1153,9 @@ int main(int nargs, char *args[])
                               }
                               #endif
                         }
+                        #ifdef TEST_AMMM
+                        }
+                        #endif
                      }
                   }
                }
