@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2001 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2001 R. Clint Whaley
  */
 
 #include "atlas_misc.h"
@@ -52,26 +27,33 @@ static void geinv
    (const enum CBLAS_ORDER Order, const int N, TYPE *A, const int lda)
 {
    int *ipiv;
-   TYPE *wrk;
+   TYPE *wrk, WQ;
    int lwrk;
 
    ipiv = malloc(sizeof(int)*N);
    ATL_assert(ipiv);
    #ifdef TimeF77
-      lwrk = N * Mjoin(PATL,GetNB)();
-      wrk = malloc(ATL_MulBySize(lwrk));
       if (Order == AtlasRowMajor) Mjoin(PATL,tstsqtran)(N, A, lda);
       ATL_assert(Mjoin(PATL,f77getrf)(AtlasColMajor, N, N, A, lda, ipiv) == 0);
-      ATL_assert(Mjoin(PATL,f77getri)
-         (AtlasColMajor, N, A, lda, ipiv, wrk, &lwrk) == 0);
-      if (Order == AtlasRowMajor) Mjoin(PATL,tstsqtran)(N, A, lda);
+      lwrk = -1;
+      ATL_assert(Mjoin(PATL,f77getri)(AtlasColMajor, N, A, lda, ipiv,
+                                      &WQ, lwrk) == 0);
+      lwrk = WQ;
+      wrk = malloc(ATL_MulBySize(lwrk));
+      ATL_assert(wrk);
+      ATL_assert(Mjoin(PATL,f77getri)(AtlasColMajor, N, A, lda, ipiv,
+                                      wrk, lwrk) == 0);
       free(wrk);
+      if (Order == AtlasRowMajor) Mjoin(PATL,tstsqtran)(N, A, lda);
    #elif defined(TimeC)
       ATL_assert(Mjoin(CLP,getrf)(Order, N, N, A, lda, ipiv) == 0);
       ATL_assert(Mjoin(CLP,getri)(Order, N, A, lda, ipiv) == 0);
   #else
-      lwrk = N * Mjoin(PATL,GetNB)();
+      lwrk = -1;
+      ATL_assert(Mjoin(PATL,getri)(Order, N, A, lda, ipiv, &WQ, &lwrk) == 0);
+      lwrk = WQ;
       wrk = malloc(ATL_MulBySize(lwrk));
+      ATL_assert(wrk);
       ATL_assert(Mjoin(PATL,getrf)(Order, N, N, A, lda, ipiv) == 0);
       ATL_assert(Mjoin(PATL,getri)(Order, N, A, lda, ipiv, wrk, &lwrk) == 0);
       free(wrk);
@@ -119,7 +101,13 @@ TYPE *GetGE(int M, int N, int lda)
 {
    TYPE *A;
    A = malloc(ATL_MulBySize(lda)*N);
-   if (A) Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   if (A)
+   {
+      #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+         ATL_NumaTouchSpread(ATL_MulBySize(lda)*N, A);
+      #endif
+      Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   }
    return(A);
 }
 
@@ -169,6 +157,9 @@ static TYPE *DupMat(enum ATLAS_ORDER Order, int M, int N, TYPE *A, int lda,
    ATL_assert(ldc >= M);
    C = malloc(ATL_MulBySize(ldc)*N);
    ATL_assert(C);
+   #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+      ATL_NumaTouchSpread(ATL_MulBySize(ldc)*N, C);
+   #endif
    for (j=0; j != N; j++)
    {
       for (i=0; i != M2; i++) C[i] = A[i];
@@ -511,11 +502,11 @@ static double RunTest
 
 static double RunTiming
    (enum CBLAS_ORDER Order, enum TEST_UPLO Uplo, int N, int lda,
-    int CacheSize, int nreps)
+    size_t CacheSize, int nreps)
 {
    TYPE *A, *a;
-   const int incA = N*lda;
-   int i, k;
+   const size_t incA = N*lda SHIFT;
+   size_t i, k;
    double t0, t1=0.0;
 
    if (nreps < 1) nreps = 1;
@@ -742,7 +733,7 @@ void GetFlags(int nargs, char **args, int *MFLOP, int *CacheSize, TYPE *thresh,
    }
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    int MFLOP, CacheSize, ldagap, nord, nuplo, N0, NN, incN;
    enum TEST_UPLO *Uplo;
@@ -752,5 +743,5 @@ main(int nargs, char **args)
             &nuplo, &Uplo, &N0, &NN, &incN);
    RunCases(CacheSize, thresh, MFLOP, ldagap, nord, Ord, nuplo,
             (enum ATLAS_UPLO*) Uplo, N0, NN, incN);
-   exit(0);
+   return(0);
 }
