@@ -1,34 +1,12 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2008 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2008 R. Clint Whaley
  */
 #include "atlas_misc.h"
 #include "atlas_lapack.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_ptalias_lapack.h"
+#endif
 #ifdef ATL_ilaenv
    #undef ATL_ilaenv
 #endif
@@ -79,6 +57,10 @@ static int Side_LA2ATL(int i)
    #define Cgetrf Mjoin(Mjoin(clapack_,PRE),getrf)
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(Cgetrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
+#elif defined(TimeBCAMM)
+   #include "atlas_bcamm.h"
+   #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
+      ATL_assert(Mjoin(PATL,tgetrf_bcAmm)(Major_, M_, N_, A_, lda_, ipiv_) == 0)
 #else
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(ATL_getrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
@@ -136,7 +118,13 @@ TYPE *GetGE(int M, int N, int lda)
 {
    TYPE *A;
    A = malloc(ATL_MulBySize(lda)*N);
-   if (A) Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   if (A)
+   {
+      #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+         ATL_NumaTouchSpread(ATL_MulBySize(lda)*N, A);
+      #endif
+      Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   }
    return(A);
 }
 
@@ -186,6 +174,9 @@ static TYPE *DupMat(enum ATLAS_ORDER Order, int M, int N, TYPE *A, int lda,
    ATL_assert(ldc >= M);
    C = malloc(ATL_MulBySize(ldc)*N);
    ATL_assert(C);
+   #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+      ATL_NumaTouchSpread(ATL_MulBySize(ldc)*N, C);
+   #endif
    for (j=0; j != N; j++)
    {
       for (i=0; i != M2; i++) C[i] = A[i];
@@ -1353,10 +1344,16 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
                int Side, int flsizeKB)
 {
 #if ATL_LINEFLUSH
-   FLSTRUCT *flp;
+   FLSTRUCT *flp=NULL;
 #endif
    TYPE *A, *wrk=NULL, dtmp, dtmp1, *tau=NULL;
    int *ipiv=NULL, itmp, wlen;
+/*
+ * If the matrix alone is larger than four times the flush size, then
+ * the matrix init should be self-flushing, so we avoid the time & memory
+ * waste of doing the flush.  This is critical for large problems.
+ */
+   int DOFLUSH = (((size_t)M)*N*ATL_sizeof < ((size_t)flsizeKB)*1024*4);
    double t0, t1;
 /*
  * Call routs that force particular flop count if requested; they return -1.0
@@ -1402,7 +1399,9 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
  * Generate operands
  */
    A = GetGE(M, N, lda);
-   flp = ATL_GetFlushStruct(A, N*lda*ATL_sizeof, NULL);
+   ATL_assert(A);
+   if (DOFLUSH)
+      flp = ATL_GetFlushStruct(A, N*((size_t)lda)*ATL_sizeof, NULL);
    if (rout == LApotrf)
       PosDefGen(CblasColMajor, Uplo_LA2ATL(Uplo), N, A, lda);
    else if (rout & LAgeqrf)
@@ -1429,21 +1428,25 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
       wlen = dtmp;
       wrk = calloc(wlen, ATL_sizeof);
       ATL_assert(wrk);
-      flp = ATL_GetFlushStruct(wrk, wlen*ATL_sizeof, flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(wrk, wlen*ATL_sizeof, flp);
       itmp = (M >= N) ? M : N;
       tau = calloc(itmp, ATL_sizeof);
-      flp = ATL_GetFlushStruct(tau, itmp*ATL_sizeof, flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(tau, itmp*ATL_sizeof, flp);
    }
    else
    {
       ipiv = calloc(M, sizeof(int));
       ATL_assert(ipiv);
-      flp = ATL_GetFlushStruct(ipiv, M*sizeof(int), flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(ipiv, M*sizeof(int), flp);
    }
 /*
  * Flush cache, and do timing
  */
-   ATL_FlushAreasByCL(flp);
+   if (DOFLUSH)
+      ATL_FlushAreasByCL(flp);
    if (rout == LApotrf)
    {
       t0 = time00();
@@ -1493,7 +1496,8 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
    if (ipiv)
       free(ipiv);
    free(A);
-   ATL_KillAllFlushStructs(flp);
+   if (DOFLUSH)
+      ATL_KillAllFlushStructs(flp);
    return(t1 - t0);
 #endif
 }
@@ -1604,6 +1608,7 @@ void WriteCFile(char *outnam, int N, int *flgs, int *ms, int *ns, int *nbs,
    ATL_assert(ln);
    sprintf(ln, "%s.h", outnam);
    nbout = my_fopen(ln, "w");
+   free(ln);
    nameNoPath = NameExcludingPath(outnam);
    fprintf(nbout, "#ifndef %s\n\n", nameNoPath);
    fprintf(nbout, "/*\n * NB selection for %s: Side='%s', Uplo='%s'\n",
@@ -1733,6 +1738,7 @@ void WriteNbFile(int F77out, char *outnam, int N, int *flgs, int *ms, int *ns,
    else
       ip = CompressDecisionTree(N, ms, nbs);
    WriteCFile(outnam, N, flgs, ms, ns, nbs, ip[0], ip+1+ip[0], ip+1);
+   free(ip);
 }
 
 int GetMyReps(int N, int *nreps)
@@ -1864,6 +1870,7 @@ void GoToTown(int *nreps, int flsizeKB, int mflopF, int ldagap,  int rout,
                               itst++, r, Uplo2Char(rout, UPLOs[u]+SDs[s]),
                               Side2Char(rout, SDs[s]+UPLOs[u]),
                               M, Ns[n], lda, ONB, time, mflop);
+                     fflush(fpout);
                      if (mflop > mfB)
                         mfB = mflop;
                   }                             /* end of reps loop */
