@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 
 #include "atlas_misc.h"
@@ -46,6 +21,16 @@ double time00();
    #define Cgetrf Mjoin(Mjoin(clapack_,PRE),getrf)
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(Cgetrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
+#elif defined(AMMIDX)
+   #define ATL_getrf_amm Mjoin(PATL,getrf_amm)
+   int ATL_getrfC_amm(const int M, const int N, TYPE *A, const int lda,
+                      int *ipiv, int IAMM);
+   #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
+      ATL_assert(ATL_getrf_amm(M_, N_, A_, lda_, ipiv_, AMMIDX) == 0)
+#elif defined(TimeBCAMM)
+   #include "atlas_bcamm.h"
+   #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
+      ATL_assert(Mjoin(PATL,tgetrf_bcAmm)(Major_, M_, N_, A_, lda_, ipiv_) == 0)
 #else
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(ATL_getrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
@@ -234,6 +219,9 @@ static TYPE lutestC(int CacheSize, int M, int N, int lda, int *npiv,
 
    Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
    normA = Mjoin(PATL,genrm1)(M, N, A, lda);
+   #ifdef DEBUG
+      Mjoin(PATL,geprint)("A0", M, N, A, lda);
+   #endif
 
    t0 = ATL_flushcache(-1);
 
@@ -244,7 +232,13 @@ static TYPE lutestC(int CacheSize, int M, int N, int lda, int *npiv,
 
    t0 = ATL_flushcache(0);
 
+   #ifdef DEBUG
+      Mjoin(PATL,geprint)("LU", M, N, A, lda);
+   #endif
    LmU = ATL_LmulUC(M, N, A, lda);  /* LmU contains L * U */
+   #ifdef DEBUG
+      Mjoin(PATL,geprint)("L*U", M, N, LmU, M);
+   #endif
    Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);  /* regenerate A, overwriting LU */
    ATL_laswp(N, A, lda, 0, MN, ipiv, 1);  /* apply swaps to A */
    resid = Mjoin(PATL,gediffnrm1)(M, N, A, lda, LmU, M);
@@ -317,7 +311,7 @@ int RunCase(int CacheSize, TYPE thresh, int MFLOP, enum ATLAS_ORDER Order,
    if (tim > 0.0) mflop = mflops / tim;
    else mflop = 0.0;
    fprintf(stdout, "%5d  %3s   %6d %6d %6d %6d %9.3f %9.3f %9.3e\n",
-           nreps, cord, M, N, lda, npiv, tim, mflop, resid);
+           (int)nreps, cord, M, N, lda, npiv, tim, mflop, resid);
    return(resid <= thresh);
 }
 
@@ -476,7 +470,7 @@ void GetFlags(int nargs, char **args, int *MFLOP, int *CacheSize, TYPE *thresh,
    if (*M0 == -1) *M0 = *MN = *incM = *MisN = 1;
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    int MFLOP, CacheSize, LdaIsM, MisN, NisM, M0, MN, incM, N0, NN, incN;
    int norders;
