@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2009 Siju Samuel
- *
  * Code contributers : Siju Samuel, Anthony M. Castaldo, R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 
 
@@ -69,7 +43,7 @@ LQ
 #ifndef TimeF77
    #include "clapack.h"
 #endif
-//#define ormCtime 1
+/*#define ormCtime 1 */
 #define ormFtime 1
 
 #ifdef ATL_FULL_LAPACK
@@ -380,7 +354,7 @@ static TYPE ORlqtest(int M, int N, int lda, int flushKB, double *time, char Csid
    printVector("TAU", TAU, MNmin);
 
 
-//   LWORKC = 1000000;            // TODO : Temporary Code  Modify  later
+/*   LWORKC = 1000000; */         /* TODO : Temporary Code  Modify  later */
    LWORKC = Mjoin(PATL,ormlq)(SIDE, TRANS, cM, cN, MNmin, &dtmp1, lda, TAU, &dtmp1, ldc,
                      &dtmp,  -1);
    LWORKC = dtmp;
@@ -453,7 +427,7 @@ static TYPE ORqltest(int M, int N, int lda, int flushKB, double *time, char Csid
          ldc = ldm;
          SIDE = CblasLeft;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN? */
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -467,7 +441,7 @@ static TYPE ORqltest(int M, int N, int lda, int flushKB, double *time, char Csid
          cN = M;
          ldc = CsideSz;
          SIDE = CblasRight;
-         TRANS = CblasNoTrans;                 //or MY_TRAN?
+         TRANS = CblasNoTrans;                 /*or MY_TRAN? */
       }
       else
       {
@@ -476,7 +450,7 @@ static TYPE ORqltest(int M, int N, int lda, int flushKB, double *time, char Csid
          ldc = CsideSz;
          SIDE = CblasRight;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN? */
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -521,7 +495,7 @@ static TYPE ORqltest(int M, int N, int lda, int flushKB, double *time, char Csid
    printVector("TAU", TAU, MNmin);
 
 
-//   LWORKC = 1000000;
+/*   LWORKC = 1000000; */
    LWORKC = Mjoin(PATL,ormql)(SIDE, TRANS, cM, cN, MNmin, &dtmp1, lda, TAU, &dtmp1, ldc,
                      &dtmp,  -1);
    LWORKC = dtmp;
@@ -606,7 +580,7 @@ static TYPE ORrqtest(int M, int N, int lda, int flushKB, double *time, char Csid
          ldc = ldn;
          SIDE = CblasLeft;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN? */
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -620,7 +594,7 @@ static TYPE ORrqtest(int M, int N, int lda, int flushKB, double *time, char Csid
          cN = N;
          ldc = CsideSz;
          SIDE = CblasRight;
-         TRANS = CblasNoTrans;                 //or MY_TRAN?
+         TRANS = CblasNoTrans;                 /*or MY_TRAN? */
       }
       else
       {
@@ -629,7 +603,7 @@ static TYPE ORrqtest(int M, int N, int lda, int flushKB, double *time, char Csid
          ldc = CsideSz;
          SIDE = CblasRight;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN? */
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -677,7 +651,7 @@ static TYPE ORrqtest(int M, int N, int lda, int flushKB, double *time, char Csid
    printVector("TAU", TAU, MNmin);
 
 
-//   LWORKC = 10000000;            // TODO : Temporary Code  Modify  later
+/*   LWORKC = 10000000; */         /* TODO : Temporary Code  Modify  later*/
    LWORKC = Mjoin(PATL,ormrq)(SIDE, TRANS, cM, cN, MNmin, &dtmp1, lda, TAU, &dtmp1, ldc,
                      &dtmp,  -1);
    LWORKC = dtmp;
@@ -758,7 +732,7 @@ TYPE ORqrtest(int M, int N, int lda, int flushKB, double *time, char Cside, char
          ldc = ldm;
          SIDE = CblasLeft;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN?*/
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -772,7 +746,7 @@ TYPE ORqrtest(int M, int N, int lda, int flushKB, double *time, char Cside, char
          cN = M;
          ldc = CsideSz;
          SIDE = CblasRight;
-         TRANS = CblasNoTrans;                 //or MY_TRAN?
+         TRANS = CblasNoTrans;                 /*or MY_TRAN?*/
       }
       else
       {
@@ -781,7 +755,7 @@ TYPE ORqrtest(int M, int N, int lda, int flushKB, double *time, char Cside, char
          ldc = CsideSz;
          SIDE = CblasRight;
          #ifdef TREAL
-            TRANS = CblasTrans;                 //or MY_TRAN?
+            TRANS = CblasTrans;                 /*or MY_TRAN? */
          #else
             TRANS = CblasConjTrans;
          #endif
@@ -825,7 +799,7 @@ TYPE ORqrtest(int M, int N, int lda, int flushKB, double *time, char Cside, char
    printMatrix("A", A, M, N, lda);
    printVector("TAU", TAU, MNmin);
 
-//   LWORKC = 100000;            // TODO : Temporary Code  Modify  later
+/*   LWORKC = 100000;  */        /* TODO : Temporary Code  Modify  later*/
    LWORKC = Mjoin(PATL,ormqr)(SIDE, TRANS, cM, cN, MNmin, &dtmp1, lda, TAU, &dtmp1, ldc,
                      &dtmp,  -1);
    LWORKC = dtmp;
@@ -942,12 +916,13 @@ int RunCases(TYPE thresh, int flushKB, int ldagap, int *NREPS,
 
                      npass += RunCase(thresh, flushKB, sides[s], uplos[u],
 
-//orig QR -QL
-//                                      M, Ns[n], M+ldagap,  'L', 'N', 100);
-//                                      M, Ns[n], M+ldagap,  'L', 'T', 100);
-//orig RQ -LQ
-//                                      M, Ns[n], M+ldagap,  'R', 'N', 100);
-//                                      M, Ns[n], M+ldagap,  'R', 'T', 100);
+/*orig QR -QL
+ *                                      M, Ns[n], M+ldagap,  'L', 'N', 100);
+ *                                      M, Ns[n], M+ldagap,  'L', 'T', 100);
+ *orig RQ -LQ
+ *                                      M, Ns[n], M+ldagap,  'R', 'N', 100);
+ *                                      M, Ns[n], M+ldagap,  'R', 'T', 100);
+ */
                                       M, Ns[n], M+ldagap,  Cside, Ctrans, CsideSz);
 
                      ntest++;
@@ -1311,53 +1286,54 @@ double Time2FlopsNew(int rout, int UPLO, int M, int N, double time, char QSIDE,
    int cM, cN;
 
 
-//ORlqtest
-//   if( Cside == 'L')
-//   {
-//         cM = N;
-//         cN = CsideSz;
-//   }
-//   else
-//   {
-//         cM = CsideSz;
-//         cN = N;
-//   }
-//
-//ORqltest
-//   if( Cside == 'L')
-//   {
-//         cM = M;
-//         cN = CsideSz;
-//   }
-//   else
-//   {
-//         cM = CsideSz;
-//         cN = M;
-//   }
-//
-//ORrqtest
-//   if( Cside == 'L')
-//   {
-//         cM = N;
-//         cN = CsideSz;
-//   }
-//   else
-//   {
-//         cM = CsideSz;
-//         cN = N;
-//   }
-//
-//ORqrtest
-//   if( Cside == 'L')
-//   {
-//         cM = M;
-//         cN = CsideSz;
-//   }
-//   else
-//   {
-//         cM = CsideSz;
-//         cN = M;
-//   }
+/*ORlqtest
+ *   if( Cside == 'L')
+ *   {
+ *         cM = N;
+ *         cN = CsideSz;
+ *   }
+ *   else
+ *   {
+ *         cM = CsideSz;
+ *         cN = N;
+ *   }
+ *
+ *ORqltest
+ *   if( Cside == 'L')
+ *   {
+ *         cM = M;
+ *         cN = CsideSz;
+ *   }
+ *   else
+ *   {
+ *         cM = CsideSz;
+ *         cN = M;
+ *   }
+ *
+ *ORrqtest
+ *   if( Cside == 'L')
+ *   {
+ *         cM = N;
+ *         cN = CsideSz;
+ *   }
+ *   else
+ *   {
+ *         cM = CsideSz;
+ *         cN = N;
+ *   }
+ *
+ *ORqrtest
+ *   if( Cside == 'L')
+ *   {
+ *         cM = M;
+ *         cN = CsideSz;
+ *   }
+ *   else
+ *   {
+ *         cM = CsideSz;
+ *         cN = M;
+ *   }
+*/
    mflop = GetFlopCount(rout, UPLO, M, N, 0, 0, CAN_NB);
    if (mflop > 0)
       mflop /= time*1e6;          /* translate flops & time to MFLOPS         */
