@@ -1,38 +1,13 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.2
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2009 Siju Samuel
- *
  * Code contributers : Siju Samuel, R. Clint Whaley, Anthony M. Castaldo
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 /*
  *  This program is a tester for QR, RQ, LQ and QL factorization routines.
  */
 
+#include <string.h>
 #include "atlas_misc.h"
 #include "atlas_lapack.h"
 #include "cblas.h"
@@ -199,7 +174,7 @@ void printVector( char * msg, TYPE *X, int N)
 void   makeVectorToNegVal( TYPE *X1, int size)
 {
    int  i, size2;
-   size2 = size SHIFT;                // 2*size, for  complex  numbers
+   size2 = size SHIFT;                /* 2*size, for  complex  numbers */
 
    for ( i= 0; i < size2; i++)
    {
@@ -251,7 +226,7 @@ void  makeIdenty(TYPE *A, int N, int lda)
 {
    int i, j, lda2;
 
-   lda2 = lda SHIFT;                  // 2*lda for complex
+   lda2 = lda SHIFT;                  /* 2*lda for complex */
 
    for(i=0; i<N; i++)
    {
@@ -424,7 +399,7 @@ void populateViForQL(TYPE *Vi, TYPE * A,  int numberOfVielement)
 void makeToUTForQR(TYPE *A , int lda, int M, int N)
 {
    int i, j;
-   int lda2 = lda SHIFT;              // 2*lda for complex
+   int lda2 = lda SHIFT;              /* 2*lda for complex */
 
    for(i=0; i<M; i++)
    {
@@ -602,7 +577,7 @@ if(TESTCORRECT)
    W = malloc(ATL_MulBySize(ldn));
    if (W == NULL) AllocationErr("W");
 
-   makeIdenty(Q, N, ldn);              //Q: Size of Q mat  and lda is N
+   makeIdenty(Q, N, ldn);              /*Q: Size of Q mat  and lda is N */
 
 /* Copy A[M, N] to L and then Modify the upper part of DIAG w.r.t top left    */
    L= DupMat(CblasColMajor, M, N, A, lda, ldm);
@@ -710,7 +685,7 @@ if(TESTCORRECT)
    resid = Mjoin(PATL,gediffnrm1)(M, N, AORIG, lda, A, lda);
    resid /= (normA * eps * Mmin(M,N));
 
-}//(TESTCORRECT)
+}/*(TESTCORRECT) */
    if(A) free(A);
    if(TAU) free(TAU);
    if(TAUNEG) free(TAUNEG);
@@ -1377,7 +1352,8 @@ int RunCase(TYPE thresh, int flushKB, int side, int uplo, int M, int N, int lda)
    printf("%2s  %3s %6d %6d %6d  %10.4e %11.2f  %9.2e\n", qs, "Col", M, N, lda,
          time, Time2Flops(LAgeqrf, uplo+side, M, N, time), resid);
    /* A temporary fix */
-   if(resid == 0. && M > 10 && N > 10)  resid = 99999999; // Setting to a big num
+   if(resid == 0. && M > 10 && N > 10)
+      resid = 99999999; /* Setting to a big num */
    return(resid <= thresh ? 1 : 0);
 }
 
