@@ -1,34 +1,12 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2000 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2000 R. Clint Whaley
  */
 #include "atlas_misc.h"
 #include "atlas_lapack.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_ptalias_lapack.h"
+#endif
 #include "cblas.h"
 #include "atlas_cblastypealias.h"
 #include "atlas_tst.h"
@@ -57,7 +35,7 @@ double time00();
    #define Cgesv Mjoin(Mjoin(clapack_,PRE),gesv)
    #define Cposv Mjoin(Mjoin(clapack_,PRE),posv)
    #define Cgels Mjoin(Mjoin(clapack_,PRE),gels)
-//   #define Cgels Mjoin(PATL,gels)
+/*   #define Cgels Mjoin(PATL,gels) */
    #define test_posv(Ord_, Uplo_, N_, NRHS_, A_, lda_, B_, ldb_) \
    { \
       int ii; \
@@ -81,11 +59,12 @@ void PrintUsage(char *nam)
 void GetFlags(int nargs, char **args, int *N0, int *NN, int *incN,
               int *R0, int *RN, int *incR, int *ldagap, int *ldbgap,
               double *thresh, int *nord, enum CBLAS_ORDER **Ord,
-              int *nuplo, int **Uplo, int *CS, int *MFLOP, int *QR)
+              int *nuplo, int **Uplo, int *CS, int *MFLOP, int *QR, int *nrep)
 {
    int i, j;
    char ch;
 
+   *nrep = 1;
    *QR = 0;
    *N0 = 100; *NN = 1000; *incN = 100;
    *R0 = *RN = *incR = 1;
@@ -106,6 +85,9 @@ void GetFlags(int nargs, char **args, int *N0, int *NN, int *incN,
       if (args[i][0] != '-') PrintUsage(args[0]);
       switch(args[i][1])
       {
+      case '#':
+         *nrep = atoi(args[++i]);
+         break;
       case 'Q':
          *QR = atoi(args[++i]);
          break;
@@ -195,7 +177,13 @@ TYPE *GetGE(int M, int N, int lda)
 {
    TYPE *A;
    A = malloc(ATL_MulBySize(lda)*N);
-   if (A) Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   if (A)
+   {
+      #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+         ATL_NumaTouchSpread(ATL_MulBySize(lda)*N, A);
+      #endif
+      Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   }
    return(A);
 }
 
@@ -245,6 +233,9 @@ static TYPE *DupMat(enum ATLAS_ORDER Order, int M, int N, TYPE *A, int lda,
    ATL_assert(ldc >= M);
    C = malloc(ATL_MulBySize(ldc)*N);
    ATL_assert(C);
+   #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+      ATL_NumaTouchSpread(ATL_MulBySize(ldc)*N, C);
+   #endif
    for (j=0; j != N; j++)
    {
       for (i=0; i != M2; i++) C[i] = A[i];
@@ -599,10 +590,12 @@ static int gelstst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
  * Generate A and X, and figure B with matmul
  */
    A = GetGE(N, N, lda);
+   ATL_assert(A);
+
    nrm = Mjoin(PATL,genrm1)(N, N, A, lda);
    X = GetGE(N, NRHS, ldb);
    B = GetGE(N, NRHS, ldb);
-   ATL_assert(A && X && B);
+   ATL_assert(X && B);
    if (Order == CblasColMajor)
       Mjoin(CBP,gemm)(Order, CblasNoTrans, CblasNoTrans, N, NRHS, N, one,
                       A, lda, X, ldb, zero, B, ldb);
@@ -658,11 +651,12 @@ static int gesvtst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
  * Generate A and X, and figure B with matmul
  */
    A = GetGE(N, N, lda);
+   ATL_assert(A);
    nrm = Mjoin(PATL,genrm1)(N, N, A, lda);
    X = GetGE(N, NRHS, ldb);
    B = GetGE(N, NRHS, ldb);
    ipiv = malloc(N*sizeof(int));
-   ATL_assert(A && X && B && ipiv);
+   ATL_assert(X && B && ipiv);
    if (Order == CblasColMajor)
       Mjoin(CBP,gemm)(Order, CblasNoTrans, CblasNoTrans, N, NRHS, N, one,
                       A, lda, X, ldb, zero, B, ldb);
@@ -692,13 +686,13 @@ static int gesvtst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
    return(ierr);
 }
 
-static int DoAllTests(int N0, int NN, int incN, int R0, int RN, int incR,
-                      int ldagap, int ldbgap, double thresh,
+static int DoAllTests(int nrep, int N0, int NN, int incN, int R0, int RN,
+                      int incR, int ldagap, int ldbgap, double thresh,
                       int nord, enum CBLAS_ORDER *Orders,
                       int nuplo, int *Uplos, int CS, int MFLOP,
                       int QR)
 {
-   int i=0, ierr=0, n, r, lda, ldb, io, iu;
+   int i=0, ierr=0, n, r, lda, ldb, io, iu, in;
 
    fprintf(stdout, "ORD  UPLO       N    NRHS     lda     ldb       TIME     MFLOP        RESID\n");
    fprintf(stdout, "===  ====  ======  ======  ======  ======  =========  ========  ===========\n\n");
@@ -713,13 +707,16 @@ static int DoAllTests(int N0, int NN, int incN, int R0, int RN, int incR,
          {
             for (iu=0; iu < nuplo; iu++, i++)
             {
-               if (Uplos[iu] == 0)
-                  ierr += gesvtst(Orders[io], n, r, lda, ldb, CS, thresh);
-               else if (Uplos[iu] == 1)
-                  ierr += gelstst(Orders[io], n, r, lda, ldb, CS, thresh);
-               else
-                  ierr += posvtst(Orders[io], Uplos[iu], n, r, lda, ldb,
-                                  CS, thresh);
+               for (in=0; in < nrep; in++)
+               {
+                  if (Uplos[iu] == 0)
+                     ierr += gesvtst(Orders[io], n, r, lda, ldb, CS, thresh);
+                  else if (Uplos[iu] == 1)
+                     ierr += gelstst(Orders[io], n, r, lda, ldb, CS, thresh);
+                  else
+                     ierr += posvtst(Orders[io], Uplos[iu], n, r, lda, ldb,
+                                     CS, thresh);
+               }
             }
          }
       }
@@ -728,18 +725,18 @@ static int DoAllTests(int N0, int NN, int incN, int R0, int RN, int incR,
    else fprintf(stdout, "\n%d TESTS RUN, %d FAILED!!\n\n", i, ierr);
    return(ierr);
 }
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    int ierr, N0, NN, incN, R0, RN, incR, ldagap, ldbgap, nord, nuplo;
-   int CS, MFLOP, QR;
+   int CS, MFLOP, QR, nrep;
    enum CBLAS_ORDER *Order;
    int *Uplo;
    double thresh;
    GetFlags(nargs, args, &N0, &NN, &incN, &R0, &RN, &incR, &ldagap, &ldbgap,
-            &thresh, &nord, &Order, &nuplo, &Uplo, &CS, &MFLOP, &QR);
-   ierr = DoAllTests(N0, NN, incN, R0, RN, incR, ldagap, ldbgap, thresh,
+            &thresh, &nord, &Order, &nuplo, &Uplo, &CS, &MFLOP, &QR, &nrep);
+   ierr = DoAllTests(nrep, N0, NN, incN, R0, RN, incR, ldagap, ldbgap, thresh,
                      nord, Order, nuplo, Uplo, CS, MFLOP, QR);
    if (Order) free(Order);
    if (Uplo) free(Uplo);
-   exit(ierr);
+   return(ierr);
 }
