@@ -1,32 +1,3 @@
-/*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- */
 /*
  * Header file for ATLAS's auxiliary routines
  */
@@ -37,8 +8,8 @@
 void ATL_xerbla(int p, char *rout, char *form, ...);
 int ATL_lcm(const int M, const int N);
 double ATL_flushcache(long long size);
-double ATL_walltime();
-double ATL_cputime();
+double ATL_walltime(void);
+double ATL_cputime(void);
 
 /*
  * Auxiliary routines that come in all four types
@@ -50,6 +21,26 @@ void ATL_strsetL(ATL_CINT M, ATL_CINT N, const float alpha,
 void ATL_strsetU(ATL_CINT M, ATL_CINT N, const float alpha,
                  const float beta, float *A, ATL_CINT lda);
 float ATL_sgemaxnrm(ATL_CINT M, ATL_CINT N, float *A, ATL_CINT lda);
+void ATL_sgeput1T_bX(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float beta, float *C, ATL_CSZT ldc);
+void ATL_samL2skLNB_bX
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float beta, float *C, ATL_iptr_t ldc);
+void ATL_sgeput1T_bN(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float beta, float *C, ATL_CSZT ldc);
+void ATL_samL2skLNB_bN
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float beta, float *C, ATL_iptr_t ldc);
+void ATL_sgeput1T_b1(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float beta, float *C, ATL_CSZT ldc);
+void ATL_samL2skLNB_b1
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float beta, float *C, ATL_iptr_t ldc);
+void ATL_sgeput1T_b0(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float beta, float *C, ATL_CSZT ldc);
+void ATL_samL2skLNB_b0
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float beta, float *C, ATL_iptr_t ldc);
 void ATL_sgeadd(const int M, const int N, const float alpha,
                 const float *A, const int lda, const float beta,
                 float *C, const int ldc);
@@ -181,6 +172,26 @@ void ATL_dtrsetL(ATL_CINT M, ATL_CINT N, const double alpha,
 void ATL_dtrsetU(ATL_CINT M, ATL_CINT N, const double alpha,
                  const double beta, double *A, ATL_CINT lda);
 double ATL_dgemaxnrm(ATL_CINT M, ATL_CINT N, double *A, ATL_CINT lda);
+void ATL_dgeput1T_bX(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double beta, double *C, ATL_CSZT ldc);
+void ATL_damL2skLNB_bX
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double beta, double *C, ATL_iptr_t ldc);
+void ATL_dgeput1T_bN(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double beta, double *C, ATL_CSZT ldc);
+void ATL_damL2skLNB_bN
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double beta, double *C, ATL_iptr_t ldc);
+void ATL_dgeput1T_b1(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double beta, double *C, ATL_CSZT ldc);
+void ATL_damL2skLNB_b1
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double beta, double *C, ATL_iptr_t ldc);
+void ATL_dgeput1T_b0(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double beta, double *C, ATL_CSZT ldc);
+void ATL_damL2skLNB_b0
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double beta, double *C, ATL_iptr_t ldc);
 void ATL_dgeadd(const int M, const int N, const double alpha,
                 const double *A, const int lda, const double beta,
                 double *C, const int ldc);
@@ -312,6 +323,26 @@ void ATL_ctrsetL(ATL_CINT M, ATL_CINT N, const float *alpha,
 void ATL_ctrsetU(ATL_CINT M, ATL_CINT N, const float *alpha,
                  const float *beta, float *A, ATL_CINT lda);
 float ATL_cgemaxnrm(ATL_CINT M, ATL_CINT N, float *A, ATL_CINT lda);
+void ATL_cgeput1T_bX(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_camL2skLNB_bX
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float *beta, float *C, ATL_iptr_t ldc);
+void ATL_cgeput1T_bN(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_camL2skLNB_bN
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float *beta, float *C, ATL_iptr_t ldc);
+void ATL_cgeput1T_b1(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_camL2skLNB_b1
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float *beta, float *C, ATL_iptr_t ldc);
+void ATL_cgeput1T_b0(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_camL2skLNB_b0
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const float *W,  const float *beta, float *C, ATL_iptr_t ldc);
 void ATL_cgeadd(const int M, const int N, const float *alpha,
                 const float *A, const int lda, const float *beta,
                 float *C, const int ldc);
@@ -443,6 +474,26 @@ void ATL_ztrsetL(ATL_CINT M, ATL_CINT N, const double *alpha,
 void ATL_ztrsetU(ATL_CINT M, ATL_CINT N, const double *alpha,
                  const double *beta, double *A, ATL_CINT lda);
 double ATL_zgemaxnrm(ATL_CINT M, ATL_CINT N, double *A, ATL_CINT lda);
+void ATL_zgeput1T_bX(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zamL2skLNB_bX
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double *beta, double *C, ATL_iptr_t ldc);
+void ATL_zgeput1T_bN(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zamL2skLNB_bN
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double *beta, double *C, ATL_iptr_t ldc);
+void ATL_zgeput1T_b1(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zamL2skLNB_b1
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double *beta, double *C, ATL_iptr_t ldc);
+void ATL_zgeput1T_b0(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zamL2skLNB_b0
+   (ATL_iptr_t N, ATL_CUINT MU, ATL_CUINT NU, ATL_CUINT shVL,
+    const double *W,  const double *beta, double *C, ATL_iptr_t ldc);
 void ATL_zgeadd(const int M, const int N, const double *alpha,
                 const double *A, const int lda, const double *beta,
                 double *C, const int ldc);
@@ -567,10 +618,89 @@ void ATL_zgescal_b0
 void ATL_zgescal_bX
    (ATL_CINT  M, ATL_CINT  N, const double *beta, double *C, ATL_CINT  ldc);
 
+
+/*
+ * Real-only aux routines
+ */
+void ATL_strcpypad4L(enum ATLAS_DIAG Diag, ATL_CINT N, const float *A,
+                     ATL_CINT lda, float *c, ATL_CINT ldc);
+void ATL_strcpypad4U(enum ATLAS_DIAG Diag, ATL_CINT N, const float *A,
+                     ATL_CINT lda, float *c, ATL_CINT ldc);
+void ATL_ssycpyUNB_aX(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_ssycpyUNB_an(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_ssycpyUNB_a1(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_ssycpyLNB_aX(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_ssycpyLNB_an(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_ssycpyLNB_a1(ATL_CSZT N, const float alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_dtrcpypad4L(enum ATLAS_DIAG Diag, ATL_CINT N, const double *A,
+                     ATL_CINT lda, double *c, ATL_CINT ldc);
+void ATL_dtrcpypad4U(enum ATLAS_DIAG Diag, ATL_CINT N, const double *A,
+                     ATL_CINT lda, double *c, ATL_CINT ldc);
+void ATL_dsycpyUNB_aX(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_dsycpyUNB_an(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_dsycpyUNB_a1(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_dsycpyLNB_aX(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_dsycpyLNB_an(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_dsycpyLNB_a1(ATL_CSZT N, const double alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+
 /*
  * Specialized complex auxiliary routines
  */
 
+void ATL_checpyUNB_aX(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyUNB_ar(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyUNB_an(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyUNB_a1(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyLNB_aX(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyLNB_ar(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyLNB_an(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_checpyLNB_a1(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyUNB_aX(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyUNB_ar(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyUNB_an(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyUNB_a1(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyLNB_aX(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyLNB_ar(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyLNB_an(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_csycpyLNB_a1(ATL_CSZT N, const float *alpha, const float *A,
+                      ATL_CSZT lda, float *C, ATL_CSZT ldc);
+void ATL_cgeput1H_bX(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_cgeput1H_bN(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_cgeput1H_b1(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_cgeput1H_b0(ATL_CSZT M, ATL_CSZT N, const float *A, ATL_CSZT lda,
+                     const float *beta, float *C, ATL_CSZT ldc);
+void ATL_cswap_cplx2real(ATL_CUINT N, float *C, ATL_CINT incc,
+                         float *R, ATL_CINT incR, float *I, ATL_CINT incI);
 void ATL_cheApAc_NB
    (const enum ATLAS_UPLO Uplo, ATL_CINT N, const float *A, ATL_CINT lda,
     const float *beta, float *C, ATL_CINT ldc);
@@ -580,8 +710,8 @@ void ATL_cheApAc
 void ATL_cgeApBc_NB
    (ATL_CINT M, ATL_CINT N, const float *A, ATL_CINT lda,
     const float *B, ATL_CINT ldb, const float *beta, float *C, ATL_CINT ldc);
-void ATL_ccplxdivide
-   (ATL_CINT N, float *b, float *X, ATL_CINT incX, float *Y, ATL_CINT incY);
+void ATL_ccplxdivide(ATL_CINT N, const float *b, float *X, ATL_CINT incX,
+                     float *Y, ATL_CINT incY);
 void ATL_ccplxinvert
    (const int N, float *X, const int incX, float *Y, const int incY);
 
@@ -775,6 +905,48 @@ void ATL_cgemove_aXi0
 void ATL_cgescal_bXi0
    (const int M, const int N, const float *beta, float *C, const int ldc);
 
+void ATL_zhecpyUNB_aX(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyUNB_ar(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyUNB_an(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyUNB_a1(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyLNB_aX(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyLNB_ar(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyLNB_an(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zhecpyLNB_a1(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyUNB_aX(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyUNB_ar(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyUNB_an(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyUNB_a1(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyLNB_aX(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyLNB_ar(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyLNB_an(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zsycpyLNB_a1(ATL_CSZT N, const double *alpha, const double *A,
+                      ATL_CSZT lda, double *C, ATL_CSZT ldc);
+void ATL_zgeput1H_bX(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zgeput1H_bN(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zgeput1H_b1(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zgeput1H_b0(ATL_CSZT M, ATL_CSZT N, const double *A, ATL_CSZT lda,
+                     const double *beta, double *C, ATL_CSZT ldc);
+void ATL_zswap_cplx2real(ATL_CUINT N, double *C, ATL_CINT incc,
+                         double *R, ATL_CINT incR, double *I, ATL_CINT incI);
 void ATL_zheApAc_NB
    (const enum ATLAS_UPLO Uplo, ATL_CINT N, const double *A, ATL_CINT lda,
     const double *beta, double *C, ATL_CINT ldc);
@@ -784,8 +956,8 @@ void ATL_zheApAc
 void ATL_zgeApBc_NB
    (ATL_CINT M, ATL_CINT N, const double *A, ATL_CINT lda,
     const double *B, ATL_CINT ldb, const double *beta, double *C, ATL_CINT ldc);
-void ATL_zcplxdivide
-   (ATL_CINT N, double *b, double *X, ATL_CINT incX, double *Y, ATL_CINT incY);
+void ATL_zcplxdivide(ATL_CINT N, const double *b, double *X, ATL_CINT incX,
+                     double *Y, ATL_CINT incY);
 void ATL_zcplxinvert
    (const int N, double *X, const int incX, double *Y, const int incY);
 
@@ -1009,22 +1181,26 @@ void ATL_eztrcollapse(const enum ATLAS_UPLO Uplo, const enum ATLAS_DIAG Diag,
    #define ATL_PTCACHEMUL
 #endif
 double ATL_flushcache(long long size);
+#ifdef ATL_USEPTHREADS
+   void ATL_NumaTouchSpread(size_t N, void *buff);
+#endif
 /*
  * If we have it, use assembly-based explicit cache-line flush algorithm
  */
 #if defined(ATL_ARCH_PPCG5) || defined(ATL_ARCH_PPCG4) || \
-    defined(ATL_GAS_PPC) || defined(ATL_SSE1) || \
-    defined(ATL_ARCH_IA64Itan) || defined(ATL_ARCH_IA64Itan2)
+    defined(ATL_GAS_PPC) || defined(ATL_SSE2) || \
+    defined(ATL_ARCH_IA64Itan) || defined(ATL_ARCH_IA64Itan2) || \
+    defined(ATL_ARM64)
 
    #define ATL_LINEFLUSH 1
    typedef struct flStruct FLSTRUCT;
    struct flStruct
    {
       void *p;
-      int length;
+      size_t length;
       FLSTRUCT *next;
    };
-   FLSTRUCT *ATL_GetFlushStruct(void *p, int length, FLSTRUCT *next);
+   FLSTRUCT *ATL_GetFlushStruct(void *p, size_t length, FLSTRUCT *next);
    void ATL_KillAllFlushStructs(FLSTRUCT *p);
    void ATL_flushCacheByAddr(size_t N, void *vp);
    void ATL_FlushAreasByCL(FLSTRUCT *fp);
