@@ -3,7 +3,7 @@
  * -- Automatically Tuned Linear Algebra Software (ATLAS)
  *    (C) Copyright 2000 All Rights Reserved
  *
- * -- ATLAS routine -- Version 3.2 -- December 25, 2000
+ * -- ATLAS routine -- Version 3.9.24 -- December 25, 2000
  *
  * Author         : Antoine P. Petitet
  * Originally developed at the University of Tennessee,
@@ -76,6 +76,7 @@
 
 #ifdef ATL_USEPTHREADS
 #define USE_L3_PTHREADS
+#define USE_L2_PTHREADS
 #endif
 
 /*
@@ -84,34 +85,25 @@
  * =====================================================================
  */
 #if   defined( USE_L1_PTHREADS  )
-#include "atlas_ptalias1.h"
-#include "atlas_ptlevel1.h"
+   #include "atlas_ptalias1.h"
 #elif defined( USE_L1_REFERENCE )
-#include "atlas_refalias1.h"
-#include "atlas_reflevel1.h"
-#else
-#include "atlas_level1.h"
+   #include "atlas_refalias1.h"
 #endif
+#include "atlas_level1.h"
 
 #if   defined( USE_L2_PTHREADS  )
-#include "atlas_ptalias2.h"
-#include "atlas_ptlevel2.h"
+   #include "atlas_ptalias2.h"
 #elif defined( USE_L2_REFERENCE )
-#include "atlas_refalias2.h"
-#include "atlas_reflevel2.h"
-#else
-#include "atlas_level2.h"
+   #include "atlas_refalias2.h"
 #endif
+#include "atlas_level2.h"
 
 #if   defined( USE_L3_PTHREADS  )
-#include "atlas_ptalias3.h"
-#include "atlas_ptlevel3.h"
+   #include "atlas_ptalias3.h"
 #elif defined( USE_L3_REFERENCE )
-#include "atlas_refalias3.h"
-#include "atlas_reflevel3.h"
-#else
-#include "atlas_level3.h"
+   #include "atlas_refalias3.h"
 #endif
+#include "atlas_level3.h"
 /*
  * =====================================================================
  * #define macro constants
@@ -209,6 +201,7 @@
 #define    ATL_F77wrap_sgbmv   atl_f77wrap_sgbmv_
 #define    ATL_F77wrap_sgemv   atl_f77wrap_sgemv_
 #define    ATL_F77wrap_sger    atl_f77wrap_sger_
+#define    ATL_F77wrap_sger2   atl_f77wrap_sger2_
 #define    ATL_F77wrap_ssbmv   atl_f77wrap_ssbmv_
 #define    ATL_F77wrap_sspmv   atl_f77wrap_sspmv_
 #define    ATL_F77wrap_ssymv   atl_f77wrap_ssymv_
@@ -248,6 +241,7 @@
 #define    ATL_F77wrap_dgbmv   atl_f77wrap_dgbmv_
 #define    ATL_F77wrap_dgemv   atl_f77wrap_dgemv_
 #define    ATL_F77wrap_dger    atl_f77wrap_dger_
+#define    ATL_F77wrap_dger2   atl_f77wrap_dger2_
 #define    ATL_F77wrap_dsbmv   atl_f77wrap_dsbmv_
 #define    ATL_F77wrap_dspmv   atl_f77wrap_dspmv_
 #define    ATL_F77wrap_dsymv   atl_f77wrap_dsymv_
@@ -288,6 +282,8 @@
 #define    ATL_F77wrap_cgemv   atl_f77wrap_cgemv_
 #define    ATL_F77wrap_cgerc   atl_f77wrap_cgerc_
 #define    ATL_F77wrap_cgeru   atl_f77wrap_cgeru_
+#define    ATL_F77wrap_cger2c  atl_f77wrap_cger2c_
+#define    ATL_F77wrap_cger2u  atl_f77wrap_cger2u_
 #define    ATL_F77wrap_chbmv   atl_f77wrap_chbmv_
 #define    ATL_F77wrap_chpmv   atl_f77wrap_chpmv_
 #define    ATL_F77wrap_chemv   atl_f77wrap_chemv_
@@ -331,6 +327,8 @@
 #define    ATL_F77wrap_zgemv   atl_f77wrap_zgemv_
 #define    ATL_F77wrap_zgerc   atl_f77wrap_zgerc_
 #define    ATL_F77wrap_zgeru   atl_f77wrap_zgeru_
+#define    ATL_F77wrap_zger2c  atl_f77wrap_zger2c_
+#define    ATL_F77wrap_zger2u  atl_f77wrap_zger2u_
 #define    ATL_F77wrap_zhbmv   atl_f77wrap_zhbmv_
 #define    ATL_F77wrap_zhpmv   atl_f77wrap_zhpmv_
 #define    ATL_F77wrap_zhemv   atl_f77wrap_zhemv_
@@ -386,6 +384,7 @@
 #define    ATL_F77wrap_sgbmv   ATL_F77WRAP_SGBMV
 #define    ATL_F77wrap_sgemv   ATL_F77WRAP_SGEMV
 #define    ATL_F77wrap_sger    ATL_F77WRAP_SGER
+#define    ATL_F77wrap_sger2   ATL_F77WRAP_SGER2
 #define    ATL_F77wrap_ssbmv   ATL_F77WRAP_SSBMV
 #define    ATL_F77wrap_sspmv   ATL_F77WRAP_SSPMV
 #define    ATL_F77wrap_ssymv   ATL_F77WRAP_SSYMV
@@ -425,6 +424,7 @@
 #define    ATL_F77wrap_dgbmv   ATL_F77WRAP_DGBMV
 #define    ATL_F77wrap_dgemv   ATL_F77WRAP_DGEMV
 #define    ATL_F77wrap_dger    ATL_F77WRAP_DGER
+#define    ATL_F77wrap_dger2   ATL_F77WRAP_DGER2
 #define    ATL_F77wrap_dsbmv   ATL_F77WRAP_DSBMV
 #define    ATL_F77wrap_dspmv   ATL_F77WRAP_DSPMV
 #define    ATL_F77wrap_dsymv   ATL_F77WRAP_DSYMV
@@ -465,6 +465,8 @@
 #define    ATL_F77wrap_cgemv   ATL_F77WRAP_CGEMV
 #define    ATL_F77wrap_cgerc   ATL_F77WRAP_CGERC
 #define    ATL_F77wrap_cgeru   ATL_F77WRAP_CGERU
+#define    ATL_F77wrap_cger2c  ATL_F77WRAP_CGER2C
+#define    ATL_F77wrap_cger2u  ATL_F77WRAP_CGER2U
 #define    ATL_F77wrap_chbmv   ATL_F77WRAP_CHBMV
 #define    ATL_F77wrap_chpmv   ATL_F77WRAP_CHPMV
 #define    ATL_F77wrap_chemv   ATL_F77WRAP_CHEMV
@@ -508,6 +510,8 @@
 #define    ATL_F77wrap_zgemv   ATL_F77WRAP_ZGEMV
 #define    ATL_F77wrap_zgerc   ATL_F77WRAP_ZGERC
 #define    ATL_F77wrap_zgeru   ATL_F77WRAP_ZGERU
+#define    ATL_F77wrap_zger2c  ATL_F77WRAP_ZGER2C
+#define    ATL_F77wrap_zger2u  ATL_F77WRAP_ZGER2U
 #define    ATL_F77wrap_zhbmv   ATL_F77WRAP_ZHBMV
 #define    ATL_F77wrap_zhpmv   ATL_F77WRAP_ZHPMV
 #define    ATL_F77wrap_zhemv   ATL_F77WRAP_ZHEMV
@@ -562,6 +566,7 @@
 #define    ATL_F77wrap_sgbmv   atl_f77wrap_sgbmv
 #define    ATL_F77wrap_sgemv   atl_f77wrap_sgemv
 #define    ATL_F77wrap_sger    atl_f77wrap_sger
+#define    ATL_F77wrap_sger2   atl_f77wrap_sger2
 #define    ATL_F77wrap_ssbmv   atl_f77wrap_ssbmv
 #define    ATL_F77wrap_sspmv   atl_f77wrap_sspmv
 #define    ATL_F77wrap_ssymv   atl_f77wrap_ssymv
@@ -601,6 +606,7 @@
 #define    ATL_F77wrap_dgbmv   atl_f77wrap_dgbmv
 #define    ATL_F77wrap_dgemv   atl_f77wrap_dgemv
 #define    ATL_F77wrap_dger    atl_f77wrap_dger
+#define    ATL_F77wrap_dger2   atl_f77wrap_dger2
 #define    ATL_F77wrap_dsbmv   atl_f77wrap_dsbmv
 #define    ATL_F77wrap_dspmv   atl_f77wrap_dspmv
 #define    ATL_F77wrap_dsymv   atl_f77wrap_dsymv
@@ -641,6 +647,8 @@
 #define    ATL_F77wrap_cgemv   atl_f77wrap_cgemv
 #define    ATL_F77wrap_cgerc   atl_f77wrap_cgerc
 #define    ATL_F77wrap_cgeru   atl_f77wrap_cgeru
+#define    ATL_F77wrap_cger2c  atl_f77wrap_cger2c
+#define    ATL_F77wrap_cger2u  atl_f77wrap_cger2u
 #define    ATL_F77wrap_chbmv   atl_f77wrap_chbmv
 #define    ATL_F77wrap_chpmv   atl_f77wrap_chpmv
 #define    ATL_F77wrap_chemv   atl_f77wrap_chemv
@@ -684,6 +692,8 @@
 #define    ATL_F77wrap_zgemv   atl_f77wrap_zgemv
 #define    ATL_F77wrap_zgerc   atl_f77wrap_zgerc
 #define    ATL_F77wrap_zgeru   atl_f77wrap_zgeru
+#define    ATL_F77wrap_zger2c  atl_f77wrap_zger2c
+#define    ATL_F77wrap_zger2u  atl_f77wrap_zger2u
 #define    ATL_F77wrap_zhbmv   atl_f77wrap_zhbmv
 #define    ATL_F77wrap_zhpmv   atl_f77wrap_zhpmv
 #define    ATL_F77wrap_zhemv   atl_f77wrap_zhemv
@@ -738,6 +748,7 @@
 #define    ATL_F77wrap_sgbmv   atl_f77wrap_sgbmv__
 #define    ATL_F77wrap_sgemv   atl_f77wrap_sgemv__
 #define    ATL_F77wrap_sger    atl_f77wrap_sger__
+#define    ATL_F77wrap_sger2   atl_f77wrap_sger2__
 #define    ATL_F77wrap_ssbmv   atl_f77wrap_ssbmv__
 #define    ATL_F77wrap_sspmv   atl_f77wrap_sspmv__
 #define    ATL_F77wrap_ssymv   atl_f77wrap_ssymv__
@@ -777,6 +788,7 @@
 #define    ATL_F77wrap_dgbmv   atl_f77wrap_dgbmv__
 #define    ATL_F77wrap_dgemv   atl_f77wrap_dgemv__
 #define    ATL_F77wrap_dger    atl_f77wrap_dger__
+#define    ATL_F77wrap_dger2   atl_f77wrap_dger2__
 #define    ATL_F77wrap_dsbmv   atl_f77wrap_dsbmv__
 #define    ATL_F77wrap_dspmv   atl_f77wrap_dspmv__
 #define    ATL_F77wrap_dsymv   atl_f77wrap_dsymv__
@@ -817,6 +829,8 @@
 #define    ATL_F77wrap_cgemv   atl_f77wrap_cgemv__
 #define    ATL_F77wrap_cgerc   atl_f77wrap_cgerc__
 #define    ATL_F77wrap_cgeru   atl_f77wrap_cgeru__
+#define    ATL_F77wrap_cger2c  atl_f77wrap_cger2c__
+#define    ATL_F77wrap_cger2u  atl_f77wrap_cger2u__
 #define    ATL_F77wrap_chbmv   atl_f77wrap_chbmv__
 #define    ATL_F77wrap_chpmv   atl_f77wrap_chpmv__
 #define    ATL_F77wrap_chemv   atl_f77wrap_chemv__
@@ -860,6 +874,8 @@
 #define    ATL_F77wrap_zgemv   atl_f77wrap_zgemv__
 #define    ATL_F77wrap_zgerc   atl_f77wrap_zgerc__
 #define    ATL_F77wrap_zgeru   atl_f77wrap_zgeru__
+#define    ATL_F77wrap_zger2c  atl_f77wrap_zger2c__
+#define    ATL_F77wrap_zger2u  atl_f77wrap_zger2u__
 #define    ATL_F77wrap_zhbmv   atl_f77wrap_zhbmv__
 #define    ATL_F77wrap_zhpmv   atl_f77wrap_zhpmv__
 #define    ATL_F77wrap_zhemv   atl_f77wrap_zhemv__
@@ -961,6 +977,11 @@ void       Mjoin( PATLF77WRAP,  ger   )
 ( F77_INTEGER *,   F77_INTEGER *,   TYPE *,          TYPE *,
   F77_INTEGER *,   TYPE *,          F77_INTEGER *,   TYPE *,
   F77_INTEGER * );
+void       Mjoin( PATLF77WRAP,  ger2  )
+( F77_INTEGER *,   F77_INTEGER *,   TYPE *,          TYPE *,
+  F77_INTEGER *,   TYPE *,          F77_INTEGER *,
+  TYPE *,          TYPE *,
+  F77_INTEGER *,   TYPE *,          F77_INTEGER *,   TYPE *, F77_INTEGER * );
 void       Mjoin( PATLF77WRAP,  sbmv  )
 ( F77_INTEGER *,   F77_INTEGER *,   F77_INTEGER *,   TYPE *,
   TYPE *,          F77_INTEGER *,   TYPE *,          F77_INTEGER *,
@@ -995,6 +1016,16 @@ void       Mjoin( PATLF77WRAP,  geru  )
 ( F77_INTEGER *,   F77_INTEGER *,   TYPE *,          TYPE *,
   F77_INTEGER *,   TYPE *,          F77_INTEGER *,   TYPE *,
   F77_INTEGER * );
+void       Mjoin( PATLF77WRAP,  ger2c )
+( F77_INTEGER *,   F77_INTEGER *,
+  TYPE *, TYPE *, F77_INTEGER *,   TYPE *,          F77_INTEGER *,
+  TYPE *, TYPE *, F77_INTEGER *,   TYPE *,          F77_INTEGER *,
+  TYPE *, F77_INTEGER * );
+void       Mjoin( PATLF77WRAP,  ger2u )
+( F77_INTEGER *,   F77_INTEGER *,
+  TYPE *, TYPE *, F77_INTEGER *,   TYPE *,          F77_INTEGER *,
+  TYPE *, TYPE *, F77_INTEGER *,   TYPE *,          F77_INTEGER *,
+  TYPE *, F77_INTEGER * );
 void       Mjoin( PATLF77WRAP,  hbmv  )
 ( F77_INTEGER *,   F77_INTEGER *,   F77_INTEGER *,   TYPE *,
   TYPE *,          F77_INTEGER *,   TYPE *,          F77_INTEGER *,
