@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 #ifndef ATLAS_LAPACK_H
    #define ATLAS_LAPACK_H
@@ -76,7 +51,7 @@ enum ATL_LAROUT
     LApotrf=(1<<8), LAsytrf=(1<<9), LAsytrd=(1<<10), LAhetrf=(1<<11),
     LAhetrd=(1<<12), LAhegst=(1<<13), LAhbgst=(1<<14), LAhpgst=(1<<15),
     LAspgst=(1<<16), LAsbgst=(1<<17), LAsygst=(1<<18), LAstebz=(1<<19),
-    LAgbtrf=(1<<20), LApbtrf=(1<21), LAtrtri=(1<<22), LAlauum=(1<<23)
+    LAgbtrf=(1<<20), LApbtrf=(1<<21), LAtrtri=(1<<22), LAlauum=(1<<23)
     };
 
 enum ATL_ISPEC
@@ -362,7 +337,7 @@ int ATL_strtriCL(const enum CBLAS_DIAG Diag, const int N, float *A,
 int ATL_slascl
    (const enum ATL_LAMATTYPE mtyp, ATL_CINT KL, ATL_CINT KU, const float den0,
     const float num0, ATL_CINT M, ATL_CINT N, float *A, ATL_CINT lda);
-
+int ATL_slaGetB(int M, int N, int K, int HINT);
 
 int ATL_dgetri(const enum CBLAS_ORDER Order, const int N, double *A, const int lda,
                const int *ipiv, double *wrk, int *lwrk);
@@ -533,7 +508,7 @@ int ATL_dtrtriCL(const enum CBLAS_DIAG Diag, const int N, double *A,
 int ATL_dlascl
    (const enum ATL_LAMATTYPE mtyp, ATL_CINT KL, ATL_CINT KU, const double den0,
     const double num0, ATL_CINT M, ATL_CINT N, double *A, ATL_CINT lda);
-
+int ATL_dlaGetB(int M, int N, int K, int HINT);
 
 int ATL_cgetri(const enum CBLAS_ORDER Order, const int N, float *A, const int lda,
                const int *ipiv, float *wrk, int *lwrk);
@@ -708,7 +683,7 @@ int ATL_ctrtriCL(const enum CBLAS_DIAG Diag, const int N, float *A,
 int ATL_clascl
    (const enum ATL_LAMATTYPE mtyp, ATL_CINT KL, ATL_CINT KU, const float den0,
     const float num0, ATL_CINT M, ATL_CINT N, float *A, ATL_CINT lda);
-
+int ATL_claGetB(int M, int N, int K, int HINT);
 
 int ATL_zgetri(const enum CBLAS_ORDER Order, const int N, double *A, const int lda,
                const int *ipiv, double *wrk, int *lwrk);
@@ -883,6 +858,6 @@ int ATL_ztrtriCL(const enum CBLAS_DIAG Diag, const int N, double *A,
 int ATL_zlascl
    (const enum ATL_LAMATTYPE mtyp, ATL_CINT KL, ATL_CINT KU, const double den0,
     const double num0, ATL_CINT M, ATL_CINT N, double *A, ATL_CINT lda);
-
+int ATL_zlaGetB(int M, int N, int K, int HINT);
 
 #endif
