@@ -56,33 +56,70 @@ static ATL_mvnode_t *GetMVNode(void)
 }
 
 /* procedure 2 */
+static void CopyMVNode(ATL_mvnode_t *p, ATL_mvnode_t *dup)
+{
+   ATL_mvnode_t *nxt=p->next;
+   if (p->str)
+      free(p->str);
+   if (p->exflags)
+      free(p->exflags);
+   if (p->genstr)
+      free(p->genstr);
+   if (p->rout)
+      free(p->rout);
+   if (p->cflags)
+      free(p->cflags);
+   if (p->comp)
+      free(p->comp);
+   if (p->auth)
+      free(p->auth);
+   memcpy(p, dup, sizeof(ATL_mvnode_t));
+   if (dup->str)
+      p->str = DupString(dup->str);
+   if (dup->exflags)
+      p->exflags = DupString(dup->exflags);
+   if (dup->genstr)
+      p->genstr = DupString(dup->genstr);
+   if (dup->rout)
+      p->rout = DupString(dup->rout);
+   if (dup->cflags)
+      p->cflags = DupString(dup->cflags);
+   if (dup->comp)
+      p->comp = DupString(dup->comp);
+   if (dup->auth)
+      p->auth = DupString(dup->auth);
+   if (dup->kname)
+      p->kname = DupString(dup->kname);
+   p->next = nxt;
+}
+/* procedure 3 */
 static ATL_mvnode_t *CloneMVNode(ATL_mvnode_t *dup)
 {
    ATL_mvnode_t *p;
    p = malloc(sizeof(ATL_mvnode_t));
    assert(p);
    memcpy(p, dup, sizeof(ATL_mvnode_t));
-   if (dup->rout)
-      p->rout = DupString(dup->rout);
-   if (dup->auth)
-      p->auth = DupString(dup->auth);
-   if (dup->comp)
-      p->comp = DupString(dup->comp);
-   if (dup->cflags)
-      p->cflags = DupString(dup->cflags);
    if (dup->str)
       p->str = DupString(dup->str);
-   if (dup->genstr)
-      p->genstr = DupString(dup->genstr);
    if (dup->exflags)
       p->exflags = DupString(dup->exflags);
+   if (dup->genstr)
+      p->genstr = DupString(dup->genstr);
+   if (dup->rout)
+      p->rout = DupString(dup->rout);
+   if (dup->cflags)
+      p->cflags = DupString(dup->cflags);
+   if (dup->comp)
+      p->comp = DupString(dup->comp);
+   if (dup->auth)
+      p->auth = DupString(dup->auth);
    if (dup->kname)
       p->kname = DupString(dup->kname);
    p->next = NULL;
    return(p);
 }
 
-/* procedure 3: clones a queue of MV structs */
+/* procedure 4: clones a queue of MV structs */
 static ATL_mvnode_t *CloneMVQueue(ATL_mvnode_t *dupb)
 {
    ATL_mvnode_t *p, *pd, *nb;
@@ -97,7 +134,7 @@ static ATL_mvnode_t *CloneMVQueue(ATL_mvnode_t *dupb)
    return(nb);
 }
 
-/* procedure 4: clones a queue of strided MV structs */
+/* procedure 5: clones a queue of strided MV structs */
 static ATL_mvnode_t *CloneStridedMVQueue
 (
    ATL_mvnode_t *dupb,   /* queue of nodes to clone */
@@ -132,7 +169,7 @@ static ATL_mvnode_t *CloneStridedMVQueue
    return(nb);
 }
 
-/* procedure 5 */
+/* procedure 6 */
 static ATL_mvnode_t *KillMVNode(ATL_mvnode_t *die)
 {
    ATL_mvnode_t *p=NULL;
@@ -160,7 +197,7 @@ static ATL_mvnode_t *KillMVNode(ATL_mvnode_t *die)
    return(p);
 }
 
-/* procedure 6: safely remove nukeme from Q, reseting all links */
+/* procedure 7: safely remove nukeme from Q, reseting all links */
 static ATL_mvnode_t *RemoveMVNodeFromQ
 (
    ATL_mvnode_t *Q,     /* queue of nodes */
@@ -190,7 +227,81 @@ static ATL_mvnode_t *RemoveMVNodeFromQ
    return(Q);
 }
 
-/* procedure 7 */
+/* procedure 8: swap two nodes in Q */
+static ATL_mvnode_t *SwapMVNodeInQ
+(
+   ATL_mvnode_t *Q,     /* queue of nodes */
+   ATL_mvnode_t *p0,    /* first node to be swap */
+   ATL_mvnode_t *p1     /* second node to be swap */
+)
+/*
+ * Puts p0 in queue at place p1 was, and p1 in place p0 was.
+ * RETURNS: possibly changed Q
+ */
+{
+   ATL_mvnode_t *p, *prv0=NULL, *prv1=NULL, *nxt;
+   unsigned int fnd=0;
+
+   if (!Q || !p0 || !p1 || p1 == p0) /* need different pair to swap */
+      return(Q);
+   for (p=Q; fnd != 3 && p; p = p->next)
+   {
+      if ((fnd&1) == 0) /* still looking for p0 */
+      {
+         if (p == p0)
+            fnd |= 1;
+         else
+         {
+            prv0 = p;
+         }
+      }
+      if ((fnd&2) == 0) /* still looking for p1 */
+      {
+         if (p == p1)
+            fnd |= 2;
+         else
+         {
+            prv1 = p;
+         }
+      }
+   }
+   if (fnd != 3)
+   {
+      fprintf(stderr, "\nFND=%u!!\n\n", fnd);
+      assert(fnd == 3);
+   }
+   if (prv0 == p1)  /* p1->p0-> */
+   {
+      if (prv1)
+         prv1->next = p0;
+      p1->next = p0->next;
+      p0->next = p1;
+   }
+   else if (prv1 == p0) /* p0->p1-> */
+   {
+      if (prv0)
+         prv0->next = p1;
+      p0->next = p1->next;
+      p1->next = p0;
+   }
+   else /* neither predecessor of other */
+   {
+      nxt = p0->next;
+      p0->next = p1->next;
+      p1->next = nxt;
+      if (prv0)
+         prv0->next = p1;
+      if (prv1)
+         prv1->next = p0;
+   }
+   if (Q == p1)
+      Q = p0;
+   else if (Q == p0)
+      Q = p1;
+   return(Q);
+}
+
+/* procedure 9 */
 static ATL_mvnode_t *KillMVNodeFromQ
 (
    ATL_mvnode_t *Q,     /* queue of nodes */
@@ -202,14 +313,42 @@ static ATL_mvnode_t *KillMVNodeFromQ
    return(Q);
 }
 
-/* procedure 8 */
+/* procedure 10: frees all ->rout entries in queue b */
+static void KillAllMVRouts(ATL_mvnode_t *b)
+{
+   while (b)
+   {
+      if (b->rout)
+      {
+         free(b->rout);
+         b->rout = NULL;
+      }
+      b = b->next;
+   }
+}
+
+/* procedure 11: frees all ->genstr entries in queue b */
+static void KillAllMVGenstr(ATL_mvnode_t *b)
+{
+   while (b)
+   {
+      if (b->genstr)
+      {
+         free(b->genstr);
+         b->genstr = NULL;
+      }
+      b = b->next;
+   }
+}
+
+/* procedure 12 */
 static void KillAllMVNodes(ATL_mvnode_t *die)
 {
    while (die)
       die = KillMVNode(die);
 }
 
-/* procedure 9 */
+/* procedure 13 */
 static void ATL_SubGoodGccInMVNodes
 (
    ATL_mvnode_t *bp   /* queue to make sub in */
@@ -232,7 +371,7 @@ static void ATL_SubGoodGccInMVNodes
    }
 }
 
-/* procedure 10 */
+/* procedure 14 */
 static void ATL_UnsubGoodGccInMVNodes
 (
    ATL_mvnode_t *bp   /* queue to make reverse sub in */
@@ -256,7 +395,7 @@ static void ATL_UnsubGoodGccInMVNodes
    }
 }
 
-/* procedure 11 */
+/* procedure 15 */
 static void ResubGoodGccInMVNodes
 (
    ATL_mvnode_t *bp   /* queue to make sub in */
@@ -279,7 +418,7 @@ static void ResubGoodGccInMVNodes
    }
 }
 
-/* procedure 12 */
+/* procedure 16 */
 static int ATL_CountNumberOfMVNodes
 (
     ATL_mvnode_t *bp   /* queue to count */
@@ -290,7 +429,7 @@ static int ATL_CountNumberOfMVNodes
    return(i);
 }
 
-/* procedure 13 */
+/* procedure 17 */
 static ATL_mvnode_t *ATL_LastMVNode(ATL_mvnode_t *bp)
 /*
  * RETURNS: pointer to last node in queue
@@ -303,7 +442,47 @@ static ATL_mvnode_t *ATL_LastMVNode(ATL_mvnode_t *bp)
    return(p);
 }
 
-/* procedure 14: finds max integer at ip0 in struct */
+/* procedure 18: adds q1 to end of q0 */
+static ATL_mvnode_t *ATL_JoinMVQs(ATL_mvnode_t *q0, ATL_mvnode_t *q1)
+{
+   ATL_mvnode_t *mp;
+   if (!q1)
+      return(q0);
+   if (!q0)
+      return(q1);
+   mp = ATL_LastMVNode(q0);
+   mp->next = q1;
+   return(q0);
+}
+
+/* procedure 19: finds node with max mflop[imf]  */
+static ATL_mvnode_t *FindMaxMflopMVQ
+(
+   ATL_mvnode_t *bp,   /* queue to be searched */
+   int imf
+)
+/*
+ * RETURNS: ptr to structure containing max value in mflop[imf]
+ */
+{
+   ATL_mvnode_t *mp, *p;
+   double mfm;
+   if (!bp)
+      return(NULL);
+   mp = bp;
+   mfm = mp->mflop[imf];
+   for (p=bp->next; p; p = p->next)
+   {
+      const double mf=p->mflop[0];
+      if (mf > mfm)
+      {
+         mfm = mf;
+         mp = p;
+      }
+   }
+   return(mp);
+}
+/* procedure 20: finds max integer at ip0 in struct */
 static ATL_mvnode_t *FindMaxIntInMVQ
 (
    ATL_mvnode_t *bp,   /* queue to be searched */
@@ -336,7 +515,7 @@ static ATL_mvnode_t *FindMaxIntInMVQ
    }
    return(mp);
 }
-/* procedure 15: finds min integer at ip0 in struct */
+/* procedure 21: finds min integer at ip0 in struct */
 static ATL_mvnode_t *FindMinIntInMVQ
 (
    ATL_mvnode_t *bp,   /* queue to be searched */
@@ -370,7 +549,7 @@ static ATL_mvnode_t *FindMinIntInMVQ
    return(mp);
 }
 
-/* procedure 16: finds first integer equal to val at ip0 in struct */
+/* procedure 22: finds first integer equal to val at ip0 in struct */
 static ATL_mvnode_t *FindIntValInMVQ
 (
    ATL_mvnode_t *bp,   /* queue to be searched */
@@ -398,7 +577,7 @@ static ATL_mvnode_t *FindIntValInMVQ
    return(NULL);
 }
 
-/* procedure 17: sorts Q from least-to-greatest on int val at ip0 in struc */
+/* procedure 23: sorts Q from least-to-greatest on int val at ip0 in struc */
 static ATL_mvnode_t *SortMVQByIntVal
 (
    ATL_mvnode_t *bp,   /* queue to be sorted */
@@ -426,7 +605,7 @@ static ATL_mvnode_t *SortMVQByIntVal
    return(sb);
 }
 
-/* procedure 18: reverses order in Q */
+/* procedure 24: reverses order in Q */
 static ATL_mvnode_t *ReverseMVQ(ATL_mvnode_t *bp)
 /*
  * RETURNS: new base ptr of reversed queue
@@ -443,7 +622,7 @@ static ATL_mvnode_t *ReverseMVQ(ATL_mvnode_t *bp)
    return(nb);
 }
 
-/* procedure 19: places all nodes wt int value val at ip0 in new queue */
+/* procedure 25: places all nodes wt int value val at ip0 in new queue */
 static ATL_mvnode_t *YankMVNodesByIntVal
 (
    ATL_mvnode_t **bp0,  /* queue to be searched */
@@ -482,7 +661,7 @@ static ATL_mvnode_t *YankMVNodesByIntVal
    return(valb);
 }
 
-/* procedure 20 */
+/* procedure 26 */
 static ATL_mvnode_t *ATL_SortMVNodesByMflop
 (
    int imf,            /* which mflop entry to sort on */
@@ -495,7 +674,7 @@ static ATL_mvnode_t *ATL_SortMVNodesByMflop
  */
 {
    ATL_mvnode_t *p, *prev, *sb=NULL;   /* ptr, prev, sorted base */
-   ATL_mvnode_t *minp;
+   ATL_mvnode_t *minp, *minprev;
    double mf;
 
 /*
@@ -514,7 +693,9 @@ static ATL_mvnode_t *ATL_SortMVNodesByMflop
          {
             minp = p;
             mf = p->mflop[imf];
+            minprev = prev;
          }
+         prev = p;
       }
 /*
  *    Remove it from unsorted queue, and add as new head of sorted
@@ -526,7 +707,7 @@ static ATL_mvnode_t *ATL_SortMVNodesByMflop
       }
       else   /* in the middle of unsorted queue */
       {
-         prev->next = minp->next;
+         minprev->next = minp->next;
          minp->next = sb;
       }
       sb = minp;
@@ -534,7 +715,7 @@ static ATL_mvnode_t *ATL_SortMVNodesByMflop
    return(sb);
 }
 
-/* procedure 21 */
+/* procedure 27 */
 static ATL_mvnode_t *ParseMVLine(char *ln)
 /*
  * Given a line from a mv index file (with multiple lines pasted together
@@ -802,31 +983,66 @@ static void PrintMVLine(FILE *fpout, ATL_mvnode_t *np)
    if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
    i += fprintf(fpout, "LDAMUL=%d ", np->ldamul);
 
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "PFTUNABLE=%d ", FLAG_IS_SET(np->flag, MVF_PFTUNABLE));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "ALIGNX2A=%d ", FLAG_IS_SET(np->flag, MVF_ALIGNX2A));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "ADDCFLAGS=%d ", FLAG_IS_SET(np->flag, MVF_ADDCFLAGS));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "GEMMBASED=%d ", FLAG_IS_SET(np->flag, MVF_GEMMBASED));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "AXPYBASED=%d ", FLAG_IS_SET(np->flag, MVF_AXPYBASED));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "ALLALIGNXY=%d ", FLAG_IS_SET(np->flag, MVF_ALLALIGNXY));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "LDYTOP=%d ", FLAG_IS_SET(np->flag, MVF_LDYTOP));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "FNU=%d ", FLAG_IS_SET(np->flag, MVF_FNU));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "X87=%d ", FLAG_IS_SET(np->flag, MVF_X87));
+   if (FLAG_IS_SET(np->flag, MVF_PFTUNABLE))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "PFTUNABLE=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_ALIGNX2A))
+   {
+      if (i+8 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "ALIGNX2A=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_ADDCFLAGS))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "ADDCFLAGS=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_GEMMBASED))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "GEMMBASED=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_AXPYBASED))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "AXPYBASED=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_ALLALIGNXY))
+   {
+      if (i+10 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "ALLALIGNXY=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_LDYTOP))
+   {
+      if (i+6 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "LDYTOP=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_FNU))
+   {
+      if (i+3 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "FNU=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, MVF_X87))
+   {
+      if (i+3 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "X87=1 ");
+   }
 
-   if (np->mflop[0]+np->mflop[1]+np->mflop[2]+np->mflop[3]+np->mflop[4]+
-       np->mflop[5]+np->mflop[6] != 0.0)
+   k = 0;  /* no need to write mflop */
+   for (j=0; j < 8; j++)
+   {
+      if (np->mflop[j] != 0.0)
+      {
+         k = 1;
+         break;
+      }
+   }
+   if (k)
    {
       if (i > 3) { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "MFLOP=%le", np->mflop[0]);
-      for (j=7; j && np->mflop[j] == 0.0; j--);
+      for (j=8-1; j && np->mflop[j] == 0.0; j--);
       for (k=1; k <= j; k++)
          i += fprintf(fpout, ",%le", np->mflop[k]);
    }
@@ -842,20 +1058,36 @@ static void PrintMVLine(FILE *fpout, ATL_mvnode_t *np)
    }
    if (np->cflags)
    {
-      if (i+strlen(np->cflags) > 70) { fprintf(fpout, " \\\n   "); i = 3; }
+      if (i+6+strlen(np->cflags) > 70)
+         { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "  CFLAGS='%s'", np->cflags);
    }
    if (np->comp)
    {
-      if (i+strlen(np->comp) > 70) { fprintf(fpout, " \\\n   "); i = 3; }
+      if (i+4+strlen(np->comp) > 70)
+         { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "  COMP='%s'", np->comp);
    }
    if (i)
       fprintf(fpout, "\n");
 }
 
-/* procedure 22 */
-void PrintMVNodes(FILE *fpout, ATL_mvnode_t *bp)
+/* procedure 28 */
+static ATL_mvnode_t *KillMVNodesByFlag(int flag, int msk, ATL_mvnode_t *bp)
+{
+   ATL_mvnode_t *p=bp;
+   while(p)
+   {
+      ATL_mvnode_t *next = p->next;
+      if ((p->flag & msk) != (flag & msk))
+        bp = KillMVNodeFromQ(bp, p);
+      p = next;
+   }
+   return(bp);
+}
+
+/* procedure 29 */
+static void PrintMVNodes(FILE *fpout, ATL_mvnode_t *bp)
 {
    while (bp)
    {
@@ -864,7 +1096,7 @@ void PrintMVNodes(FILE *fpout, ATL_mvnode_t *bp)
    }
 }
 
-/* procedure 23 */
+/* procedure 30 */
 static void WriteMVFile(char *file, ATL_mvnode_t *nq)
 {
    FILE *fpout;
@@ -883,7 +1115,7 @@ static void WriteMVFile(char *file, ATL_mvnode_t *nq)
       fclose(fpout);
 }
 
-/* procedure 24 */
+/* procedure 31 */
 static void WriteMVFileWithPath
    (char pre, char *path, char *file, ATL_mvnode_t *nq)
 {
@@ -892,7 +1124,7 @@ static void WriteMVFileWithPath
    WriteMVFile(ln, nq);
 }
 
-/* procedure 25 */
+/* procedure 32 */
 static ATL_mvnode_t *ReadMVFile(char *file)
 /*
  * Reads in a standard ATLAS parsable MV index file, and returns a
@@ -913,7 +1145,7 @@ static ATL_mvnode_t *ReadMVFile(char *file)
    nq = p = GetMVNode();
    while (ln = GetJoinedLines(fpin))
    {
-      if (ln[0] != '#')
+      if (ln[0] != '#' && ln[0] != '\0')
       {
          p->next = ParseMVLine(ln);
          p = p->next;
@@ -923,7 +1155,7 @@ static ATL_mvnode_t *ReadMVFile(char *file)
    return(KillMVNode(nq));
 }
 
-/* procedure 26 */
+/* procedure 33 */
 static ATL_mvnode_t *ReadMVFileWithPath
    (char pre, char *path, char *file)
 {
@@ -933,7 +1165,7 @@ static ATL_mvnode_t *ReadMVFileWithPath
 }
 
 
-/* procedure 27 */
+/* procedure 34 */
 static ATL_mvnode_t *DelRepeatedMVKernels(ATL_mvnode_t *bp)
 /*
  * Deletes any repeated IDs
@@ -958,7 +1190,7 @@ static ATL_mvnode_t *DelRepeatedMVKernels(ATL_mvnode_t *bp)
    return(bp);
 }
 
-/* procedure 28 */
+/* procedure 35 */
 static ATL_mvnode_t *DelBadArchMVKernels(ATL_mvnode_t *bp)
 /*
  * Weeds out kernels that require SSE/assembly that we haven't got
@@ -966,6 +1198,9 @@ static ATL_mvnode_t *DelBadArchMVKernels(ATL_mvnode_t *bp)
 {
    int asmb=0, die;
    ATL_mvnode_t *p, *prev;
+   #ifdef ATL_GAS_ARM64
+      asmb |= (1<<8);
+   #endif
    #ifdef ATL_GAS_ARM
       asmb |= (1<<7);
    #endif
@@ -1021,7 +1256,7 @@ static ATL_mvnode_t *DelBadArchMVKernels(ATL_mvnode_t *bp)
 }
 
 #define MAXBASES 4
-/* procedure 29 */
+/* procedure 36 */
 static int ATL_MVSplitContexts
 (
    ATL_mvnode_t *kb,   /* pointer to all read in kernels */
@@ -1077,7 +1312,7 @@ static int ATL_MVSplitContexts
    return(nbases);
 }
 
-/* procedure 30 */
+/* procedure 37 */
 static ATL_mvnode_t *ATL_MVLinkContexts
 (
    ATL_mvnode_t *kp1, /* all out-of-cache kernels */
@@ -1110,7 +1345,7 @@ static ATL_mvnode_t *ATL_MVLinkContexts
 }
 #undef MAXBASES
 
-/* procedure 31 */
+/* procedure 38 */
 static ATL_mvnode_t *FindFastestMVKernel
 (  char pre,             /* precision prefix */
    ATL_mvnode_t *bp,  /* kernel queue */
@@ -1153,7 +1388,7 @@ static ATL_mvnode_t *FindFastestMVKernel
    return(kmax);
 }
 
-/* procedure 32 */
+/* procedure 39 */
 static int MVflag2size(int flag)
 /*
  * RETURNS: size of type using precision/type bits in flag
@@ -1166,7 +1401,7 @@ static int MVflag2size(int flag)
    return(size);
 }
 
-/* procedure 33 */
+/* procedure 40 */
 static char MVflag2pre(int flag)
 /*
  * RETURNS: correct precision/type prefix based on flag
@@ -1178,7 +1413,7 @@ static char MVflag2pre(int flag)
    return(FLAG_IS_SET(flag, MVF_COMPLEX) ? 'z' : 'd');
 }
 
-/* procedure 34 */
+/* procedure 41 */
 static int pre2MVflag(char pre, int flag)
 /*
  * RETURNS: flag modified to reflect type/precision indicated by pre
@@ -1189,15 +1424,15 @@ static int pre2MVflag(char pre, int flag)
    return(flag);
 }
 
-/* procedure 35 */
+/* procedure 42 */
 static void SetAllMVTypeFlags(char pre, ATL_mvnode_t *bp)
 {
    ATL_mvnode_t *p;
    for (p=bp; p; p = p->next)
       p->flag = pre2MVflag(pre, p->flag);
 }
-/* procedure 36 */
-void SortByTrans
+/* procedure 43 */
+static void SortByTrans
 (
    ATL_mvnode_t *bp,    /* original kernels wt mixture of trans cases */
    ATL_mvnode_t **bN0,  /* No trans cases */
