@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2003 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2003 R. Clint Whaley
  */
 #ifndef ATLAS_PKBLAS_H
 #define ATLAS_PKBLAS_H
@@ -61,16 +36,20 @@ enum PACK_UPLO {PackUpper=121, PackLower=122, PackGen=123};
 #endif
 
 #ifdef TCPLX
-   #define MindexPL(I_,J_,lda_) ( (((J_)*((lda_)+(lda_)-(J_)-1))) + (I_)+(I_) )
-   #define MindexPU(I_,J_,lda_) ( ((((lda_)+(lda_)+(J_)-1)*(J_))) + (I_)+(I_) )
+   #define MindexPL(I_,J_,lda_) \
+      ( (((J_)*(((size_t)(lda_))+(lda_)-(J_)-1))) + (I_)+(I_) )
+   #define MindexPU(I_,J_,lda_) \
+      ( (((((size_t)(lda_))+(lda_)+(J_)-1)*(J_))) + (I_)+(I_) )
 #else
-   #define MindexPL(I_,J_,lda_) ( (((J_)*((lda_)+(lda_)-(J_)-1))>>1) + (I_) )
-   #define MindexPU(I_,J_,lda_) ( ((((lda_)+(lda_)+(J_)-1)*(J_))>>1) + (I_) )
+   #define MindexPL(I_,J_,lda_) \
+      ( (((J_)*(((size_t)(lda_))+(lda_)-(J_)-1))>>1) + (I_) )
+   #define MindexPU(I_,J_,lda_) \
+      ( (((((size_t)(lda_))+(lda_)+(J_)-1)*(J_))>>1) + (I_) )
 #endif
 #define MindexP(uplo_,I_,J_,lda_) \
    ( (uplo_) == PackUpper ? MindexPU(I_,J_,lda_) : \
      ( (uplo_) == PackLower ? MindexPL(I_,J_,lda_) : \
-                              (((J_)*(lda_)+(I_))SHIFT) ) )
+                              (((J_)*((size_t)(lda_))+(I_))SHIFT) ) )
 #define Mpld(uplo_,J_,lda_) (uplo_) == PackUpper ? (lda_)+(J_) : \
               ( (uplo_) == PackLower ? (lda_)-(J_) : (lda_) )
 
