@@ -1,8 +1,12 @@
 #ifndef ATLAS_PREFETCH_H
 #define ATLAS_PREFETCH_H
+#ifdef ATL_ARCH_TI_C66_BM
+   #define ATL_NOPREFETCH 1
+#endif
 /*
  * Altivec prefetch model not well utilized by SSE-like prefetch, so have
  * special commands for it.
+ * NOTE: modern machines really using VSX, so don't do this.
  */
 #if defined(ATL_AltiVec)
    #include "atlas_altivec.h"
@@ -46,7 +50,34 @@
          __asm__ __volatile__ ("prefetcht1 %0" : : "m" (*((char *)(mem))))
       #define ATL_pfl2WO ATL_pfl2W
       #define ATL_GOT_L2PREFETCH
+      #define ATL_pflXR(mem) \
+         __asm__ __volatile__ ("prefetcht2 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pflXW(mem) \
+         __asm__ __volatile__ ("prefetcht2 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pflXWO ATL_pflXW
+      #define ATL_GOT_LXPREFETCH
    #endif
+#elif defined(ATL_AVXZ)
+      #define ATL_pfl1R(mem) \
+         __asm__ __volatile__ ("vprefetch0 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pfl1W(mem) \
+         __asm__ __volatile__ ("vprefetche0 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pfl1WO ATL_pfl1W
+      #define ATL_GOT_L1PREFETCH
+
+      #define ATL_pfl2R(mem) \
+         __asm__ __volatile__ ("vprefetch1 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pfl2W(mem) \
+         __asm__ __volatile__ ("vprefetche1 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pfl2WO ATL_pfl2W
+      #define ATL_GOT_L2PREFETCH
+
+      #define ATL_pflXR(mem) \
+         __asm__ __volatile__ ("vprefetch1 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pflXW(mem) \
+         __asm__ __volatile__ ("vprefetche1 %0" : : "m" (*((char *)(mem))))
+      #define ATL_pflXWO ATL_pflXW
+      #define ATL_GOT_LXPREFETCH
 #elif defined(__SUNPRO_C) && defined(__sparc) /* && __SUNPRO_CC > 0x600 */
    #include <sun_prefetch.h>
    #define ATL_pfl1R(mem) sparc_prefetch_read_many((void*)(mem))
@@ -55,6 +86,22 @@
    #define ATL_pfl2R(mem) sparc_prefetch_read_many((void*)(mem))
    #define ATL_pfl2W(mem) sparc_prefetch_write_many((void*)(mem))
    #define ATL_GOT_L2PREFETCH
+#elif defined(ATL_GAS_ARM64)
+   #define ATL_GOT_L1PREFETCH
+   #define ATL_pfl1R(mem) \
+      __asm__ __volatile__ ("PRFM PLDL1KEEP, %0" : : "m" (*((char *)(mem))))
+   #define ATL_pfl1W(mem) \
+      __asm__ __volatile__ ("PRFM PSTL1STRM, %0" : : "m" (*((char *)(mem))))
+   #define ATL_GOT_L2PREFETCH
+   #define ATL_pfl2R(mem) \
+      __asm__ __volatile__ ("PRFM PLDL2KEEP, %0" : : "m" (*((char *)(mem))))
+   #define ATL_pfl2W(mem) \
+      __asm__ __volatile__ ("PRFM PSTL2STRM, %0" : : "m" (*((char *)(mem))))
+   #define ATL_GOT_LXPREFETCH
+   #define ATL_pflXR(mem) \
+      __asm__ __volatile__ ("PRFM PLDL3KEEP, %0" : : "m" (*((char *)(mem))))
+   #define ATL_pflXW(mem) \
+      __asm__ __volatile__ ("PRFM PSTL3STRM, %0" : : "m" (*((char *)(mem))))
 #elif defined(ATL_GAS_ARM)
    #define ATL_pfl1R(mem) \
       __asm__ __volatile__ ("pld %0" : : "m" (*((char *)(mem))))
@@ -154,15 +201,29 @@
    #define ATL_GOT_L1PREFETCH
    #define ATL_L1LS 32
    #define ATL_L2LS 64
-#elif defined(__GNUC__)  /* last ditch, use gcc predefined func */
+#elif defined(ATL_ARCH_IbmZ196) || defined(ATL_ARCH_IbmZ10)
    #define ATL_pfl1R(mem) __builtin_prefetch(mem, 0, 3)
    #define ATL_pfl1W(mem) __builtin_prefetch(mem, 1, 3)
    #define ATL_GOT_L1PREFETCH
+   #define ATL_L1LS 256
+   #define ATL_L2LS 256
+/*
+ * As last resort, try using gcc intrinsic, but only if we aren't on a
+ * generic architecture that does not possess prefetch
+ */
+#elif defined(__GNUC__) && !defined(ATL_ARCH_x86x87)
+   #define ATL_pfl1R(mem) __builtin_prefetch(mem, 0, 3)
+   #define ATL_pfl1W(mem) __builtin_prefetch(mem, 1, 1)
+   #define ATL_GOT_L1PREFETCH
 #endif
 #if defined(ATL_pfl1W) && !defined(ATL_pfl1WO)
    #define ATL_pfl1WO ATL_pfl1W
 #endif
 
+#ifdef ATL_NOPREFETCH
+   #define ATL_NOL1PREFETCH 1
+   #define ATL_NOL2PREFETCH 1
+#endif
 #ifdef ATL_NOL1PREFETCH
    #ifdef ATL_GOT_L1PREFETCH
       #undef ATL_pfl1R
@@ -187,6 +248,12 @@
 #ifndef ATL_GOT_L2PREFETCH  /* dummy calls cpp takes out of code */
    #define ATL_pfl2R(mem)
    #define ATL_pfl2W(mem)
+   #define ATL_pfl2WO(mem)
+#endif
+#ifndef ATL_GOT_LXPREFETCH  /* dummy calls cpp takes out of code */
+   #define ATL_pflXR(mem)
+   #define ATL_pflXW(mem)
+   #define ATL_pflXWO(mem)
 #endif
 
 /*
@@ -198,5 +265,71 @@
 #ifndef ATL_L2LS
    #define ATL_L2LS ATL_L1LS
 #endif
+/*
+ * Defined macro to divide by line size using shifts, just in case compiler
+ * doesn't do strength reduction
+ */
+#if ATL_L1LS == 2
+   #define ATL_MulByL1LS(i_) ((i_)<<1)
+   #define ATL_DivByL1LS(i_) ((i_)>>1)
+#elif ATL_L1LS == 4
+   #define ATL_MulByL1LS(i_) ((i_)<<1)
+   #define ATL_DivByL1LS(i_) ((i_)>>1)
+#elif ATL_L1LS == 8
+   #define ATL_MulByL1LS(i_) ((i_)<<2)
+   #define ATL_DivByL1LS(i_) ((i_)>>2)
+#elif ATL_L1LS == 16
+   #define ATL_MulByL1LS(i_) ((i_)<<3)
+   #define ATL_DivByL1LS(i_) ((i_)>>3)
+#elif ATL_L1LS == 32
+   #define ATL_MulByL1LS(i_) ((i_)<<4)
+   #define ATL_DivByL1LS(i_) ((i_)>>4)
+#elif ATL_L1LS == 64
+   #define ATL_MulByL1LS(i_) ((i_)<<5)
+   #define ATL_DivByL1LS(i_) ((i_)>>5)
+#elif ATL_L1LS == 128
+   #define ATL_MulByL1LS(i_) ((i_)<<6)
+   #define ATL_DivByL1LS(i_) ((i_)>>6)
+#elif ATL_L1LS == 256
+   #define ATL_MulByL1LS(i_) ((i_)<<7)
+   #define ATL_DivByL1LS(i_) ((i_)>>7)
+#elif ATL_L1LS == 512
+   #define ATL_MulByL1LS(i_) ((i_)<<8)
+   #define ATL_DivByL1LS(i_) ((i_)>>8)
+#elif ATL_L1LS == 1024
+   #define ATL_MulByL1LS(i_) ((i_)<<9)
+   #define ATL_DivByL1LS(i_) ((i_)>>9)
+#endif
+#if ATL_L2LS == 2
+   #define ATL_MulByL2LS(i_) ((i_)<<1)
+   #define ATL_DivByL2LS(i_) ((i_)>>1)
+#elif ATL_L2LS == 4
+   #define ATL_MulByL2LS(i_) ((i_)<<1)
+   #define ATL_DivByL2LS(i_) ((i_)>>1)
+#elif ATL_L2LS == 8
+   #define ATL_MulByL2LS(i_) ((i_)<<2)
+   #define ATL_DivByL2LS(i_) ((i_)>>2)
+#elif ATL_L2LS == 16
+   #define ATL_MulByL2LS(i_) ((i_)<<3)
+   #define ATL_DivByL2LS(i_) ((i_)>>3)
+#elif ATL_L2LS == 32
+   #define ATL_MulByL2LS(i_) ((i_)<<4)
+   #define ATL_DivByL2LS(i_) ((i_)>>4)
+#elif ATL_L2LS == 64
+   #define ATL_MulByL2LS(i_) ((i_)<<5)
+   #define ATL_DivByL2LS(i_) ((i_)>>5)
+#elif ATL_L2LS == 128
+   #define ATL_MulByL2LS(i_) ((i_)<<6)
+   #define ATL_DivByL2LS(i_) ((i_)>>6)
+#elif ATL_L2LS == 256
+   #define ATL_MulByL2LS(i_) ((i_)<<7)
+   #define ATL_DivByL2LS(i_) ((i_)>>7)
+#elif ATL_L2LS == 512
+   #define ATL_MulByL2LS(i_) ((i_)<<8)
+   #define ATL_DivByL2LS(i_) ((i_)>>8)
+#elif ATL_L2LS == 1024
+   #define ATL_MulByL2LS(i_) ((i_)<<9)
+   #define ATL_DivByL2LS(i_) ((i_)>>9)
+#endif
 
 #endif
