@@ -67,33 +67,70 @@ static ATL_r1node_t *GetR1Node(void)
 }
 
 /* procedure 2 */
+static void CopyR1Node(ATL_r1node_t *p, ATL_r1node_t *dup)
+{
+   ATL_r1node_t *nxt=p->next;
+   if (p->str)
+      free(p->str);
+   if (p->exflags)
+      free(p->exflags);
+   if (p->genstr)
+      free(p->genstr);
+   if (p->rout)
+      free(p->rout);
+   if (p->cflags)
+      free(p->cflags);
+   if (p->comp)
+      free(p->comp);
+   if (p->auth)
+      free(p->auth);
+   memcpy(p, dup, sizeof(ATL_r1node_t));
+   if (dup->str)
+      p->str = DupString(dup->str);
+   if (dup->exflags)
+      p->exflags = DupString(dup->exflags);
+   if (dup->genstr)
+      p->genstr = DupString(dup->genstr);
+   if (dup->rout)
+      p->rout = DupString(dup->rout);
+   if (dup->cflags)
+      p->cflags = DupString(dup->cflags);
+   if (dup->comp)
+      p->comp = DupString(dup->comp);
+   if (dup->auth)
+      p->auth = DupString(dup->auth);
+   if (dup->kname)
+      p->kname = DupString(dup->kname);
+   p->next = nxt;
+}
+/* procedure 3 */
 static ATL_r1node_t *CloneR1Node(ATL_r1node_t *dup)
 {
    ATL_r1node_t *p;
    p = malloc(sizeof(ATL_r1node_t));
    assert(p);
    memcpy(p, dup, sizeof(ATL_r1node_t));
-   if (dup->rout)
-      p->rout = DupString(dup->rout);
-   if (dup->auth)
-      p->auth = DupString(dup->auth);
-   if (dup->comp)
-      p->comp = DupString(dup->comp);
-   if (dup->cflags)
-      p->cflags = DupString(dup->cflags);
    if (dup->str)
       p->str = DupString(dup->str);
-   if (dup->genstr)
-      p->genstr = DupString(dup->genstr);
    if (dup->exflags)
       p->exflags = DupString(dup->exflags);
+   if (dup->genstr)
+      p->genstr = DupString(dup->genstr);
+   if (dup->rout)
+      p->rout = DupString(dup->rout);
+   if (dup->cflags)
+      p->cflags = DupString(dup->cflags);
+   if (dup->comp)
+      p->comp = DupString(dup->comp);
+   if (dup->auth)
+      p->auth = DupString(dup->auth);
    if (dup->kname)
       p->kname = DupString(dup->kname);
    p->next = NULL;
    return(p);
 }
 
-/* procedure 3: clones a queue of R1 structs */
+/* procedure 4: clones a queue of R1 structs */
 static ATL_r1node_t *CloneR1Queue(ATL_r1node_t *dupb)
 {
    ATL_r1node_t *p, *pd, *nb;
@@ -108,7 +145,7 @@ static ATL_r1node_t *CloneR1Queue(ATL_r1node_t *dupb)
    return(nb);
 }
 
-/* procedure 4: clones a queue of strided R1 structs */
+/* procedure 5: clones a queue of strided R1 structs */
 static ATL_r1node_t *CloneStridedR1Queue
 (
    ATL_r1node_t *dupb,   /* queue of nodes to clone */
@@ -143,7 +180,7 @@ static ATL_r1node_t *CloneStridedR1Queue
    return(nb);
 }
 
-/* procedure 5 */
+/* procedure 6 */
 static ATL_r1node_t *KillR1Node(ATL_r1node_t *die)
 {
    ATL_r1node_t *p=NULL;
@@ -171,7 +208,7 @@ static ATL_r1node_t *KillR1Node(ATL_r1node_t *die)
    return(p);
 }
 
-/* procedure 6: safely remove nukeme from Q, reseting all links */
+/* procedure 7: safely remove nukeme from Q, reseting all links */
 static ATL_r1node_t *RemoveR1NodeFromQ
 (
    ATL_r1node_t *Q,     /* queue of nodes */
@@ -201,7 +238,81 @@ static ATL_r1node_t *RemoveR1NodeFromQ
    return(Q);
 }
 
-/* procedure 7 */
+/* procedure 8: swap two nodes in Q */
+static ATL_r1node_t *SwapR1NodeInQ
+(
+   ATL_r1node_t *Q,     /* queue of nodes */
+   ATL_r1node_t *p0,    /* first node to be swap */
+   ATL_r1node_t *p1     /* second node to be swap */
+)
+/*
+ * Puts p0 in queue at place p1 was, and p1 in place p0 was.
+ * RETURNS: possibly changed Q
+ */
+{
+   ATL_r1node_t *p, *prv0=NULL, *prv1=NULL, *nxt;
+   unsigned int fnd=0;
+
+   if (!Q || !p0 || !p1 || p1 == p0) /* need different pair to swap */
+      return(Q);
+   for (p=Q; fnd != 3 && p; p = p->next)
+   {
+      if ((fnd&1) == 0) /* still looking for p0 */
+      {
+         if (p == p0)
+            fnd |= 1;
+         else
+         {
+            prv0 = p;
+         }
+      }
+      if ((fnd&2) == 0) /* still looking for p1 */
+      {
+         if (p == p1)
+            fnd |= 2;
+         else
+         {
+            prv1 = p;
+         }
+      }
+   }
+   if (fnd != 3)
+   {
+      fprintf(stderr, "\nFND=%u!!\n\n", fnd);
+      assert(fnd == 3);
+   }
+   if (prv0 == p1)  /* p1->p0-> */
+   {
+      if (prv1)
+         prv1->next = p0;
+      p1->next = p0->next;
+      p0->next = p1;
+   }
+   else if (prv1 == p0) /* p0->p1-> */
+   {
+      if (prv0)
+         prv0->next = p1;
+      p0->next = p1->next;
+      p1->next = p0;
+   }
+   else /* neither predecessor of other */
+   {
+      nxt = p0->next;
+      p0->next = p1->next;
+      p1->next = nxt;
+      if (prv0)
+         prv0->next = p1;
+      if (prv1)
+         prv1->next = p0;
+   }
+   if (Q == p1)
+      Q = p0;
+   else if (Q == p0)
+      Q = p1;
+   return(Q);
+}
+
+/* procedure 9 */
 static ATL_r1node_t *KillR1NodeFromQ
 (
    ATL_r1node_t *Q,     /* queue of nodes */
@@ -213,14 +324,42 @@ static ATL_r1node_t *KillR1NodeFromQ
    return(Q);
 }
 
-/* procedure 8 */
+/* procedure 10: frees all ->rout entries in queue b */
+static void KillAllR1Routs(ATL_r1node_t *b)
+{
+   while (b)
+   {
+      if (b->rout)
+      {
+         free(b->rout);
+         b->rout = NULL;
+      }
+      b = b->next;
+   }
+}
+
+/* procedure 11: frees all ->genstr entries in queue b */
+static void KillAllR1Genstr(ATL_r1node_t *b)
+{
+   while (b)
+   {
+      if (b->genstr)
+      {
+         free(b->genstr);
+         b->genstr = NULL;
+      }
+      b = b->next;
+   }
+}
+
+/* procedure 12 */
 static void KillAllR1Nodes(ATL_r1node_t *die)
 {
    while (die)
       die = KillR1Node(die);
 }
 
-/* procedure 9 */
+/* procedure 13 */
 static void ATL_SubGoodGccInR1Nodes
 (
    ATL_r1node_t *bp   /* queue to make sub in */
@@ -243,7 +382,7 @@ static void ATL_SubGoodGccInR1Nodes
    }
 }
 
-/* procedure 10 */
+/* procedure 14 */
 static void ATL_UnsubGoodGccInR1Nodes
 (
    ATL_r1node_t *bp   /* queue to make reverse sub in */
@@ -267,7 +406,7 @@ static void ATL_UnsubGoodGccInR1Nodes
    }
 }
 
-/* procedure 11 */
+/* procedure 15 */
 static void ResubGoodGccInR1Nodes
 (
    ATL_r1node_t *bp   /* queue to make sub in */
@@ -290,7 +429,7 @@ static void ResubGoodGccInR1Nodes
    }
 }
 
-/* procedure 12 */
+/* procedure 16 */
 static int ATL_CountNumberOfR1Nodes
 (
     ATL_r1node_t *bp   /* queue to count */
@@ -301,7 +440,7 @@ static int ATL_CountNumberOfR1Nodes
    return(i);
 }
 
-/* procedure 13 */
+/* procedure 17 */
 static ATL_r1node_t *ATL_LastR1Node(ATL_r1node_t *bp)
 /*
  * RETURNS: pointer to last node in queue
@@ -314,7 +453,47 @@ static ATL_r1node_t *ATL_LastR1Node(ATL_r1node_t *bp)
    return(p);
 }
 
-/* procedure 14: finds max integer at ip0 in struct */
+/* procedure 18: adds q1 to end of q0 */
+static ATL_r1node_t *ATL_JoinR1Qs(ATL_r1node_t *q0, ATL_r1node_t *q1)
+{
+   ATL_r1node_t *mp;
+   if (!q1)
+      return(q0);
+   if (!q0)
+      return(q1);
+   mp = ATL_LastR1Node(q0);
+   mp->next = q1;
+   return(q0);
+}
+
+/* procedure 19: finds node with max mflop[imf]  */
+static ATL_r1node_t *FindMaxMflopR1Q
+(
+   ATL_r1node_t *bp,   /* queue to be searched */
+   int imf
+)
+/*
+ * RETURNS: ptr to structure containing max value in mflop[imf]
+ */
+{
+   ATL_r1node_t *mp, *p;
+   double mfm;
+   if (!bp)
+      return(NULL);
+   mp = bp;
+   mfm = mp->mflop[imf];
+   for (p=bp->next; p; p = p->next)
+   {
+      const double mf=p->mflop[0];
+      if (mf > mfm)
+      {
+         mfm = mf;
+         mp = p;
+      }
+   }
+   return(mp);
+}
+/* procedure 20: finds max integer at ip0 in struct */
 static ATL_r1node_t *FindMaxIntInR1Q
 (
    ATL_r1node_t *bp,   /* queue to be searched */
@@ -347,7 +526,7 @@ static ATL_r1node_t *FindMaxIntInR1Q
    }
    return(mp);
 }
-/* procedure 15: finds min integer at ip0 in struct */
+/* procedure 21: finds min integer at ip0 in struct */
 static ATL_r1node_t *FindMinIntInR1Q
 (
    ATL_r1node_t *bp,   /* queue to be searched */
@@ -381,7 +560,7 @@ static ATL_r1node_t *FindMinIntInR1Q
    return(mp);
 }
 
-/* procedure 16: finds first integer equal to val at ip0 in struct */
+/* procedure 22: finds first integer equal to val at ip0 in struct */
 static ATL_r1node_t *FindIntValInR1Q
 (
    ATL_r1node_t *bp,   /* queue to be searched */
@@ -409,7 +588,7 @@ static ATL_r1node_t *FindIntValInR1Q
    return(NULL);
 }
 
-/* procedure 17: sorts Q from least-to-greatest on int val at ip0 in struc */
+/* procedure 23: sorts Q from least-to-greatest on int val at ip0 in struc */
 static ATL_r1node_t *SortR1QByIntVal
 (
    ATL_r1node_t *bp,   /* queue to be sorted */
@@ -437,7 +616,7 @@ static ATL_r1node_t *SortR1QByIntVal
    return(sb);
 }
 
-/* procedure 18: reverses order in Q */
+/* procedure 24: reverses order in Q */
 static ATL_r1node_t *ReverseR1Q(ATL_r1node_t *bp)
 /*
  * RETURNS: new base ptr of reversed queue
@@ -454,7 +633,7 @@ static ATL_r1node_t *ReverseR1Q(ATL_r1node_t *bp)
    return(nb);
 }
 
-/* procedure 19: places all nodes wt int value val at ip0 in new queue */
+/* procedure 25: places all nodes wt int value val at ip0 in new queue */
 static ATL_r1node_t *YankR1NodesByIntVal
 (
    ATL_r1node_t **bp0,  /* queue to be searched */
@@ -493,7 +672,7 @@ static ATL_r1node_t *YankR1NodesByIntVal
    return(valb);
 }
 
-/* procedure 20 */
+/* procedure 26 */
 static ATL_r1node_t *ATL_SortR1NodesByMflop
 (
    int imf,            /* which mflop entry to sort on */
@@ -506,7 +685,7 @@ static ATL_r1node_t *ATL_SortR1NodesByMflop
  */
 {
    ATL_r1node_t *p, *prev, *sb=NULL;   /* ptr, prev, sorted base */
-   ATL_r1node_t *minp;
+   ATL_r1node_t *minp, *minprev;
    double mf;
 
 /*
@@ -525,7 +704,9 @@ static ATL_r1node_t *ATL_SortR1NodesByMflop
          {
             minp = p;
             mf = p->mflop[imf];
+            minprev = prev;
          }
+         prev = p;
       }
 /*
  *    Remove it from unsorted queue, and add as new head of sorted
@@ -537,7 +718,7 @@ static ATL_r1node_t *ATL_SortR1NodesByMflop
       }
       else   /* in the middle of unsorted queue */
       {
-         prev->next = minp->next;
+         minprev->next = minp->next;
          minp->next = sb;
       }
       sb = minp;
@@ -545,7 +726,7 @@ static ATL_r1node_t *ATL_SortR1NodesByMflop
    return(sb);
 }
 
-/* procedure 21 */
+/* procedure 27 */
 static ATL_r1node_t *ParseR1Line(char *ln)
 /*
  * Given a line from a r1 index file (with multiple lines pasted together
@@ -773,25 +954,51 @@ static void PrintR1Line(FILE *fpout, ATL_r1node_t *np)
    if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
    i += fprintf(fpout, "LDAMUL=%d ", np->ldamul);
 
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "PFTUNABLE=%d ", FLAG_IS_SET(np->flag, R1F_PFTUNABLE));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "ALIGNX2A=%d ", FLAG_IS_SET(np->flag, R1F_ALIGNX2A));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "ADDCFLAGS=%d ", FLAG_IS_SET(np->flag, R1F_ADDCFLAGS));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "FNU=%d ", FLAG_IS_SET(np->flag, R1F_FNU));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "INCYISONE=%d ", FLAG_IS_SET(np->flag, R1F_INCYISONE));
-   if (i > 70) { fprintf(fpout, " \\\n   "); i = 3; }
-   i += fprintf(fpout, "X87=%d ", FLAG_IS_SET(np->flag, R1F_X87));
+   if (FLAG_IS_SET(np->flag, R1F_PFTUNABLE))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "PFTUNABLE=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, R1F_ALIGNX2A))
+   {
+      if (i+8 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "ALIGNX2A=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, R1F_ADDCFLAGS))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "ADDCFLAGS=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, R1F_FNU))
+   {
+      if (i+3 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "FNU=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, R1F_INCYISONE))
+   {
+      if (i+9 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "INCYISONE=1 ");
+   }
+   if (FLAG_IS_SET(np->flag, R1F_X87))
+   {
+      if (i+3 > 72) { fprintf(fpout, " \\\n   "); i = 3; }
+      i += fprintf(fpout, "X87=1 ");
+   }
 
-   if (np->mflop[0]+np->mflop[1]+np->mflop[2]+np->mflop[3]+np->mflop[4]+
-       np->mflop[5]+np->mflop[6] != 0.0)
+   k = 0;  /* no need to write mflop */
+   for (j=0; j < 8; j++)
+   {
+      if (np->mflop[j] != 0.0)
+      {
+         k = 1;
+         break;
+      }
+   }
+   if (k)
    {
       if (i > 3) { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "MFLOP=%le", np->mflop[0]);
-      for (j=7; j && np->mflop[j] == 0.0; j--);
+      for (j=8-1; j && np->mflop[j] == 0.0; j--);
       for (k=1; k <= j; k++)
          i += fprintf(fpout, ",%le", np->mflop[k]);
    }
@@ -807,20 +1014,36 @@ static void PrintR1Line(FILE *fpout, ATL_r1node_t *np)
    }
    if (np->cflags)
    {
-      if (i+strlen(np->cflags) > 70) { fprintf(fpout, " \\\n   "); i = 3; }
+      if (i+6+strlen(np->cflags) > 70)
+         { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "  CFLAGS='%s'", np->cflags);
    }
    if (np->comp)
    {
-      if (i+strlen(np->comp) > 70) { fprintf(fpout, " \\\n   "); i = 3; }
+      if (i+4+strlen(np->comp) > 70)
+         { fprintf(fpout, " \\\n   "); i = 3; }
       i += fprintf(fpout, "  COMP='%s'", np->comp);
    }
    if (i)
       fprintf(fpout, "\n");
 }
 
-/* procedure 22 */
-void PrintR1Nodes(FILE *fpout, ATL_r1node_t *bp)
+/* procedure 28 */
+static ATL_r1node_t *KillR1NodesByFlag(int flag, int msk, ATL_r1node_t *bp)
+{
+   ATL_r1node_t *p=bp;
+   while(p)
+   {
+      ATL_r1node_t *next = p->next;
+      if ((p->flag & msk) != (flag & msk))
+        bp = KillR1NodeFromQ(bp, p);
+      p = next;
+   }
+   return(bp);
+}
+
+/* procedure 29 */
+static void PrintR1Nodes(FILE *fpout, ATL_r1node_t *bp)
 {
    while (bp)
    {
@@ -829,7 +1052,7 @@ void PrintR1Nodes(FILE *fpout, ATL_r1node_t *bp)
    }
 }
 
-/* procedure 23 */
+/* procedure 30 */
 static void WriteR1File(char *file, ATL_r1node_t *nq)
 {
    FILE *fpout;
@@ -848,7 +1071,7 @@ static void WriteR1File(char *file, ATL_r1node_t *nq)
       fclose(fpout);
 }
 
-/* procedure 24 */
+/* procedure 31 */
 static void WriteR1FileWithPath
    (char pre, char *path, char *file, ATL_r1node_t *nq)
 {
@@ -857,7 +1080,7 @@ static void WriteR1FileWithPath
    WriteR1File(ln, nq);
 }
 
-/* procedure 25 */
+/* procedure 32 */
 static ATL_r1node_t *ReadR1File(char *file)
 /*
  * Reads in a standard ATLAS parsable R1 index file, and returns a
@@ -878,7 +1101,7 @@ static ATL_r1node_t *ReadR1File(char *file)
    nq = p = GetR1Node();
    while (ln = GetJoinedLines(fpin))
    {
-      if (ln[0] != '#')
+      if (ln[0] != '#' && ln[0] != '\0')
       {
          p->next = ParseR1Line(ln);
          p = p->next;
@@ -888,7 +1111,7 @@ static ATL_r1node_t *ReadR1File(char *file)
    return(KillR1Node(nq));
 }
 
-/* procedure 26 */
+/* procedure 33 */
 static ATL_r1node_t *ReadR1FileWithPath
    (char pre, char *path, char *file)
 {
@@ -898,7 +1121,7 @@ static ATL_r1node_t *ReadR1FileWithPath
 }
 
 
-/* procedure 27 */
+/* procedure 34 */
 static ATL_r1node_t *DelRepeatedR1Kernels(ATL_r1node_t *bp)
 /*
  * Deletes any repeated IDs
@@ -923,7 +1146,7 @@ static ATL_r1node_t *DelRepeatedR1Kernels(ATL_r1node_t *bp)
    return(bp);
 }
 
-/* procedure 28 */
+/* procedure 35 */
 static ATL_r1node_t *DelBadArchR1Kernels(ATL_r1node_t *bp)
 /*
  * Weeds out kernels that require SSE/assembly that we haven't got
@@ -931,6 +1154,9 @@ static ATL_r1node_t *DelBadArchR1Kernels(ATL_r1node_t *bp)
 {
    int asmb=0, die;
    ATL_r1node_t *p, *prev;
+   #ifdef ATL_GAS_ARM64
+      asmb |= (1<<8);
+   #endif
    #ifdef ATL_GAS_ARM
       asmb |= (1<<7);
    #endif
@@ -986,7 +1212,7 @@ static ATL_r1node_t *DelBadArchR1Kernels(ATL_r1node_t *bp)
 }
 
 #define MAXBASES 4
-/* procedure 29 */
+/* procedure 36 */
 static int ATL_R1SplitContexts
 (
    ATL_r1node_t *kb,   /* pointer to all read in kernels */
@@ -1042,7 +1268,7 @@ static int ATL_R1SplitContexts
    return(nbases);
 }
 
-/* procedure 30 */
+/* procedure 37 */
 static ATL_r1node_t *ATL_R1LinkContexts
 (
    ATL_r1node_t *kp1, /* all out-of-cache kernels */
@@ -1075,7 +1301,7 @@ static ATL_r1node_t *ATL_R1LinkContexts
 }
 #undef MAXBASES
 
-/* procedure 31 */
+/* procedure 38 */
 static ATL_r1node_t *FindFastestR1Kernel
 (  char pre,             /* precision prefix */
    ATL_r1node_t *bp,  /* kernel queue */
@@ -1118,7 +1344,7 @@ static ATL_r1node_t *FindFastestR1Kernel
    return(kmax);
 }
 
-/* procedure 32 */
+/* procedure 39 */
 static int R1flag2size(int flag)
 /*
  * RETURNS: size of type using precision/type bits in flag
@@ -1131,7 +1357,7 @@ static int R1flag2size(int flag)
    return(size);
 }
 
-/* procedure 33 */
+/* procedure 40 */
 static char R1flag2pre(int flag)
 /*
  * RETURNS: correct precision/type prefix based on flag
@@ -1143,7 +1369,7 @@ static char R1flag2pre(int flag)
    return(FLAG_IS_SET(flag, R1F_COMPLEX) ? 'z' : 'd');
 }
 
-/* procedure 34 */
+/* procedure 41 */
 static int pre2R1flag(char pre, int flag)
 /*
  * RETURNS: flag modified to reflect type/precision indicated by pre
@@ -1154,7 +1380,7 @@ static int pre2R1flag(char pre, int flag)
    return(flag);
 }
 
-/* procedure 35 */
+/* procedure 42 */
 static void SetAllR1TypeFlags(char pre, ATL_r1node_t *bp)
 {
    ATL_r1node_t *p;
@@ -1162,7 +1388,7 @@ static void SetAllR1TypeFlags(char pre, ATL_r1node_t *bp)
       p->flag = pre2R1flag(pre, p->flag);
 }
 
-/* procedure 36 */
+/* procedure 43 */
 void PutKernNameInStr(ATL_r1node_t *r1B)
 /*
  * Fills in the proper name for all kernels in r1->str
