@@ -15,12 +15,14 @@
       #define ATL_NEON 1
    #endif
 #endif
-//#undef ATL_AVX
-//#undef ATL_SSE3
-//#undef ATL_SSE2
-//#undef ATL_SSE1
-//#define ATL_FRCGNUVEC 1
-//#define ATL_VLEN 32
+#if 0
+#undef ATL_AVX
+#undef ATL_SSE3
+#undef ATL_SSE2
+#undef ATL_SSE1
+#define ATL_FRCGNUVEC 1
+#define ATL_VLEN 32
+#endif
 /*
  * This header files contains wrappers to allow you to use SIMD vector
  * extensions in a very simplified way in a type-independent manner.
@@ -82,6 +84,11 @@
           ((defined(DREAL) || defined(DCPLX)) && ATL_VLEN != 2)
          #define ATL_FRCGNUVEC
       #endif
+   #elif defined(ATL_AVXZ)
+      #if ((defined(SREAL) || defined(SCPLX)) && ATL_VLEN != 16) || \
+          ((defined(DREAL) || defined(DCPLX)) && ATL_VLEN != 8)
+         #define ATL_FRCGNUVEC
+      #endif
    #elif defined(ATL_AVXMAC)
       #if ((defined(SREAL) || defined(SCPLX)) && ATL_VLEN != 8) || \
           ((defined(DREAL) || defined(DCPLX)) && ATL_VLEN != 4)
@@ -127,6 +134,12 @@
       #else
          #define ATL_VLEN 2
       #endif
+   #elif defined(ATL_AVXZ)
+      #if defined(SREAL) || defined(SCPLX)
+         #define ATL_VLEN 16
+      #else
+         #define ATL_VLEN 8
+      #endif
    #elif defined(ATL_AVXMAC)
       #if defined(SREAL) || defined(SCPLX)
          #define ATL_VLEN 8
@@ -244,18 +257,21 @@
    #ifdef ATL_SSE3
       #undef ATL_SSE3
    #endif
+   #ifdef ATL_AVXZ
+      #undef ATL_AVXZ
+   #endif
    #ifdef ATL_AVX
       #undef ATL_AVX
    #endif
    #ifdef ATL_AVXMAC
       #undef ATL_AVXMAC
    #endif
-   #ifdef ATL_VSXZ
-      #undef ATL_VSXZ
-   #endif
    #ifdef ATL_VSX
       #undef ATL_VSX
    #endif
+   #ifdef ATL_VXZ
+      #undef ATL_VXZ
+   #endif
    #if ATL_VLEN < 2 && defined(ATL_FRCGNUVEC)
        #undef ATL_FRCGNUVEC
    #endif
@@ -309,39 +325,72 @@
          d_ = vec_add(d_, t_) ; \
       }
       #if ATL_FULLGCCVSX
-         #define ATL_vvrsum4(s0_, s1_, s2_, s3_) \
-         {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
-            t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
-            s0_ = vec_vmrglw(s0_, s1_);          /* s1d,s0d,s1c,s0c}*/ \
-            s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
-            h_ = vec_vmrghw(s2_, s3_);           /*{s3b,s2b,s3a,s2a}*/\
-            s2_ = vec_vmrglw(s2_, s3_);          /*{s3d,s2d,s3c,s2c}*/ \
-            s2_ = ATL_vadd(s2_, s2_, h_);        /*{s3bd,s2bd,s3ac,s2ac}*/\
-            t_ =  vec_xxpermdi(s0_, s2_, 0);     /*{s3ac,s2ac,s1ac,s0ac}*/\
-            s0_ = vec_xxpermdi(s0_, s2_, 3);     /*{s3bd,s2bd,s1bd,s0bd}*/ \
-            ATL_vadd(s0_, s0_, t_); \
-            s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
-         }
-         #define ATL_vvrsum2(s0_, s1_) \
-         {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
-            t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
-            s0_ = vec_vmrglw(s0_, s1_);          /*{s1d,s0d,s1c,s0c}*/ \
-            s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
-            t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s1ac,s0ac,s1ac,s0ac}*/\
-            s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s1bd,s0bd,s1bd,s0bd}*/ \
-            ATL_vadd(s0_, s0_, t_); \
-            s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
-         }
-         #define ATL_vvrsum1(s0_) \
-         {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
-            t_ = vec_vmrghw(s0_, s0_);           /*{s0b,s0b,s0a,s0a}*/\
-            s0_ = vec_vmrglw(s0_, s0_);          /*{s0d,s0d,s0c,s0c}*/ \
-            s0_ = ATL_vadd(s0_, s0_, t_);        /*{s0bd,s0bd,s0ac,s0ac}*/\
-            t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s0ac,s0ac,s0ac,s0ac}*/\
-            s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s0bd,s0bd,s0bd,s0bd}*/ \
-            ATL_vadd(s0_, s0_, t_); \
-            s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
-         }
+         #ifdef ATL_ARCH_POWER8
+            #define ATL_vvrsum4(s0_, s1_, s2_, s3_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s1_);          /* s1d,s0d,s1c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
+               h_ = vec_vmrghw(s2_, s3_);           /*{s3b,s2b,s3a,s2a}*/\
+               s2_ = vec_vmrglw(s2_, s3_);          /*{s3d,s2d,s3c,s2c}*/ \
+               s2_ = ATL_vadd(s2_, s2_, h_);        /*{s3bd,s2bd,s3ac,s2ac}*/\
+               t_ =  vec_xxpermdi(s0_, s2_, 0);     /*{s3ac,s2ac,s1ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s2_, 3);     /*{s3bd,s2bd,s1bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+               s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
+            }
+            #define ATL_vvrsum2(s0_, s1_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s1_);          /*{s1d,s0d,s1c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
+               t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s1ac,s0ac,s1ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s1bd,s0bd,s1bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+               s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
+            }
+            #define ATL_vvrsum1(s0_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s0_);           /*{s0b,s0b,s0a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s0_);          /*{s0d,s0d,s0c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s0bd,s0bd,s0ac,s0ac}*/\
+               t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s0ac,s0ac,s0ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s0bd,s0bd,s0bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+               s0_ = vec_xxpermdi(s0_, s0_, 2);     /* pwr8 endian-insanity */ \
+            }
+         #else
+            #define ATL_vvrsum4(s0_, s1_, s2_, s3_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s1_);          /* s1d,s0d,s1c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
+               h_ = vec_vmrghw(s2_, s3_);           /*{s3b,s2b,s3a,s2a}*/\
+               s2_ = vec_vmrglw(s2_, s3_);          /*{s3d,s2d,s3c,s2c}*/ \
+               s2_ = ATL_vadd(s2_, s2_, h_);        /*{s3bd,s2bd,s3ac,s2ac}*/\
+               t_ =  vec_xxpermdi(s0_, s2_, 0);     /*{s3ac,s2ac,s1ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s2_, 3);     /*{s3bd,s2bd,s1bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+            #define ATL_vvrsum2(s0_, s1_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s1_);           /*{s1b,s0b,s1a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s1_);          /*{s1d,s0d,s1c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s1bd,s0bd,s1ac,s0ac}*/\
+               t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s1ac,s0ac,s1ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s1bd,s0bd,s1bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+            #define ATL_vvrsum1(s0_) \
+            {  ATL_VTYPE t_, h_;                    /*{s0d,s0c,s0b,s0a}*/\
+               t_ = vec_vmrghw(s0_, s0_);           /*{s0b,s0b,s0a,s0a}*/\
+               s0_ = vec_vmrglw(s0_, s0_);          /*{s0d,s0d,s0c,s0c}*/ \
+               s0_ = ATL_vadd(s0_, s0_, t_);        /*{s0bd,s0bd,s0ac,s0ac}*/\
+               t_ =  vec_xxpermdi(s0_, s0_, 0);     /*{s0ac,s0ac,s0ac,s0ac}*/\
+               s0_ = vec_xxpermdi(s0_, s0_, 3);     /*{s0bd,s0bd,s0bd,s0bd}*/ \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+         #endif
       #endif
       #define ATL_vsplat0(d_, s_) d_ = vec_splat(s_, 0)
       #define ATL_vsplat1(d_, s_) d_ = vec_splat(s_, 1)
@@ -354,18 +403,32 @@
          d_ = vec_add(d_, s_) ; \
       }
       #if ATL_FULLGCCVSX
-         #define ATL_vvrsum2(s0_, s1_) \
-         {  ATL_VTYPE t_;\
-            t_ =  vec_xxpermdi(s0_, s1_, 0); \
-            s0_ = vec_xxpermdi(s0_, s1_, 3); \
-            ATL_vadd(s0_, s0_, t_); \
-            s0_ = vec_xxpermdi(s0_, s0_, 2); /* pwr8 endian-insanity */ \
-         }
-         #define ATL_vvrsum1(s0_) \
-         {  ATL_VTYPE t_;\
-            t_ =  vec_xxpermdi(s0_, s0_, 0); \
-            ATL_vadd(s0_, s0_, t_); \
-         }
+         #ifdef ATL_ARCH_POWER8
+            #define ATL_vvrsum2(s0_, s1_) \
+            {  ATL_VTYPE t_;\
+               t_ =  vec_xxpermdi(s0_, s1_, 0); \
+               s0_ = vec_xxpermdi(s0_, s1_, 3); \
+               ATL_vadd(s0_, s0_, t_); \
+               s0_ = vec_xxpermdi(s0_, s0_, 2); /* pwr8 endian-insanity */ \
+            }
+            #define ATL_vvrsum1(s0_) \
+            {  ATL_VTYPE t_;\
+               t_ =  vec_xxpermdi(s0_, s0_, 0); \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+         #else
+            #define ATL_vvrsum2(s0_, s1_) \
+            {  ATL_VTYPE t_;\
+               t_ =  vec_xxpermdi(s0_, s1_, 0); \
+               s0_ = vec_xxpermdi(s0_, s1_, 3); \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+            #define ATL_vvrsum1(s0_) \
+            {  ATL_VTYPE t_;\
+               t_ = vec_splat(s0_, 1); \
+               ATL_vadd(s0_, s0_, t_); \
+            }
+         #endif
       #else
          #define ATL_vvrsum1(s0_) \
          {  ATL_VTYPE t_;\
@@ -519,13 +582,396 @@
    #endif
    #define ATL_vuld(v_, p_) ATL_vld(v_, p_)
    #define ATL_vust(p_, v_) ATL_vst(p_, v_)
+#elif defined(ATL_AVXZ) && \
+      ( (ATL_VLEN == 16 && (defined(SREAL) || defined(SCPLX))) || \
+        (ATL_VLEN ==  8 && (defined(DREAL) || defined(DCPLX))) )
+   #include <immintrin.h>
+   #if defined(SREAL) || defined(SCPLX)
+      #define ATL_VTYPE __m512
+      #define ATL_vzero(v_) v_ = _mm512_setzero_ps()
+      #define ATL_vbcast(v_, p_) v_ =  _mm512_set1_ps(*(p_))
+      #define ATL_vuld(v_, p_) v_ = _mm512_loadu_ps(p_)
+      #define ATL_vld(v_, p_) v_ = _mm512_load_ps(p_)
+      #define ATL_vust(p_, v_) _mm512_storeu_ps(p_, v_)
+      #define ATL_vst(p_, v_) _mm512_store_ps(p_, v_)
+      #define ATL_vadd(d_, s1_, s2_) d_ =  _mm512_add_ps(s1_, s2_)
+      #define ATL_vsub(d_, s1_, s2_) d_ =  _mm512_sub_ps(s1_, s2_)
+      #define ATL_vmul(d_, s1_, s2_) d_ =  _mm512_mul_ps(s1_, s2_)
+      #ifdef ATL_AVXMAC
+         #define ATL_vmac(d_, s1_, s2_) \
+            d_ = _mm512_fmadd_ps(s1_, s2_, d_)
+      #else
+         #define ATL_vmac(d_, s1_, s2_) \
+         { ATL_VTYPE t_; \
+            t_ = _mm512_mul_ps(s1_, s2_); \
+            d_ = _mm512_add_ps(t_, d_); \
+         }
+      #endif
+      #define ATL_vvrsum8_256(s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_) \
+      { \
+         s0_ = _mm256_hadd_ps(s0_, s1_); \
+            /*{s1gh,s1ef,s0gh,s0ef,s1cd,s1ab,s0cd,s0ab}*/\
+         s2_ = _mm256_hadd_ps(s2_, s3_); \
+            /*{s3gh,s3ef,s2gh,s2ef,s3cd,s3ab,s2cd,s2ab}*/\
+         s4_ = _mm256_hadd_ps(s4_, s5_); \
+            /*{s5gh,s5ef,s4gh,s4ef,s5cd,s5ab,s4cd,s4ab}*/\
+         s6_ = _mm256_hadd_ps(s6_, s7_); \
+            /*{s7gh,s7ef,s6gh,s6ef,s7cd,s7ab,s6cd,s6ab}*/\
+         s0_ = _mm256_hadd_ps(s0_, s2_); \
+            /*{s3e-h,s2e-h,s1e-h,s0e-g,s3a-d,s2a-d,s1a-d,s0a-d}*/\
+         s4_ = _mm256_hadd_ps(s4_, s6_); \
+            /*{s7e-h,s6e-h,s5e-h,s4e-g,s7a-d,s6a-d,s5a-d,s4a-d}*/\
+         s1_ = _mm256_permute2f128_ps(s0_, s4_, 0x31); \
+            /*{s7e-h,s6e-h,s5e-h,s4e-g,s3e-h,s2e-h,s1e-h,s0e-g}*/\
+         s0_ = _mm256_permute2f128_ps(s0_, s4_, 0x20); \
+            /*{s7a-d,s6a-d,s5a-d,s4a-d,s3a-d,s2a-d,s1a-d,s0a-d}*/\
+         s0_ = _mm256_add_ps(s0_, s1_); \
+      }
+      #define ATL_vvrsum4_256(s0_, s1_, s2_, s3_) \
+      { \
+         s0_ = _mm256_hadd_ps(s0_, s1_); \
+            /*{s1gh,s1ef,s0gh,s0ef,s1cd,s1ab,s0cd,s0ab}*/\
+         s2_ = _mm256_hadd_ps(s2_, s3_); \
+            /*{s3gh,s3ef,s2gh,s2ef,s3cd,s3ab,s2cd,s2ab}*/\
+         s0_ = _mm256_hadd_ps(s0_, s2_); \
+            /*{s3e-h,s2e-h,s1e-h,s0e-g,s3a-d,s2a-d,s1a-d,s0a-d}*/\
+         s1_ = _mm256_permute2f128_ps(s0_, s0_, 0x31); \
+            /*{s3e-h,s2e-h,s1e-h,s0e-g,s3e-h,s2e-h,s1e-h,s0e-g}*/\
+         s0_ = _mm256_permute2f128_ps(s0_, s0_, 0x20); \
+            /*{s3a-d,s2a-d,s1a-d,s0a-d,s3a-d,s2a-d,s1a-d,s0a-d}*/\
+         s0_ = _mm256_add_ps(s0_, s1_); \
+      }
+      #define ATL_vvrsum2_256(s0_, s1_) \
+      { \
+         s0_ = _mm256_hadd_ps(s0_, s1_); \
+            /*{s1gh,s1ef,s0gh,s0ef,s1cd,s1ab,s0cd,s0ab}*/\
+         s0_ = _mm256_hadd_ps(s0_, s0_); \
+            /*{s1e-h,s0e-h,s1e-h,s0e-g,s1a-d,s0a-d,s1a-d,s0a-d}*/\
+         s1_ = _mm256_permute2f128_ps(s0_, s0_, 0x31); \
+            /*{s1e-h,s0e-h,s1e-h,s0e-g,s1e-h,s0e-h,s1e-h,s0e-g}*/\
+         s0_ = _mm256_permute2f128_ps(s0_, s0_, 0x20); \
+            /*{s1a-d,s0a-d,s1a-d,s0a-d,s1a-d,s0a-d,s1a-d,s0a-d}*/\
+         s0_ = _mm256_add_ps(s0_, s1_); \
+      }
+      #define ATL_vrsum1(d_, s_) \
+      { __m256 t0_, t1_; __m128 x0_, x1_; \
+         t0_ = _mm512_extractf32x8_ps(s_, 0); \
+         t1_ = _mm512_extractf32x8_ps(s_, 1); \
+         t0_ = _mm256_add_ps(t0_, t1_); \
+         x0_ = _mm256_extractf128_ps(t0_, 0); \
+         x1_ = _mm256_extractf128_ps(t0_, 1); \
+         x0_ = _mm_add_ps(x0_, x1_); \
+         x0_ = _mm_hadd_ps(x0_, x0_); /* {X,X,x0dc,x0ab} */ \
+         x0_ = _mm_hadd_ps(x0_, x0_); /* {X,X,X,x0abcd} */ \
+         d_ = x0_[0]; \
+      }
+      #define ATL_vvrsum1(s0_) \
+      { __m256 t0_, t1_; __m128 x0_, x1_; \
+         t0_ = _mm512_extractf32x8_ps(s0_, 0); \
+         t1_ = _mm512_extractf32x8_ps(s0_, 1); \
+         t0_ = _mm256_add_ps(t0_, t1_); \
+         x0_ = _mm256_extractf128_ps(t0_, 0); \
+         x1_ = _mm256_extractf128_ps(t0_, 1); \
+         x0_ = _mm_add_ps(x0_, x1_); \
+         x0_ = _mm_hadd_ps(x0_, x0_); /* {X,X,x0dc,x0ab} */ \
+         x0_ = _mm_hadd_ps(x0_, x0_); /* {X,X,X,x0abcd} */ \
+         s0_ = _mm512_insertf32x4(s0_, x0_, 0); \
+      }
+      #define ATL_vvrsum2(s0_, s1_) \
+      { __m256 t0_, t1_, t2_, t3_; \
+         t0_ = _mm512_extractf32x8_ps(s0_, 0); \
+         t1_ = _mm512_extractf32x8_ps(s0_, 1); \
+         t0_ = _mm256_add_ps(t0_, t1_); \
+         t2_ = _mm512_extractf32x8_ps(s1_, 0); \
+         t1_ = _mm512_extractf32x8_ps(s1_, 1); \
+         t2_ = _mm256_add_ps(t2_, t1_); \
+         ATL_vvrsum2_256(t0_, t2_); \
+         s0_ = _mm512_insertf32x8(s0_, t0_, 0); \
+      }
+      #define ATL_vvrsum4(s0_, s1_, s2_, s3_) \
+      { __m256 t0_, t1_, t2_, t3_, t4_, t5_, t6_, t7_; \
+         t0_ = _mm512_extractf32x8_ps(s0_, 0); \
+         t4_ = _mm512_extractf32x8_ps(s0_, 1); \
+         t0_ = _mm256_add_ps(t0_, t4_); \
+         t1_ = _mm512_extractf32x8_ps(s1_, 0); \
+         t4_ = _mm512_extractf32x8_ps(s1_, 1); \
+         t1_ = _mm256_add_ps(t1_, t4_); \
+         t2_ = _mm512_extractf32x8_ps(s2_, 0); \
+         t4_ = _mm512_extractf32x8_ps(s2_, 1); \
+         t2_ = _mm256_add_ps(t2_, t4_); \
+         t3_ = _mm512_extractf32x8_ps(s3_, 0); \
+         t4_ = _mm512_extractf32x8_ps(s3_, 1); \
+         t3_ = _mm256_add_ps(t3_, t4_); \
+         ATL_vvrsum4_256(t0_, t1_, t2_, t3_); \
+         s0_ = _mm512_insertf32x8(s0_, t0_, 0); \
+      }
+      #define ATL_vvrsum8_512_256(t0_, s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_) \
+      { __m256 t1_, t2_, t3_, t4_, t5_, t6_, t7_, t8_; \
+         t0_ = _mm512_extractf32x8_ps(s0_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s0_, 1); \
+         t0_ = _mm256_add_ps(t0_, t8_); \
+         t1_ = _mm512_extractf32x8_ps(s1_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s1_, 1); \
+         t1_ = _mm256_add_ps(t1_, t8_); \
+         t2_ = _mm512_extractf32x8_ps(s2_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s2_, 1); \
+         t2_ = _mm256_add_ps(t2_, t8_); \
+         t3_ = _mm512_extractf32x8_ps(s3_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s3_, 1); \
+         t3_ = _mm256_add_ps(t3_, t8_); \
+         t4_ = _mm512_extractf32x8_ps(s4_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s4_, 1); \
+         t4_ = _mm256_add_ps(t4_, t8_); \
+         t5_ = _mm512_extractf32x8_ps(s5_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s5_, 1); \
+         t5_ = _mm256_add_ps(t5_, t8_); \
+         t6_ = _mm512_extractf32x8_ps(s6_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s6_, 1); \
+         t6_ = _mm256_add_ps(t6_, t8_); \
+         t7_ = _mm512_extractf32x8_ps(s7_, 0); \
+         t8_ = _mm512_extractf32x8_ps(s7_, 1); \
+         t7_ = _mm256_add_ps(t7_, t8_); \
+         ATL_vvrsum8_256(t0_, t1_, t2_, t3_, t4_, t5_, t6_, t7_); \
+      }
+      #define ATL_vvrsum8(s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_) \
+      { __m256 t0_;\
+         ATL_vvrsum8_512_256(t0_, s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_); \
+         s0_ = _mm512_insertf32x8(s0_, t0_, 0); \
+      }
+      #define ATL_vvrsum16(s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_, \
+                           s8_, s9_, sA_, sB_, sC_, sD_, sE_, sF_) \
+      { __m256 t0_;\
+         ATL_vvrsum8_512_256(t0_, s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_); \
+         s0_ = _mm512_insertf32x8(s0_, t0_, 0); \
+         ATL_vvrsum8_512_256(t0_, s8_, s9_, sA_, sB_, sC_, sD_, sE_, sF_); \
+         s0_ = _mm512_insertf32x8(s0_, t0_, 1); \
+      }
+      #define ATL_vsplat0(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,0)); \
+      }
+      #define ATL_vsplat1(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0x55); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,0)); \
+      }
+      #define ATL_vsplat2(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xAA); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,0)); \
+      }
+      #define ATL_vsplat3(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xFF); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,0)); \
+      }
+      #define ATL_vsplat4(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,1)); \
+      }
+      #define ATL_vsplat5(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0x55); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,1)); \
+      }
+      #define ATL_vsplat6(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xAA); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,1)); \
+      }
+      #define ATL_vsplat7(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xFF); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,1)); \
+      }
+      #define ATL_vsplat8(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,2)); \
+      }
+      #define ATL_vsplat9(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0x55); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,2)); \
+      }
+      #define ATL_vsplat10(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xAA); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,2)); \
+      }
+      #define ATL_vsplat11(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xFF); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,2)); \
+      }
+      #define ATL_vsplat12(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,3)); \
+      }
+      #define ATL_vsplat13(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0x55); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,3)); \
+      }
+      #define ATL_vsplat14(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xAA); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,3)); \
+      }
+      #define ATL_vsplat15(d_, s_) \
+      { \
+         d_ = _mm512_shuffle_ps(s_, s_, 0xFF); \
+         d_ = _mm512_broadcast_f32x4(_mm512_extractf32x4_ps(d_,3)); \
+      }
+   #else        /* double precision */
+      #if ATL_VLEN == 8
+         #define ATL_VTYPE __m512d
+      #else
+         #error "AVX SUPPORTS ONLY VLEN=8 FOR DOUBLE!"
+      #endif
+      #define ATL_vzero(v_) v_ = _mm512_setzero_pd()
+      #define ATL_vbcast(v_, p_) v_ =  _mm512_set1_pd(*(p_))
+      #define ATL_vuld(v_, p_) v_ = _mm512_loadu_pd(p_)
+      #define ATL_vld(v_, p_) v_ = _mm512_load_pd(p_)
+      #define ATL_vust(p_, v_) _mm512_storeu_pd(p_, v_)
+      #define ATL_vst(p_, v_) _mm512_store_pd(p_, v_)
+      #define ATL_vadd(d_, s1_, s2_) d_ =  _mm512_add_pd(s1_, s2_)
+      #define ATL_vsub(d_, s1_, s2_) d_ =  _mm512_sub_pd(s1_, s2_)
+      #define ATL_vmul(d_, s1_, s2_) d_ =  _mm512_mul_pd(s1_, s2_)
+      #ifdef ATL_AVXMAC
+         #define ATL_vmac(d_, s1_, s2_) \
+            d_ = _mm512_fmadd_pd(s1_, s2_, d_)
+      #else
+         #define ATL_vmac(d_, s1_, s2_) \
+         { ATL_VTYPE t_; \
+            t_ = _mm512_mul_pd(s1_, s2_); \
+            d_ = _mm512_add_pd(t_, d_); \
+         }
+      #endif
+      #define ATL_vvrsum4_256(s0_, s1_, s2_, s3_) \
+      { \
+         s0_ = _mm256_hadd_pd(s0_, s1_); /*{s1cd,s0cd,s1ab,s0ab}*/ \
+         s2_ = _mm256_hadd_pd(s2_, s3_); /*{s3cd,s2cd,s3ab,s2ab}*/ \
+         s1_ = _mm256_permute2f128_pd(s0_, s2_,0x31);/*{s3cd,s2cd,s1cd,s0cd}*/ \
+         s0_ = _mm256_permute2f128_pd(s0_, s2_,0x20);/*{s3ab,s2ab,s1ab,s0ab}*/ \
+         s0_ = _mm256_add_pd(s0_, s1_); \
+      }
+      #define ATL_vvrsum2_256(s0_, s1_) \
+      { \
+         s0_ = _mm256_hadd_pd(s0_, s1_); /*{s1cd,s0cd,s1ab,s0ab}*/ \
+         s1_ = _mm256_permute2f128_pd(s0_, s1_,0x31);/*{s3cd,s2cd,s1cd,s0cd}*/ \
+         s0_ = _mm256_add_pd(s0_, s1_); \
+      }
+      #define ATL_vvrsum4_512_256(t0_, s0_, s1_, s2_, s3_) \
+      {  __m256d tt_, t1_, t2_, t3_; \
+         t0_ = _mm512_extractf64x4_pd(s0_, 0); \
+         tt_ = _mm512_extractf64x4_pd(s0_, 1); \
+         t0_ = _mm256_add_pd(t0_, tt_); \
+         t1_ = _mm512_extractf64x4_pd(s1_, 0); \
+         tt_ = _mm512_extractf64x4_pd(s1_, 1); \
+         t1_ = _mm256_add_pd(t1_, tt_); \
+         t2_ = _mm512_extractf64x4_pd(s2_, 0); \
+         tt_ = _mm512_extractf64x4_pd(s2_, 1); \
+         t2_ = _mm256_add_pd(t2_, tt_); \
+         t3_ = _mm512_extractf64x4_pd(s3_, 0); \
+         tt_ = _mm512_extractf64x4_pd(s3_, 1); \
+         t3_ = _mm256_add_pd(t3_, tt_); \
+         ATL_vvrsum4_256(t0_, t1_, t2_, t3_); \
+      }
+      #define ATL_vvrsum4(s0_, s1_, s2_, s3_) \
+      { \
+         __m256d t0_; \
+         ATL_vvrsum4_512_256(t0_, s0_, s1_, s2_, s3_); \
+         s0_ = _mm512_insertf64x4(s0_, t0_, 0); \
+      }
+      #define ATL_vvrsum8(s0_, s1_, s2_, s3_, s4_, s5_, s6_, s7_) \
+      {  __m256d t0_; \
+         ATL_vvrsum4_512_256(t0_, s0_, s1_, s2_, s3_); \
+         s0_ = _mm512_insertf64x4(s0_, t0_, 0); \
+         ATL_vvrsum4_512_256(t0_, s4_, s5_, s6_, s7_); \
+         s0_ = _mm512_insertf64x4(s0_, t0_, 1); \
+      }
+      #define ATL_vvrsum2(s0_, s1_) \
+      {  __m256d t0_, t1_, t2_; \
+         t0_ = _mm512_extractf64x4_pd(s0_, 0); \
+         t1_ = _mm512_extractf64x4_pd(s0_, 1); \
+         t0_ = _mm256_add_pd(t0_, t1_); \
+         t2_ = _mm512_extractf64x4_pd(s1_, 0); \
+         t1_ = _mm512_extractf64x4_pd(s1_, 1); \
+         t2_ = _mm256_add_pd(t2_, t1_); \
+         ATL_vvrsum2_256(t0_, t2_); \
+         s0_ = _mm512_insertf64x4(s0_, t0_, 0); \
+      }
+      #define ATL_vrsum1(d_, s_) \
+      { __m256d t0_, t1_; __m128d x0_, x1_; \
+         t0_ = _mm512_extractf64x4_pd(s_, 0); \
+         t1_ = _mm512_extractf64x4_pd(s_, 1); \
+         t0_ = _mm256_add_pd(t0_, t1_); \
+         x0_ = _mm256_extractf128_pd(t0_, 0); \
+         x1_ = _mm256_extractf128_pd(t0_, 1); \
+         x0_ = _mm_add_pd(x0_, x1_); \
+         x0_ = _mm_hadd_pd(x0_, x0_); \
+         d_ = x0_[0];  \
+      }
+      #define ATL_vvrsum1(s0_) \
+      { __m256d t0_, t1_; __m128d x0_, x1_; \
+         t0_ = _mm512_extractf64x4_pd(s0_, 0); \
+         t1_ = _mm512_extractf64x4_pd(s0_, 1); \
+         t0_ = _mm256_add_pd(t0_, t1_); \
+         x0_ = _mm256_extractf128_pd(t0_, 0); \
+         x1_ = _mm256_extractf128_pd(t0_, 1); \
+         x0_ = _mm_add_pd(x0_, x1_); \
+         x0_ = _mm_hadd_pd(x0_, x0_); \
+         s0_ = _mm512_insertf64x2(s0_, x0_, 0); \
+      }
+      #define ATL_vsplat0(d_, s_) \
+      { \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(s_, 0), 0)); \
+      }
+      #define ATL_vsplat2(d_, s_) \
+      { \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(s_, 0), 1)); \
+      }
+      #define ATL_vsplat1(d_, s_) \
+      { \
+         d_ = _mm512_unpackhi_pd(s_, s_); \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(d_, 0), 0)); \
+      }
+      #define ATL_vsplat3(d_, s_) \
+      { \
+         d_ = _mm512_unpackhi_pd(s_, s_); \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(d_, 0), 1)); \
+      }
+      #define ATL_vsplat4(d_, s_) \
+      { \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(s_, 1), 0)); \
+      }
+      #define ATL_vsplat6(d_, s_) \
+      { \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(s_, 1), 1)); \
+      }
+      #define ATL_vsplat5(d_, s_) \
+      { \
+         d_ = _mm512_unpackhi_pd(s_, s_); \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(d_, 1), 0)); \
+      }
+      #define ATL_vsplat7(d_, s_) \
+      { \
+         d_ = _mm512_unpackhi_pd(s_, s_); \
+         d_ = _mm512_broadcastsd_pd(_mm256_extractf128_pd(_mm512_extractf64x4_pd(d_, 1), 1)); \
+      }
+   #endif
 #elif defined(ATL_AVXMAC) || defined(ATL_AVX)
    #include <immintrin.h>
    #if defined(SREAL) || defined(SCPLX)
-      #if ATL_VLEN != 8
-         #error "VLEN != 8 not supported for AVX or AVX2!"
+      #if ATL_VLEN == 8
+         #define ATL_VTYPE __m256
+      #else
+         #error "VLEN != 8 not supported for AVX/AVX2!"
       #endif
-      #define ATL_VTYPE __m256
       #define ATL_vzero(v_) v_ = _mm256_setzero_ps()
       #define ATL_vbcast(v_, p_) v_ =  _mm256_broadcast_ss(p_)
       #define ATL_vuld(v_, p_) v_ = _mm256_loadu_ps(p_)
@@ -644,10 +1090,15 @@
          d_ = _mm256_insertf128_ps(d_, _mm256_extractf128_ps(d_,1), 0); \
       }
    #else        /* double precision */
-      #if ATL_VLEN != 4
-         #error "AVX SUPPORTS ONLY VLEN=4 FOR DOUBLE!"
+      #if ATL_VLEN == 4
+         #define ATL_VTYPE __m256d
+      #elif ATL_VLEN == 8
+         #define ATL_VTYPE __m512d
+      #elif ATL_VLEN == 2
+         #define ATL_VTYPE __m128d
+      #else
+         #error "AVX SUPPORTS ONLY VLEN=[2,4,8] FOR DOUBLE!"
       #endif
-      #define ATL_VTYPE __m256d
       #define ATL_vzero(v_) v_ = _mm256_setzero_pd()
       #define ATL_vbcast(v_, p_) v_ =  _mm256_broadcast_sd(p_)
       #define ATL_vuld(v_, p_) v_ = _mm256_loadu_pd(p_)
