@@ -1,39 +1,14 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  * Code contributers : R. Clint Whaley, Antoine P. Petitet
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 
 #ifndef ATLAS_TST_H
    #define ATLAS_TST_H
 
 #include "atlas_enum.h"
+#include "atlas_amm.h"
 
 double time00();
 #ifndef UseCRand
@@ -49,6 +24,18 @@ double time00();
    #define dumb_rand() ( 0.5 - ((double)rand())/((double)RAND_MAX) )
 #endif
 
+int *ATL_getIpiv(ATL_UINT N, ATL_UINT seed);
+void ATL_ipivinit(ATL_UINT N, int *ipiv);
+void ATL_print2dBV(int M, int N, void *bv);
+void ATL_print1dBV(int COL, int N, void *bv);
+void Mjoin(PATL,ipprint)(FILE *fp, ipinfo_t *ip, int flg, char *nm,
+                         ATL_iptr_t M, ATL_iptr_t N, const TYPE *w);
+void *ATL_sipsycmpBV(int verb, double tol, ipinfo_t *ip, int flg, const TYPE *a,
+                    ATL_iptr_t N, const TYPE *A);
+void *ATL_sgecmpBV(int,double,int,int,const float*,int,const float*,int);
+void *ATL_strcmpBV
+   (int, double, const enum ATLAS_UPLO, int, int, const float*, int,
+    const float*, int);
 void ATL_ststsqtran(const int N, float *A, const int lda);
 void ATL_sgeprint
    (char *mat, const int M, const int N, const float *A, const int lda);
@@ -100,6 +87,14 @@ void ATL_svdiff(const int N, const float *X, const int incX,
                 const float *Y, const int incY, float *Z, const int incZ);
 void ATL_sgediff(const int M, const int N, const float *A, const int lda,
                  const float *B, const int ldb, float *C, const int ldc);
+void Mjoin(PATL,ipprint)(FILE *fp, ipinfo_t *ip, int flg, char *nm,
+                         ATL_iptr_t M, ATL_iptr_t N, const TYPE *w);
+void *ATL_dipsycmpBV(int verb, double tol, ipinfo_t *ip, int flg, const TYPE *a,
+                    ATL_iptr_t N, const TYPE *A);
+void *ATL_dgecmpBV(int,double,int,int,const double*,int,const double*,int);
+void *ATL_dtrcmpBV
+   (int, double, const enum ATLAS_UPLO, int, int, const double*, int,
+    const double*, int);
 void ATL_dtstsqtran(const int N, double *A, const int lda);
 void ATL_dgeprint
    (char *mat, const int M, const int N, const double *A, const int lda);
@@ -151,6 +146,14 @@ void ATL_dvdiff(const int N, const double *X, const int incX,
                 const double *Y, const int incY, double *Z, const int incZ);
 void ATL_dgediff(const int M, const int N, const double *A, const int lda,
                  const double *B, const int ldb, double *C, const int ldc);
+void Mjoin(PATL,ipprint)(FILE *fp, ipinfo_t *ip, int flg, char *nm,
+                         ATL_iptr_t M, ATL_iptr_t N, const TYPE *w);
+void *ATL_cipsycmpBV(int verb, double tol, ipinfo_t *ip, int flg, const TYPE *a,
+                    ATL_iptr_t N, const TYPE *A);
+void *ATL_cgecmpBV(int,double,int,int,const float*,int,const float*,int);
+void *ATL_ctrcmpBV
+   (int, double, const enum ATLAS_UPLO, int, int, const float*, int,
+    const float*, int);
 void ATL_ctstsqtran(const int N, float *A, const int lda);
 void ATL_cgeprint
    (char *mat, const int M, const int N, const float *A, const int lda);
@@ -202,6 +205,14 @@ void ATL_cvdiff(const int N, const float *X, const int incX,
                 const float *Y, const int incY, float *Z, const int incZ);
 void ATL_cgediff(const int M, const int N, const float *A, const int lda,
                  const float *B, const int ldb, float *C, const int ldc);
+void Mjoin(PATL,ipprint)(FILE *fp, ipinfo_t *ip, int flg, char *nm,
+                         ATL_iptr_t M, ATL_iptr_t N, const TYPE *w);
+void *ATL_zipsycmpBV(int verb, double tol, ipinfo_t *ip, int flg, const TYPE *a,
+                    ATL_iptr_t N, const TYPE *A);
+void *ATL_zgecmpBV(int,double,int,int,const double*,int,const double*,int);
+void *ATL_ztrcmpBV
+   (int, double, const enum ATLAS_UPLO, int, int, const double*, int,
+    const double*, int);
 void ATL_ztstsqtran(const int N, double *A, const int lda);
 void ATL_zgeprint
    (char *mat, const int M, const int N, const double *A, const int lda);
@@ -259,7 +270,7 @@ void ATL_zgediff(const int M, const int N, const double *A, const int lda,
  */
 int ATL_sf77getri
    (const enum ATLAS_ORDER, const int, float*, const int, int*,
-    float*, int*);
+    float*, int);
 int ATL_sf77getrf
    (const enum ATLAS_ORDER, const int, const int, float*, const int, int*);
 int ATL_sf77potrf(const enum ATLAS_UPLO, const int, float*, const int);
@@ -271,7 +282,7 @@ int ATL_sf77gesv(const int, const int, float*, const int, int*, float*, const in
 int ATL_sf77gels(const enum ATLAS_TRANS, const int, const int, const int, float*, const int, float*, const int);
 int ATL_df77getri
    (const enum ATLAS_ORDER, const int, double*, const int, int*,
-    double*, int*);
+    double*, int);
 int ATL_df77getrf
    (const enum ATLAS_ORDER, const int, const int, double*, const int, int*);
 int ATL_df77potrf(const enum ATLAS_UPLO, const int, double*, const int);
@@ -283,7 +294,7 @@ int ATL_df77gesv(const int, const int, double*, const int, int*, double*, const
 int ATL_df77gels(const enum ATLAS_TRANS, const int, const int, const int, double*, const int, double*, const int);
 int ATL_cf77getri
    (const enum ATLAS_ORDER, const int, float*, const int, int*,
-    float*, int*);
+    float*, int);
 int ATL_cf77getrf
    (const enum ATLAS_ORDER, const int, const int, float*, const int, int*);
 int ATL_cf77potrf(const enum ATLAS_UPLO, const int, float*, const int);
@@ -295,7 +306,7 @@ int ATL_cf77gesv(const int, const int, float*, const int, int*, float*, const in
 int ATL_cf77gels(const enum ATLAS_TRANS, const int, const int, const int, float*, const int, float*, const int);
 int ATL_zf77getri
    (const enum ATLAS_ORDER, const int, double*, const int, int*,
-    double*, int*);
+    double*, int);
 int ATL_zf77getrf
    (const enum ATLAS_ORDER, const int, const int, double*, const int, int*);
 int ATL_zf77potrf(const enum ATLAS_UPLO, const int, double*, const int);
