@@ -1,36 +1,12 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 #include "atlas_misc.h"
 #ifdef ATL_USEPTHREADS
    #include "atlas_ptalias3.h"
 #endif
+#include "atlas_level3.h"
 #include "cblas.h"
 
 #include "atlas_level1.h"
@@ -124,8 +100,8 @@ void cblas_cgemm(const enum CBLAS_ORDER Order,
                                 "TransB must be %d, %d or %d, but is set to %d",
                                 CblasNoTrans, CblasTrans, CblasConjTrans, TB);
          if ( (ldb < N) || (ldb < 1) )
-            info = cblas_errprn(11,info, "ldb must be >= MAX(K,1): ldb=%d K=%d",
-                                ldb, K);
+            info = cblas_errprn(11,info, "ldb must be >= MAX(N,1): ldb=%d N=%d",
+                                ldb, N);
       }
       if ( (ldc < M) || (ldc < 1) )
          info = cblas_errprn(14, info,"ldc must be >= MAX(M,1): ldc=%d M=%d",
