@@ -1,35 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.3
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2010 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 #include "atlas_misc.h"
 #include "atlas_lvl2.h"
 #include "atlas_lvl3.h"
+#include "atlas_cache.h"
 #if defined(ATL_INL1)
    #include Mstr(Mjoin(Mjoin(atlas_,PRE),r1_L1.h))
 #elif defined(ATL_INL2)
@@ -45,12 +21,12 @@
  */
    #ifdef CacheEdge
       #if CacheEdge > 4194304 || CacheEdge == 0
-         #define MY_CE (4*ATL_MulBySize(ATL_L1elts))
+         #define MY_CE (4*ATL_MulBySize(L1C_ELTS))
       #else
          #define MY_CE CacheEdge
       #endif
    #else
-      #define MY_CE (4*ATL_MulBySize(ATL_L1elts))
+      #define MY_CE (4*ATL_MulBySize(L1C_ELTS))
    #endif
 #endif
 
@@ -101,7 +77,7 @@ void ATL_ger
 #ifdef ATL_CHOICE
    const size_t opsize = (M*N+M+N)*sizeof(TYPE)SHIFT;
 
-   if (opsize <= ATL_MulBySize(ATL_L1elts))
+   if (opsize <= ATL_MulBySize(L1C_ELTS))
       Mjoin(MY_GER,_L1)(M, N, alpha0, X, incX, Y, incY, A, lda);
    else if (opsize <= MY_CE)
       Mjoin(MY_GER,_L2)(M, N, alpha0, X, incX, Y, incY, A, lda);
