@@ -1,37 +1,15 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #include "atlas_prefetch.h"
 #ifdef Left_
-static void trsmLU_2(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLU_2(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left, 'Upper', with 1 col prefetch, written with all dependencies shown,
  * so that compiler can optimize.
@@ -43,9 +21,9 @@ static void trsmLU_2(const int N, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar22=A[6], ai22=A[7];
    TYPE xr1, xi1, xr2, xi2;
    TYPE t0, p0;
-   const int ldb2=ldb+ldb;
+   ATL_CSZT ldb2=ldb+ldb;
    TYPE *bn=B+ldb2;
-   const int pfd=ldb2+ldb2;
+   ATL_CSZT  pfd=ldb2+ldb2;
    int j;
 
    p0 = B[2];
@@ -85,7 +63,7 @@ static void trsmLU_2(const int N, const TYPE *A, TYPE *B, const int ldb)
    *B   = xr1; B[1] = xi1;
    B[2] = xr2; B[3] = xi2;
 }
-static void trsmLU_3(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLU_3(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left, 'Upper', with 1 col prefetch, written with all dependencies shown,
  * so that compiler can optimize.
@@ -98,9 +76,9 @@ static void trsmLU_3(const int N, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar33=A[16], ai33=A[17];
    TYPE xr1, xi1, xr2, xi2, xr3, xi3;
    TYPE t0, p0;
-   const int ldb2=ldb+ldb;
+   ATL_CSZT ldb2=ldb+ldb;
    TYPE *bn=B+ldb2;
-   const int pfd=ldb2+ldb2;
+   ATL_CSZT  pfd=ldb2+ldb2;
    int j;
 
    p0 = B[4];
@@ -160,7 +138,7 @@ static void trsmLU_3(const int N, const TYPE *A, TYPE *B, const int ldb)
    B[2] = xr2; B[3] = xi2;
    B[4] = xr3; B[5] = xi3;
 }
-static void trsmLU_4(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLU_4(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left, 'Upper', with 1 col prefetch, written with all dependencies shown,
  * so that compiler can optimize.
@@ -176,9 +154,9 @@ static void trsmLU_4(const int N, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar44=A[30], ai44=A[31];
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, xr4, xi4;
    TYPE t0, p0;
-   const int ldb2=ldb+ldb;
+   ATL_CSZT ldb2=ldb+ldb;
    TYPE *bn=B+ldb2;
-   const int pfd = ldb2+ldb2;
+   ATL_CSZT  pfd = ldb2+ldb2;
    int j;
 
    p0 = B[6];
@@ -262,7 +240,7 @@ static void trsmLU_4(const int N, const TYPE *A, TYPE *B, const int ldb)
    B[4] = xr3; B[5] = xi3;
    B[6] = xr4; B[7] = xi4;
 }
-static void trsmLL_2(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLL_2(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left', 'Lower', with 1 column prefetch, written with all dependencies
  * shown, so that the compiler can optimize.
@@ -272,11 +250,11 @@ static void trsmLL_2(const int N, const TYPE *A, TYPE *B, const int ldb)
 {
    const TYPE ar11=*A, ai11=A[1], ar21=A[2], ai21=A[3];
    const TYPE ar22=A[6], ai22=A[7];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2;
    TYPE t0, p0;
    TYPE *pBn=B+ldb2;
-   const int pfd=ldb2+ldb2;
+   ATL_CSZT  pfd=ldb2+ldb2;
    int j;
 
    p0 = *B;
@@ -316,7 +294,7 @@ static void trsmLL_2(const int N, const TYPE *A, TYPE *B, const int ldb)
    *B   = xr1; B[1] = xi1;
    B[2] = xr2; B[3] = xi2;
 }
-static void trsmLL_3(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLL_3(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left', 'Lower', with 1 column prefetch, written with all dependencies
  * shown, so that the compiler can optimize.
@@ -327,11 +305,11 @@ static void trsmLL_3(const int N, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar11=*A, ai11=A[1], ar21=A[2], ai21=A[3], ar31=A[4], ai31=A[5];
    const TYPE ar22=A[ 8], ai22=A[ 9], ar32=A[10], ai32=A[11];
    const TYPE ar33=A[16], ai33=A[17];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3;
    TYPE t0, p0;
    TYPE *pBn=B+ldb2;
-   const int pfd=ldb2+ldb2;
+   ATL_CSZT  pfd=ldb2+ldb2;
    int j;
 
    p0 = *B;
@@ -391,7 +369,7 @@ static void trsmLL_3(const int N, const TYPE *A, TYPE *B, const int ldb)
    B[2] = xr2; B[3] = xi2;
    B[4] = xr3; B[5] = xi3;
 }
-static void trsmLL_4(const int N, const TYPE *A, TYPE *B, const int ldb)
+static void trsmLL_4(ATL_CSZT  N, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Left', 'Lower', with 1 column prefetch, written with all dependencies
  * shown, so that the compiler can optimize.
@@ -405,11 +383,11 @@ static void trsmLL_4(const int N, const TYPE *A, TYPE *B, const int ldb)
               ar42=A[14], ai42=A[15];
    const TYPE ar33=A[20], ai33=A[21], ar43=A[22], ai43=A[23];
    const TYPE ar44=A[30], ai44=A[31];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, xr4, xi4;
    TYPE t0, p0;
    TYPE *pBn=B+ldb2;
-   const int pfd = ldb2+ldb2;
+   ATL_CSZT  pfd = ldb2+ldb2;
    int j;
 
    p0 = *B;
@@ -494,7 +472,7 @@ static void trsmLL_4(const int N, const TYPE *A, TYPE *B, const int ldb)
 #endif
 
 #ifdef Right_
-static void trsmRU_2(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRU_2(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Upper', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 2x2, with 1/alpha already applied,
@@ -503,7 +481,7 @@ static void trsmRU_2(const int M, const TYPE *A, TYPE *B, const int ldb)
 {
    const TYPE ar11=*A, ai11=A[1], ar12=A[4], ai12=A[5];
    const TYPE ar22=A[6], ai22=A[7];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, t0;
    TYPE *pB0=B, *pB1 = B+ldb2;
    int i;
@@ -534,7 +512,7 @@ static void trsmRU_2(const int M, const TYPE *A, TYPE *B, const int ldb)
    #undef PFD
 }
 
-static void trsmRU_3(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRU_3(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Upper', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 3x3, with 1/alpha already applied,
@@ -544,7 +522,7 @@ static void trsmRU_3(const int M, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar11=*A, ai11=A[1], ar12=A[6], ai12=A[7], ar13=A[12], ai13=A[13];
    const TYPE ar22=A[ 8], ai22=A[ 9], ar23=A[14], ai23=A[15];
    const TYPE ar33=A[16], ai33=A[17];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, t0;
    TYPE *pB0=B, *pB1 = B+ldb2, *pB2=pB1+ldb2;
    int i;
@@ -585,7 +563,7 @@ static void trsmRU_3(const int M, const TYPE *A, TYPE *B, const int ldb)
    #undef PFD
 }
 
-static void trsmRU_4(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRU_4(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Upper', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 4x4, with 1/alpha already applied,
@@ -598,7 +576,7 @@ static void trsmRU_4(const int M, const TYPE *A, TYPE *B, const int ldb)
               ar24=A[26], ai24=A[27];
    const TYPE ar33=A[20], ai33=A[21], ar34=A[28], ai34=A[29];
    const TYPE ar44=A[30], ai44=A[31];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, xr4, xi4, t0;
    TYPE *pB0=B, *pB1 = B+ldb2, *pB2=pB1+ldb2, *pB3=pB2+ldb2;
    int i;
@@ -651,7 +629,7 @@ static void trsmRU_4(const int M, const TYPE *A, TYPE *B, const int ldb)
    #undef PFD
 }
 
-static void trsmRL_2(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRL_2(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Lower', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 2x2, with 1/alpha already applied,
@@ -660,7 +638,7 @@ static void trsmRL_2(const int M, const TYPE *A, TYPE *B, const int ldb)
 {
    const TYPE ar11=*A, ai11=A[1], ar21=A[2], ai21=A[3];
    const TYPE ar22=A[6], ai22=A[7];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE *pB0=B, *pB1 = B+ldb2;
    TYPE xr1, xi1, xr2, xi2, t0;
    int i;
@@ -690,7 +668,7 @@ static void trsmRL_2(const int M, const TYPE *A, TYPE *B, const int ldb)
    }
    #undef PFD
 }
-static void trsmRL_3(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRL_3(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Lower', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 3x3, with 1/alpha already applied,
@@ -700,7 +678,7 @@ static void trsmRL_3(const int M, const TYPE *A, TYPE *B, const int ldb)
    const TYPE ar11=*A, ai11=A[1], ar21=A[2], ai21=A[3], ar31=A[4], ai31=A[5];
    const TYPE ar22=A[ 8], ai22=A[ 9], ar32=A[10], ai32=A[11];
    const TYPE ar33=A[16], ai33=A[17];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, t0;
    TYPE *pB0=B, *pB1 = B+ldb2, *pB2=pB1+ldb2;
    int i;
@@ -741,7 +719,7 @@ static void trsmRL_3(const int M, const TYPE *A, TYPE *B, const int ldb)
    #undef PFD
 }
 
-static void trsmRL_4(const int M, const TYPE *A, TYPE *B, const int ldb)
+static void trsmRL_4(ATL_CSZT  M, const TYPE *A, TYPE *B, ATL_CSZT ldb)
 /*
  * 'Right', 'Lower', written with all dependencies shown, so that the
  * compiler can optimize.  A is known to be 4x4, with 1/alpha already applied,
@@ -754,7 +732,7 @@ static void trsmRL_4(const int M, const TYPE *A, TYPE *B, const int ldb)
               ar42=A[14], ai42=A[15];
    const TYPE ar33=A[20], ai33=A[21], ar43=A[22], ai43=A[23];
    const TYPE ar44=A[30], ai44=A[31];
-   const int ldb2 = ldb+ldb;
+   ATL_CSZT ldb2 = ldb+ldb;
    TYPE xr1, xi1, xr2, xi2, xr3, xi3, xr4, xi4, t0;
    TYPE *pB0=B, *pB1 = B+ldb2, *pB2=pB1+ldb2, *pB3=pB2+ldb2;
    int i;
@@ -810,12 +788,12 @@ static void trsmRL_4(const int M, const TYPE *A, TYPE *B, const int ldb)
 #endif
 
 static void trsmcpUN
-   (const int N, const TYPE *alpha, const TYPE *A, const int lda, TYPE *C)
+   (ATL_CSZT  N, const TYPE *alpha, const TYPE *A, ATL_CSZT lda, TYPE *C)
 /*
  * copies Upper, NoTranspose matrix to Upper, NoTrans matrix, applying 1/alpha
  */
 {
-   const int lda2 = lda+lda, N2 = N+N;
+   ATL_CSZT lda2 = lda+lda, N2 = N+N;
    const TYPE *a=A+lda2;
    TYPE *c=C+N2;
    TYPE tmp[2];
@@ -835,19 +813,19 @@ static void trsmcpUN
 }
 
 static void trsmcpUT
-   (const enum ATLAS_TRANS Trans, const int N, const TYPE *alpha,
-    const TYPE *A, const int lda, TYPE *C)
+   (const enum ATLAS_TRANS Trans, ATL_CSZT  N, const TYPE *alpha,
+    const TYPE *A, ATL_CSZT lda, TYPE *C)
 /*
  * copies Upper, Transpose matrix to Lower, NoTrans matrix, applying 1/alpha
  */
 {
-   const int lda2 = lda+lda;
+   ATL_CSZT lda2 = lda+lda;
    const TYPE *a=A+lda2;
    TYPE *c=C+2;
    TYPE tmp[2];
    int j;
-   void (*move)(const int N, const SCALAR alpha, const TYPE *X, const int incX,
-                TYPE *Y, const int incY);
+   void (*move)(ATL_CSZT  N, const SCALAR alpha, const TYPE *X, ATL_CSZT  incX,
+                TYPE *Y, ATL_CSZT  incY);
 
    if (Trans == AtlasConjTrans) move = Mjoin(PATL,moveConj);
    else move = Mjoin(PATL,cpsc);
@@ -865,19 +843,19 @@ static void trsmcpUT
    }
 }
 
-static void trsmcpLT(const enum ATLAS_TRANS Trans, const int N,
-                     const TYPE *alpha, const TYPE *A, const int lda, TYPE *C)
+static void trsmcpLT(const enum ATLAS_TRANS Trans, ATL_CSZT  N,
+                     const TYPE *alpha, const TYPE *A, ATL_CSZT lda, TYPE *C)
 /*
  * copies Lower, Trans matrix to Upper, Notrans matrix, applies 1/alpha
  */
 {
-   const int lda2 = lda+lda+2, N2 = N+N+2;
+   ATL_CSZT lda2 = lda+lda+2, N2 = N+N+2;
    const TYPE *a=A+2;
    TYPE *c=C+N+N;
    TYPE tmp[2];
    int j;
-   void (*move)(const int N, const SCALAR alpha, const TYPE *X, const int incX,
-                TYPE *Y, const int incY);
+   void (*move)(ATL_CSZT  N, const SCALAR alpha, const TYPE *X, ATL_CSZT  incX,
+                TYPE *Y, ATL_CSZT  incY);
 
    if (Trans == AtlasConjTrans) move = Mjoin(PATL,moveConj);
    else move = Mjoin(PATL,cpsc);
@@ -894,13 +872,13 @@ static void trsmcpLT(const enum ATLAS_TRANS Trans, const int N,
       c += N2;
    }
 }
-static void trsmcpLN(const int N, const TYPE *alpha, const TYPE *A,
-                     const int lda, TYPE *C)
+static void trsmcpLN(ATL_CSZT  N, const TYPE *alpha, const TYPE *A,
+                     ATL_CSZT lda, TYPE *C)
 /*
  * copies Lower, NoTrans matrix to Lower, Notrans matrix, applies 1/alpha
  */
 {
-   const int lda2 = lda+lda+2, N2 = N+N+2;
+   ATL_CSZT lda2 = lda+lda+2, N2 = N+N+2;
    const TYPE *a=A+2;
    TYPE *c=C+2;
    TYPE tmp[2];
@@ -921,7 +899,7 @@ static void trsmcpLN(const int N, const TYPE *alpha, const TYPE *A,
 
 static enum ATLAS_DIAG trsmcopy
    (enum ATLAS_UPLO Uplo, enum ATLAS_TRANS Trans, enum ATLAS_DIAG Diag,
-    const int N, const TYPE *alpha, const TYPE *A, const int lda, TYPE *C)
+    ATL_CSZT  N, const TYPE *alpha, const TYPE *A, ATL_CSZT lda, TYPE *C)
 {
    enum ATLAS_DIAG diag=AtlasNonUnit;
    if (Uplo == AtlasLower)
@@ -967,13 +945,13 @@ static enum ATLAS_DIAG trsmcopy
    #define trsmLL_2 trsmRL_2
 void Mjoin(PATL,CtrsmKR)
    (enum ATLAS_UPLO Uplo, enum ATLAS_TRANS Trans, enum ATLAS_DIAG Diag,
-    const int N, const int M, const SCALAR alpha, const TYPE *A, const int lda,
-    TYPE *B, const int ldb)
+    ATL_CSZT  N, ATL_CSZT  M, const SCALAR alpha, const TYPE *A, ATL_CSZT lda,
+    TYPE *B, ATL_CSZT ldb)
 #else
 void Mjoin(PATL,CtrsmKL)
    (enum ATLAS_UPLO Uplo, enum ATLAS_TRANS Trans, enum ATLAS_DIAG Diag,
-    const int M, const int N, const SCALAR alpha, const TYPE *A, const int lda,
-    TYPE *B, const int ldb)
+    ATL_CSZT  M, ATL_CSZT  N, const SCALAR alpha, const TYPE *A, ATL_CSZT lda,
+    TYPE *B, ATL_CSZT ldb)
 #endif
 {
    TYPE tmp[2], ra, ia;
