@@ -1,33 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef TREAL
 
@@ -37,14 +15,14 @@
    #else
    void Mjoin(PATL_sdtrcopyU2L_N,NM)
    #endif
-   (const int N, const double alpha0, const float *A, const int lda, double *C)
+   (ATL_CSZT  N, const double alpha0, const float *A, ATL_CSZT lda, double *C)
 #else
    #ifdef UnitDiag_
    void Mjoin(Mjoin(PATL,trcopyU2L_U),NM)
    #else
    void Mjoin(Mjoin(PATL,trcopyU2L_N),NM)
    #endif
-      (const int N, const SCALAR alpha0, const TYPE *A, const int lda, TYPE *C)
+      (ATL_CSZT  N, const SCALAR alpha0, const TYPE *A, ATL_CSZT lda, TYPE *C)
 #endif
 /*
  * Takes an Upper matrix, transposes it, and copies it to a dense matrix with
@@ -52,7 +30,7 @@
  */
 {
    int i, j;
-   const int ldap1 = lda+1;
+   ATL_CSZT ldap1 = lda+1;
    const register TYPE alpha=alpha0;
    const TYPE *Ac = A, *Ar;
    TYPE *rC, *cC=C;
@@ -99,7 +77,7 @@
       void ATL_cztrcopyU2L_N
    #endif
 #endif
-   (const int N, const float *A, const int lda, TYPE *C)
+   (ATL_CSZT  N, const float *A, ATL_CSZT lda, TYPE *C)
 #else
 #ifdef UnitDiag_
    #ifdef ConjTrans_
@@ -114,7 +92,7 @@
       void Mjoin(PATL,trcopyU2L_N)
    #endif
 #endif
-   (const int N, const TYPE *A, const int lda, TYPE *C)
+   (ATL_CSZT  N, const TYPE *A, ATL_CSZT lda, TYPE *C)
 #endif
 /*
  * Takes an Upper matrix, transposes it, and copies it to a dense matrix with
@@ -122,7 +100,7 @@
  */
 {
    int i, j2;
-   const int lda2=lda<<1, N2=N<<1, ldap1=lda2+2;
+   ATL_CSZT lda2=lda<<1, N2=N<<1, ldap1=lda2+2;
    #define ldc N2
    const TYPE *a;
    #ifdef UnitDiag_
