@@ -1,48 +1,26 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef TREAL
 
 #ifdef Upper_
 
 #define ATL_trmv_scal Mjoin(ATL_dtrmv_scalUN,UnitNM)
-static void ATL_trmv_scal(const int N, const SCALAR alpha0, const TYPE *A,
-                          const int lda, TYPE *X)
+static void ATL_trmv_scal(ATL_CSZT  N, const SCALAR alpha0, const TYPE *A,
+                          ATL_CSZT lda, TYPE *X)
 /*
  * X <- alpha * A * X
  */
 {
    int i, j;
-   const int n = (N>>2)<<2;
-   const int incAd = (lda+1)<<2;
+   ATL_CSZT  n = (N>>2)<<2;
+   ATL_CSZT  incAd = (lda+1)<<2;
    register TYPE y0, y1, y2, y3, x0;
    const register TYPE alpha=alpha0;
    const TYPE *Ad=A, *Ac;
@@ -150,7 +128,7 @@ static void ATL_trmv_scal(const int N, const SCALAR alpha0, const TYPE *A,
    }
 }
 
-void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(const int N, TYPE *A, const int lda)
+void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(ATL_CSZT  N, TYPE *A, ATL_CSZT lda)
 {
    int j;
    register int i;
@@ -161,7 +139,7 @@ void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(const int N, TYPE *A, const int lda)
    #else
       #define DIAG "N"
       TYPE Ajj;
-      const int ldap1 = lda + 1;
+      ATL_CSZT ldap1 = lda + 1;
       TYPE one=1.0;
    #endif
    TYPE *Ac=A;
@@ -189,11 +167,11 @@ void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(const int N, TYPE *A, const int lda)
 #else
 
 #define ATL_trmv_scal Mjoin(ATL_dtrmv_scalLN,UnitNM)
-static void ATL_trmv_scal(const int N, const SCALAR alpha0, const TYPE *A,
-                          const int lda, TYPE *X)
+static void ATL_trmv_scal(ATL_CSZT  N, const SCALAR alpha0, const TYPE *A,
+                          ATL_CSZT lda, TYPE *X)
 {
    int i;
-   const int n = (N>>2)<<2;
+   ATL_CSZT  n = (N>>2)<<2;
    TYPE *x = X, *stX = X+N-4;
    const TYPE *Ac, *a=A;
    const register TYPE alpha=alpha0;
@@ -302,12 +280,12 @@ static void ATL_trmv_scal(const int N, const SCALAR alpha0, const TYPE *A,
    }
 }
 #ifndef NoUse
-void Mjoin(Mjoin(PATL,trinvertL),UnitNM)(const int N, TYPE *A, const int lda)
+void Mjoin(Mjoin(PATL,trinvertL),UnitNM)(ATL_CSZT  N, TYPE *A, ATL_CSZT lda)
 {
    int j;
    register int i;
    const int ione=1;
-   const int ldap1=lda+1;
+   ATL_CSZT ldap1=lda+1;
    #ifdef UnitDiag_
       #define DIAG "U"
       #define Ajj -1.0
@@ -346,17 +324,18 @@ void Mjoin(Mjoin(PATL,trinvertL),UnitNM)(const int N, TYPE *A, const int lda)
 
 #ifdef Upper_
 
-void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(const int N, TYPE *A, const int lda)
+void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(ATL_CSZT  N, TYPE *A, ATL_CSZT lda)
 {
    int j;
    register int i;
-   const int ione=1, lda2=lda<<1;
+   const int ione=1;
+   ATL_CSZT lda2=lda+lda;
    #ifdef UnitDiag_
       #define DIAG "U"
       const TYPE Ajj[2] = {-1.0, 0.0};
    #else
       #define DIAG "N"
-      const int ldap1 = lda2+2;
+      ATL_CSZT ldap1 = lda2+2;
       TYPE Ajj[2], *Ad=A;
    #endif
    TYPE *Ac=A;
@@ -393,12 +372,12 @@ void Mjoin(Mjoin(PATL,trinvertU),UnitNM)(const int N, TYPE *A, const int lda)
 
 #else
 
-void Mjoin(Mjoin(PATL,trinvertL),UnitNM)(const int N, TYPE *A, const int lda)
+void Mjoin(Mjoin(PATL,trinvertL),UnitNM)(ATL_CSZT  N, TYPE *A, ATL_CSZT lda)
 {
    int j;
    register int i;
    const int ione=1;
-   const int ldap1=(lda<<1)+2;
+   ATL_CSZT ldap1=(lda<<1)+2;
    #ifdef UnitDiag_
       #define DIAG "U"
       const TYPE Ajj[2] = {-1.0, 0.0};
