@@ -1,33 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef Upper_
    #ifdef Transpose_
@@ -84,8 +62,8 @@
 #endif
 
 void Mjoin(Mjoin(PATL,trmmR),ATLP)
-   (const int M, const int N, const void *valpha, const void *A, const int lda,
-    void *C, const int ldc)
+   (ATL_CSZT  M, ATL_CSZT  N, const void *valpha, const void *A, ATL_CSZT lda,
+    void *C, ATL_CSZT ldc)
 {
    #ifdef TREAL
       const SCALAR alpha=*( (const SCALAR *)valpha );
@@ -105,10 +83,14 @@ void Mjoin(Mjoin(PATL,trmmR),ATLP)
       #ifdef TREAL
          if ( SCALAR_IS_ONE(alpha) ) Mjoin(ATL_trcopy,_a1)(N, alpha, A, lda, a);
          else Mjoin(ATL_trcopy,_aX)(N, alpha, A, lda, a);
-         CAgemmNN(M, N, N, one, C, ldc, a, N, zero, C, ldc);
+         ATL_assert(N <= ATL_VWopgen_MAX_KB); /* aliasing chk */
+         ATL_almm(AtlasNoTrans, AtlasNoTrans, M, N, N, one, C, ldc, a, N,
+                  zero, C, ldc);
       #else
          ATL_trcopy(N, A, lda, a);
-         CAgemmNN(M, N, N, valpha, C, ldc, a, N, zero, C, ldc);
+         ATL_assert(N <= ATL_VWopgen_MAX_KB); /* aliasing chk */
+         ATL_almm(AtlasNoTrans, AtlasNoTrans, M, N, N, valpha, C, ldc, a, N,
+                  zero, C, ldc);
       #endif
       free(va);
    }
