@@ -1,33 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef TCPLX
 
@@ -39,11 +17,11 @@
    #define ATL_trmv Mjoin(PATL,Mjoin(trmv_scalUNN,NM))
 #endif
 
-void ATL_trmv(const int N, const SCALAR alpha, const TYPE *A, const int lda,
+void ATL_trmv(ATL_CSZT  N, const SCALAR alpha, const TYPE *A, ATL_CSZT lda,
               TYPE *X)
 {
    int i, j;
-   const int lda2=(lda<<1), n=(N>>1)<<1;
+   ATL_CSZT lda2=(lda<<1), n=(N>>1)<<1;
    const TYPE zero=0.0;
    TYPE *x;
    const TYPE *Ad=A, *a;
@@ -198,11 +176,11 @@ void ATL_trmv(const int N, const SCALAR alpha, const TYPE *A, const int lda,
    #define ATL_trmv Mjoin(PATL,Mjoin(trmv_scalLNN,NM))
 #endif
 
-void ATL_trmv(const int N, const SCALAR alpha, const TYPE *A, const int lda,
+void ATL_trmv(ATL_CSZT  N, const SCALAR alpha, const TYPE *A, ATL_CSZT lda,
               TYPE *X)
 {
    int i, j;
-   const int n = (N>>1)<<1, lda2=lda<<1;
+   ATL_CSZT  n = (N>>1)<<1, lda2=lda<<1;
    #ifdef UnitDiag_
       #define ii i
    #else
