@@ -1,38 +1,16 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef TREAL
 
 void Mjoin(Mjoin(PATL,trputU),BNM)
-   (const int N, const TYPE *v, const SCALAR beta0, TYPE *A, const int lda)
+   (ATL_CSZT  N, const TYPE *v, const SCALAR beta0, TYPE *A, ATL_CSZT lda)
 /*
  * Writes the upper part of an NxN workspace v to the upper part of the array A
  */
@@ -62,13 +40,13 @@ void Mjoin(Mjoin(PATL,trputU),BNM)
 #else
    void Mjoin(Mjoin(PATL,trputU),BNM)
 #endif
-   (const int N, const TYPE *v, const SCALAR beta0, TYPE *A, const int lda)
+   (ATL_CSZT  N, const TYPE *v, const SCALAR beta0, TYPE *A, ATL_CSZT lda)
 /*
  * Writes the upper part of an NxN workspace v to the upper part of the array A
  */
 {
    int i, j2;
-   const int N2=N<<1, lda2=lda<<1;
+   ATL_CSZT  N2=N<<1, lda2=lda<<1;
    #ifdef Herm_
       const TYPE zero=0.0;
    #endif
