@@ -2,8 +2,9 @@
 #include "atlas_prefetch.h"
 #define RTYPE register TYPE
 
-#if defined(__GNUC__) || \
-    (defined(__STDC_VERSION__) && (__STDC_VERSION__/100 >= 1999))
+#ifdef __GNUC__
+   #define ATL_SINLINE static __inline__
+#elif defined(__STDC_VERSION__) && (__STDC_VERSION__/100 >= 1999)
    #define ATL_SINLINE static inline
 #else
    #define ATL_SINLINE static
@@ -53,86 +54,197 @@ ATL_SINLINE void ATL_rk4(ATL_CINT M, const TYPE *A, ATL_CINT lda,
       register __m256d rB;
 
       rB = _mm256_unpacklo_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB); rA1 = _mm256_load_pd(pA1);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA1);
       rB = _mm256_unpacklo_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB); rC40 =_mm256_load_pd(pC0+4);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      rC40 =_mm256_load_pd(pC0+4);
       rB = _mm256_unpacklo_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC02 = _mm256_sub_pd(rC02, rB); rA0 = _mm256_load_pd(pA2);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA0, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA2);
 
       rB = _mm256_unpackhi_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB); rC41 =_mm256_load_pd(pC1+4);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      rC41 =_mm256_load_pd(pC1+4);
       rB = _mm256_unpackhi_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB); rC42 =_mm256_load_pd(pC2+4);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      rC42 =_mm256_load_pd(pC2+4);
       rB = _mm256_unpackhi_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); rA1 = _mm256_load_pd(pA3);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA3);
 
       rB = _mm256_unpacklo_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB22, rB22);
       rB = _mm256_mul_pd(rB, rA0);
       rC02 = _mm256_sub_pd(rC02, rB); rA0 = _mm256_load_pd(pA0+4);
 
       rB = _mm256_unpackhi_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB); _mm256_store_pd(pC0, rC00);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      _mm256_store_pd(pC0, rC00);
       rB = _mm256_unpackhi_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB); _mm256_store_pd(pC1, rC01);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      _mm256_store_pd(pC1, rC01);
       rB = _mm256_unpackhi_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); rA1 = _mm256_load_pd(pA1+4);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA1+4);
 /*
  *    2nd row of C regs
  */
       rB = _mm256_unpacklo_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC40 = _mm256_sub_pd(rC40, rB); _mm256_store_pd(pC2, rC02);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA0, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
+      _mm256_store_pd(pC2, rC02);
       rB = _mm256_unpacklo_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC41 = _mm256_sub_pd(rC41, rB); rC00 = _mm256_load_pd(pC0+8);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA0, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
+      rC00 = _mm256_load_pd(pC0+8);
       rB = _mm256_unpacklo_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC42 = _mm256_sub_pd(rC42, rB); rA0 = _mm256_load_pd(pA2+4);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA0, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA2+4);
 
       rB = _mm256_unpackhi_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC40 = _mm256_sub_pd(rC40, rB); rC01 = _mm256_load_pd(pC1+8);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA1, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
+      rC01 = _mm256_load_pd(pC1+8);
       rB = _mm256_unpackhi_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC41 = _mm256_sub_pd(rC41, rB); rC02 = _mm256_load_pd(pC2+8);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA1, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
+      rC02 = _mm256_load_pd(pC2+8);
       rB = _mm256_unpackhi_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC42 = _mm256_sub_pd(rC42, rB); rA1 = _mm256_load_pd(pA3+4);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA1, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA3+4);
 
       rB = _mm256_unpacklo_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC40 = _mm256_sub_pd(rC40, rB);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA0, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC41 = _mm256_sub_pd(rC41, rB);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA0, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC42 = _mm256_sub_pd(rC42, rB); rA0 = _mm256_load_pd(pA0+8);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA0, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA0+8);
 
       rB = _mm256_unpackhi_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC40 = _mm256_sub_pd(rC40, rB); _mm256_store_pd(pC0+4, rC40);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA1, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
+      _mm256_store_pd(pC0+4, rC40);
       rB = _mm256_unpackhi_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC41 = _mm256_sub_pd(rC41, rB); _mm256_store_pd(pC1+4, rC41);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA1, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
+      _mm256_store_pd(pC1+4, rC41);
       rB = _mm256_unpackhi_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC42 = _mm256_sub_pd(rC42, rB); _mm256_store_pd(pC2+4, rC42);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA1, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      _mm256_store_pd(pC2+4, rC42);
    }
 /*
  * Drain C load/use pipeline
@@ -142,130 +254,296 @@ ATL_SINLINE void ATL_rk4(ATL_CINT M, const TYPE *A, ATL_CINT lda,
       register __m256d rB;
 
       rB = _mm256_unpacklo_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB); rA1 = _mm256_load_pd(pA1);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA1);
       rB = _mm256_unpacklo_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC02 = _mm256_sub_pd(rC02, rB); rA0 = _mm256_load_pd(pA2);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA0, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA2);
 
       rB = _mm256_unpackhi_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
       rB = _mm256_unpackhi_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
       rB = _mm256_unpackhi_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); rA1 = _mm256_load_pd(pA3);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA3);
 
       rB = _mm256_unpacklo_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC02 = _mm256_sub_pd(rC02, rB);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA0, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
 
       rB = _mm256_unpackhi_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB); _mm256_store_pd(pC0, rC00);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      _mm256_store_pd(pC0, rC00);
       rB = _mm256_unpackhi_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB); _mm256_store_pd(pC1, rC01);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      _mm256_store_pd(pC1, rC01);
       rB = _mm256_unpackhi_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); _mm256_store_pd(pC2, rC02);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      _mm256_store_pd(pC2, rC02);
    }
    else /* M-MM = 8, drain pipe over 2 iterations */
    {
       register __m256d rB;
 
       rB = _mm256_unpacklo_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB); rA1 = _mm256_load_pd(pA1);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA1);
       rB = _mm256_unpacklo_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB); rC40 =_mm256_load_pd(pC0+4);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+         rC40 =_mm256_load_pd(pC0+4);
       rB = _mm256_unpacklo_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC02 = _mm256_sub_pd(rC02, rB); rA0 = _mm256_load_pd(pA2);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA0, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA2);
 
       rB = _mm256_unpackhi_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB); rC41 =_mm256_load_pd(pC1+4);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      rC41 =_mm256_load_pd(pC1+4);
       rB = _mm256_unpackhi_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB); rC42 =_mm256_load_pd(pC2+4);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      rC42 =_mm256_load_pd(pC2+4);
       rB = _mm256_unpackhi_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); rA1 = _mm256_load_pd(pA3);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA3);
 
       rB = _mm256_unpacklo_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC00 = _mm256_sub_pd(rC00, rB);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA0, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC01 = _mm256_sub_pd(rC01, rB);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA0, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC02 = _mm256_sub_pd(rC02, rB); rA0 = _mm256_load_pd(pA0+4);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA0, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA0+4);
 
       rB = _mm256_unpackhi_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC00 = _mm256_sub_pd(rC00, rB); _mm256_store_pd(pC0, rC00);
+      #ifdef ATL_AVXMAC
+         rC00 = _mm256_fnmadd_pd(rB, rA1, rC00);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC00 = _mm256_sub_pd(rC00, rB);
+      #endif
+      _mm256_store_pd(pC0, rC00);
       rB = _mm256_unpackhi_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC01 = _mm256_sub_pd(rC01, rB); _mm256_store_pd(pC1, rC01);
+      #ifdef ATL_AVXMAC
+         rC01 = _mm256_fnmadd_pd(rB, rA1, rC01);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC01 = _mm256_sub_pd(rC01, rB);
+      #endif
+      _mm256_store_pd(pC1, rC01);
       rB = _mm256_unpackhi_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC02 = _mm256_sub_pd(rC02, rB); rA1 = _mm256_load_pd(pA1+4);
+      #ifdef ATL_AVXMAC
+         rC02 = _mm256_fnmadd_pd(rB, rA1, rC02);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC02 = _mm256_sub_pd(rC02, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA1+4);
 /*
  *    2nd row of C regs
  */
       rB = _mm256_unpacklo_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC40 = _mm256_sub_pd(rC40, rB); _mm256_store_pd(pC2, rC02);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA0, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
+      _mm256_store_pd(pC2, rC02);
       rB = _mm256_unpacklo_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC41 = _mm256_sub_pd(rC41, rB);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA0, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC42 = _mm256_sub_pd(rC42, rB); rA0 = _mm256_load_pd(pA2+4);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA0, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      rA0 = _mm256_load_pd(pA2+4);
 
       rB = _mm256_unpackhi_pd(rB00, rB00);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC40 = _mm256_sub_pd(rC40, rB);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA1, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
       rB = _mm256_unpackhi_pd(rB01, rB01);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC41 = _mm256_sub_pd(rC41, rB);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA1, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
       rB = _mm256_unpackhi_pd(rB02, rB02);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC42 = _mm256_sub_pd(rC42, rB); rA1 = _mm256_load_pd(pA3+4);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA1, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      rA1 = _mm256_load_pd(pA3+4);
 
       rB = _mm256_unpacklo_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC40 = _mm256_sub_pd(rC40, rB);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA0, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC41 = _mm256_sub_pd(rC41, rB);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA0, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
       rB = _mm256_unpacklo_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA0);
-      rC42 = _mm256_sub_pd(rC42, rB);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA0, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA0);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
 
       rB = _mm256_unpackhi_pd(rB20, rB20);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC40 = _mm256_sub_pd(rC40, rB); _mm256_store_pd(pC0+4, rC40);
+      #ifdef ATL_AVXMAC
+         rC40 = _mm256_fnmadd_pd(rB, rA1, rC40);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC40 = _mm256_sub_pd(rC40, rB);
+      #endif
+      _mm256_store_pd(pC0+4, rC40);
       rB = _mm256_unpackhi_pd(rB21, rB21);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC41 = _mm256_sub_pd(rC41, rB); _mm256_store_pd(pC1+4, rC41);
+      #ifdef ATL_AVXMAC
+         rC41 = _mm256_fnmadd_pd(rB, rA1, rC41);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC41 = _mm256_sub_pd(rC41, rB);
+      #endif
+      _mm256_store_pd(pC1+4, rC41);
       rB = _mm256_unpackhi_pd(rB22, rB22);
-      rB = _mm256_mul_pd(rB, rA1);
-      rC42 = _mm256_sub_pd(rC42, rB); _mm256_store_pd(pC2+4, rC42);
+      #ifdef ATL_AVXMAC
+         rC42 = _mm256_fnmadd_pd(rB, rA1, rC42);
+      #else
+         rB = _mm256_mul_pd(rB, rA1);
+         rC42 = _mm256_sub_pd(rC42, rB);
+      #endif
+      _mm256_store_pd(pC2+4, rC42);
    }
 }
 #elif defined(ATL_SSE2) && defined(DREAL)
@@ -955,6 +1233,38 @@ ATL_SINLINE void ATL_rk4(ATL_CINT M, const TYPE *A, ATL_CINT lda,
 #endif
 
 #if NRHS == 3
+/*
+ * Solve 4x4 L with 3 RHS symbolically for unit diagonal
+ * Answer is output into rBxx regs, which are live on input and output
+ */
+#define  ATL_trsmL4U(L_, ldl_, r_, ldr_) \
+{ \
+   const RTYPE L10=L_[1], L20=L_[2], L30=L_[3]; \
+   const RTYPE L21=L_[ldl_+2], L31=a[ldl_+3]; \
+   const RTYPE L32=L_[2*(ldl_)+3]; \
+/* \
+ * x1 = (b1 - L10 * x0) \
+ */ \
+   rB10 = (rB10 - L10*rB00);  \
+   rB11 = (rB11 - L10*rB01); \
+   rB12 = (rB12 - L10*rB02); \
+   ATL_pfl1W(r_ + ((ldr_)<<2)); \
+/* \
+ * x2 = (b2 - L20*x0 - L21*x1) \
+ */ \
+   rB20 = (rB20 - L20*rB00 - L21*rB10); \
+   rB21 = (rB21 - L20*rB01 - L21*rB11); \
+   rB22 = (rB22 - L20*rB02 - L21*rB12); \
+   ATL_pfl1W(r_ + ldr_+((ldr_)<<2)); \
+/* \
+ * x3 = (b3 - L30*x0 - L31*x1 - L32*x2) \
+ */ \
+   rB30 = (rB30 - L30*rB00 - L31*rB10 - L32*rB20); \
+   rB31 = (rB31 - L30*rB01 - L31*rB11 - L32*rB21); \
+   rB32 = (rB32 - L30*rB02 - L31*rB12 - L32*rB22); \
+   ATL_pfl1W(r_ + ((ldr_)<<1)+((ldr_)<<2)); \
+}  /* complete 4x4 NRHS=3 solve block */
+
 /*
  * Solve 4x4 L with 3 RHS symbolically
  * Answer is output into rBxx regs, which are live on input and output
@@ -1033,6 +1343,42 @@ ATL_SINLINE void ATL_rk4(ATL_CINT M, const TYPE *A, ATL_CINT lda,
    rB02 = (rB02 - U01*rB12 - U02*rB22 - U03*rB32) * U00; \
 }  /* complete M=4, N=3 solve block */
 #elif NRHS == 4
+/*
+ * Solve 4x4 L with 4 RHS symbolically for unit diagonal
+ * Answer is output into rBxx regs, which are live on input and output
+ */
+#define  ATL_trsmL4U(L_, ldl_, r_, ldr_) \
+{ \
+   const RTYPE L10=L_[1], L20=L_[2], L30=L_[3]; \
+   const RTYPE L21=L_[ldl_+2], L31=a[ldl_+3]; \
+   const RTYPE L32=L_[2*(ldl_)+3]; \
+   ATL_pfl1W(r_ + ((ldr_)<<2)); \
+/* \
+ * x1 = (b1 - L10 * x0) \
+ */ \
+   rB10 = (rB10 - L10*rB00);  \
+   rB11 = (rB11 - L10*rB01); \
+   rB12 = (rB12 - L10*rB02); \
+   rB13 = (rB13 - L10*rB03); \
+   ATL_pfl1W(r_ + ldr_ +((ldr_)<<2)); \
+/* \
+ * x2 = (b2 - L20*x0 - L21*x1) \
+ */ \
+   rB20 = (rB20 - L20*rB00 - L21*rB10); \
+   rB21 = (rB21 - L20*rB01 - L21*rB11); \
+   rB22 = (rB22 - L20*rB02 - L21*rB12); \
+   rB23 = (rB23 - L20*rB03 - L21*rB13); \
+   ATL_pfl1W(r_ + ((ldr_)<<1)+((ldr_)<<2)); \
+/* \
+ * x3 = (b3 - L30*x0 - L31*x1 - L32*x2) \
+ */ \
+   rB30 = (rB30 - L30*rB00 - L31*rB10 - L32*rB20); \
+   rB31 = (rB31 - L30*rB01 - L31*rB11 - L32*rB21); \
+   ATL_pfl1W(r_ + ldr_+((ldr_)<<1)+((ldr_)<<2)); \
+   rB32 = (rB32 - L30*rB02 - L31*rB12 - L32*rB22); \
+   rB33 = (rB33 - L30*rB03 - L31*rB13 - L32*rB23); \
+}  /* complete 4x4 solve block */
+
 /*
  * Solve 4x4 L with 4 RHS symbolically
  * Answer is output into rBxx regs, which are live on input and output
@@ -1121,7 +1467,15 @@ ATL_SINLINE void ATL_rk4(ATL_CINT M, const TYPE *A, ATL_CINT lda,
 }  /* complete 4x4 solve block */
 #endif
 
-static void ATL_trsmLLN
+#ifdef ATL_KERN_L4U
+   #define ATL_trsmLLNU Mjoin(PATL,ktrsmLLNU_rk4)
+   #define trsmL4 ATL_trsmL4U  /* use unit-diag kernel */
+void ATL_trsmLLNU
+#else
+   #define ATL_trsmLLN Mjoin(PATL,ktrsmLLN_rk4)
+   #define trsmL4 ATL_trsmL4  /* use non-unit-diag kernel */
+void ATL_trsmLLN
+#endif
 (
    ATL_CINT  M,   /* size of orig triangular matrix A */
    ATL_CINT  N,   /* number of RHS in B */
@@ -1187,7 +1541,7 @@ static void ATL_trsmLLN
 /*
  *       Solve M=4 NRHS=4 TRSM symbolically
  */
-         ATL_trsmL4(a, lda, b, ldb);
+         trsmL4(a, lda, b, ldb);
 /*
  *       Write solved 4x4 block out to original workspace (final answer)
  *       Handle most common case with only one if
@@ -1637,8 +1991,11 @@ static void cpypadL
       a[j] = ATL_rone;
    }
 }
-
+#ifdef ATL_KERN_L4U
+int Mjoin(PATL,trsmKLU_rk4)   /* returns 0 on success */
+#else
 int Mjoin(PATL,trsmKL_rk4)   /* returns 0 on success */
+#endif
 (
    enum ATLAS_SIDE Side,
    enum ATLAS_UPLO Uplo,
@@ -1686,7 +2043,11 @@ int Mjoin(PATL,trsmKL_rk4)   /* returns 0 on success */
       cpypadL(Diag, M, A, lda, a, M4);
       if (t)
          free(t);
-      ATL_trsmLLN(M, N, alpha, a, B, ldb, w);
+      #ifdef ATL_KERN_L4U
+         ATL_trsmLLNU(M, N, alpha, a, B, ldb, w);
+      #else
+         ATL_trsmLLN(M, N, alpha, a, B, ldb, w);
+      #endif
    }
    else  /* Uplo == AtlasUpper */
    {
