@@ -1,40 +1,17 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                   (C) Copyright 2000 Antoine P. Petitet
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2000 Antoine P. Petitet
  * Code contributers : Antoine P. Petitet, R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 
 void Mjoin(Mjoin(Mjoin(Mjoin(Mjoin(PATL,trsmK),SideNM),UploNM),N),UnitNM)
-   (const int M, const int N, const SCALAR alpha, const TYPE *A, const int lda,
-    TYPE *B, const int ldb)
+   (ATL_CSZT  M, ATL_CSZT  N, const SCALAR alpha, const TYPE *A, ATL_CSZT lda,
+    TYPE *B, ATL_CSZT ldb)
 /*
  * This source file is multiply compiled to create the following routines:
  * ATL_trsmKRUNU : trsm Side='Right', Uplo='Upper', Trans='N', Unit='Unit'
@@ -90,12 +67,12 @@ void Mjoin(Mjoin(Mjoin(Mjoin(Mjoin(PATL,trsmK),SideNM),UploNM),N),UnitNM)
 
 #ifdef Left_
    #ifdef Upper_
-      const int ldap1 = lda + 1;
+      ATL_CSZT ldap1 = lda + 1;
       const TYPE * Ad;
    #endif
 #else
    #ifdef Lower_
-      const int ldap1 = lda + 1;
+      ATL_CSZT ldap1 = lda + 1;
       const TYPE * Ad;
    #endif
 #endif
