@@ -1,33 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 #include "atlas_kern3.h"
+#ifdef USE_AMM
+   #undef USE_AMM
+#endif
 
 #ifdef Transpose_
    #define ATL_TRANS AtlasTrans
@@ -119,9 +97,9 @@
 
 #ifdef TREAL
 void Mjoin(ATL_trcopy,_a1)
-   (const int N, const SCALAR alpha0, const TYPE *A, const int lda, TYPE *C);
+   (ATL_CSZT  N, const SCALAR alpha0, const TYPE *A, ATL_CSZT lda, TYPE *C);
 #else
-void ATL_trcopy (const int N, const TYPE *A, const int lda, TYPE *C);
+void ATL_trcopy (ATL_CSZT  N, const TYPE *A, ATL_CSZT lda, TYPE *C);
 #endif
 
 #ifdef BIG_TRSM
@@ -131,8 +109,8 @@ void ATL_trcopy (const int N, const TYPE *A, const int lda, TYPE *C);
 #else
    void Mjoin(Mjoin(PATL,trsmL),ATLP)
 #endif
-   (const int M, const int N, const void *valpha, const void *A, const int lda,
-    void *C, const int ldc)
+   (ATL_CSZT  M, ATL_CSZT  N, const void *valpha, const void *A, ATL_CSZT lda,
+    void *C, ATL_CSZT ldc)
 {
    const TYPE *alpha = valpha;
 #ifdef TREAL
@@ -147,12 +125,15 @@ void ATL_trcopy (const int N, const TYPE *A, const int lda, TYPE *C);
       Mjoin(PATL,reftrsm)(AtlasLeft, ATL_UPLO, ATL_TRANS, ATL_UNIT,
                           M, N, *alpha, A, lda, C, ldc);
 #else
-   if (N > (M<<2) && M <= 4)
-      Mjoin(PATL,CtrsmKL)(ATL_UPLO, ATL_TRANS, ATL_UNIT,
-                          M, N, valpha, A, lda, C, ldc);
-   else
+   int ierr=1;
+   int Mjoin(PATL,trsmKL_rk2)(enum ATLAS_SIDE Side, enum ATLAS_UPLO Uplo,
+      enum ATLAS_TRANS TA, enum ATLAS_DIAG Diag, ATL_CINT M, ATL_CINT N,
+      const SCALAR alpha, const TYPE *A, ATL_CINT lda, TYPE *B, ATL_CINT ldb);
+   if (N >= 8 && M >= 8)  /* must get reuse of A, and call rk2 */
+      ierr = Mjoin(PATL,trsmKL_rk2)(AtlasLeft, ATL_UPLO, ATL_TRANS, ATL_UNIT,
+                                    M, N, valpha, A, lda, C, ldc);
+   if (ierr)
       Mjoin(PATL,reftrsm)(AtlasLeft, ATL_UPLO, ATL_TRANS, ATL_UNIT,
                           M, N, valpha, A, lda, C, ldc);
-
 #endif
 }
