@@ -47,23 +47,25 @@
 /*
  * Include files
  */
+#include "atlas_misc.h"
 #include "atlas_rblas3.h"
 #include "atlas_kernel3.h"
+#include "atlas_level3.h"
 
 void ATL_rsyr2kLN
 (
    RC3_SYR2K_T                * RTYP,
-   const int                  N,
-   const int                  K,
+   ATL_CSZT                   N,
+   ATL_CSZT                   K,
    const void                 * ALPHA,
    const void                 * A,
-   const int                  LDA,
+   ATL_CSZT                   LDA,
    const void                 * B,
-   const int                  LDB,
+   ATL_CSZT                   LDB,
    const void                 * BETA,
    void                       * C,
-   const int                  LDC,
-   const int                  RB
+   ATL_CSZT                   LDC,
+   ATL_CSZT                   RB
 )
 {
 /*
