@@ -47,21 +47,24 @@
 /*
  * Include files
  */
+#include "atlas_misc.h"
 #include "atlas_rblas3.h"
 #include "atlas_kernel3.h"
+#include "atlas_level3.h"
+#define FORCE_RECUR 1
 
 void ATL_rsyrkUT
 (
    RC3_SYRK_T                 * RTYP,
-   const int                  N,
-   const int                  K,
+   ATL_CSZT                   N,
+   ATL_CSZT                   K,
    const void                 * ALPHA,
    const void                 * A,
-   const int                  LDA,
+   ATL_CSZT                   LDA,
    const void                 * BETA,
    void                       * C,
-   const int                  LDC,
-   const int                  RB
+   ATL_CSZT                   LDC,
+   ATL_CSZT                   RB
 )
 {
 /*
@@ -88,8 +91,11 @@ void ATL_rsyrkUT
  * .. Executable Statements ..
  *
  */
-   if( ( n1 = N - RB ) <= 0 )
-   { RTYP->Tsyrk( N, K, ALPHA, A, LDA, BETA, C, LDC ); return; }
+   n1 = N - RB;
+   #ifdef FORCE_RECUR
+   if( n1 <= 0 )
+   #endif
+   { if(!RTYP->Tsyrk( N, K, ALPHA, A, LDA, BETA, C, LDC )) return; }
 
    n2 = N - ( n1 = RB + ( n1 / ( RB << 1 ) ) * RB ); size = RTYP->size;
 
