@@ -47,20 +47,22 @@
 /*
  * Include files
  */
+#include "atlas_misc.h"
 #include "atlas_rblas3.h"
 #include "atlas_kernel3.h"
+#include "atlas_level3.h"
 
 void ATL_rtrsmLLN
 (
    RC3_TRSM_T                 * RTYP,
-   const int                  M,
-   const int                  N,
+   ATL_CSZT                   M,
+   ATL_CSZT                   N,
    const void                 * ALPHA,
    const void                 * A,
-   const int                  LDA,
+   ATL_CSZT                   LDA,
    void                       * B,
-   const int                  LDB,
-   const int                  RB
+   ATL_CSZT                   LDB,
+   ATL_CSZT                   RB
 )
 {
 /*
