@@ -3,7 +3,7 @@
  * -- Automatically Tuned Linear Algebra Software (ATLAS)
  *    (C) Copyright 2000 All Rights Reserved
  *
- * -- ATLAS routine -- Version 3.2 -- December 25, 2000
+ * -- ATLAS routine -- Version 3.9.24 -- December 25, 2000
  *
  * Author         : Antoine P. Petitet
  * Contributor(s) : R. Clint Whaley
@@ -47,20 +47,22 @@
 /*
  * Include files
  */
+#include "atlas_misc.h"
 #include "atlas_rblas3.h"
 #include "atlas_kernel3.h"
+#include "atlas_level3.h"
 
 void ATL_rtrsmLLT
 (
    RC3_TRSM_T                 * RTYP,
-   const int                  M,
-   const int                  N,
+   ATL_CSZT                   M,
+   ATL_CSZT                   N,
    const void                 * ALPHA,
    const void                 * A,
-   const int                  LDA,
+   ATL_CSZT                   LDA,
    void                       * B,
-   const int                  LDB,
-   const int                  RB
+   ATL_CSZT                   LDB,
+   ATL_CSZT                   RB
 )
 {
 /*
