@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.2
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2009 Siju Samuel
- *
  * Code contributers : Siju Samuel, Anthony M. Castaldo, R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 
 #include "atlas_misc.h"
@@ -36,6 +10,18 @@
 #include "atlas_lapack.h"
 #include "atlas_lvl3.h"
 #include "atlas_qrrmeth.h"
+/* #include Mstr(Mjoin(ATLAS_PRE,oprk_perf.h)) */
+#include Mstr(Mjoin(ATLAS_PRE,opgen_view.h))
+#if ATL_VWopgen_MAX_KB <= 88
+   #define ATL_QRKB ATL_VWopgen_100KB
+   #define ATL_QRLCM ATL_VWopgen_100LCMMN
+#elif ATL_VWopgen_99KB <= 88
+   #define ATL_QRKB ATL_VWopgen_99KB
+   #define ATL_QRLCM ATL_VWopgen_99LCMMN
+#else
+   #define ATL_QRKB ATL_VWopgen_98KB
+   #define ATL_QRLCM ATL_VWopgen_98LCMMN
+#endif
 
 #ifdef ATL_USEPTHREADS
    #include "atlas_threads.h"
@@ -142,15 +128,15 @@ int ATL_geqlr(ATL_CINT M, ATL_CINT N, TYPE *A, ATL_CINT LDA, TYPE  *TAU,
        * For small problems, make M a mult of MU (many x86 have NU=1 anyway!).
        */
 
-         if (minMN >= NB+NB) /* big prob, put remainder on right.             */
+         if (minMN >= 2*ATL_QRKB) /* big prob, put remainder on right. */
          {
-            leftMN = ATL_MulByNB(ATL_DivByNB(minMN>>1));
+            leftMN = ((minMN>>1)/ATL_QRKB)*ATL_QRKB;
             right = minMN - leftMN;
             left  = N -right;
          }
          else /* small prob, keep M mult of MU (MU more critical than NU)     */
          {
-            right = ((minMN>>1)/ATL_mmMU)*ATL_mmMU;
+            right = ((minMN>>1)/ATL_QRLCM)*ATL_QRLCM;
             leftMN = minMN - right;
             left = N - right;
          }
