@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 #include "atlas_misc.h"
 #include "atlas_lvl3.h"
@@ -33,13 +8,21 @@
 #include "atlas_level1.h"
 #include "atlas_lapack.h"
 #include "atlas_lamch.h"
-#include Mstr(Mjoin(Mjoin(atlas_,PRE),sysinfo.h))
+#include "atlas_cache.h"
 #ifdef ATL_USEPTHREADS
    #include "atlas_pthreads.h"
    #include "atlas_tcacheedge.h"
+   #include "atlas_pca.h"
+   #ifdef ATL_laswp
+      #undef ATL_laswp
+   #endif
+   #define ATL_laswp Mjoin(PATL,tlaswp)
 #else
    #include "atlas_cacheedge.h"
 #endif
+#ifndef CacheEdge
+   #define CacheEdge 262144
+#endif
 
 
 #ifdef TREAL
@@ -58,7 +41,7 @@
    #define ATL_CplxInv(in, out) Mjoin(PATL,cplxinvert)(1, in, 1, out, 1);
 #endif
 
-#ifdef TREAL
+#if defined(TREAL)
 
 static int LU2(const int M, TYPE *A, const int lda, int *ipiv)
 /*
@@ -278,11 +261,11 @@ int ATL_getrfC(const int M, const int N, TYPE *A, const int lda, int *ipiv)
    #endif
    TYPE *Ac, *An;
 
-   if (((size_t)M)*N <= ATL_L1elts)
+   if (((size_t)M)*N <= L1C_ELTS)
       return(Mjoin(PATL,getf2)(M, N, A, lda, ipiv));
-   #ifdef ATL_USEPTHREADS
-      if (N <= (NB<<2) && N >= 16 && M-N >= ATL_PCAMin &&
-          ((size_t)ATL_MulBySize(M)*N) <= CacheEdge*ATL_NTHREADS)
+   #if defined(ATL_USEPTHREADS) && defined(ATL_USEPCA)
+      if (M-N > ATL_PCAMin && ((N > 1 && N < 12) &&
+          ((size_t)ATL_MulBySize(M)*N) <= CacheEdge*ATL_NTHREADS))
       {
          if (N >= 16)
             ierr = Mjoin(PATL,tgetf2)(M, N, A, lda, ipiv);
@@ -294,8 +277,9 @@ int ATL_getrfC(const int M, const int N, TYPE *A, const int lda, int *ipiv)
    if (MN > ATL_luMmin)
    {
       Nleft = MN >> 1;
-      #ifdef NB
-         if (Nleft > NB) Nleft = ATL_MulByNB(ATL_DivByNB(Nleft));
+      #ifdef ATL_VWipgen_100LCMMN
+         if (Nleft > ATL_VWipgen_100LCMMN)
+            Nleft = (Nleft/ATL_VWipgen_100LCMMN)*ATL_VWipgen_100LCMMN;
       #endif
       Nright = N - Nleft;
       i = ATL_getrfC(M, Nleft, A, lda, ipiv);  /* factor left to L & U */
@@ -315,7 +299,12 @@ int ATL_getrfC(const int M, const int N, TYPE *A, const int lda, int *ipiv)
       for (i=Nleft; i != MN; i++) ipiv[i] += Nleft;
       ATL_laswp(Nleft, A, lda, Nleft, MN, ipiv, 1);
    }
-#ifdef TREAL
+/*
+ * Don't use LU2 for threaded code: it is serial, and will become costly
+ * due to Amdahl's law for large-scale problems.  Recur to 1 and use BLAS
+ * for parallelism
+ */
+#if defined(TREAL) && !defined(ATL_USEPTHREADS)
    else if (MN == 2)
    {
       Nleft = 2;
