@@ -1,34 +1,10 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2001 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2001 R. Clint Whaley
  */
 #include "atlas_lapack.h"
 #include "atlas_lvl3.h"
+#include Mstr(Mjoin(ATLAS_PRE,ipgen_view.h))
 
 static void trcpzeroU(const int M, const int N, TYPE *U, const int ldu,
                       TYPE *C, const int ldc)
@@ -69,9 +45,16 @@ int ATL_getriR(const int N, TYPE *A, const int lda, const int *ipiv,
  *    Find largest NB we can use with our provided workspace
  */
     jb = lwrk / N;
-    if (jb >= NB) nb = ATL_MulByNB(ATL_DivByNB(jb));
-    else if (jb >= ATL_mmMU) nb = (jb/ATL_mmMU)*ATL_mmMU;
-    else nb = jb;
+    nb = Mjoin(PATL,laGetB)(N, 0, N, 0);
+    if (jb < nb)
+    {
+       if (jb >= ATL_VWipgen_100LCMMN)
+          nb = (jb/ATL_VWipgen_100LCMMN)*ATL_VWipgen_100LCMMN;
+       else if (jb >= 4)
+          nb = 4;
+       else
+          nb = jb;
+    }
     if (!nb) return(-6);  /* need at least 1 row of workspace */
 /*
  *    Only first iteration will have partial block, unroll it
