@@ -2,14 +2,14 @@
  * HERK actually uses real gemm, so use real blocking factors
  */
 #ifdef SCPLX
-   #define CMM_H
-   #include "smm.h"
+   #include "atlas_samm_sum.h"
 #elif defined(DCPLX)
-   #define ZMM_H
-   #include "zmm.h"
+   #include "atlas_damm_sum.h"
 #endif
 #include "atlas_misc.h"
+#define ATL_NOAMM 1
 #include "atlas_lvl3.h"
+#undef ATL_NOAMM
 #include "atlas_level3.h"
 #include "atlas_level1.h"
 #include "atlas_lapack.h"
@@ -24,36 +24,10 @@
 #else
    #define llt_syrk cblas_syrk
    #define llt_trans AtlasTrans
-   #define lda2 lda
 #endif
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1999 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1999 R. Clint Whaley
  */
 static int ATL_potrf2(const int N, TYPE *A, const int lda)
 {
@@ -230,13 +204,13 @@ static int ATL_potrfL_2(TYPE *A, const int lda)
 int ATL_potrfL(const int N, TYPE *A, const int lda)
 {
    TYPE *An, *Ar;
+   const size_t lda2=(lda SHIFT);
    int Nleft, Nright, ierr;
    #ifdef TREAL
       #define lda2 lda
       #define ONE ATL_rone
    #else
       static const TYPE ONE[2] = {ATL_rone, ATL_rzero};
-      const int lda2=lda<<1;
    #endif
 
 #ifdef TREAL
@@ -246,8 +220,9 @@ int ATL_potrfL(const int N, TYPE *A, const int lda)
 #endif
    {
       Nleft = N >> 1;
-      #ifdef NB
-         if (Nleft > NB<<1) Nleft = ATL_MulByNB(ATL_DivByNB(Nleft));
+      #ifdef ATL_VWipgen_100LCMMN
+         if (Nleft > ATL_VWipgen_100LCMMN<<1)
+            Nleft = (Nleft/ATL_VWipgen_100LCMMN)*ATL_VWipgen_100LCMMN;
       #endif
       Nright = N - Nleft;
       ierr = ATL_potrfL(Nleft, A, lda);
