@@ -1,5 +1,4 @@
-#include "atlas_misc.h"
-#include "atlas_threads.h"
+#include "atlas_tprim.h"
 int ATL_thread_join(ATL_thread_t *thr)   /* waits on completion of thread */
 {
 #ifdef ATL_WINTHREADS
