@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2009 Siju Samuel
- *
  * Code contributers : Siju Samuel, Anthony M. Castaldo, R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 /*-----------------------------------------------------------------------------
  * This is the C translation of the standard LAPACK Fortran routine:
@@ -161,6 +135,7 @@
 #include "atlas_threads.h"
 #include "atlas_tlvl3.h"
 #include "atlas_level2.h"
+#include "atlas_lamch.h"
 #include "math.h"
 #include "errno.h"
 
@@ -281,9 +256,10 @@ typedef struct                              /* Threading structure.           */
    } /* END #define dnrm2_combine */
 
 
-//-----------------------------------------------------------------------------
-//  ATL_dgeqr2_dnrm2: Compute a dnrm2, but retain scale and value.
-// ---------------------------------------------------------------------------
+/*----------------------------------------------------------------------------
+ *  ATL_dgeqr2_dnrm2: Compute a dnrm2, but retain scale and value.
+ * ---------------------------------------------------------------------------
+ */
 
 static void ATL_geql2_dnrm2_ql(ATL_DGEQL2_t *ts)
 {
@@ -312,7 +288,7 @@ static void ATL_geql2_dnrm2_ql(ATL_DGEQL2_t *ts)
             {
                SSQ += (aX/Scale)*(aX/Scale);
             }
-         } //for
+         } /*for */
       }
    } else                                   /* This is the Rank 0 Operation   */
    {
@@ -334,7 +310,7 @@ static void ATL_geql2_dnrm2_ql(ATL_DGEQL2_t *ts)
                SSQ += sX*sX;                /* Square and add to SSQ.         */
             }
          }
-      }//for
+      }/*for */
    }
 
    ts->Scale = Scale;
@@ -422,7 +398,7 @@ static void ATL_geql2_Cache(ATL_DGEQL2_t *myTS)
  */
 static int Mjoin(PATL,StructIsInitGEQL2)(void *vp)
 {
-   //return 1;
+   /*return 1; */
    return(((ATL_DGEQL2_t*)vp)->active);
 }
 
@@ -431,7 +407,7 @@ static int Mjoin(PATL,StructIsInitGEQL2)(void *vp)
  * ATL_geql2Worker: Persistent for the duration of the DGEQL2 operation.
  * Argument is pointer to a structure which contains our data section.
  */
-//void* ATL_geql2Worker(void *myArg)
+/*void* ATL_geql2Worker(void *myArg) */
 static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 {
    ATL_thread_t *tp=vp;
@@ -444,17 +420,18 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
    int   myCopy=myTS->copy;
    volatile ATL_DGEQL2_t *ptnr;             /* partner log2 combine.          */
    TYPE  *A, *scaleA, *T;                   /* Work variables.                */
-   TYPE XNORM, BETA, BETAp, RSAFMN;
-   TYPE *TAU = myTS->TAU;                   /* Short cut.                     */
+   TYPE XNORM, BETA, BETAp;
+   TYPE *TAU = myTS->TAU;                   /* Short cut.              */
    TYPE ALPHA;
+   TYPE w;
 
    #ifdef TREAL
       const TYPE ONE = ATL_rone;
       TYPE AII ;
       TYPE TAUVAL ;
       TYPE myTAUi;
-      TYPE w;                               /* Scaling Factor                 */
-      TYPE wfact;                           /* Scaling Factor                 */
+      TYPE sc;
+      TYPE RSAFMN;
       const TYPE ZERO = ATL_rzero;
    #else
       const TYPE ONE[2] = {ATL_rone, ATL_rzero};
@@ -465,8 +442,8 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 
       TYPE ALPHAI, ALPHAR;
       TYPE ALPHADIV[2] ;
-      TYPE w;                               /* Scaling Factor                 */
-      TYPE wfact[2];                        /* Scaling Factor                 */
+      TYPE sc[2];
+      TYPE RSAFMN[2];
       const TYPE ZERO[2] = {ATL_rzero, ATL_rzero};
    #endif
 
@@ -544,29 +521,38 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 
       myTS->dnrm2++;                        /* Signal I am done.              */
 
+
       /****************************************/
       /********  S Y N C   P O I N T  *********/
       /********  S Y N C   P O I N T  *********/
       /********  S Y N C   P O I N T  *********/
       /****************************************/
-/*----------------------------------------------------------------------------*/
-/*    Note: We want to avoid syncing after DGER, but                          */
-/*    it is using zTS->WORK as one vector. So it is                           */
-/*    important zTS->WORK is NOT corrupted before this                        */
-/*    sync. All threads converge here for the next col                        */
-/*    and so will be done with zTS->WORK at this point                        */
-/*    and each individually will be ready to start its                        */
-/*    dnrm2 work on its share.                                                */
-/*----------------------------------------------------------------------------*/
+
+      /*-----------------------------------------------------------------*/
+      /* Note: We can avoid syncing immediately after the DGER, but DGER */
+      /* does use zTS->WORK as its vector. So here is the point where we */
+      /* have to make sure everybody's DGER is done, and they have done  */
+      /* their norm2 work. All threads converge here for the next column */
+      /* and so will be done with their DGER, and zTS->WORK is free.     */
+      /*-----------------------------------------------------------------*/
       while (zTS->dnrm2 < myTS->dnrm2);     /* Wait for zero to finish.       */
 
-/*----------------------------------------------------------------------------*/
-/*    At this point, zTS->zDiag is valid.                                     */
-/*    However, if we are going to skip the column because SSQ == 0, then we   */
-/*    will require another synchronization in order to make sure everybody    */
-/*    sees the SSQ before we replace it.                                      */
-/*----------------------------------------------------------------------------*/
-      if (zTS->SSQ == 0.)                   /* If result was zero,            */
+      /*-------------------------------------------------------------*/
+      /* At this point, zTS->SSQ is computed. If it is zero, then no */
+      /* rotation is needed, and TAU should be set to zero, and we   */
+      /* just skip to the next column.                               */
+      /* HOWEVER, core zero may be fast on the next compare, and     */
+      /* change zTS->SSQ before some other core gets a chance to see */
+      /* it was zero. So we must have a barrier here before we go on.*/
+      /*                                                             */
+      /* In Complex we require both SSQ == 0 and IMAG(A[i,i]) == 0.  */
+      /*-------------------------------------------------------------*/
+      /* If H should be Identity, set TAU to zero and go to next column. */
+      #ifdef TREAL
+      if (zTS->SSQ == 0.)
+      #else /* COMPLEX */
+      if (zTS->SSQ == 0. && zTS->zDiag[1] == 0.)
+      #endif /* REAL or CPLX */
       {
          zees = myRank;                     /* Init the test flags.           */
          pair = 1;                          /* Starting pair.                 */
@@ -584,21 +570,27 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
             pair <<= 1;                     /* Double the pair idx.           */
          }
 
-         if (myRank == 0)
+         if (myRank == 0)                   /* Replace A[i,i] we repl with 1. */
          {
             #ifdef TREAL
             *(A + M -1 - idup) =  myTS->zDiag;
+            TAU[i] = 0.;                    /* clear TAU so H[i]=Identity.    */
             #else
             *(A + ((M -1 - idup) SHIFT)) =  myTS->zDiag[0];
             *(A + ((M -1 - idup) SHIFT) +1) =  myTS->zDiag[1];
+            TAU[(i SHIFT)] = 0.;            /* clear TAU so H[i]=Identity.    */
+            TAU[(i SHIFT)+1] = 0.;
             #endif
          }
 
          myTS->dnrm2++;                     /* Signal I am done.              */
          while (zTS->dnrm2 < myTS->dnrm2);  /* Wait for zero to finish.       */
-         continue;                          /* ..Nothing to do on column.     */
-      }
+         continue;                          /* ..Proceed to next column.      */
+      } /* END if H=Identity no need to process column. */
 
+/*----------------------------------------------------------------------------*/
+/* Here, H is not identity, we 'continued' the loop if it was.                */
+/*----------------------------------------------------------------------------*/
       XNORM = (zTS->Scale)*sqrt(zTS->SSQ);  /* Compute the norm.              */
 
 /*----------------------------------------------------------------------------*/
@@ -608,98 +600,185 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 
    #ifdef TREAL
 /*----------------------------------------------------------------------------*/
-/*    The following code is inlined from ATL_larfg.                           */
+/*    The following code is inlined from ATL_larfg; the main difference is    */
+/*    that we use zTS->zDiag instead of ALPHA, and recompute is parallel.     */
 /*----------------------------------------------------------------------------*/
-
       BETAp = ATL_lapy2((zTS->zDiag), XNORM);    /* Get sqrt(a^2+b^2)         */
-/*    Above : already computed using norm                                     */
+      BETA = BETAp;                              /* Assume diagonal < 0 ...   */
+      if ((zTS->zDiag) >= 0.) BETA = -BETAp;     /* .. If >= 0, change sign.  */
 
-      BETA = BETAp;                         /* Assume diagonal < 0            */
-      if ((zTS->zDiag) < 0.) BETA = 0.-BETAp;    /* Change sign ..            */
-      ALPHA = zTS->zDiag + BETA;
-      if ( BETA < 0.0)
+      KNT = 0;                                   /* Init power to zero. */
+      if (BETAp < ATL_laSAFMIN)
       {
-         BETA = 0. - BETA;
-         myTAUi = (0. - ALPHA)/BETA;
-      }
-      else
-      {
-         ALPHA = XNORM * (XNORM/(ALPHA));
-         myTAUi   = ALPHA/BETA;
-         ALPHA = 0.0 - ALPHA ;
+         RSAFMN = ATL_rone / ATL_laSAFMIN;       /* Set a maximum. */
+
+         /*---------------------------------------------------------------*/
+         /* BETAp is the same for all cores, so this loop can be executed */
+         /* independently. However, XNORM must be computed in concert.    */
+         /* The new BETAp will be at most 1, at least SAFMIN.             */
+         /*---------------------------------------------------------------*/
+         while (BETAp < ATL_laSAFMIN)
+         {
+            KNT++;                              /* increment power. */
+            if ( myTS->active == 1)             /* If I am active, */
+            {
+               cblas_scal(mScale, RSAFMN,
+                                    scaleA, 1); /* Scale my share. */
+            }
+
+            BETA *= RSAFMN;
+            BETAp *= RSAFMN;
+            if (myRank==0) zTS->zDiag *= RSAFMN;   /* Only done by core 0! */
+         }
+
+         /* i starts from N -1. */
+         ATL_geql2_dnrm2_ql(myTS);             /* Do my share of new norm2. */
+
+         zees = myRank;                        /* Init the test flags.    */
+         pair = 1;                             /* Starting pair.          */
+         while ( (zees & 1) == 0 &&            /* If I must wait on pair, */
+            (myRank)+pair < ATL_NTHREADS)      /* ..and pair exists,      */
+         {
+            ptnr = myTS+pair;                  /* Point at my partner.    */
+
+            if (ptnr->active == 1)             /* If ptnr was used,       */
+            {
+               while (ptnr->dnrm2 <= myTS->dnrm2);  /* Wait for it.       */
+                  dnrm2_combine_ql;
+            }
+
+            zees >>= 1;                        /* Shift a zero out.       */
+            pair <<= 1;                        /* Double the pair idx.    */
+         }
+
+         myTS->dnrm2++;                        /* Signal I am done.       */
+
+         /*************************************/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /*************************************/
+         while (zTS->dnrm2 < myTS->dnrm2);         /* Wait on zero to finish. */
+
+         XNORM = (zTS->Scale)*sqrt(zTS->SSQ);      /* Compute the norm. */
+         BETAp = ATL_lapy2((zTS->zDiag), XNORM);   /* Get sqrt(a^2+b^2) */
+         BETA = BETAp;                             /* Assume diagonal < 0 ... */
+         if ((zTS->zDiag) >= 0.) BETA = 0.-BETAp;  /* ..If >= 0, change sign. */
       }
 
-      if (myRank == 0) TAU[i] = myTAUi;     /* Set if rank zero.              */
-
-      wfact = 1./(ALPHA);                   /* Compute scaling factor.        */
+      myTAUi = (BETA-(zTS->zDiag)) / BETA;         /* Compute TAU[i]. */
+      if (myRank == 0) TAU[i] = myTAUi;            /* Set if I own TAU. */
 
+      sc = ATL_rone/((zTS->zDiag)-BETA);           /* Find scaling factor. */
       if ( myTS->active == 1)
       {
-         cblas_scal(mScale, wfact, scaleA, 1);  /* Scale the vector.          */
+         cblas_scal(mScale, sc, scaleA, 1);        /* Scale the vector. */
       }
 
-      if (myRank == 0) AII = BETA;          /* If core 0, remember.           */
-   #else
-      ALPHAR =  zTS->zDiag[0];
-      ALPHAI =  zTS->zDiag[1];
-
-      BETAp = ATL_lapy3(zTS->zDiag[0], zTS->zDiag[1], XNORM);
-                                                     /* Get sqrt(a^2+b^2,c^2) */
-/*    Above : already computed using norm                                     */
-
-      BETA = BETAp;                         /* Assume diagonal < 0            */
-      if ((zTS->zDiag[0]) < 0.) BETA = 0.-BETAp; /* Change sign ..            */
-
-      ALPHA = zTS->zDiag[0] + BETA;
-      if ( BETA < 0.0)
+      if (myRank == 0)                             /* If I own diagonal, */
       {
-         BETA = 0. - BETA;
-         myTAUi[0] = (0. - ALPHA)/BETA;
-         myTAUi[1] = (0. - zTS->zDiag[1])/BETA;
-      }
-      else
-      {
-         ALPHAR = zTS->zDiag[1] * ( zTS->zDiag[1] / ALPHA ) ;
-         ALPHAR = ALPHAR + XNORM * ( XNORM / ALPHA ) ;
-
-/*       TAU = DCMPLX( ALPHR/BETA, -ALPHI/BETA )                              */
-         myTAUi[0] = ALPHAR/BETA;
-         myTAUi[1] = (0. - zTS->zDiag[1])/BETA;
-
-         ALPHA = 0.0 - ALPHAR;
+         AII = BETA;                               /* Set new A[i,i] element. */
+         for (j=0; j<KNT; j++)                     /* Rescaling loop...       */
+            AII *= ATL_laSAFMIN;                   /* ...Adjust it.           */
       }
 
+   #else /* COMPLEX VERSION of LARFG, modeled on clarfg.f. */
+      ALPHAR =  zTS->zDiag[0];                     /* Real portion. */
+      ALPHAI =  zTS->zDiag[1];                     /* Imaginary portion. */
 
-      if (myRank == 0)                      /* Set if rank zero.              */
+      BETAp = ATL_lapy3(ALPHAR, ALPHAI, XNORM);    /* sqrt(a^2+b^2,c^2) */
+      BETA = BETAp;                                /* Assume ALPHAR < 0.*/
+      if (ALPHAR >= 0.) BETA = -BETAp;             /* If >=0, Change sign. */
+
+      KNT = 0;                                     /* Init power to zero. */
+      if (BETAp < ATL_laSAFMIN)
       {
+         RSAFMN[0] = ATL_rone/ATL_laSAFMIN;        /* Set a maximum. */
+         RSAFMN[1] = ATL_rzero;                    /* ..for scaling. */
 
-/*      Taking conjugate for TAU for RQ                                       */
-/*      This value is passed as output                                        */
-	     #ifdef BUILD_RQ2
-            TAU[(i SHIFT)] = myTAUi[0];
-            TAU[(i SHIFT) +1] = 0 - myTAUi[1];
+         /*---------------------------------------------------------------*/
+         /* BETAp is the same for all cores, so this loop can be executed */
+         /* independently. However, XNORM must be computed in concert.    */
+         /* The new BETAp will be at most 1, at least SAFMIN.             */
+         /*---------------------------------------------------------------*/
+         while (BETAp < ATL_laSAFMIN)
+         {
+            KNT++;                                 /* increment power. */
+            if ( myTS->active == 1)                /* If I am active, */
+            {
+               cblas_scal(mScale, RSAFMN,
+                                    scaleA, 1);    /* Scale my share. */
+            }
+
+            BETA *= RSAFMN[0];
+            BETAp *= RSAFMN[0];
+            ALPHAR *= RSAFMN[0];
+            ALPHAI *= RSAFMN[0];
+         }
+
+         /* i starts from N -1. */
+         ATL_geql2_dnrm2_ql(myTS);             /* Do my share of new norm2. */
+
+         zees = myRank;                        /* Init the test flags.      */
+         pair = 1;                             /* Starting pair.            */
+         while ( (zees & 1) == 0 &&            /* If I must wait on pair,   */
+            (myRank)+pair < ATL_NTHREADS)      /* ..and pair exists,        */
+         {
+            ptnr = myTS+pair;                  /* Point at my partner.      */
+
+            if (ptnr->active == 1)             /* If ptnr was used,         */
+            {
+               while (ptnr->dnrm2 <= myTS->dnrm2);  /* Wait for it.         */
+                  dnrm2_combine_ql;
+            }
+
+            zees >>= 1;                        /* Shift a zero out.         */
+            pair <<= 1;                        /* Double the pair idx.      */
+         }
+
+         myTS->dnrm2++;                        /* Signal I am done.         */
+
+         /*************************************/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /*************************************/
+         while (zTS->dnrm2 < myTS->dnrm2);         /* Wait on zero to finish.*/
+
+         XNORM = (zTS->Scale)*sqrt(zTS->SSQ);      /* Compute the norm. */
+         BETAp = ATL_lapy3(ALPHAR, ALPHAI, XNORM); /* sqrt(a^2+b^2,c^2) */
+         BETA = BETAp;                             /* Assume ALPHAR < 0.*/
+         if (ALPHAR >= 0.) BETA = 0.-BETAp;        /* If >=0, Change sign. */
+      }
+
+      myTAUi[0] = (BETA-ALPHAR) / BETA;      /* Compute real part. */
+      myTAUi[1] = (0.-ALPHAI) / BETA;        /* Compute imag part. */
+
+      ALPHADIV[0] = ALPHAR - BETA;           /* prepare for 1/(alpha-beta) */
+      ALPHADIV[1] = ALPHAI;                  /* ...                        */
+      ATL_ladiv(ONE, ALPHADIV, sc);          /* compute scaling factor.    */
+      if (myTS->active == 1)                 /* If I have some of vector,  */
+      {
+         cblas_scal(mScale, sc, scaleA, 1);  /* ..scale it. */
+      }
+
+      if (myRank == 0)                       /* If I own TAU, */
+      {
+         TAU[(i SHIFT)] = myTAUi[0];
+         #ifdef BUILD_RQ2
+            TAU[(i SHIFT) +1] = -myTAUi[1];  /* RQ2 needs conjugate. */
          #else
-            TAU[(i SHIFT)] = myTAUi[0];
-            TAU[(i SHIFT) +1] = myTAUi[1];
+            TAU[(i SHIFT) +1] = myTAUi[1];   /* otherwise use normal. */
          #endif
       }
-      ALPHADIV[0] = ALPHA;
-      ALPHADIV[1] = ALPHAI;
 
-/*    Perform complex division before scaling the X vector                    */
-      ATL_ladiv( ONE,  ALPHADIV  ,wfact);       /* ALPHA will have the result */
-
-      if ( myTS->active == 1)
+      if (myRank == 0)                       /* If I own diagonal, */
       {
-         cblas_scal(mScale, wfact, scaleA, 1);  /* Scale the vector.          */
-      }
-
-      if (myRank == 0)
-      {
-         AII[0] = BETA;                     /* If core 0, remember.           */
+         for (j=0; j<KNT; j++) BETA *= ATL_laSAFMIN;  /* Rescale BETA. */
+         AII[0] = BETA;                               /* save for later. */
          AII[1] = 0.0;
       }
-   #endif
+   #endif /* COMPLEX version of larfg. */
 
 /*----------------------------------------------------------------------------*/
 /* Now we apply dlarf, if we are not on the last column. This is              */
@@ -914,8 +993,8 @@ static void* ATL_geql2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
             #ifdef BUILD_RQ2
                 for (i_loop = 0; i_loop < idup; i_loop++)
                 {
-                     (myTS->WORK+(os SHIFT))[(i_loop SHIFT) + 0] =
-                                 0.0 - (myTS->WORK+(os SHIFT))[(i_loop SHIFT) + 0];
+                     (myTS->WORK+(os SHIFT))[(i_loop SHIFT)    ] =
+                         -(myTS->WORK+(os SHIFT))[(i_loop SHIFT)    ];
                 }
             #endif
          #endif
@@ -1053,7 +1132,7 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
 /* We use a data-owner split, each thread gets 1/p of the data,               */
 /* and does all computation related to it.                                    */
 /*----------------------------------------------------------------------------*/
-   TYPE *myA = A, *allMem, *workMem;
+   TYPE *myA = A, *allMem=NULL, *workMem;
    int i, j, k, b0, b, th;
    long unsigned int CPU;
    size_t mem[ATL_NTHREADS], totmem, workSize;
@@ -1075,7 +1154,7 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
    }
 
    if (b == 0) th = 1;
-   if (th == 1 || (M < N) )                             /* If impossible to split,        */
+   if (th == 1 || (M < N) )       /* If impossible to split, use serial. */
    {
 
       #ifdef BUILD_RQ2
@@ -1199,7 +1278,8 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
    ATL_goparallel(th, ATL_geql2Worker, ts, NULL);
 
 
-   if (myCopy) free(allMem);                /* release copied area.           */
+   if (allMem)
+      free(allMem);                         /* release copied area.           */
    free(workMem);                           /* release work area.             */
    return(0);                               /* Done with dgeql2.              */
 } /* END ATL_t_dgeql2 */
