@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2009 Siju Samuel
- *
  * Code contributers : Siju Samuel, Anthony M. Castaldo, R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 /*
  * This is the C translation of the standard LAPACK Fortran routine:
@@ -158,6 +132,7 @@
 #include "atlas_threads.h"
 #include "atlas_tlvl3.h"
 #include "atlas_level2.h"
+#include "atlas_lamch.h"
 #include "math.h"
 #include "errno.h"
 
@@ -246,8 +221,8 @@ double time00(void);                        /* prototype external rtn.        */
       ".align 4096;\n\t"            \
       : : : "%eax" )
 
-//int ATL_geqr2_Order[ATL_NTHREADS]={0,4,2,6,1,5,3,7};
-//int ATL_geqr2_Order[ATL_NTHREADS]={0,2,1,3};
+/*int ATL_geqr2_Order[ATL_NTHREADS]={0,4,2,6,1,5,3,7}; */
+/*int ATL_geqr2_Order[ATL_NTHREADS]={0,2,1,3}; */
 
 typedef struct                              /* Threading structure.           */
 {
@@ -295,7 +270,7 @@ typedef struct                              /* Threading structure.           */
    }                                        /* END #define dnrm2_combine      */
 
 
-//double dgeqr2_SAFMIN=-1.;
+/*double dgeqr2_SAFMIN=-1.; */
 
 /*-----------------------------------------------------------------------------
  * ATL_dgeqr2_dnrm2: Compute a dnrm2, but retain scale and value.
@@ -327,7 +302,7 @@ static void ATL_geqr2_dnrm2(ATL_DGEQR2_t *ts)
             {
                SSQ += (aX/Scale)*(aX/Scale);
             }
-         } //for
+         } /*for */
       }
    } else                                   /* This is the Rank 0 Operation   */
    {
@@ -348,12 +323,12 @@ static void ATL_geqr2_dnrm2(ATL_DGEQR2_t *ts)
                SSQ += sX*sX;                /* Square and add to SSQ.         */
             }
          }
-      }//for
+      }/*for */
    }
 
    ts->Scale = Scale;
    ts->SSQ   = SSQ;
-//    fprintf(stderr, "scale %f ssq %f \n", Scale, SSQ );
+/*    fprintf(stderr, "scale %f ssq %f \n", Scale, SSQ ); */
 }                                           /* END ATL_dgeqr2_dnrm2           */
 
 
@@ -439,7 +414,7 @@ static void ATL_geqr2_Cache(ATL_DGEQR2_t *myTS)
  */
 static int Mjoin(PATL,StructIsInitGEQR2)(void *vp)
 {
-   //return 1;
+   /*return 1; */
    return(((ATL_DGEQR2_t*)vp)->active);
 }
 
@@ -448,7 +423,7 @@ static int Mjoin(PATL,StructIsInitGEQR2)(void *vp)
  * ATL_geqr2Worker: Persistent for the duration of the DGEQR2 operation.
  * Argument is pointer to a structure which contains our data section.
  */
-//void* ATL_geqr2Worker(void *myArg)
+/*void* ATL_geqr2Worker(void *myArg) */
 static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 {
    ATL_thread_t *tp = vp;
@@ -460,17 +435,17 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
    int   myCopy = myTS->copy;
    volatile ATL_DGEQR2_t *ptnr;                 /* partner log2 combine.     */
    TYPE  *A, *scaleA,  *T;                      /* Work variables.           */
-   TYPE XNORM, BETA, BETAp, RSAFMN;
+   TYPE XNORM, BETA, BETAp;
    TYPE *TAU = myTS->TAU;                       /* Short cut.                */
    TYPE ALPHA;
-
+   TYPE w;                                      /* Used in dnrm2_combine mac.*/
    #ifdef TREAL
       const TYPE ONE = ATL_rone;
       TYPE AII ;
       TYPE TAUVAL ;
       TYPE myTAUi;
-      TYPE w;                               /* Scaling Factor                 */
-      TYPE wfact;                           /* Scaling Factor                 */
+      TYPE sc;
+      TYPE RSAFMN;
       const TYPE ZERO = ATL_rzero;
    #else
       const TYPE ONE[2] = {ATL_rone, ATL_rzero};
@@ -481,8 +456,8 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 
       TYPE ALPHAI, ALPHAR;
       TYPE ALPHADIV[2] ;
-      TYPE w;                               /* Scaling Factor                 */
-      TYPE wfact[2];                        /* Scaling Factor                 */
+      TYPE sc[2];
+      TYPE RSAFMN[2];
       const TYPE ZERO[2] = {ATL_rzero, ATL_rzero};
    #endif
 
@@ -553,30 +528,37 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
 
       myTS->dnrm2++;                        /* Signal I am done.              */
 
-/***************************************                                      */
-/********  S Y N C   P O I N T  ********                                      */
-/********  S Y N C   P O I N T  ********                                      */
-/********  S Y N C   P O I N T  ********                                      */
-/***************************************                                      */
-/*----------------------------------------------------------------------------*/
-/* Note: We want to avoid syncing after DGER, but                             */
-/* it is using zTS->WORK as one vector. So it is                              */
-/* important zTS->WORK is NOT corrupted before this                           */
-/* sync. All threads converge here for the next col                           */
-/* and so will be done with zTS->WORK at this point                           */
-/* and each individually will be ready to start its                           */
-/* dnrm2 work on its share.                                                   */
-/*----------------------------------------------------------------------------*/
+      /*************************************/
+      /******  S Y N C   P O I N T  ********/
+      /******  S Y N C   P O I N T  ********/
+      /******  S Y N C   P O I N T  ********/
+      /*************************************/
+
+      /*-----------------------------------------------------------------*/
+      /* Note: We can avoid syncing immediately after the DGER, but DGER */
+      /* does use zTS->WORK as its vector. So here is the point where we */
+      /* have to make sure everybody's DGER is done, and they have done  */
+      /* their norm2 work. All threads converge here for the next column */
+      /* and so will be done with their DGER, and zTS->WORK is free.     */
+      /*-----------------------------------------------------------------*/
       while (zTS->dnrm2 < myTS->dnrm2);     /* Wait for zero to finish.       */
 
-
-/*----------------------------------------------------------------------------*/
-/*    At this point, zTS->zDiag is valid.                                     */
-/*    However, if we are going to skip the column because SSQ == 0, then we   */
-/*    will require another synchronization in order to make sure everybody    */
-/*    sees the SSQ before we replace it.                                      */
-/*----------------------------------------------------------------------------*/
-      if (zTS->SSQ == 0.)                   /* If result was zero,            */
+      /*-------------------------------------------------------------*/
+      /* At this point, zTS->SSQ is computed. If it is zero, then no */
+      /* rotation is needed, and TAU should be set to zero, and we   */
+      /* just skip to the next column.                               */
+      /* HOWEVER, core zero may be fast on the next compare, and     */
+      /* change zTS->SSQ before some other core gets a chance to see */
+      /* it was zero. So we must have a barrier here before we go on.*/
+      /*                                                             */
+      /* In Complex we require both SSQ == 0 and IMAG(A[i,i]) == 0.  */
+      /*-------------------------------------------------------------*/
+      /* If H should be Identity, set TAU to zero and go to next column. */
+      #ifdef TREAL
+      if (zTS->SSQ == 0.)
+      #else /* COMPLEX */
+      if (zTS->SSQ == 0. && zTS->zDiag[1] == 0.)
+      #endif /* REAL or CPLX */
       {
          zees = myRank;                     /* Init the test flags.           */
          pair = 1;                          /* Starting pair.                 */
@@ -594,117 +576,208 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
             pair <<= 1;                     /* Double the pair idx.           */
          }
 
-         if (myRank == 0)
+         if (myRank == 0)                   /* Restore A[i,i] we replaced w 1.*/
          {
             #ifdef TREAL
             *(A) =  zTS->zDiag;
+            TAU[i] = 0.;                    /* clear TAU so H[i]=Identity.    */
             #else
             *(A )  = zTS->zDiag[0];
             *(A+1) = zTS->zDiag[1];
+            TAU[(i SHIFT)] = 0.;            /* clear TAU so H[i]=Identity.    */
+            TAU[(i SHIFT)+1] = 0.;
             #endif
          }
 
          myTS->dnrm2++;                     /* Signal I am done.              */
          while (zTS->dnrm2 < myTS->dnrm2);  /* Wait for zero to finish.       */
-         continue;                          /* ..Nothing to do on column.     */
-      }
+         continue;                          /* ..Proceed to next column.      */
+      } /* END if H=Identity no need to process column. */
 
+/*----------------------------------------------------------------------------*/
+/* Here, H is not identity, we 'continued' the loop if it was.                */
+/*----------------------------------------------------------------------------*/
       XNORM = (zTS->Scale)*sqrt(zTS->SSQ);  /* Compute the norm.              */
 
+      #ifdef TREAL
 /*----------------------------------------------------------------------------*/
-/* The following code is inlined from ATL_larfg.                              */
-/* Here, SSQ is non-zero, we exited the loop above if that occurred.          */
-/*----------------------------------------------------------------------------*/
-   #ifdef TREAL
-/*----------------------------------------------------------------------------*/
-/*    The following code is inlined from ATL_larfg.                           */
+/*    The following code is inlined from ATL_larfg; the main difference is    */
+/*    that we use zTS->zDiag instead of ALPHA, and recompute is parallel.     */
 /*----------------------------------------------------------------------------*/
       BETAp = ATL_lapy2((zTS->zDiag), XNORM);    /* Get sqrt(a^2+b^2)         */
-      BETA = BETAp;                              /* Assume diagonal < 0       */
-      if ((zTS->zDiag) < 0.) BETA = 0.-BETAp;    /* Change sign.              */
+      BETA = BETAp;                              /* Assume diagonal < 0 ...   */
+      if ((zTS->zDiag) >= 0.) BETA = -BETAp;     /* .. If >= 0, change sign.  */
 
-      ALPHA = zTS->zDiag + BETA;
-      if ( BETA < 0.0)
+      KNT = 0;                                   /* Init power to zero. */
+      if (BETAp < ATL_laSAFMIN)
       {
-         BETA = 0. - BETA;
-         myTAUi = (0. - ALPHA)/BETA;
-      }
-      else
-      {
-         ALPHA = XNORM * (XNORM/(ALPHA));
-         myTAUi   = ALPHA/BETA;
-         ALPHA = 0.0 - ALPHA ;
+         RSAFMN = ATL_rone / ATL_laSAFMIN;       /* Set a maximum. */
+
+         /*---------------------------------------------------------------*/
+         /* BETAp is the same for all cores, so this loop can be executed */
+         /* independently. However, XNORM must be computed in concert.    */
+         /* The new BETAp will be at most 1, at least SAFMIN.             */
+         /*---------------------------------------------------------------*/
+         while (BETAp < ATL_laSAFMIN)
+         {
+            KNT++;                              /* increment power. */
+            if ( myTS->active == 1)             /* If I am active, */
+            {
+               cblas_scal(mScale, RSAFMN,
+                                    scaleA, 1); /* Scale my share. */
+            }
+
+            BETA *= RSAFMN;
+            BETAp *= RSAFMN;
+            if (myRank==0) zTS->zDiag *= RSAFMN;   /* Only done by core 0! */
+         }
+
+         ATL_geqr2_dnrm2(myTS);                /* Do my share of new norm2. */
+
+         zees = myRank;                        /* Init the test flags.    */
+         pair = 1;                             /* Starting pair.          */
+         while ( (zees & 1) == 0 &&            /* If I must wait on pair, */
+            (myRank)+pair < ATL_NTHREADS)      /* ..and pair exists,      */
+         {
+            ptnr = myTS+pair;                  /* Point at my partner.    */
+
+            if (ptnr->active == 1)             /* If ptnr was used,       */
+            {
+               while (ptnr->dnrm2 <= myTS->dnrm2);  /* Wait for it.       */
+                  dnrm2_combine;
+            }
+
+            zees >>= 1;                        /* Shift a zero out.       */
+            pair <<= 1;                        /* Double the pair idx.    */
+         }
+
+         myTS->dnrm2++;                        /* Signal I am done.       */
+
+         /*************************************/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /*************************************/
+         while (zTS->dnrm2 < myTS->dnrm2);         /* Wait on zero to finish. */
+
+         XNORM = (zTS->Scale)*sqrt(zTS->SSQ);      /* Compute the norm. */
+         BETAp = ATL_lapy2((zTS->zDiag), XNORM);   /* Get sqrt(a^2+b^2) */
+         BETA = BETAp;                             /* Assume diagonal < 0 ... */
+         if ((zTS->zDiag) >= 0.) BETA = 0.-BETAp;  /* ..If >= 0, change sign. */
       }
 
-      if (myRank == 0) TAU[i] = myTAUi;     /* Set if rank zero.              */
-
-      wfact = 1./(ALPHA);                   /* Compute scaling factor.        */
+      myTAUi = (BETA-(zTS->zDiag)) / BETA;         /* Compute TAU[i]. */
+      if (myRank == 0) TAU[i] = myTAUi;            /* Set if I own TAU. */
 
+      sc = ATL_rone/((zTS->zDiag)-BETA);           /* Find scaling factor. */
       if ( myTS->active == 1)
       {
-         cblas_scal(mScale, wfact, scaleA, 1);   /* Scale the vector.         */
+         cblas_scal(mScale, sc, scaleA, 1);        /* Scale the vector. */
       }
 
-      if (myRank == 0) AII = BETA;          /* If core 0, remember.           */
-   #else
-      ALPHAR =  zTS->zDiag[0];
-      ALPHAI =  zTS->zDiag[1];
-
-      BETAp = ATL_lapy3(zTS->zDiag[0], zTS->zDiag[1], XNORM);
-                                            /* Get sqrt(a^2+b^2,c^2)          */
-
-      BETA = BETAp;                         /* Assume diagonal < 0            */
-      if ((zTS->zDiag[0]) < 0.) BETA = 0.-BETAp; /* Change sign ..            */
-
-      ALPHA = zTS->zDiag[0] + BETA;
-      if ( BETA < 0.0)
+      if (myRank == 0)                             /* If I own diagonal, */
       {
-         BETA = 0. - BETA;
-         myTAUi[0] = (0. - ALPHA)/BETA;
-         myTAUi[1] = (0. - zTS->zDiag[1])/BETA;
+         AII = BETA;                               /* Set new A[i,i] element. */
+         for (j=0; j<KNT; j++)                     /* Rescaling loop...       */
+            AII *= ATL_laSAFMIN;                   /* ...Adjust it.           */
       }
-      else
+
+   #else /* COMPLEX VERSION of LARFG, modeled on clarfg.f. */
+      ALPHAR =  zTS->zDiag[0];                     /* Real portion. */
+      ALPHAI =  zTS->zDiag[1];                     /* Imaginary portion. */
+
+      BETAp = ATL_lapy3(ALPHAR, ALPHAI, XNORM);    /* sqrt(a^2+b^2,c^2) */
+      BETA = BETAp;                                /* Assume ALPHAR < 0.*/
+      if (ALPHAR >= 0.) BETA = -BETAp;             /* If >=0, Change sign. */
+
+      KNT = 0;                                     /* Init power to zero. */
+      if (BETAp < ATL_laSAFMIN)
       {
-         ALPHAR = zTS->zDiag[1] * ( zTS->zDiag[1] / ALPHA ) ;
-         ALPHAR = ALPHAR + XNORM * ( XNORM / ALPHA ) ;
+         RSAFMN[0] = ATL_rone/ATL_laSAFMIN;        /* Set a maximum. */
+         RSAFMN[1] = ATL_rzero;                    /* ..for scaling. */
 
-/* TAU = DCMPLX( ALPHR/BETA, -ALPHI/BETA )                                    */
-         myTAUi[0] = ALPHAR/BETA;
-         myTAUi[1] = (0. - zTS->zDiag[1])/BETA;
+         /*---------------------------------------------------------------*/
+         /* BETAp is the same for all cores, so this loop can be executed */
+         /* independently. However, XNORM must be computed in concert.    */
+         /* The new BETAp will be at most 1, at least SAFMIN.             */
+         /*---------------------------------------------------------------*/
+         while (BETAp < ATL_laSAFMIN)
+         {
+            KNT++;                                 /* increment power. */
+            if ( myTS->active == 1)                /* If I am active, */
+            {
+               cblas_scal(mScale, RSAFMN,
+                                    scaleA, 1);    /* Scale my share. */
+            }
 
-         ALPHA = 0.0 - ALPHAR;
+            BETA *= RSAFMN[0];
+            BETAp *= RSAFMN[0];
+            ALPHAR *= RSAFMN[0];
+            ALPHAI *= RSAFMN[0];
+         }
+
+         ATL_geqr2_dnrm2(myTS);                /* Do my share of new norm2. */
+
+         zees = myRank;                        /* Init the test flags.      */
+         pair = 1;                             /* Starting pair.            */
+         while ( (zees & 1) == 0 &&            /* If I must wait on pair,   */
+            (myRank)+pair < ATL_NTHREADS)      /* ..and pair exists,        */
+         {
+            ptnr = myTS+pair;                  /* Point at my partner.      */
+
+            if (ptnr->active == 1)             /* If ptnr was used,         */
+            {
+               while (ptnr->dnrm2 <= myTS->dnrm2);  /* Wait for it.         */
+                  dnrm2_combine;
+            }
+
+            zees >>= 1;                        /* Shift a zero out.         */
+            pair <<= 1;                        /* Double the pair idx.      */
+         }
+
+         myTS->dnrm2++;                        /* Signal I am done.         */
+
+         /*************************************/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /******  S Y N C   P O I N T  ********/
+         /*************************************/
+         while (zTS->dnrm2 < myTS->dnrm2);         /* Wait on zero to finish.*/
+
+         XNORM = (zTS->Scale)*sqrt(zTS->SSQ);      /* Compute the norm. */
+         BETAp = ATL_lapy3(ALPHAR, ALPHAI, XNORM); /* sqrt(a^2+b^2,c^2) */
+         BETA = BETAp;                             /* Assume ALPHAR < 0.*/
+         if (ALPHAR >= 0.) BETA = 0.-BETAp;        /* If >=0, Change sign. */
       }
 
-      if (myRank == 0)                      /* Set if rank zero.              */
+      myTAUi[0] = (BETA-ALPHAR) / BETA;      /* Compute real part. */
+      myTAUi[1] = (0.-ALPHAI) / BETA;        /* Compute imag part. */
+
+      ALPHADIV[0] = ALPHAR - BETA;           /* prepare for 1/(alpha-beta) */
+      ALPHADIV[1] = ALPHAI;                  /* ...                        */
+      ATL_ladiv(ONE, ALPHADIV, sc);          /* compute scaling factor.    */
+      if (myTS->active == 1)                 /* If I have some of vector,  */
+      {
+         cblas_scal(mScale, sc, scaleA, 1);  /* ..scale it. */
+      }
+
+      if (myRank == 0)                       /* If I own TAU, */
       {
          TAU[(i SHIFT)] = myTAUi[0];
          #ifdef BUILD_LQ2
-            TAU[(i SHIFT) +1] = 0. - myTAUi[1];
+            TAU[(i SHIFT) +1] = -myTAUi[1];  /* LQ2 needs conjugate. */
          #else
-            TAU[(i SHIFT) +1] = myTAUi[1];
+            TAU[(i SHIFT) +1] = myTAUi[1];   /* otherwise use normal. */
          #endif
       }
-      ALPHADIV[0] = ALPHA;
-      ALPHADIV[1] = ALPHAI;
 
-/*
- *    Perform complex division before scaling the X vector.
- *    ALPHA will have the result
- */
-
-      ATL_ladiv( ONE,  ALPHADIV  ,wfact);
-
-      if ( myTS->active == 1)
+      if (myRank == 0)                       /* If I own diagonal, */
       {
-            cblas_scal(mScale, wfact, scaleA, 1);/* Scale the vector.         */
-      }
-
-      if (myRank == 0)
-      {
-         AII[0] = BETA;                     /* If core 0, remember.           */
+         for (j=0; j<KNT; j++) BETA *= ATL_laSAFMIN;  /* Rescale BETA. */
+         AII[0] = BETA;                               /* save for later. */
          AII[1] = 0.0;
       }
-   #endif
+   #endif /* COMPLEX version of larfg. */
 
 /*----------------------------------------------------------------------------*/
 /*   Now we apply dlarf, if we are not on the last column. This is            */
@@ -746,9 +819,9 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
       if (i < (N-1) )                       /* If dlarf necessary,            */
    #endif
       {
-         newN = N-i-1;                      /* Width of update array & w.     */
+         newN = N-i-1;                      /* Width of update array & vect w.*/
          MY_gemvT(mUpdate, newN,
-                  ONE,   A+(LDA SHIFT), LDA, A, 1, ZERO, myTS->WORK, 1);
+                  ONE, A+(LDA SHIFT), LDA, A, 1, ZERO, myTS->WORK, 1);
 
 /*----------------------------------------------------------------------------*/
 /*       Now combine with other threads.                                      */
@@ -777,15 +850,15 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
          /******  S Y N C   P O I N T  ******/
          /******  S Y N C   P O I N T  ******/
          /***********************************/
-/*----------------------------------------------------------------------------*/
-/*       We can't start GER until all of                                      */
-/*       'w' is finished. Wait for zero.                                      */
-/*----------------------------------------------------------------------------*/
+         /*---------------------------------*/
+         /* We can't start GER until all of */
+         /* 'w' is finished. Wait for zero. */
+         /*---------------------------------*/
          while (zTS->dgemv < i);            /* Wait for zero to build 'w'.    */
 
-/*----------------------------------------------------------------------------*/
-/*       'w' now in WORK. Use for GER.                                        */
-/*----------------------------------------------------------------------------*/
+         /*---------------------------------*/
+         /* 'w' now in WORK. Use for GER.   */
+         /*---------------------------------*/
          #ifdef TREAL
          MY_ger(mUpdate, newN, 0.-myTAUi, A, 1, zTS->WORK, 1,
                 A+(LDA SHIFT), LDA);
@@ -797,32 +870,23 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
                 A+(LDA SHIFT), LDA);
          #endif
 
-         #ifdef TREAL
-            if (myRank == 0) *A = AII;      /* Core 0, restore diag now.      */
-         #else
-            if (myRank == 0)
-            {
-               *(A ) = AII[0];              /* Core 0, restore diag now.      */
-               *(A + 1) = AII[1];           /* Core 0, restore diag now.      */
-            }
-         #endif
-/*----------------------------------------------------------------------------*/
-/*       Once we finish it is safe for us to start our                        */
-/*       next column and dnrm2 on our share. We  will                         */
-/*       sync up with other threads to complete that.                         */
-/*----------------------------------------------------------------------------*/
-      } else                      /* END IF we need to apply dlarf.           */
-      {
-         #ifdef TREAL
-            if (myRank == 0) *A = AII;      /* Core 0, restore diag now.      */
-         #else
-            if (myRank == 0)
-            {
-               *(A ) = AII[0];              /* Core 0, restore diag now.      */
-               *(A + 1) = AII[1];           /* Core 0, restore diag now.      */
-            }
-         #endif
-      }
+         /*-----------------------------------------------*/
+         /* Once we finish it is safe for us to start our */
+         /* next column and dnrm2 on our share. We  will  */
+         /* sync up with other threads to complete that.  */
+         /*-----------------------------------------------*/
+      } /* END IF we needed to apply dlarf. */
+
+      #ifdef TREAL
+         if (myRank == 0) *A = AII;      /* Core 0, restore diag now.      */
+      #else
+         if (myRank == 0)
+         {
+            *(A ) = AII[0];              /* Core 0, restore diag now.      */
+            *(A + 1) = AII[1];           /* Core 0, restore diag now.      */
+         }
+      #endif
+
       /*
        * for computing T,  for LQ replace myTAUi with correct TAU for the
        * complex part.
@@ -898,23 +962,20 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
          int os=(N+3)&(-4);                 /* Find even offset into work.    */
 
          #ifdef TREAL
-         MY_gemvT( mUpdate,  i,
-         0.-myTAUi,
-         A-i*LDA, LDA,
-         A, 1, 0.0,
-         myTS->WORK+os, 1);
+         MY_gemvT(mUpdate, i, 0.-myTAUi,
+                  A-i*LDA, LDA, A, 1, 0.0, myTS->WORK+os, 1);
          #else
               #ifdef BUILD_LQ2
                  negTAUi[0]= 0.0 + myTAUi[0];
-                 negTAUi[1]= 0.0 - myTAUi[1]; /* conjugate for complex Not required */
+                 negTAUi[1]= 0.0 - myTAUi[1]; /* conj for cplx Not required */
               #else
                  negTAUi[0]= 0.0 - myTAUi[0];
-                 negTAUi[1]= 0.0 - myTAUi[1]; /* conjugate for complex Not required */
+                 negTAUi[1]= 0.0 - myTAUi[1]; /* conj for cplx Not required */
               #endif
               MY_gemvT(mUpdate,  i, negTAUi, A-(i*(LDA SHIFT)), LDA,
                        A, 1, ZERO, myTS->WORK+(os SHIFT), 1);
 
-                          #ifdef BUILD_LQ2
+          #ifdef BUILD_LQ2
 		          for (i_loop = 0; i_loop < i; i_loop++)
 		          {
 		              (myTS->WORK+(os SHIFT))[(i_loop SHIFT) + 0] =
@@ -968,21 +1029,17 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
             cblas_trmv(CblasColMajor, CblasUpper, CblasNoTrans, CblasNonUnit,
                         i, T, LDT, T+(i*(LDT SHIFT)), 1);
 
+            /* Force TAU[i] on diagonal. */
             #ifdef TREAL
-	       *(T+i+i*LDT)=myTAUi;                /* Force TAU[i] on diagonal       */
+	         *(T+i+i*LDT)=myTAUi;
             #else
-               *(T+(i SHIFT)+(i*(LDT SHIFT)))=
-                        myTAUi[0];          /* Force TAU[i] on diagonal.      */
-               *(T+(i SHIFT)+(i*(LDT SHIFT)) + 1)=
-                        myTAUi[1];          /* Force TAU[i] on diagonal.      */
+            *(T+(i SHIFT)+(i*(LDT SHIFT))    ) = myTAUi[0];
+            *(T+(i SHIFT)+(i*(LDT SHIFT)) + 1) = myTAUi[1];
             #endif
 
-         }                        /* ** END IF zero must update T. **         */
-
-      }                                     /* ** END if building T **        */
-   }                                        /* END FOR each column.           */
-
-
+         } /* END IF zero must update T. */
+      } /* END IF building T. */
+   } /* END FOR each column. */
 
 
 /*----------------------------------------------------------------------------*/
@@ -994,7 +1051,7 @@ static void* ATL_geqr2Worker(ATL_LAUNCHSTRUCT_t *lp, void *vp)
    }
 
    return(NULL);                            /* Implicit thread exit.          */
-}                                           /* end *** ATL_geqr2Worker **     */
+} /* END ATL_geqr2Worker() */
 
 
 /**************************************************************************** */
@@ -1025,7 +1082,7 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
    ATL_DGEQR2_t ts[ATL_NTHREADS];
    long long t0, t1;
 
-// fprintf (stderr,"______in %s M=%d  N=%d \n", Mstr(ATL_tgexx2), M,N);
+/* fprintf (stderr,"______in %s M=%d  N=%d \n", Mstr(ATL_tgexx2), M,N);   */
    if (M < 0)
    {
       fprintf(stderr, "%s: M<0 (%i)\n", Mstr(ATL_tgexx2), M);
@@ -1058,7 +1115,7 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
 /* We use a data-owner split, each thread gets 1/p of the data,               */
 /* and does all computation related to it.                                    */
 /*----------------------------------------------------------------------------*/
-   TYPE *myA = A, *myOldA = A, *allMem, *workMem;
+   TYPE *myA = A, *myOldA = A, *allMem=NULL, *workMem;
    int i, j, k, b0, b, th;
    long unsigned int CPU;
    size_t mem[ATL_NTHREADS], totmem, workSize;
@@ -1080,7 +1137,7 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
    }
 
    if (b == 0) th = 1;
-   if (th == 1 || (N> M))                             /* If impossible to split,        */
+   if (th == 1 || (N> M))         /* If impossible to split, use serial. */
    {
 
       #ifdef BUILD_LQ2
@@ -1192,9 +1249,8 @@ int ATL_tgexx2(int M, int N, TYPE *A, int LDA, TYPE *TAU, TYPE *WORK,
    ATL_goparallel(th, ATL_geqr2Worker, ts, NULL);
 
 
-   #if defined(local_copy)
-   free(allMem);                            /* release copied area.           */
-   #endif                                   /* defined(local_copy)            */
+   if (allMem)
+      free(allMem);                         /* release copied area.           */
    free(workMem);                           /* release work area.             */
    return(0);                               /* Done with dgeqr2.              */
 }  /* END ATL_t_dgeqr2. */
