@@ -1,33 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.3
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2012, 2011 R. Clint Whaley
- *
  * Code contributers : R. Clint Whaley, Tom Wallace
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 #ifndef ATL_GAS_ARM
    #error "This routine requires GAS/ARM assembly"
@@ -42,6 +16,9 @@
    #error "NB must be a multiple of 4"
 #endif
 
+#ifndef ATL_ARM_SOFTFP
+   #define ATL_ARM_HARDFP 1
+#endif
 /*
  * This routine is a simple 4x4 register blocked routine, with two iterations
  * of software pipelining on the load/use of A and B for the arm.
@@ -113,7 +90,7 @@ void ATL_USERMM (const int M, const int N, const int K, const TYPE alpha,
 #include "atlas_asm.h"
 .code 32
 .fpu vfpv3
-#ifndef ATL_ARM_SOFTFP
+#ifdef ATL_ARM_HARDFP
 .eabi_attribute 28, 1
 #endif
 .text
@@ -126,7 +103,7 @@ ATL_asmdecor(ATL_USERMM):
 /*
  * Load needed vals to registers
  */
-#ifndef ATL_ARM_SOFTFP
+#ifdef ATL_ARM_HARDFP
    add M0, SP, #FSIZE
    ldmIA M0, {lda,pB0,pA00,pC0,ldc} /* lda,B,ldb,C,ldc */
    vmov M0, zero, d1  /* put beta in M0, zero */
@@ -177,7 +154,7 @@ ATL_asmdecor(ATL_USERMM):
          #else
             add PTR, pC0, ldc
             #ifdef BETAX
-               #ifndef ATL_ARM_SOFTFP
+               #ifdef ATL_ARM_HARDFP
                   fldd rb0, [SP, #0]
                #else
                   fldd rb0, [SP, #(FSIZE+24)]
@@ -367,7 +344,7 @@ ATL_asmdecor(ATL_USERMM):
       mov M, M0
    bne NLOOP
 
-   #ifndef ATL_ARM_SOFTFP
+   #ifdef ATL_ARM_HARDFP
       pop {r0,r1}  /* clear beta off stack */
    #endif
    ldmIA SP!, {r4-r11,r14}
