@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 2001 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2001 R. Clint Whaley
  */
 
 #include "atlas_asm.h"
@@ -279,7 +254,18 @@ void ATL_USERMM(const int M, const int N, const int K, const TYPE alpha,
 #endif
 .text
 #ifdef ATL_GAS_LINUX_PPC
-   #if defined(ATL_USE64BITS)
+   #if defined(ATL_USE64BITS) && _CALL_ELF != 2
+    /*
+      *      Official Program Descripter section, seg fault w/o it on Linux/PPC64
+        */
+        @@ -199,6 +199,12 @@ Mjoin(.,ATL_USERMM):
+            #else
+                .globl  ATL_USERMM
+                 ATL_USERMM:
+                 +   #if _CALL_ELF == 2
+                 +      .type ATL_USERMM,@function
+                 +0:    addis 2,12,.TOC.-0b@ha
+
 /*
  *       No idea what this does, but seg fault without it (I think it is
  *       partially resp for making code callable from both static & PIC code)
@@ -298,9 +284,9 @@ Mjoin(.,ATL_USERMM):
    #else
 .globl	ATL_USERMM
 ATL_USERMM:
-      #define IROFF FROFF+144
-      #define FROFF 8
-      #define FSIZE 224
+      #define IROFF 8
+      #define FROFF IROFF+48
+      #define FSIZE (((IROFF+FROFF+144+15)/16)*16)
 	mflr	r0
 	stw	r0, 4(r1)
 	stwu	r1, -FSIZE(r1)
