@@ -1,33 +1,11 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                   (C) Copyright 2000 Peter Soendergaard
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2000 Peter Soendergaard
  */
 
+#ifndef ATL_SSE1
+   #error "This routine requires SSE1!"
+#endif
 #define SSE
 #include "SSE3Dnow.h"
 #include "atlas_misc.h"
@@ -52,7 +30,7 @@ void ATL_USERMM
    /*  align_jumps = 0  */
    /*  prec = 'single'  */
    /*  m_cleanup = {}  */
-   /*  used_outside_len = 60  */
+   /*  used_outside_len = 56  */
    /*  k_cleanup = {'mustfit': 56, 'method': 'acc1', 'compile': 1}  */
    /*  outputdir = 'Linux_PIII/'  */
    /*  arch = 'sse'  */
@@ -60,7 +38,7 @@ void ATL_USERMM
    /*  atlasname = 'Linux_PIII'  */
    /*  method = 'acc'  */
    /*  used_lastuse = 'a'  */
-   /*  outside_len = 60  */
+   /*  outside_len = 56  */
    /*  lastuse = 1  */
    /*  veclen = 4  */
    /*  sched = ['fuse', 'spread']  */
@@ -79,8 +57,8 @@ void ATL_USERMM
    float *pC1 = C+(ldc SHIFT);
    const float *stM = A + M*KB;
    const float *stN = B + NB*KB;
-   const int incAm = 1*KB-KB+60;
-   const int incBm = -KB+60;
+   const int incAm = 1*KB-KB+56;
+   const int incBm = -KB+56;
    const int incCm = (1 SHIFT);
    const int incAn = -M*KB;
    const int incBn = 2*KB;
@@ -188,16 +166,10 @@ void ATL_USERMM
          vec_mul_mr_a(pB0+48+KB,reg4);
          vec_add_rr(reg4,reg1);
          vec_mov_mr_a(pA0+52,reg4);
-         vec_mov_mr_a(pB0+52,reg3);
-         vec_mul_rr(reg4,reg3);
-         vec_add_rr(reg3,reg0);
-         vec_mul_mr_a(pB0+52+KB,reg4);
-         vec_add_rr(reg4,reg1);
-         vec_mov_mr_a(pA0+56,reg4);
-         vec_mov_mr_a(pB0+56,reg2);
+         vec_mov_mr_a(pB0+52,reg2);
          vec_mul_rr(reg4,reg2);
          vec_add_rr(reg2,reg0);
-         vec_mul_mr_a(pB0+56+KB,reg4);
+         vec_mul_mr_a(pB0+52+KB,reg4);
          vec_add_rr(reg4,reg1);
          vec_sum(reg0);
          vec_sum(reg1);
