@@ -68,27 +68,44 @@ ATL_flagnode_t *GetGccOptFlags(void)
  "# Flags to probe once optimization level is selected",
  "-fno-cprop-registers", "-fcprop-registers",
  "-fcrossjumping", "-fno-crossjumping",
+ "-fmodulo-sched -fmodulo-sched-allow-regmoves",
+ "-fcse-follow-jumps",
+ "-free",
+ "-fira-algorithm=priority", "-fira-algorithm=CB",
+ "-fira-region=all", "-fira-region=mixed", "-fira-region=one",
+ "-fira-hoist-pressure", "-fira-loop-pressure",
+ "-fselective-scheduling", "-fselective-scheduling2",
+ "-fselective-scheduling -fsel-sched-pipelining",
+ "-fselective-scheduling2 -fsel-sched-pipelining",
+ "-fstrict-overflow",
  "-fmodulo-sched", "-fno-branch-count-reg", "-fno-move-loop-invariants",
+ "-fdelayed-branch",
  "-fno-peephole -fno-peephole2", "-fno-peephole", "-fno-peephole2",
- "-funsafe-loop-optimizations", "-fomit-frame-pointer",
+ "-faggressive-loop-optimizations", "-funsafe-loop-optimizations",
+ "-fomit-frame-pointer",
  "-foptimize-register-move", "-fno-optimize-register-move",
  "-fprefetch-loop-arrays", "-fno-prefetch-loop-arrays",
  "-frename-registers", "-fno-rename-registers",
  "-fno-reorder-blocks", "-fno-sched-interblock", "-fno-sched-spec",
  "-fsched-spec-load", "-fsched2-use-superblocks", "-fsched2-use-traces",
+ "-fselective-scheduling -fsel-sched-pipelining",
+ "-fselective-scheduling -fsel-sched-pipelining -fsel-sched-pipelining-outer-loops",
+ "-fsel-sched-pipelining-outer-loops",
  "-fsee", "-freschedule-modulo-scheduled-loops", "-fsingle-precision-constant",
  "-fstrict-aliasing", "-funroll-all-loops", "-funroll-loops",
  "-fno-split-ivs-in-unroller",
  "-fvariable-expansion-in-unroller", "-fno-variable-expansion-in-unroller",
  "-fno-tree-pre", "-fno-tree-fre", "-fno-tree-loop-optimize",
- "-ftree-loop-linear", "-fno-tree-loop-linear", "-fno-tree-loop-im",
+ "-fno-tree-loop-linear", "-fno-tree-loop-im",
  "-ftree-loop-ivcanon", "-fno-tree-loop-ivcanon", "-fivopts", "-fno-ivopts",
  "-fno-tree-dominator-opts", "-ftree-dse", "-fno-tree-dse",
  "-fno-tree-copyrename", "-fno-tree-sink", "-fno-tree-ch", "-fno-tree-ter",
  "-fno-tree-lrs", "-ftree-vectorize -fno-tree-vect-loop-version",
  "-ftree-vectorize -ftree-vect-loop-version", "-fno-tree-salias",
  "-fweb", "-fno-web", "-fno-rerun-loop-opt", "-frerun-loop-opt",
+ "-falign-functions=64",
  "-falign-loops=4", "-falign-loops=8", "-falign-loops=16", "-falign-loops=32",
+ "-falign-jumps=4", "-falign-jumps=8", "-falign-jumps=16",
  NULL
 };
    int i;
@@ -319,8 +336,8 @@ void PrintUsage(char *name, int ierr, char *flag)
    fprintf(stderr, "      using baseflags as given in next mandatory arg\n");
    fprintf(stderr,
       "   -m <file> : get mmcase from file (default res/<pre>gMMRES.sum\n");
-//   fprintf(stderr,
-//   "   -S [g/G/l] : do O(N) or O(N^2) Gready or O(N) linear search\n");
+/*   fprintf(stderr,
+     "   -S [g/G/l] : do O(N) or O(N^2) Gready or O(N) linear search\n"); */
    exit(-1);
 }
 
@@ -445,7 +462,7 @@ ATL_flagnode_t *LinearFlagSearch
          free(mmp->cflags);
       mmp->cflags = DupString(flags);
       mf = TimeMMKernel(verb, 1, mmp, pre, mmp->mbB, mmp->nbB, mmp->kbB,
-                        0, 0, 0, 1, -1, -1);
+                        1, -1, -1);
       if (verb > 0)
          printf("%s%4d. mf=%.2f, newflags='%s'\n", spc, i, mf, fp->flags);
       if (mf > mfB*mulB)
@@ -507,7 +524,7 @@ char *GreedyLinearFlagSearch
             free(mmp->cflags);
          mmp->cflags = flags;
          mf = TimeMMKernel(verb, 1, mmp, pre, mmp->mbB, mmp->nbB, mmp->kbB,
-                           0, 0, 0, 1, -1, -1);
+                           1, -1, -1);
          if (mf0 == 0.0)
             mf0 = mf;
          if (verb > 0)
@@ -535,7 +552,7 @@ char *GreedyLinearFlagSearch
                free(mmp->cflags);
             mmp->cflags = flags;
             mf = TimeMMKernel(verb, 1, mmp, pre, mmp->mbB, mmp->nbB, mmp->kbB,
-                              0, 0, 0, 1, -1, -1);
+                              1, -1, -1);
             if (verb > 0)
             {
                printf("%s%4d. mf=%.2f, flags='%s", spc, i, mf, fp->flags);
@@ -581,7 +598,7 @@ char *GreedyLinearFlagSearch
          free(mmp->cflags);
       mmp->cflags = DupString(flags);
       mf = TimeMMKernel(verb, 1, mmp, pre, mmp->mbB, mmp->nbB, mmp->kbB,
-                        0, 0, 0, 1, -1, -1);
+                        1, -1, -1);
       if (mf0 == 0.0)
          mf0 = mf;
       if (verb > 0)
@@ -679,6 +696,7 @@ int main(int nargs, char **args)
       fprintf(stderr, "CANNOT READ FILE '%s'!\n", mmfile);
       exit(-1);
    }
+   MMFillInGenStrings(pre, mmp);
    if (verb > 0)
    {
       printf("\nFINDING BEST FLAGS USING MATMUL KERNEL:\n");
