@@ -1,37 +1,13 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2010, 2009 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
 #include "atlas_misc.h"
+#include "atlas_cache.h"
 #include Mstr(Mjoin(Mjoin(atlas_,PRE),sysinfo.h))
 
 static char *resfile=NULL;
@@ -60,7 +36,7 @@ struct FA_allocs *NewAlloc(size_t size, struct FA_allocs *next,
    int n, i;
    const int malign = align >= misalign ? align : misalign;
 
-   n = size + align + malign;
+   n = size + align + align + malign;
    i = (n >> 3)<<3;
    if (n != i)
       n += n - i;
@@ -74,12 +50,13 @@ struct FA_allocs *NewAlloc(size_t size, struct FA_allocs *next,
    ap->memA = align ? (void*) ((((size_t) cp)/align)*align + align) : cp;
 /*
  * Misalign to misalign
+ * We often need to make sure to unaligned addresses share the same modulo
+ * so that they have the *same* degree of misalignment (so that their alignment
+ * can be fixed by simple peeling), and so in this case force the address
+ * modulo the misalign to be the exact align value.
  */
    if (misalign)
-   {
-      if (((size_t)ap->memA)%misalign == 0)
-         ap->memA = ((char*)ap->memA) + align;
-   }
+      ap->memA = (void*)((((size_t)ap->memA)/malign)*malign + malign + align);
    ap->next = next;
    return(ap);
 }
@@ -241,7 +218,7 @@ static double mysum(ATL_CINT N, double *d)
 double mvtime(
    int verb,            /* verbosity */
    int nreps,           /* number of reps to do for one timing sample */
-   ATL_INT flushelts,   /* size of area to flush to avoid cache reuse */
+   size_t flushelts,    /* size of area to flush to avoid cache reuse */
    ATL_INT M,           /* # of rows of array A */
    ATL_INT N,           /* # of cols of array A */
    TYPE *alpha,
@@ -357,11 +334,19 @@ double mvtime(
  * through memory; mem starts with maxalign-aligned memory, so that we can
  * guarantee all further alignments
  */
-   maxalign = (MAx >= MAa) ? MAx : MAa;
-   maxalign = (maxalign >= MAy) ? maxalign : MAy;
-   maxalign = (maxalign >= FAy) ? maxalign : FAy;
-   maxalign = (maxalign >= FAx) ? maxalign : FAx;
-   maxalign = (maxalign >= FAa) ? maxalign : FAa;
+   maxalign = (MAx) ? MAx : 1;
+   if (MAy)
+      maxalign = ATL_lcm(MAy,maxalign);
+   if (MAa)
+      maxalign = ATL_lcm(MAa,maxalign);
+   if (FAx)
+      maxalign = ATL_lcm(FAx,maxalign);
+   if (FAy)
+      maxalign = ATL_lcm(FAy,maxalign);
+   if (FAa)
+      maxalign = ATL_lcm(FAa,maxalign);
+   if (maxalign == 1)
+      maxalign = 0;
    j = (FAx) ? FAx : sizeof(TYPE);
    if (MAx)
       for (i=0; (i % j != 0 || i%MAx == 0); i += sizeof(TYPE));
@@ -469,7 +454,7 @@ double mvtime(
 }
 #endif
 
-void DoTimes(int verb, ATL_INT flshelts, ATL_INT ntim, ATL_INT nrep,
+void DoTimes(int verb, size_t flshelts, ATL_INT ntim, ATL_INT nrep,
              ATL_INT mu, ATL_INT nu, ATL_INT M, ATL_INT N, TYPE *alpha,
              TYPE *beta, ATL_INT lda,
              int FAa, int MAa, int FAx, int MAx, int FAy, int MAy)
@@ -537,7 +522,7 @@ void PrintUsage(char *name, char *arg, int i)
 }
 
 void GetFlags(int nargs, char **args, int *verb,
-              ATL_INT *flushelts, ATL_INT *celts, ATL_INT *pgelts,
+              size_t *flushelts, ATL_INT *celts, ATL_INT *pgelts,
               ATL_INT *mu, ATL_INT *nu, ATL_INT *ntim, ATL_INT *nrep,
               enum ATLAS_TRANS *TA, ATL_INT *m, ATL_INT *n, ATL_INT *lda,
               TYPE *beta,
@@ -545,6 +530,7 @@ void GetFlags(int nargs, char **args, int *verb,
 {
    double mfF=ATL_nkflop/1000.0, flops;
    ATL_INT j, h;
+   size_t il;
    int i;
    char ch;
 
@@ -554,7 +540,7 @@ void GetFlags(int nargs, char **args, int *verb,
    #else
       *pgelts = 4*ATL_DivBySize(1024);
    #endif
-   *celts = 0.75*ATL_L1elts;
+   *celts = 0.75*L1C_ELTS;
    #ifdef L2SIZE
       *flushelts = L2SIZE;
    #else
@@ -609,9 +595,9 @@ void GetFlags(int nargs, char **args, int *verb,
       case 'C' :  /* set flushsz in KB */
          if (++i >= nargs)
             PrintUsage(args[0], "out of flags in -C ", i-1);
-         j = atoi(args[i]);
-         if (j >= 0)
-            *flushelts = j*ATL_DivBySize(1024);
+         il = atol(args[i]);
+         if (il >= 0)
+            *flushelts = il*ATL_DivBySize(1024);
          break;
       case 'p' :  /* set pagesz in KB */
          if (++i >= nargs)
@@ -716,7 +702,8 @@ void GetFlags(int nargs, char **args, int *verb,
 }
 int main(int nargs, char **args)
 {
-   ATL_INT flushelts, celts, pgelts, mu, nu, ntim, nrep, m, n, lda;
+   size_t flushelts;
+   ATL_INT celts, pgelts, mu, nu, ntim, nrep, m, n, lda;
    int FAa, MAa, FAx, MAx, FAy, MAy;    /* Force & Max align for ops */
    int verb;
    enum ATLAS_TRANS TA;
