@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Automatically Tuned Linear Algebra Software v3.11.41
  * Copyright (C) 2010 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
  */
 #include "atlas_misc.h"
 #include "atlas_tst.h"
@@ -57,7 +32,7 @@ struct FA_allocs *NewAlloc(size_t size, struct FA_allocs *next,
    int n, i;
    const int malign = align >= misalign ? align : misalign;
 
-   n = size + align + malign;
+   n = size + align + align + malign;
    i = (n >> 3)<<3;
    if (n != i)
       n += n - i;
@@ -71,12 +46,13 @@ struct FA_allocs *NewAlloc(size_t size, struct FA_allocs *next,
    ap->memA = align ? (void*) ((((size_t) cp)/align)*align + align) : cp;
 /*
  * Misalign to misalign
+ * We often need to make sure to unaligned addresses share the same modulo
+ * so that they have the *same* degree of misalignment (so that their alignment
+ * can be fixed by simple peeling), and so in this case force the address
+ * modulo the misalign to be the exact align value.
  */
    if (misalign)
-   {
-      if (((size_t)ap->memA)%misalign == 0)
-         ap->memA = ((char*)ap->memA) + align;
-   }
+      ap->memA = (void*)((((size_t)ap->memA)/malign)*malign + malign + align);
    ap->next = next;
    return(ap);
 }
@@ -435,14 +411,14 @@ void PrintUsage(char *name, int ierr, char *flag)
 
 
 /* procedure 1 */
-int *GetIntList1(int ival)
+static int *GetIntList1(int ival)
 /*
  * returns integer array with iarr[0] = 1, iarr[1] = ival
  */
 {
    int *iarr;
    iarr = malloc(2*sizeof(int));
-   ATL_assert(iarr);
+   assert(iarr);
    iarr[0] = 1;
    iarr[1] = ival;
    return(iarr);
@@ -450,14 +426,14 @@ int *GetIntList1(int ival)
 
 #ifdef TYPE
 /* procedure 2 */
-TYPE *GetTypeList1(const SCALAR val)
+static TYPE *GetTypeList1(const SCALAR val)
 /*
  * Returns a TYPE array with arr[0] = 1.0, arr[1] = val
  */
 {
    TYPE *arr;
    arr = malloc(ATL_MulBySize(2));
-   ATL_assert(arr);
+   assert(arr);
    arr[0] = 1;
    #ifdef TCPLX
       arr[2] = *val;
@@ -470,14 +446,14 @@ TYPE *GetTypeList1(const SCALAR val)
 #endif
 
 /* procedure 3 */
-int *GetIntList2(int ival1, int ival2)
+static int *GetIntList2(int ival1, int ival2)
 /*
  * returns integer array with iarr[0] = 1, iarr[1] = ival1, ival[2] = ival2
  */
 {
    int *iarr;
    iarr = malloc(3*sizeof(int));
-   ATL_assert(iarr);
+   assert(iarr);
    iarr[0] = 1;
    iarr[1] = ival1;
    iarr[2] = ival2;
@@ -485,7 +461,7 @@ int *GetIntList2(int ival1, int ival2)
 }
 
 /* procedure 4 */
-int *DupIntList(int *list)
+static int *DupIntList(int *list)
 /*
  * Duplicates list of integers, list[0] holds the length, not including 0
  */
@@ -502,7 +478,7 @@ int *DupIntList(int *list)
 }
 
 /* procedure 5 */
-int *GetIntList(int nargs, char **args, int i, int nmul)
+static int *GetIntList(int nargs, char **args, int i, int nmul)
 /*
  * Gets a list of integers, whose length is given by atoi(args[i])*nmul
  * list is this length+1, since 0'th location gets atoi(args[i])
@@ -513,9 +489,9 @@ int *GetIntList(int nargs, char **args, int i, int nmul)
    if (++i >= nargs)
       PrintUsage(args[0], i, NULL);
    n = atoi(args[i]) * nmul;
-   ATL_assert(n > 0);
+   assert(n > 0);
    iarr = malloc(sizeof(int)*(n+1));
-   ATL_assert(iarr);
+   assert(iarr);
 
    iarr[0] = n / nmul;
    for (k=0; k < n; k++)
@@ -529,7 +505,7 @@ int *GetIntList(int nargs, char **args, int i, int nmul)
 
 #ifdef TYPE
 /* procedure 6 */
-TYPE *GetTypeList(int nargs, char **args, int i, int nmul)
+static TYPE *GetTypeList(int nargs, char **args, int i, int nmul)
 /*
  * Gets a list of TYPEs, whose length is given by atoi(args[i])*nmul
  * list is this length+1, since 0'th location gets atof(args[i])
@@ -541,9 +517,9 @@ TYPE *GetTypeList(int nargs, char **args, int i, int nmul)
    if (++i >= nargs)
       PrintUsage(args[0], i, NULL);
    n = atoi(args[i]) * nmul;
-   ATL_assert(n > 0);
+   assert(n > 0);
    arr = malloc(ATL_MulBySize(n+1));
-   ATL_assert(arr);
+   assert(arr);
 
    arr[0] = n / nmul;
    for (k=0; k < n; k++)
@@ -557,14 +533,14 @@ TYPE *GetTypeList(int nargs, char **args, int i, int nmul)
 #endif
 
 /* procedure 7 */
-int *IntRange2IntList(int N0, int NN, int incN)
+static int *IntRange2IntList(int N0, int NN, int incN)
 {
    int i, n;
    int *iarr;
 
    for (i=N0, n=0; i <= NN; i += incN) n++;
    iarr = malloc(sizeof(int)*(n+1));
-   ATL_assert(iarr);
+   assert(iarr);
    iarr[0] = n;
    for (i=N0, n=1 ; i <= NN; i += incN, n++)
       iarr[n] = i;
