@@ -5,6 +5,7 @@
 #include "atlas_misc.h"
 #include "atlas_r1parse.h"
 #include "atlas_r2testtime.h"
+#include "atlas_cache.h"
 #include "atlas_ssysinfo.h"
 
 /*
@@ -82,7 +83,13 @@ static double TimeMyKernel
 )
 {
    double mf;
-   static int fmflop[5] = {0,0,0,0,0};
+   #ifdef WALL
+      static int fmflop[5] = {0,0,0,0,0};
+   #else
+      static int fmflop[5] =
+         {ATL_nkflop/1000.0,ATL_nkflop/1000.0,ATL_nkflop/1000.0,
+          ATL_nkflop/10.0,ATL_nkflop};
+   #endif
 
    mf = TimeR2Kernel(verb, FTIME, r1p, pre, M, N, lda, percL1, nrep,
                      fmflop[imf], cflush);
@@ -1184,6 +1191,9 @@ ATL_r1node_t *TunePF
  * This routine tunes the prefetch parameters for kb
  */
 {
+   #ifdef ATL_NOPREFETCH
+      return(kb);
+   #endif
 /*
  * First tune the instruction types, then the prefetch distances
  */
@@ -1947,7 +1957,6 @@ void ExhaustiveTime(char pre, char *fin, char *fout, int lvl)
    WriteMflopExp(fpout);
    fclose(fpout);
 }
-
 int main(int nargs, char **args)
 {
    ATL_r1node_t *ocb, *i2b, *i1b, *syb, *r1cA, *r1cB;
