@@ -8,9 +8,13 @@ TYPE ATL_UNRM2(const int N, const TYPE *X, const int incX)
  */
 {
    int i;
-   #if defined(SREAL) || defined(SCPLX) || defined(ATL_OS_WinNT)
+   #if defined(SREAL) || defined(SCPLX) || defined(ATL_OS_WinNT) || \
+       defined(ATL_OS_Win64)
       double t0=ATL_rzero;
    #else
+      #if !defined(__STDC_VERSION__) || __STDC_VERSION__ < 199900
+         long double sqrtl(long double x);
+      #endif
       #define sqrt sqrtl
       long double t0=ATL_rzero;
    #endif
