@@ -9,9 +9,13 @@ TYPE ATL_UNRM2(const int N, const TYPE *X, const int incX)
  */
 {
    int n;
-   #if defined(SREAL) || defined(SCPLX) || defined(ATL_OS_WinNT)
+   #if defined(SREAL) || defined(SCPLX) || defined(ATL_OS_WinNT) || \
+       defined(ATL_OS_Win64)
       register double t0=0.0, t1=0.0, t2=0.0, t3=0.0;
    #else
+      #if !defined(__STDC_VERSION__) || __STDC_VERSION__ < 199900
+         long double sqrtl(long double x);
+      #endif
       #define sqrt sqrtl
       register long double t0=0.0, t1=0.0, t2=0.0, t3=0.0;
    #endif
