@@ -588,7 +588,7 @@ FILENODE *TstFile(char pre, FILENODE *fbase, int *N)
       i = sprintf(ln, "make %caxpbytest urout=%s opt=\"", pre, fp->rout);
       if (fp->incX == 0) i += sprintf(ln+i, " -X 4 1 -1 2 -3");
       if (fp->incY == 0) i += sprintf(ln+i, " -Y 4 1 -1 3 -2");
-      if (pre == 'c' && pre == 'z') /* complex alphas */
+      if (pre == 'c' || pre == 'z') /* complex alphas */
       {
          if (fp->alpha == 0) /* imag = 0 */
             i += sprintf(ln+i, " -a 3 1.0 0.0 -1.0 0.0 0.9 0.0");
@@ -597,7 +597,7 @@ FILENODE *TstFile(char pre, FILENODE *fbase, int *N)
       }
       else if (fp->alpha ==  1) i += sprintf(ln+i, " -a 1 1.0");
       else if (fp->alpha == -1) i += sprintf(ln+i, " -a 1 -1.0");
-      if (pre == 'c' && pre == 'z') /* complex betas */
+      if (pre == 'c' || pre == 'z') /* complex betas */
       {
          if (fp->beta == 0) /* imag = 0 */
             i += sprintf(ln+i, " -b 3 1.0 0.0 -1.0 0.0 0.8 0.0");
@@ -786,8 +786,9 @@ void GenCplxIf(char pre, char *spc, FILE *fpout,
 
    if ( HaveABcase(AlphaX, 0, n, ia, ib) )
    {
-      fprintf(fpout, "%s   }\n%s   else /* cplx alpha, real beta cases\n",
+      fprintf(fpout, "%s   }\n%s   else /* cplx alpha, real beta cases */\n",
               spc, spc);
+      fprintf(fpout, "%s   {\n", spc);
       GenIncCases(pre, spc-6, "", fpout, AlphaX, 0, n, ix, iy, ia, ib);
    }
    fprintf(fpout, "%s   }\n", spc);
