@@ -159,7 +159,7 @@ FILENODE *ReadFile(char pre)
 {
    FILE *fpin;
    FILENODE *fbase, *fn, *fp=NULL;
-   char ln[512], rout[256], auth[256], cc[256], ccflags[256];
+   char ln[512], rout[256], auth[256], cc[256], ccflags[512];
    char *pcc, *pccflags;
    int i, n, ID, alpha, beta, incX, incY;
 
@@ -590,7 +590,7 @@ FILENODE *TstFile(char pre, FILENODE *fbase, int *N)
       i = sprintf(ln, "make %crottest urout=%s opt=\"", pre, fp->rout);
       if (fp->incX == 0) i += sprintf(ln+i, " -X 4 1 -1 2 -3");
       if (fp->incY == 0) i += sprintf(ln+i, " -Y 4 1 -1 3 -2");
-      if (pre == 'c' && pre == 'z') /* complex alphas */
+      if (pre == 'c' || pre == 'z') /* complex alphas */
       {
          if (fp->alpha == 0) /* imag = 0 */
             i += sprintf(ln+i, " -a 3 1.0 0.0 -1.0 0.0 0.9 0.0");
@@ -599,7 +599,7 @@ FILENODE *TstFile(char pre, FILENODE *fbase, int *N)
       }
       else if (fp->alpha ==  1) i += sprintf(ln+i, " -a 1 1.0");
       else if (fp->alpha == -1) i += sprintf(ln+i, " -a 1 -1.0");
-      if (pre == 'c' && pre == 'z') /* complex betas */
+      if (pre == 'c' || pre == 'z') /* complex betas */
       {
          if (fp->beta == 0) /* imag = 0 */
             i += sprintf(ln+i, " -b 3 1.0 0.0 -1.0 0.0 0.8 0.0");
