@@ -1,34 +1,12 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.10.2
- *                    (C) Copyright 2009 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 2009 R. Clint Whaley
  */
 #include "atlas_misc.h"
 #include "atlas_lapack.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_ptalias_lapack.h"
+#endif
 #ifdef ATL_ilaenv
    #undef ATL_ilaenv
 #endif
@@ -80,6 +58,10 @@ static int Side_LA2ATL(int i)
    #define Cgetrf Mjoin(Mjoin(clapack_,PRE),getrf)
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(Cgetrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
+#elif defined(TimeBCAMM)
+   #include "atlas_bcamm.h"
+   #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
+      ATL_assert(Mjoin(PATL,tgetrf_bcAmm)(Major_, M_, N_, A_, lda_, ipiv_) == 0)
 #else
    #define test_getrf(Major_, M_, N_, A_, lda_, ipiv_) \
       ATL_assert(ATL_getrf(Major_, M_, N_, A_, lda_, ipiv_) == 0)
@@ -137,7 +119,13 @@ TYPE *GetGE(int M, int N, int lda)
 {
    TYPE *A;
    A = malloc(ATL_MulBySize(lda)*N);
-   if (A) Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   if (A)
+   {
+      #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+         ATL_NumaTouchSpread(ATL_MulBySize(lda)*N, A);
+      #endif
+      Mjoin(PATL,gegen)(M, N, A, lda, M*N+lda);
+   }
    return(A);
 }
 
@@ -187,6 +175,9 @@ static TYPE *DupMat(enum ATLAS_ORDER Order, int M, int N, TYPE *A, int lda,
    ATL_assert(ldc >= M);
    C = malloc(ATL_MulBySize(ldc)*N);
    ATL_assert(C);
+   #if defined(ATL_USEPTHREADS) && !defined(ATL_NONUMATOUCH)
+      ATL_NumaTouchSpread(ATL_MulBySize(ldc)*N, C);
+   #endif
    for (j=0; j != N; j++)
    {
       for (i=0; i != M2; i++) C[i] = A[i];
@@ -275,7 +266,6 @@ static void MakeHEDiagDom
    int j;
    const int lda2=(lda SHIFT), ldap1=((lda+1)SHIFT);
 
-   Mjoin(PATL,gegen)(N, N, A, lda, N*N+lda);
    if (Order == CblasRowMajor)
    {
       if (Uplo == CblasLower) Uplo = CblasUpper;
@@ -706,33 +696,46 @@ int GetGoodNForFlopRate(int rout, double nsec, int restM, int restN)
       pre = 'd';
    #endif
    #ifdef ATL_USEPTHREADS
-      k = ((4000+NB-1)/NB)*NB;
+      #ifdef ATL_PTR64
+         k = 20000;
+      #else
+         k = 8000;
+      #endif
    #else
-      k = ((2000+NB-1)/NB)*NB;
+      #ifdef ATL_PTR64
+         k = 8000;
+      #else
+         k = 2000;
+      #endif
    #endif
-   mmp = ReadMMFileWithPath(pre, "../blas/gemm/res", "MMRES.sum");
+   mmp = ReadMMFileWithPath(pre, "../blas/gemm/res", "ipgen.sum");
    if (!mmp)
-     return(k);
-   if (mmp->next)
-      floprate = mmp->next->mflop[0];
-   else
-      floprate = mmp->mflop[0];
+   {
+      printf("lansrch goes with default size due to lack of timing!\n");
+      return(k);
+   }
+   while (mmp->next)
+      mmp = KillMMNode(mmp);
+   floprate = mmp->mflop[0];
    KillAllMMNodes(mmp);
    #ifdef ATL_USEPTHREADS
       floprate *= ATL_NCPU;
+      printf("Parallel flop rate at %g over %u cores\n", floprate, ATL_NCPU);
+   #else
+      printf("Serial flop rate at %g\n", floprate);
    #endif
    if (floprate < 100)  /* nonsensical result */
       return(k);
-   k = (600/NB)*NB;
+   k = 600;
    do
    {
-       k += NB;
+       k += 80;
        m = (restM) ? restM : k;
        n = (restN) ? restN : k;
-       time = GetFlopCount(rout, 0, m, n, 0, 0, NB) / (floprate*1000000.0);
+       time = GetFlopCount(rout, 0, m, n, 0, 0, 1) / (floprate*1000000.0);
    }
    while (time < nsec);
-   k -= NB;
+   k -= 80;
 /*
  * Don't let K be too huge, or extremely tiny regardless of computation
  */
@@ -758,9 +761,9 @@ void GetFlags(int nargs, char **args, int *nreps, int *flsizeKB, int *mflop,
    *ldagap = 0;
    *flsizeKB = L2SIZE/1024;
    #ifdef ATL_USEPTHREADS
-      *maxN = ((4000+NB-1)/NB)*NB;
+      *maxN = 4000;
    #else
-      *maxN = ((2000+NB-1)/NB)*NB;
+      *maxN = 2000;
    #endif
    *maxN = 0;
    *rout = LAgeqrf;
@@ -890,9 +893,15 @@ void GetFlags(int nargs, char **args, int *nreps, int *flsizeKB, int *mflop,
    }
    if (!(*maxN))
    {
-      *maxN = GetGoodNForFlopRate(*rout, 3.00, *restM, *restN);
+      #ifdef ATL_USEPTHREADS
+         *maxN = GetGoodNForFlopRate(*rout, 3.25, *restM, *restN);
+      #else
+         *maxN = GetGoodNForFlopRate(*rout, 4.50, *restM, *restN);
+      #endif
       fprintf(stderr, "maxN = %d\n", *maxN);
    }
+   if (NBs)
+      free(NBs);
    if (!ups)
       ups = GetIntList1(LAUpper);
    if (!sds)
@@ -1359,10 +1368,16 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
                int Side, int flsizeKB)
 {
 #if ATL_LINEFLUSH
-   FLSTRUCT *flp;
+   FLSTRUCT *flp=NULL;
 #endif
    TYPE *A, *wrk=NULL, dtmp, dtmp1, *tau=NULL;
    int *ipiv=NULL, itmp, wlen;
+/*
+ * If the matrix alone is larger than four times the flush size, then
+ * the matrix init should be self-flushing, so we avoid the time & memory
+ * waste of doing the flush.  This is critical for large problems.
+ */
+   int DOFLUSH = (((size_t)M)*N*ATL_sizeof < ((size_t)flsizeKB)*1024*4);
    double t0, t1;
 /*
  * Call routs that force particular flop count if requested; they return -1.0
@@ -1409,7 +1424,8 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
  */
    A = GetGE(M, N, lda);
    ATL_assert(A);
-   flp = ATL_GetFlushStruct(A, N*lda*ATL_sizeof, NULL);
+   if (DOFLUSH)
+      flp = ATL_GetFlushStruct(A, N*((size_t)lda)*ATL_sizeof, NULL);
    if (rout == LApotrf)
       PosDefGen(CblasColMajor, Uplo_LA2ATL(Uplo), N, A, lda);
    else if (rout & LAgeqrf)
@@ -1436,21 +1452,25 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
       wlen = dtmp;
       wrk = calloc(wlen, ATL_sizeof);
       ATL_assert(wrk);
-      flp = ATL_GetFlushStruct(wrk, wlen*ATL_sizeof, flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(wrk, wlen*ATL_sizeof, flp);
       itmp = (M >= N) ? M : N;
       tau = calloc(itmp, ATL_sizeof);
-      flp = ATL_GetFlushStruct(tau, itmp*ATL_sizeof, flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(tau, itmp*ATL_sizeof, flp);
    }
    else
    {
       ipiv = calloc(M, sizeof(int));
       ATL_assert(ipiv);
-      flp = ATL_GetFlushStruct(ipiv, M*sizeof(int), flp);
+      if (DOFLUSH)
+         flp = ATL_GetFlushStruct(ipiv, M*sizeof(int), flp);
    }
 /*
  * Flush cache, and do timing
  */
-   ATL_FlushAreasByCL(flp);
+   if (DOFLUSH)
+      ATL_FlushAreasByCL(flp);
    if (rout == LApotrf)
    {
       t0 = time00();
@@ -1500,7 +1520,8 @@ double GetTime(int rout, int mflopF, int lda, int M, int N, int nb, int Uplo,
    if (ipiv)
       free(ipiv);
    free(A);
-   ATL_KillAllFlushStructs(flp);
+   if (DOFLUSH)
+      ATL_KillAllFlushStructs(flp);
    return(t1 - t0);
 #endif
 }
@@ -1611,6 +1632,7 @@ void WriteCFile(char *outnam, int N, int *flgs, int *ms, int *ns, int *nbs,
    ATL_assert(ln);
    sprintf(ln, "%s.h", outnam);
    nbout = my_fopen(ln, "w");
+   free(ln);
    nameNoPath = NameExcludingPath(outnam);
    fprintf(nbout, "#ifndef %s\n\n", nameNoPath);
    fprintf(nbout, "/*\n * NB selection for %s: Side='%s', Uplo='%s'\n",
@@ -1740,6 +1762,7 @@ void WriteNbFile(int F77out, char *outnam, int N, int *flgs, int *ms, int *ns,
    else
       ip = CompressDecisionTree(N, ms, nbs);
    WriteCFile(outnam, N, flgs, ms, ns, nbs, ip[0], ip+1+ip[0], ip+1);
+   free(ip);
 }
 
 int GetMyReps(int N, int *nreps)
@@ -1759,6 +1782,38 @@ int GetMyReps(int N, int *nreps)
 }
 
 
+#ifdef SCPLX
+   #define C_PRE 'c'
+#elif defined(DCPLX)
+   #define C_PRE 'z'
+#elif defined(SREAL)
+   #define C_PRE 's'
+#else
+   #define C_PRE 'd'
+#endif
+char *latimeName(int rout, int uplo, int side, int M, int N, int lda, int kb,
+                 int nsample)
+/*
+ * Get the name for a lapack block timing file of form:
+ *   res/<pre><rout><Sd><Up>_<M>x<N>_<lda>_<kb>_<nsample>.tim
+ */
+{
+   char *rn, *on;
+   int len, i;
+   rn = Bitmap2Char(FSRout, (rout<<8)|uplo|side);
+   len = 17 + strlen(rn) + NumDecDigits(M) + NumDecDigits(N) + NumDecDigits(lda)
+         + NumDecDigits(kb) + NumDecDigits(nsample);
+   on = malloc(len*sizeof(char));
+   assert(on);
+   if (rout == LApotrf)
+      i = sprintf(on, "res/%c%s%c_%ux%u_%u_%u_%u.tim", C_PRE, rn,
+                  (uplo==LAUpper)?'U':'L', M, N, lda, kb, nsample);
+   else
+      i = sprintf(on, "res/%c%s_%ux%u_%u_%u_%u.tim", C_PRE, rn, M, N, lda, kb,
+                  nsample);
+   assert(i < len);
+   return(on);
+}
 double GetMultSampleTimes
 (
    int verb,                    /* 0 : no output, else print mflop on line */
@@ -1780,27 +1835,351 @@ double GetMultSampleTimes
 {
    double t0, t1, time, mintime, avgtime;
    int i;
+   char *fn;
    extern int ONB;
+   FILE *fp;
 
-   ONB = nb;
-   avgtime = mintime = 0.0;
-   t0 = time00();
-   for (i=0; i < nsample; i++)
+   fn = latimeName(rout, uplo, side, M, N, lda, nb, nsample);
+   fp = fopen(fn, "r");
+   if (fp)
    {
-      time = GetTime(rout, mflopF, lda, M, N, nb, uplo, side, flshszKB);
-      if (verb)
-         printf("%.2f ", Time2Flops(rout, uplo, M, N, time));
-      avgtime += time;
-      if (mintime)
-         mintime = (mintime <= time) ? mintime : time;
-      else
-         mintime = time;
-      t1 = time00() - t0;
-      if (nsecs && t1 > nsecs) break;
+      avgtime = mintime = 0.0;
+      for (i=0; i < nsample; i++)
+      {
+         if (fscanf(fp, " %le", &time) != 1)
+         {
+            fclose(fp);
+            if (!i)
+               fp = NULL;
+            break;
+         }
+         if (verb)
+            printf("%.2f ", Time2Flops(rout, uplo, M, N, time));
+         avgtime += time;
+         mintime = (mintime <= time && i) ? mintime : time;
+      }
+      if (i == nsample)          /* don't nullify so that */
+         fclose(fp);   /* timing won't be rerun */
    }
+/*
+ * If file didn't exist, or wasn't intelligible, run timing
+ */
+   if (!fp)
+   {
+      fp = fopen(fn, "w");
+      ONB = nb;
+      avgtime = mintime = 0.0;
+      t0 = time00();
+      for (i=0; i < nsample; i++)
+      {
+         time = GetTime(rout, mflopF, lda, M, N, nb, uplo, side, flshszKB);
+         if (verb)
+            printf("%.2f ", Time2Flops(rout, uplo, M, N, time));
+         avgtime += time;
+         if (mintime)
+            mintime = (mintime <= time) ? mintime : time;
+         else
+            mintime = time;
+         t1 = time00() - t0;
+         fprintf(fp, "%le\n", time);
+/*
+ *       Time runs too long, skip extra samples
+ */
+         if (nsecs && t1 > nsecs)
+            break;
+      }
+      fclose(fp);
+   }
+   free(fn);
    return(retavg ? avgtime/i : mintime);
 }
 
+
+#include Mstr(Mjoin(Mjoin(atlas_,PRE),opgen_view.h))
+#include Mstr(Mjoin(Mjoin(atlas_,PRE),ipgen_view.h))
+
+ATL_UINT fastopIdxInRange(ATL_UINT minIdx, ATL_UINT maxIdx)
+/*
+ * RETURNS: index of smallest SPF in opgen_view within idx range [min,max]
+ */
+{
+   unsigned int i, idx, imin=minIdx;
+   float minSPF=2e16;
+
+   for (i=minIdx; i <= maxIdx; i++)
+   {
+      const unsigned int *ip;
+      const unsigned int idx = ATL_VWopgen_IDXMUL(i);
+      float spf;
+      ip = ATL_VIEW_opgen + idx;
+      spf = *((float*)ip);
+      if (spf < minSPF)
+      {
+         minSPF = spf;
+         imin = i;
+      }
+   }
+   assert(minSPF != 2e16);
+   return(imin);
+}
+
+int ipfindNB  /* RETURNS: ipgen KB to use for this problem */
+(
+   int rout,                    /* routine to time */
+   int nreps,                   /* # of times to repeat timing */
+   int nsecs,                   /* max # secs to spend repeating */
+   int flshszKB,                /* KB to flush */
+   int mflopF,                  /* mflops to force in one timing*/
+   int uplo, int side,
+   int M, int N, int lda,       /* prob dims */
+   int minKB, int maxKB,        /* min/max KB to try with opgen range */
+   double *TIMB                 /* if non-NULL OUTPUT: min time */
+)
+{
+   ATL_UINT i, kbB = Mmin(M,N)>>1;
+   double tim, timB=0.0, tim0=0.0, tim1=0.0;
+   maxKB = Mmin(maxKB, kbB);
+   maxKB = Mmin(maxKB, 1024);
+   kbB = 0;
+   if (minKB < ATL_VWipgen_BEST_KB)
+   {
+      for (i=0; i < ATL_VWipgen_NCASES; i++)
+      {
+         ATL_UINT kb;
+         kb = ATL_GetVWipgenKB(i);
+         if (kb+kb >= minKB)
+            break;
+      }
+      for (; i < ATL_VWipgen_NCASES; i++)
+      {
+         ATL_UINT kb;
+         kb = ATL_GetVWipgenKB(i);
+         kb += kb;
+         if (kb > maxKB)
+            break;
+         tim = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                                   uplo, side, M, N, lda, kb);
+         printf("      nb=%d: %.2f MFLOP.\n", kb,Time2Flops(rout,uplo,M,N,tim));
+         if (!kbB || tim < timB)
+         {
+            kbB = kb;
+            timB = tim;
+         }
+         tim0 = tim1;
+         tim1 = tim;
+      }
+   }
+   if (3*ATL_VWipgen_BEST_KB <= maxKB)
+   {
+      ATL_UINT kb;
+      kb=ATL_VWipgen_BEST_KB*3;
+      do
+      {
+         kb += ATL_VWipgen_BEST_KB;
+         tim = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                                   uplo, side, M, N, lda, kb);
+         printf("      nb=%d: %.2f MFLOP.\n", kb,Time2Flops(rout,uplo,M,N,tim));
+         if (!kbB || tim < timB)
+         {
+            kbB = kb;
+            timB = tim;
+         }
+         else if (tim > tim1 && tim > tim0 && tim > timB*1.03 && tim0 != 0.0)
+             break;
+         tim0 = tim1;
+         tim1 = tim;
+      }
+      while (kb <= maxKB);
+   }
+   if (TIMB)
+      *TIMB = timB;
+   return(kbB);
+}
+
+int opfindNB  /* RETURNS: opgen KB to use for this problem */
+(
+   int rout,                    /* routine to time */
+   int nreps,                   /* # of times to repeat timing */
+   int nsecs,                   /* max # secs to spend repeating */
+   int flshszKB,                /* KB to flush */
+   int mflopF,                  /* mflops to force in one timing*/
+   int uplo, int side,
+   int M, int N, int lda,       /* prob dims */
+   int minKB, int maxKB,        /* min/max KB to try with opgen range */
+   double *TIMB                 /* if non-NULL OUTPUT: min time */
+)
+/*
+ * RETURNS: best KB for this problem size in KBs within [minIdx,maxIdx] range.
+ * IDEA: scope view
+ */
+{
+   unsigned int kb = Mmin(M,N)>>1, kbB=0, inc, i;
+   double tim, tim0=0.0, tim1=0.0, timB = 0.0;
+   if (!kb)
+      return(1);
+   maxKB = Mmin(kb, maxKB);
+   maxKB = Mmin(maxKB, ATL_VWopgen_MAX_KB);
+   printf("   FINDING KB [%u,%u] FOR M=%d, N=%d, LDA=%d:\n", minKB, maxKB,
+          M, N, lda);
+
+/*
+ * Check KB=1,2, which are GER & GER2 respectively
+ */
+   if (minKB < 3)
+   {
+      kbB = 1;
+      timB = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                                uplo, side, M, N, lda, 1);
+      printf("      nb=%d: %.2f MFLOP.\n", 1, Time2Flops(rout,uplo,M,N,timB));
+      tim0 = timB;
+      tim1 = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                                uplo, side, M, N, lda, 2);
+      printf("      nb=%d: %.2f MFLOP.\n", 2, Time2Flops(rout,uplo,M,N,tim1));
+      if (tim1 < timB)
+      {
+         timB = tim1;
+         kbB = 2;
+      }
+   }
+   assert(maxKB > minKB);
+   minKB = (minKB <= 3) ? 0 : minKB-3;
+   maxKB = (maxKB > 3) ? maxKB-3 : 0;
+/*
+ * See how course to do initial search
+ */
+   inc = maxKB - minKB;
+   if (inc <= 4)
+      inc = 1;
+   else if (inc <= 8)
+      inc = 2;
+   else if (inc <= 32)
+      inc = 4;
+   else if (inc <= 64)
+      inc = 8;
+   else if (inc <= 128)
+      inc = 16;
+   else
+      inc = 32;
+
+   for (i=minKB; i < maxKB; i += inc)
+   {
+      unsigned int idx;
+      idx = fastopIdxInRange(i, i+inc);
+      kb = i+3;
+      tim = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                               uplo, side, M, N, lda, kb);
+      printf("      nb=%d: %.2f MFLOP.\n", kb, Time2Flops(rout,uplo,M,N,tim));
+      if (!kbB || tim < timB)
+      {
+         timB = tim;
+         kbB = kb;
+      }
+/*
+ *    Bail if present time noticably slower than best, and we have been going
+ *    down in performance for last 3 tries
+ */
+      if (tim > 1.03*timB && tim > tim1 && tim > tim0)
+         if (timB > 0.0 && tim1 > 0.0 && tim0 > 0.0)
+            break;
+      tim0 = tim1;
+      tim1 = tim;
+   }
+/*
+ * Refine inc
+ */
+   while (inc > 1)
+   {
+      maxKB = minKB + inc;
+      minKB = (minKB >= inc) ? minKB-inc : 0;
+      inc = (inc >= 16) ? (inc>>2) : (inc>>1);
+      for (i=minKB; i < maxKB; i += inc)
+      {
+         unsigned int idx;
+         idx = fastopIdxInRange(i, i+inc);
+         kb = idx+3;
+         tim = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                                  uplo, side, M, N, lda, kb);
+         printf("      nb=%d: %.2f MFLOP.\n", kb,Time2Flops(rout,uplo,M,N,tim));
+         if (!kbB || tim < timB)
+         {
+            timB = tim;
+            kbB = kb;
+         }
+      }
+   }
+   if (TIMB)
+      *TIMB = timB;
+   assert(kbB);
+   return(kbB);
+}
+
+/*
+ * RETURNS: best-performing block factor, starting with best-performing
+ *          opgen, and then trying any 2*KB larger than that found in ipgen
+ */
+ATL_UINT findMaxKB              /* RETURNS: best performing KB */
+(
+   int rout,                    /* routine to time */
+   int nreps,                   /* # of times to repeat timing */
+   int nsecs,                   /* max # secs to spend repeating */
+   int flshszKB,                /* KB to flush */
+   int mflopF,                  /* mflops to force in one timing*/
+   int uplo, int side,
+   int M, int N, int lda        /* prob dims */
+)
+{
+   double timB, tim, tim0, tim1;
+   ATL_UINT kbB=ATL_VWopgen_BEST_KB, kb, KB, i;
+   ATL_CUINT maxKB=Mmin(M,N)>>1;
+
+   if (kbB > (N>>1))  /* if N too small for large blockings */
+      return(kbB);    /* just say search with opgen_view */
+/*
+ * Try best-performing outer-product KB
+ */
+   printf("   FINDING MAX KB FOR M=%d, N=%d, LDA=%d:\n", M, N, lda);
+   timB = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                             uplo, side, M, N, lda, kbB);
+   printf("      nb=%d: %.2f MFLOP.\n", kbB, Time2Flops(rout,uplo,M,N,timB));
+   tim0 = tim1 = timB;
+/*
+ * If best inner-product KB is larger than outer-product, try it even though
+ * we'll call ipgemm with outer-product formulation
+ */
+   if (ATL_VIEW_BEST_KB > kbB)
+   {
+      tim = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                               uplo, side, M, N, lda, ATL_VIEW_BEST_KB);
+      printf("      nb=%d: %.2f MFLOP.\n", ATL_VIEW_BEST_KB,
+             Time2Flops(rout, uplo, M, N, tim));
+      if (tim < timB)
+      {
+         timB = tim;
+         kbB = ATL_VIEW_BEST_KB;
+      }
+      tim1 = tim;
+   }
+   kb = ipfindNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side, M, N, lda,
+                 ATL_VWopgen_BEST_KB, 1024, &tim);
+   if (tim < timB)
+   {
+      kbB = kb;
+      timB = tim;
+   }
+   if (kbB <= ATL_VWopgen_BEST_KB)
+   {
+      kb = opfindNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side, M, N, lda,
+                    1, kbB, &tim);
+      if (tim < timB)
+      {
+         kbB = kb;
+         timB = tim;
+      }
+   }
+   printf("   MAX KB=%u, MFLOP=%.2f\n", kbB,Time2Flops(rout, uplo, M, N, timB));
+   return(kb);
+}
+
 int findNB
 (
    int rout,                    /* routine to time */
@@ -1810,156 +2189,50 @@ int findNB
    int mflopF,                  /* mflops to force in one timing*/
    int uplo, int side,
    int M, int N, int lda,       /* prob dims */
-   int minNB, int maxNB,        /* smallest & largest NB to try */
-   double *tLeft,               /* % time minNB took for this problem size */
-   double *tRight               /* time maxNB took for this problem size */
+   int minKB, int maxKB,        /* smallest & largest KB to try */
+   double *spdMin,              /* time(minKB) / time(bestKB) */
+   double *spdMax               /* time(maxKB) / time(bestKB) */
 )
 /*
- * tLeft is the time minNB took for this problem size as a percentage of
- * the minimum time (1.02 would be 2% slower); tRight same for maxNB
- * used to determine if the blocking factors are really different
+ * Find best KB for this exact problem size, assuming in range [minKB,maxKB]
  */
 {
-   int i, nbB, n, mul1, mul2, nbL, nbR, TINYCASE;
-   double time, timeL, timeR, mintime, lasttime;
-
-   timeL = timeR = 0.0;
-   printf("   FINDING NB FOR M=%d, N=%d, LDA=%d:\n", M, N, lda);
-/*
- * Try all multiples of NB between min & max NB
- */
-   TINYCASE = Mmin(M,N) < 2*NB;
-   i = (minNB/NB)*NB;  /* smallest mul of NB >= floor(min,NB) */
-   if (i < 1)
-      i = NB;
-   nbB = Mmin(M,N)/2;
-   if (!nbB)
-      nbB = 1;
-   if (nbB > i)
-      nbB = i;
+   int kb=0;
+   double tmin, timeL=0.0, timMax, timMin;
+   printf("SEARCHING KB RANGE [%u,%u] for D=(%u,%u), LD=%u\n", minKB, maxKB,
+          M, N, lda);
+   timMin = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                               uplo, side, M, N, lda, minKB);
+   timMax = GetMultSampleTimes(0, 0, nreps, nsecs, flshszKB, mflopF, rout,
+                               uplo, side, M, N, lda, minKB);
+   if (minKB <= ATL_VWopgen_MAX_KB)
+      kb = opfindNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side, M, N, lda,
+                    minKB, maxKB, &timeL);
+   if (maxKB > ATL_VWopgen_MAX_KB)
+   {
+      int kbR;
+      kbR = ipfindNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side, M, N,
+                     lda, minKB, maxKB, &tmin);
+      if (kb)
+      {
+         if (!kbR || tmin > timeL)
+            tmin = timeL;
+         else
+            kb = kbR;
+      }
+      else
+         kb = kbR;
+   }
    else
-      i = nbB;
-   printf("      nb=%d: ", i);
-   mintime = GetMultSampleTimes(1, 0, nreps, nsecs, flshszKB, mflopF, rout,
-                                uplo, side, M, N, lda, i);
-   printf("--> %.2f MFLOP.\n", Time2Flops(rout, uplo, M, N, mintime));
-   n = Mmin(N,M)>>1;
-   n = Mmin(maxNB, n);
-   n = (n/NB)*NB;
-   for (i += NB; i <= n; i += NB)
-   {
-      printf("      nb=%d: ", i);
-      time = GetMultSampleTimes(1, 0, nreps, nsecs, flshszKB, mflopF, rout,
-                                uplo, side, M, N, lda, i);
-      printf("--> %.2f MFLOP.\n", Time2Flops(rout, uplo, M, N, time));
-      if (i == minNB)
-        timeL = time;
-      if (i == maxNB)
-         timeR = time;
-      if (mintime*1.05 < time) break;  /* quit once we see big perf loss */
-/*
- *    Force larger NB to be significantly faster before we use it
- */
-      if (time < mintime)
-      {
-         if (i < nbB || time*1.01 < mintime)
-         {
-            mintime = time;
-            nbB = i;
-         }
-      }
-   }
-
-   if (nbB <= 3*NB)  /* don't bother to refine for very large NB */
-   {
-      printf("\n      NB refinement search:\n");
-/*
- *    Try all multiples of lcm(MU,NU) the NBs on either side of the best found
- *    NB.  If lcm is large, and nbB is small, also try all modulo 4 cases.
- *    Require non-multiples of NB to be noticiably better than multiples
- */
-      mul1 = ATL_lcm(ATL_mmMU, ATL_mmNU);
-      n = Mmin(M,N);
-      mul2 = (nbB >= 2*NB || n > 1000) ? 8 : 4;  /* granularity of refinement */
-      mul2 = (nbB >= 3*NB) ? 12 : mul2; /* make more granular for huge NB */
-      while (mul1 < mul2)
-         mul1 *= 2;
-      if (mul1 > 6 && nbB <= NB && n <= 1000)
-         mul2 = 4;   /* now, mul2 is a second thing to try beyond mul1 */
-      else
-         mul2 = 0;
-      n = Mmin(nbB,N);
-      if (mul2)
-         n = nbB + NB - mul2;
-      else
-         n = nbB + NB - mul1;
-      if (n > N/2)
-         n = N/2;
-      if (n > M/2)
-         n = M/2;
-      i = nbB - NB + mul1;
-      if (i < 1) i = 1;
-      printf("      *** i=%d, nbB=%d, NB=%d, mul1=%d, mul2=%d ***\n",
-             i, nbB, NB, mul1, mul2);
-      if (TINYCASE)
-      {
-         i = 1;
-         n = Mmin(M,N)/2;
-         mul1 = 1;
-         mul2 = 0;
-      }
-      for (; i <= n; i++)
-      {
-         if (i == nbB) continue;  /* already timed */
-         if (mul2)
-         {
-            if (i%mul2 && i%mul1) continue;
-         }
-         else if (i%mul1) continue;
-
-         printf("      nb=%d: ", i);
-         time = GetMultSampleTimes(1, 0, nreps, nsecs, flshszKB, mflopF, rout,
-                                   uplo, side, M, N, lda, i);
-         printf(" --> %.2f MFLOP.\n", Time2Flops(rout, uplo, M, N, time));
-         if (i == minNB)
-           timeL = time;
-         if (i == maxNB)
-            timeR = time;
-         if (time*1.02 < mintime)  /* non-mul-NB must be much better */
-         {
-            mintime = time;
-            nbB = i;
-         }
-      }
-   }
-   if (timeL == 0.0)
-   {
-      printf("      nb=%d: ", minNB);
-      timeL = GetMultSampleTimes(1, 0, nreps, nsecs, flshszKB, mflopF, rout,
-                                 uplo, side, M, N, lda, minNB);
-      printf(" --> %.2f MFLOP.\n", Time2Flops(rout, uplo, M, N, timeL));
-      if (timeL < mintime)
-      {
-         mintime = timeL;
-         nbB = minNB;
-      }
-   }
-   if (timeR == 0.0)
-   {
-      printf("      nb=%d: ", maxNB);
-      timeR = GetMultSampleTimes(1, 0, nreps, nsecs, flshszKB, mflopF, rout,
-                                 uplo, side, M, N, lda, maxNB);
-      printf(" --> %.2f MFLOP.\n", Time2Flops(rout, uplo, M, N, timeR));
-      if (timeR < mintime)
-      {
-         mintime = timeR;
-         nbB = maxNB;
-      }
-   }
-   *tLeft  = timeL / mintime;
-   *tRight = timeR / mintime;
-   printf("   FOR M=%d, N=%d, LDA=%d, BEST NB=%d\n\n", M, N, lda, nbB);
-   return(nbB);
+      tmin = timeL;
+   assert(kb);
+   if (spdMin)
+      *spdMin = timMin / tmin;
+   if (spdMax)
+      *spdMax = timMax / tmin;
+   printf("BEST KB=%u (%.2fMFLOP) for D=(%u,%u), LD=%u\n", kb,
+          Time2Flops(rout, uplo, M, N, tmin), M, N, lda);
+   return(kb);
 }
 
 int *FindAllNBs
@@ -1983,14 +2256,14 @@ int *FindAllNBs
  * boundary points
  */
 {
-   int m, n, k, lda, i, iM;
+   int m, n, k, lda, i, iM, kb;
    int *tp;
    double Lperc, Rperc;
 
-   k = (nnb[iR] + nnb[iL])>>1;
-   k = (k/NB)*NB;          /* keep N a mul of NB to avoid cleanup affects */
-   if (k-nnb[iL] < NB || nnb[iR]-k < NB)
-      return(nnb);   /* don't get finer grained than NB in search */
+   k = (nnb[iR] - nnb[iL])>>1;  /* gap between endpoints */
+   if (k < 32)                  /* if within 32, close enough */
+      return(nnb);              /* return done */
+   k += nnb[iL];
    m = (restM) ? restM : k;
    n = (restN) ? restN : k;
    lda = m + ldagap;
@@ -2012,11 +2285,11 @@ int *FindAllNBs
  */
    iM = i;
    nnb[iM] = k;
-   nnb[iM+1] = findNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,
-                      m, n, lda, nnb[iL+1], nnb[iR+1], &Lperc, &Rperc);
+   kb = findNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,
+               m, n, lda, nnb[iL+1], nnb[iR+1], &Lperc, &Rperc);
+   nnb[iM+1] = kb;
    nnb[1] = i + 2;
-   printf("   M=%d, N=%d, lda=%d, NB=%d\n", m, n, lda, nnb[iM+1]);
-
+   printf("   M=%d, N=%d, lda=%d, NB=%d\n", m, n, lda, kb);
 /*
  * If middle NB greater than left NB, and loss of using different NB is more
  * than 2%, must search space between
@@ -2056,32 +2329,39 @@ int *FindNBByN
  */
 {
    int *nnb;
-   int m, n, lda;
+   int i, m, n, lda, kbmax;
    double t0, tN, percL, percR;
+   m = (restM) ? restM : maxN;
+   n = (restN) ? restN : maxN;
+   lda = m + ldagap;
+   kbmax = findMaxKB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,m,n,lda);
 /*
  * Find best NB for minimum problem (dimension 4)
  */
    nnb = malloc(sizeof(int)*4000);
-   nnb[2] = 25;
-   m = (restM) ? restM : 25;
-   n = (restN) ? restN : 25;
-   lda = m + ldagap;
-   nnb[3] = findNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,
-                   m, n, lda, 1, 12, &percL, &percR);
-/*
- * Find best NB for maximum size problem
- */
-   nnb[4] = maxN;
-   m = (restM) ? restM : maxN;
-   n = (restN) ? restN : maxN;
-   lda = m + ldagap;
-   nnb[5] = findNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side, m, n, lda,
-                   nnb[3], 10*NB, &percL, &percR);
-
    nnb[0] = 4000;   /* length of array in pos 0 */
    nnb[1] = 6;      /* number of entries used so far in pos 1 */
+   nnb[2] = 25;
+   nnb[4] = maxN;
+   nnb[5] = kbmax;
+   m = (restM) ? restM : 24;
+   n = (restN) ? restN : 24;
+   lda = m + ldagap;
+   nnb[3] = findNB(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,
+                   m, n, lda, 1, kbmax, NULL, NULL);
    nnb = FindAllNBs(rout, nreps, nsecs, flshszKB, mflopF, uplo, side,
                     restM, restN, ldagap, 2, 4, nnb);
+   #if 0
+/*
+ * Translate indices into actual NBs
+ */
+   for (n=nnb[1], i=3; i < n; i += 2)
+   {
+      int j = nnb[i];
+      assert(j >= 0 && j < ATL_AMM_NCASES);
+      nnb[i] = ATL_AMM_KBs[j];
+   }
+#endif
    return(nnb);
 }
 
@@ -2145,8 +2425,6 @@ int TransNNB(int rout, int restM, int restN, int side, int uplo, int *nnb,
    ATL_assert(ms);
    ns = malloc(sizeof(int)*n);
    ATL_assert(ns);
-   ns = malloc(sizeof(int)*n);
-   ATL_assert(ns);
    nbs = malloc(sizeof(int)*n);
    ATL_assert(nbs);
    flags = malloc(sizeof(int)*n);
@@ -2188,11 +2466,15 @@ int main(int nargs, char **args)
             &maxN, &UPLOs, &SIDEs, &F77out, &outnam, &restM, &restN);
    nnb = FindNBByN(rout, nsample, nsecs, flsizeKB, mflopF, SIDEs[1],
                    UPLOs[1], maxN, restM, restN, ldagap);
+   if (!nnb)
+      return(-1);
    SortNNB(nnb);
    PrintNNB(nnb);
    n = TransNNB(rout, restM, restN, SIDEs[1], UPLOs[1], nnb,
                 &flags, &Ms, &Ns, &NBs);
    free(nnb);
+   free(UPLOs);
+   free(SIDEs);
    WriteNbFile(F77out, outnam, n, flags, Ms, Ns, NBs);
    free(Ms);
    free(Ns);
