@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 
 #include <stdio.h>
@@ -35,6 +10,7 @@
 #include <assert.h>
 #include "atlas_type.h"
 #include "atlas_fopen.h"
+#include "atlas_sys.h"
 int GetL1CacheSize(int MaxL1Size)
 {
    FILE *L1f;
@@ -48,12 +24,28 @@ int GetL1CacheSize(int MaxL1Size)
    }
    L1f = fopen("res/L1CacheSize", "r");
    assert(L1f != NULL);
-   fscanf(L1f, "%d", &L1Size);
+   assert(fscanf(L1f, "%d", &L1Size) == 1);
    fclose(L1f);
    fprintf(stderr, "\n      Read in L1 Cache size as = %dKB.\n",L1Size);
    return(L1Size);
 }
 
+size_t GetPageSize(void)
+{
+   size_t pgsz=4096;
+   char *sp;
+   sp = atlsys_1L(NULL, "getconf PAGESIZE", 0, 1);
+   if (!sp)
+      sp = atlsys_1L(NULL, "getconf PAGE_SIZE", 0, 1);
+   if (sp)
+   {
+      unsigned long sz;
+      if (sscanf(sp, "%lu", &sz) == 1)
+         pgsz = sz;
+      free(sp);
+   }
+   return(pgsz);
+}
 void getfpinfo0(char pre, int *muladd, int *lat, int *lbnreg, double *mf)
 {
    char fnam[128];
@@ -102,7 +94,7 @@ void getfpinfo(char pre, int *muladd, int *lat, int *lbnreg, int *nkflop)
 }
 
 void CreateHeader(char pre, char *fnam, int L1Size, int muladd, int lat,
-                  int lbnreg, int nkflop, int mmnreg)
+                  int lbnreg, int nkflop, int mmnreg, size_t pgsz)
 {
    FILE *fpout;
 
@@ -112,11 +104,11 @@ void CreateHeader(char pre, char *fnam, int L1Size, int muladd, int lat,
            toupper(pre), toupper(pre));
    if (muladd) fprintf(fpout, "#define ATL_MULADD\n");
    else fprintf(fpout, "#define ATL_NOMULADD\n");
-   fprintf(fpout, "#define ATL_L1elts %d\n", L1Size);
    fprintf(fpout, "#define ATL_fplat  %d\n", lat);
    fprintf(fpout, "#define ATL_lbnreg %d\n", lbnreg);
    fprintf(fpout, "#define ATL_mmnreg %d\n", mmnreg);
    fprintf(fpout, "#define ATL_nkflop %d\n", nkflop);
+   fprintf(fpout, "#define ATL_pgsz %lu\n", pgsz);
    fprintf(fpout, "\n#endif\n");
    fclose(fpout);
 }
@@ -138,13 +130,14 @@ int getmmnreg(char pre)
    }
    fp = fopen(fnam, "r");
    assert(fp);
-   fscanf(fp, " %d", &mmnregs);
+   assert(fscanf(fp, " %d", &mmnregs) == 1);
    fclose(fp);
    return(mmnregs);
 }
 
 int main(int nargs, char **args)
 {
+   size_t pgsz;
    int MaxL1Size;
    int muladd, lat, lbnreg, L1Size, mmnreg, nkflop;
    FILE *fpout;
@@ -157,14 +150,10 @@ int main(int nargs, char **args)
    }
    pre = *args[1];
 
-   L1Size = 1024 * GetL1CacheSize(64);
-   if (pre == 'd') L1Size /= ATL_dsize;
-   else if (pre == 's') L1Size /= ATL_ssize;
-   else if (pre == 'z') L1Size /= ATL_csize;
-   else if (pre == 'c') L1Size /= ATL_zsize;
    getfpinfo(pre, &muladd, &lat, &lbnreg, &nkflop);
-   CreateHeader(pre, args[2], L1Size, muladd, lat, lbnreg, nkflop, 0);
+   pgsz = GetPageSize();
+   CreateHeader(pre, args[2], L1Size, muladd, lat, lbnreg, nkflop, 0, pgsz);
    mmnreg = getmmnreg(pre);
-   CreateHeader(pre, args[2], L1Size, muladd, lat, lbnreg, nkflop, mmnreg);
+   CreateHeader(pre, args[2], L1Size, muladd, lat, lbnreg, nkflop, mmnreg,pgsz);
    return(0);
 }
