@@ -1,31 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.9.51
- *                    (C) Copyright 1997 R. Clint Whaley
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *   1. Redistributions of source code must retain the above copyright
- *      notice, this list of conditions and the following disclaimer.
- *   2. Redistributions in binary form must reproduce the above copyright
- *      notice, this list of conditions, and the following disclaimer in the
- *      documentation and/or other materials provided with the distribution.
- *   3. The name of the ATLAS group or the names of its contributers may
- *      not be used to endorse or promote products derived from this
- *      software without specific written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
+ * Automatically Tuned Linear Algebra Software v3.11.41
+ * (C) Copyright 1997 R. Clint Whaley
  */
 
 #include <stdio.h>
@@ -34,11 +9,13 @@
 #define REPS 4096
 
 #ifndef time00
-   #define time00 ATL_cputime
+   #define time00 ATL_walltime
 #endif
-double time00();
+double time00(void);
 
-// #define USE_OLD
+#if 0
+#define USE_OLD
+#endif
 #ifdef USE_OLD
 int L1CacheSize_read(int MAXL1CACHE, double tol)
 {
@@ -58,6 +35,11 @@ int L1CacheSize_read(int MAXL1CACHE, double tol)
       dlen = (dlen >> 3)<<3;
       len = dlen * 8;
       d = p = malloc(len);
+      if (p == NULL)
+      {
+         fprintf(stderr, "malloc failed, len=%d\n", len);
+         assert(p);
+      }
       st = p + dlen;
 
       for (k=0; k != dlen; k += 8)
